(*
DockableForm.pas
Copyright (C) 2006  Christian Brugger <brugger.christian(AT)web.de>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version. This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
details. You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA 02110, USA
*)
unit DockableForm;

interface

uses Forms, Classes;

type
  TDockableForm = class(TForm)
  private
    FNameIndex: integer;
  public
    constructor Create(AOwner: TComponent); override;
    property NameIndex: integer read FNameIndex write FNameIndex;
    procedure Check; virtual; abstract;
    procedure UnCheck; virtual; abstract;
    procedure RefreshForm; virtual; abstract;
    procedure FormShow(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  end;

implementation

constructor TDockableForm.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FNameIndex := -1;
  OnShow := FormShow;
  OnClose := FormClose;
end;

procedure TDockableForm.FormShow(Sender: TObject);
begin
  Check;
  RefreshForm;
  Tag := 0;
end;

procedure TDockableForm.FormHide(Sender: TObject);
begin
  if Tag = 2 then // means that Form will be Closed!
    Tag := 0
  else Tag := 1;
end;

procedure TDockableForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  UnCheck;
  if Visible then // if visible, OnHide will be called
    Tag := 2
  else Tag := 0;
end;

end.
