(*
DynDockPanel.pas
Copyright (C) 2006  Christian Brugger <brugger.christian(AT)web.de>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version. This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
details. You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA 02110, USA
*)
unit DynDockPanel;

interface

uses ExtCtrls, Classes, Controls, Contnrs, DockManagerPro, Windows, Forms;

type

  { Forward declarations }

  TSubDockPanel = class;
  TDynDockPanel = class;
  TDynClientPanel = class;

  {DynDock declarations}

  TDynDockPanel = class(TPanel)
  private
    FSubList: TObjectList;
    procedure AddSuroundingPanels(ASubDockPanel: TSubDockPanel);
    procedure RemoveSuroundingPanels(ASubDockPanel: TSubDockPanel);
    procedure SortPanels;
    function VisibleFormCount: integer;
    procedure ResetAllBounds;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SaveToStream(AStream: TStream);
    procedure LoadFromStream(AStream: TStream);
  end;


  TSubDockPanel = class(TObject)
  private
    // to get a horizontal Bar we need a Panel, wich has a Left or Right align
    // but we want to arange the SubDockPanels from Top to Bottom, so we need
    // a HostPanel with the align alTop and a ClientPanel with the align alLeft
    FHostPanel: TPanel;
    FClientPanel: TDynClientPanel;
    // if the Panel is the last one, its align must be alClient to fill the
    // remaining space
    FIsLast: boolean;
    // We uses this DockManager to have a nicer Bar
    FDockManagerPro: TDockManagerPro;
    // To manager Panels under themselves, we need pointers to the
    // ancestor and successor and to the DynDockPanel
    FAncPanel, FSuccPanel: TSubDockPanel;
    FDynDockPanel: TDynDockPanel;
    procedure ClientPanelDockOver(Sender: TObject; Source: TDragDockObject;
      X, Y: Integer; State: TDragState; var Accept: Boolean);
    procedure ClientPanelDockDrop(Sender: TObject; Source: TDragDockObject;
      X, Y: Integer);
    procedure ClientPanelUnDock(Sender: TObject; Client: TControl;
      NewTarget: TWinControl; var Allow: Boolean);

    procedure FormShowMod(Sender: TObject);
    procedure FormHideMod(Sender: TObject);
    procedure FormCloseMod(Sender: TObject; var Action: TCloseAction);

    procedure SetIsLast(AIsLast: boolean);
    function GetHeight: integer;
    procedure SetHeight(AHeight: integer);
    function GetTop: integer;
    procedure SetTop(ATop: integer);
  public
    constructor Create(AOwner: TComponent; AIsLast: boolean); reintroduce;
    destructor Destroy; override;
    property AncPanel: TSubDockPanel read FAncPanel write FAncPanel;
    property SuccPanel: TSubDockPanel read FSuccPanel write FSuccPanel;
    property DynDockPanel: TDynDockPanel read FDynDockPanel write FDynDockPanel;
    property IsLast: boolean read FIsLast write SetIsLast;
    property Height: integer read GetHeight write SetHeight;
    property Top: integer read GetTop write SetTop;
  end;

  TDynClientPanel = class(TPanel)
  private
    FSubDockPanel: TSubDockPanel;
  public
    property SubDockPanel: TSubDockPanel read FSubDockPanel write FSubDockPanel;
  end;

implementation

uses Haupt, DockableForm, OptionsGeneral, OptionsStandardModules,
  Options, OptionsClockGenerator;

{--------------------- TDynDockPanel main declarations ------------------------}

	constructor TDynDockPanel.Create(AOwner: TComponent);
	var ASubPanel: TSubDockPanel;
	begin
		inherited Create(AOwner);
		FSubList := TObjectList.Create;

		self.Parent := AOwner as TWinControl;
		self.Width := 0;
		self.BevelOuter := bvNone;

		ASubPanel := TSubDockPanel.Create(self, true);
		FSubList.Add(ASubPanel);
	end;
	
	destructor TDynDockPanel.Destroy;
	begin
		FSubList.Free;
		inherited Destroy;
	end;

	procedure TDynDockPanel.AddSuroundingPanels(ASubDockPanel: TSubDockPanel);
  var AAncPanel, ASuccPanel: TSubDockPanel;
  begin                             
                                    // create new ancestor
    AAncPanel := TSubDockPanel.Create(self, false);
		with AAncPanel do
		begin
			AncPanel := ASubDockPanel.AncPanel;
			SuccPanel := ASubDockPanel;
      Height := 3;
      Top := ASubDockPanel.Top;
		end;
		FSubList.Add(AAncPanel);

                                    // create new successor
    ASuccPanel := TSubDockPanel.Create(self, ASubDockPanel.IsLast);
		with ASuccPanel do
		begin
			AncPanel := ASubDockPanel;
			SuccPanel := ASubDockPanel.SuccPanel;
      Height := 3;
      Top := ASubDockPanel.Top + ASubDockPanel.Height;
		end;
		FSubList.Add(ASuccPanel);

                                    
    if ASubDockPanel.AncPanel <> Nil then   // modify old ancestor
      ASubDockPanel.AncPanel.SuccPanel := AAncPanel;
    if ASubDockPanel.SuccPanel <> Nil then  // modify old successor
      ASubDockPanel.SuccPanel.AncPanel := ASuccPanel;
    with ASubDockPanel do                   // modify middle SubPanel
    begin
      AncPanel := AAncPanel;
      SuccPanel := ASuccPanel;
      IsLast := false;
    end;
  end;

	procedure TDynDockPanel.RemoveSuroundingPanels(ASubDockPanel: TSubDockPanel);
  var AAncPanel, ASuccPanel: TSubDockPanel;
  begin
    AAncPanel := ASubDockPanel.AncPanel;
    ASuccPanel := ASubDockPanel.FSuccPanel;

    with ASubDockPanel do                       // modify middle Panel
    begin
      AncPanel := AncPanel.AncPanel;
      SuccPanel := SuccPanel.SuccPanel;
      if SuccPanel <> Nil then
        Height := 3        // not last one
      else IsLast := true; // last one
    end;
    if AAncPanel.AncPanel <> Nil then           // modify new Anc
      AAncPanel.AncPanel.SuccPanel := ASubDockPanel;
    if ASuccPanel.SuccPanel <> Nil then         // modify new Succ
      ASuccPanel.SuccPanel.AncPanel := ASubDockPanel;

    FSubList.Extract(AAncPanel);
    if AAncPanel <> Nil then AAncPanel.Free;    // destroy old Anc
    FSubList.Extract(ASuccPanel);
    if ASuccPanel <> Nil then ASuccPanel.Free;  // destroy old Succ
  end;

  procedure TDynDockPanel.SortPanels;
  var I: integer;
    AObject: TSubDockPanel;
  begin
    I := FSubList.Count - 1;
    // find Last Object in List;
    while (I >= 0) and not ((FSubList[I] as TSubDockPanel).FIsLast) do
      dec(I);
    // if not Found exit
    if I < 0 then
      exit;

    AObject := FSubList[I] as TSubDockPanel;
    while AObject.AncPanel <> Nil do
    begin
      FSubList.Extract(AObject);
      FSubList.Insert(0, AObject);
      AObject := AObject.AncPanel;
    end;
    FSubList.Extract(AObject);
    FSubList.Insert(0, AObject);
  end;

  function TDynDockPanel.VisibleFormCount;
  var I: integer;
  begin
    Result := 0;
    for I := 0 to FSubList.Count - 1 do
      inc(Result, (FSubList[I] as TSubDockPanel).FClientPanel.VisibleDockClientCount);
  end;

  procedure TDynDockPanel.SaveToStream(AStream: TStream);
  var I: integer;
    ANameIndex: byte;
  begin
    SortPanels;

    I := 1;
    while I < FSubList.Count do
    begin
      ANameIndex := ((FSubList[I] as TSubDockPanel).FClientPanel.DockClients[0] as TDockableForm).NameIndex;
      AStream.Write(ANameIndex, SizeOf(ANameIndex));
      inc(I, 2);
    end;
  end;

  procedure TDynDockPanel.ResetAllBounds;
  var i: Integer;
  begin
    for i := 0 to FSubList.Count - 1 do
      (FSubList[I] as TSubDockPanel).FClientPanel.DockManager.ResetBounds(true);
  end;

  procedure TDynDockPanel.LoadFromStream(AStream: TStream);
  var ANameIndex: byte;
    ADockableForm: TDockableForm;
  begin
    AStream.Position := 0;
    while AStream.Position + SizeOf(ANameIndex) <= AStream.Size do
    begin
      SortPanels;
      AStream.Read(ANameIndex, SizeOf(ANameIndex));
      case ANameIndex of
        0: ADockableForm := FormOptionsGeneral;
        1: ADockableForm := FormOptionsStandardModules;
        2: ADockableForm := FormOptionsClockGenerator;
        else ADockableForm := Nil;
      end;

      if ADockableForm <> Nil  then
      with ADockableForm do
        begin
          Left := -1000; // move outside the sreen, to hide form, after visible and not docked, yet
          Top := -1000;
          Show;
          ManualDock((FSubList[FSubList.Count - 1] as TSubDockPanel).FClientPanel, Nil, alTop);
        end;
    end;
  end;

{--------------------- TSubDockPanel main declaration -------------------------}
	
	constructor TSubDockPanel.Create(AOwner: TComponent; AIsLast: boolean);
	begin
		inherited Create;
    
		FIsLast := AIsLast;
    FDynDockPanel := AOwner as TDynDockPanel;
    FAncPanel := Nil;
    FSuccPanel := Nil;

		FHostPanel := TPanel.Create(AOwner);                // create HostPanel
		with FHostPanel do
		begin
			Parent := AOwner as TWinControl;
      Height := 0;
      Top := 0;
      Left := 0;
      Width := DynDockPanel.Width;
			if FIsLast then
				Align := alClient
			else Align := alTop;
		 	BevelOuter := bvNone;
		end;

		FClientPanel := TDynClientPanel.Create(FHostPanel); // create ClientPanel
		with FClientPanel do
		begin
			Parent := FHostPanel;
      SubDockPanel := self;
			Align := alClient;
			DockSite := true;
			BevelOuter := bvNone;
			OnDockOver := ClientPanelDockOver;
			OnDockDrop := ClientPanelDockDrop;
			OnUnDock := ClientPanelUnDock;
		end;
																							          // create DockManagerPro
 	 	FDockManagerPro := TDockManagerPro.Create(FHostPanel);
		with FDockManagerPro do
		begin
			Control := FClientPanel;
			Font.Name := 'Tahoma';
			CaptionStartColor := rgb(210,210,210);
			CaptionEndColor := rgb(230,230,230);
			TransparentButton := true;
		end;
	end;

	destructor TSubDockPanel.Destroy;
	begin
		FClientPanel.Free;
		FHostPanel.Free;
		inherited Destroy;
	end;
	
	procedure TSubDockPanel.ClientPanelDockOver(Sender: TObject;
		Source: TDragDockObject; X, Y: Integer; State: TDragState;
		var Accept: Boolean);
	var
		ARect: TRect;
	begin
		// Only accept only one DockClient, wich must be a TDockableForm
    // Dont accept, if Form is already Docked at a surounding Panel!
	 	Accept := (Source.Control is TDockableForm) and (Sender is TDynClientPanel) and ((Sender as TPanel).DockClientCount = 0)
      and not ( ((Source.Control as TDockableForm).Parent is TPanel) and
      ( ( (((Source.Control as TDockableForm).Parent as TDynClientPanel).SubDockPanel.AncPanel <> Nil) and (((Source.Control as TDockableForm).Parent as TDynClientPanel).SubDockPanel.AncPanel.FClientPanel = Sender) )
      or ( (((Source.Control as TDockableForm).Parent as TDynClientPanel).SubDockPanel.SuccPanel <> Nil) and (((Source.Control as TDockableForm).Parent as TDynClientPanel).SubDockPanel.SuccPanel.FClientPanel = Sender) ) ));
		if Accept then
			// Modify the DockRect to preview dock area.
      if (DynDockPanel.Align = alRight) and (DynDockPanel.Width = 0) then
      begin // ARect must be left, if DynDockPanel is right and not visible
        ARect.TopLeft := FClientPanel.ClientToScreen(Point(0 - Source.Control.LRDockWidth, 0));
			  ARect.BottomRight := FClientPanel.ClientToScreen(
				  Point(0, Source.Control.TBDockHeight));
      end else begin
			  ARect.TopLeft := FClientPanel.ClientToScreen(Point(0, 0));
			  ARect.BottomRight := FClientPanel.ClientToScreen(
				  Point(Source.Control.LRDockWidth, Source.Control.TBDockHeight));
		end else ARect := rect(0,0,0,0);
		Source.DockRect := ARect;
	end;
	
	procedure TSubDockPanel.ClientPanelDockDrop(Sender: TObject;
		Source: TDragDockObject; X, Y: Integer);
	begin
		// OnDockDrop gets called AFTER the client has actually docked,
		// so we create new Panels above and under current Panel
    DynDockPanel.AddSuroundingPanels(self);
    // Adjust Height to new DragObject
    FHostPanel.Height := Source.Control.TBDockHeight - 5;
    // Set new Events
    (Source.Control as TForm).OnShow := FormShowMod;
    (Source.Control as TForm).OnHide := FormHideMod;
    (Source.Control as TForm).OnClose := FormCloseMod;
    // make Panel "visible" after first Client has docked
    If DynDockPanel.VisibleFormCount = 1 then
        DynDockPanel.Width := 200;
		// Make DockManager repaints it's clients
    DynDockPanel.ResetAllBounds;
	end;

	procedure TSubDockPanel.ClientPanelUnDock(Sender: TObject; Client: TControl;
		NewTarget: TWinControl; var Allow: Boolean);
	begin
    // OnUnDock gets called BEFORE the client is undocked, in order to optionally
    // disallow the undock. DockClientCount is never 0 when called from this event.
		// We do not allow undock, if NewTarget is a full or one of the surounding Panels
    Allow := not (NewTarget is TDynClientPanel) or ((Client is TDockableForm) and (NewTarget is TDynClientPanel) and ((NewTarget as TPanel).DockClientCount = 0)
      and not ( ((Client as TDockableForm).Parent is TPanel) and
      ( ( (((Client as TDockableForm).Parent as TDynClientPanel).SubDockPanel.AncPanel <> Nil) and (((Client as TDockableForm).Parent as TDynClientPanel).SubDockPanel.AncPanel.FClientPanel = NewTarget) )
      or ( (((Client as TDockableForm).Parent as TDynClientPanel).SubDockPanel.SuccPanel <> Nil) and (((Client as TDockableForm).Parent as TDynClientPanel).SubDockPanel.SuccPanel.FClientPanel = NewTarget) ) )));
    if Allow then
    begin
      // destroy the Panels above and under current Panel
	    DynDockPanel.RemoveSuroundingPanels(self);
      // Restore old Events
      (Client as TForm).OnShow := (Client as TDockableForm).FormShow;
      (Client as TForm).OnHide := (Client as TDockableForm).FormHide;
      (Client as TForm).OnClose := (Client as TDockableForm).FormClose;
      
      // make Panel "invisible" after last visible Client has undocked
      If (Client as TDockableForm).Visible and (DynDockPanel.VisibleFormCount = 1) then
        DynDockPanel.Width := 0;
		  // Make DockManager repaints it's clients
      DynDockPanel.ResetAllBounds;
    end;
	end;

  procedure TSubDockPanel.FormShowMod(Sender: TObject);
  begin
    // call standard OnShow event
    (Sender as TDockableForm).FormShow(Sender);
    // Restor old Panel Height
    FHostPanel.Height := FClientPanel.DockClients[0].TBDockHeight - 5;
    // make Panel "visible" after first Client has shown
    If DynDockPanel.VisibleFormCount = 1 then
    begin
      DynDockPanel.Width := 200;
      (Sender as TDockableForm).RefreshForm;
    end;
    DynDockPanel.ResetAllBounds;
  end;

  procedure TSubDockPanel.FormHideMod(Sender: TObject);
  begin
    // Set Panel Hight to zero
    FHostPanel.Height := 0;
    // call standard OnClose event
    (Sender as TDockableForm).FormHide(Sender);
    // make Panel "invisible" if no visible Client exists
    If DynDockPanel.VisibleFormCount = 0 then
      DynDockPanel.Width := 0;
  end;

  procedure TSubDockPanel.FormCloseMod(Sender: TObject; var Action: TCloseAction);
  begin
    // ManualFloat object, before closing
    (Sender as TDockableForm).ManualFloat(rect(-(Sender as TForm).UndockWidth, -(Sender as TForm).UndockHeight, 0, 0));
    // call standard OnClose event
    (Sender as TDockableForm).FormClose(Sender, Action);
    DynDockPanel.ResetAllBounds;
  end;


  procedure TSubDockPanel.SetIsLast(AIsLast: Boolean);
  begin
    FIsLast := AIsLast;
    if AIsLast then
      FHostPanel.Align := alClient
    else FHostPanel.Align := alTop;
  end;

  function TSubDockPanel.GetHeight;
  begin
    Result := FHostPanel.Height;
  end;

  procedure TSubDockPanel.SetHeight(AHeight: Integer);
  begin
    FHostPanel.Height := AHeight
  end;

  function TSubDockPanel.GetTop;
  begin
    Result := FHostPanel.Top;
  end;

  procedure TSubDockPanel.SetTop(ATop: Integer);
  begin
    FHostPanel.Top := ATop;
  end;
	
end.
