unit OptionsClockGenerator;

interface

uses
  SysUtils, Controls, Forms, StdCtrls, ComCtrls, ExtCtrls, DockableForm,
  ImgList, ToolWin, Classes;

type
  TFormOptionsClockGenerator = class(TDockableForm)
    ToolBar1: TToolBar;
    btnSingleClock: TToolButton;
    btnClockSequence: TToolButton;
    btnPermanentClock: TToolButton;
    btnStopClock: TToolButton;
    ImageList1: TImageList;
    edSequenceCount: TEdit;
    Label3: TLabel;
    Label2: TLabel;
    Label4: TLabel;
    edClockLength: TEdit;
    Label5: TLabel;
    GroupBox1: TGroupBox;
    Panel1: TPanel;
    procedure btnSingleClockClick(Sender: TObject);
    procedure btnClockSequenceClick(Sender: TObject);
    procedure btnPermanentClockClick(Sender: TObject);
    procedure btnStopClockClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure edClockLengthChange(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
    procedure Check; override;
    procedure UnCheck; override;
    procedure RefreshForm; override;
    procedure EnableAllButtons(AStatus: boolean);
    procedure EnableAll(AStatus: boolean);
    procedure Done;
  end;

var
  FormOptionsClockGenerator: TFormOptionsClockGenerator;

implementation

{$R *.dfm}

uses Haupt;

procedure TFormOptionsClockGenerator.Check;
begin
  Form1.aShowOptionsClockGenerator.Checked := true;
end;

procedure TFormOptionsClockGenerator.UnCheck;
begin
  Form1.aShowOptionsClockGenerator.Checked := false;
end;

procedure TFormOptionsClockGenerator.btnSingleClockClick(Sender: TObject);
var Value: integer;
begin
  EnableAllButtons(false);
  if TryStrToInt(edClockLength.Text,Value) then
    Form1.Simulation.StartClockGenerator(StrToInt(edClockLength.Text), 1);
  btnSingleClock.Down := false;
end;

procedure TFormOptionsClockGenerator.btnClockSequenceClick(Sender: TObject);
var Value: integer;
begin
  EnableAllButtons(false);
  if TryStrToInt(edSequenceCount.Text,Value) and TryStrToInt(edClockLength.Text,Value) then
    Form1.Simulation.StartClockGenerator(StrToInt(edClockLength.Text), StrToInt(edSequenceCount.Text));
  btnClockSequence.Down := false;
end;

procedure TFormOptionsClockGenerator.btnPermanentClockClick(Sender: TObject);
var Value: integer;
begin
  EnableAllButtons(false);
  if TryStrToInt(edClockLength.Text,Value) then
    Form1.Simulation.StartClockGenerator(StrToInt(edClockLength.Text), -1);
  btnPermanentClock.Down := false;
end;

procedure TFormOptionsClockGenerator.btnStopClockClick(Sender: TObject);
begin
  EnableAllButtons(true);
  Form1.Simulation.StopClockGenerator;
  btnStopClock.Down := false;
end;

procedure TFormOptionsClockGenerator.FormCreate(Sender: TObject);
begin
  ClientHeight := 150;
  NameIndex := 2;
end;

procedure TFormOptionsClockGenerator.EnableAllButtons(AStatus: boolean);
begin
  btnSingleClock.Enabled := AStatus;
  btnClockSequence.Enabled := AStatus;
  btnPermanentClock.Enabled := AStatus;
  btnStopClock.Enabled := not AStatus;
end;

procedure TFormOptionsClockGenerator.EnableAll(AStatus: Boolean);
begin
  btnSingleClock.Enabled := AStatus;
  btnClockSequence.Enabled := AStatus;
  btnPermanentClock.Enabled := AStatus;
  btnStopClock.Enabled := false;

  edSequenceCount.Enabled := AStatus;
  edClockLength.Enabled := AStatus;

  Label2.Enabled := AStatus;
  Label3.Enabled := AStatus;
  Label4.Enabled := AStatus;
  Label5.Enabled := AStatus;
end;

procedure TFormOptionsClockGenerator.Done;
begin
  EnableAllButtons(true);
end;

procedure TFormOptionsClockGenerator.edClockLengthChange(Sender: TObject);
var IntOut: integer;
begin
  if TryStrToInt(edClockLength.Text, IntOut) then
    if IntOut < 1 then
      IntOut := 1         // TimerInterval muss at least be 1 ms
    else       
  else IntOut := 1;
  Form1.Simulation.ClockGeneratorInterval := IntOut;
end;

procedure TFormOptionsClockGenerator.RefreshForm;
begin
  ToolBar1.ButtonHeight := 30;
  GroupBox1.Width := Panel1.Width - 10;
  ToolBar1.Left := (GroupBox1.Width - ToolBar1.Width) div 2;
end;

end.
