unit OptionsGeneral;

interface

uses
  SysUtils, Classes, StdCtrls, ComCtrls, ExtCtrls, DockableForm, IniFiles,
  Controls, Dialogs;

type
  TFormOptionsGeneral = class(TDockableForm)
    cbAutoSizeX: TCheckBox;
    lbWidth: TLabel;
    edWidth: TEdit;
    edHeight: TEdit;
    lbHeight: TLabel;
    cbPaintGrid: TCheckBox;
    tBarDistance: TTrackBar;
    btnOkay: TButton;
    btnCancel: TButton;
    bgGroesse: TGroupBox;
    gbGitter: TGroupBox;
    Label8: TLabel;
    lbDistance: TLabel;
    Panel1: TPanel;
    cbAutoSizeY: TCheckBox;
    procedure cbAutoSizeXClick(Sender: TObject);
    procedure cbPaintGridClick(Sender: TObject);
    procedure btnOkayClick(Sender: TObject);
    procedure tBarDistanceChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure cbAutoSizeYClick(Sender: TObject);
    procedure edWidthKeyPress(Sender: TObject; var Key: Char);
    procedure btnCancelClick(Sender: TObject);
  private
    { Private-Deklarationen }
    procedure SetChecked(ACheckBox: TCheckBox; AStatus: boolean);
    procedure UpdateResolution;
  public
    { Public-Deklarationen }
    procedure Check; override;
    procedure UnCheck; override;
    procedure RefreshForm; override;
    procedure SaveSettingsToFile(AFile: string);
    procedure LoadSettingsFromFile(AFile: string);
  end;

var
  FormOptionsGeneral: TFormOptionsGeneral;

implementation

{$R *.dfm}

uses Haupt, ObjectList;

procedure TFormOptionsGeneral.Check;
begin
  Form1.aShowOptionsGeneral.Checked := true;
end;

procedure TFormOptionsGeneral.UnCheck;
begin
  Form1.aShowOptionsGeneral.Checked := false;
end;

procedure TFormOptionsGeneral.SetChecked(ACheckBox: TCheckBox; AStatus: Boolean);
begin
  with ACheckBox do
    if Checked xor AStatus then
    begin
      Tag := 1;
      Checked := AStatus;
    end;
end;

procedure TFormOptionsGeneral.cbAutoSizeXClick(Sender: TObject);
begin
  if cbAutoSizeX.Tag <> 0 then
    cbAutoSizeX.Tag := 0
  else if cbAutoSizeX.Checked then
		begin
			edWidth.Enabled := false;
			lbwidth.Enabled := false;
			Form1.Simulation.Resolution := point(0, Form1.Simulation.Resolution.Y);
			RefreshForm;
		end else begin
			edWidth.Enabled := true;
			lbwidth.Enabled := true;
			edWidth.Text := IntToStr(Form1.Simulation.Resolution.X);
			if edWidth.Text = '0' then
				edWidth.Text := '';
		end;
end;

procedure TFormOptionsGeneral.cbAutoSizeYClick(Sender: TObject);
begin
  if cbAutoSizeY.Tag <> 0 then
    cbAutoSizeY.Tag := 0
  else if cbAutoSizeY.Checked then
		begin
			edHeight.Enabled := false;
			lbHeight.Enabled := false;
			Form1.Simulation.Resolution := point(Form1.Simulation.Resolution.X , 0);
			RefreshForm;
		end else begin
			edHeight.Enabled := true;
			lbHeight.Enabled := true;
			edHeight.Text := IntToStr(Form1.Simulation.Resolution.Y);
			if edHeight.Text = '0' then
				edHeight.Text := '';
		end;
end;

procedure TFormOptionsGeneral.cbPaintGridClick(Sender: TObject);
begin
  ObjectList.GPaintGrid := cbPaintGrid.Checked;
  Form1.Simulation.Paint;
end;

procedure TFormOptionsGeneral.btnCancelClick(Sender: TObject);
begin
  RefreshForm;
end;

procedure TFormOptionsGeneral.btnOkayClick(Sender: TObject);
begin
  UpdateResolution;

  Form1.Simulation.Paint;
  RefreshForm;
end;

procedure TFormOptionsGeneral.edWidthKeyPress(Sender: TObject; var Key: Char);
begin
  if key = #13 then
  begin
    key := #0;
    btnOkayClick(Sender);
  end else if key = #27 then
    begin
      key := #0;
      btnCancelClick(Sender);
    end;
end;

procedure TFormOptionsGeneral.FormCreate(Sender: TObject);
begin
  ClientHeight := 275;
  NameIndex := 0;
end;

procedure TFormOptionsGeneral.FormShow(Sender: TObject);
begin
  RefreshForm;
end;

procedure TFormOptionsGeneral.tBarDistanceChange(Sender: TObject);
begin
  lbDistance.Caption := IntToStr(tBarDistance.Position);
  ObjectList.GDistance := tBarDistance.Position;
  UpdateResolution;
  Form1.Simulation.Paint;
end;

procedure TFormOptionsGeneral.UpdateResolution;
var IntOut: integer;
begin
  if TryStrToInt(edWidth.Text, IntOut) and (IntOut > 0) then        // X
    // value valid and Interface allready in right Form!
    Form1.Simulation.Resolution := point(IntOut * ObjectList.gDistance, Form1.Simulation.Resolution.Y)
  else begin
    Form1.Simulation.Resolution := point(0, Form1.Simulation.Resolution.Y);  
    SetChecked(cbAutoSizeX, true);
    edWidth.Enabled := false;
    lbWidth.Enabled := false;
  end;

  if TryStrToInt(edHeight.Text, IntOut) and (IntOut > 0) then      // Y
    // value valid and Interface allready in right Form!
    Form1.Simulation.Resolution := point(Form1.Simulation.Resolution.X, IntOut * ObjectList.gDistance)
  else begin
    Form1.Simulation.Resolution := point(Form1.Simulation.Resolution.X, 0);  
    SetChecked(cbAutoSizeY, true);
    edHeight.Enabled := false;
    lbHeight.Enabled := false;
  end;
end;

procedure TFormOptionsGeneral.RefreshForm;
begin
  edWidth.Text := IntToStr(Form1.Simulation.Resolution.X div ObjectList.gDistance);
  edHeight.Text := IntToStr(Form1.Simulation.Resolution.Y div ObjectList.gDistance);
  if edWidth.Text = '0' then
  begin
    SetChecked(cbAutoSizeX, true);
    edWidth.Enabled := false;
    lbWidth.Enabled := false;
    edWidth.Text := ' - ';
  end else begin            
    SetChecked(cbAutoSizeX, false);
    edWidth.Enabled := true;
    lbWidth.Enabled := true;
  end;
  if edHeight.Text = '0' then
  begin
    SetChecked(cbAutoSizeY, true);
    edHeight.Enabled := false;
    lbHeight.Enabled := false;
    edHeight.Text := ' - ';
  end else begin  
    SetChecked(cbAutoSizeY, false);
    edHeight.Enabled := true;
    lbHeight.Enabled := true;
  end;

  cbPaintGrid.Checked := ObjectList.GPaintGrid;
  tBarDistance.Position := ObjectList.GDistance;
end;

procedure TFormOptionsGeneral.SaveSettingsToFile(AFile: string);
var IniFile: TIniFile;
begin
  IniFile := TIniFile.Create(AFile);
  try
    IniFile.WriteInteger('Simulation', 'Distance', ObjectList.gDistance);
    IniFile.WriteBool('Simulation', 'PaintGrid', ObjectList.gPaintGrid);
  finally
    IniFile.Free;
  end;
end;

procedure TFormOptionsGeneral.LoadSettingsFromFile(AFile: string);
var IniFile: TIniFile;
begin
  IniFile := TIniFile.Create(AFile);
  try
    ObjectList.gDistance := IniFile.ReadInteger('Simulation', 'Distance', ObjectList.gDistance);
    ObjectList.gPaintGrid := IniFile.ReadBool('Simulation', 'PaintGrid', ObjectList.gPaintGrid);
    RefreshForm;
  finally
    IniFile.Free;
  end;
end;

end.
