unit OptionsStandardModules;

interface

uses
  Windows, SysUtils, Graphics, Controls, Dialogs, ComCtrls, StdCtrls, ExtCtrls,
  DockableForm, IniFiles, ImgList, ToolWin, Classes;

type
  TFormOptionsStandardModules = class(TDockableForm)
    tbStandardLogikModules: TToolBar;
    btnAnd: TToolButton;
    btnOr: TToolButton;
    btnXor: TToolButton;
    lbInputCount: TLabel;
    edSize: TEdit;
    tBarSize: TTrackBar;
    tbLedModule: TToolBar;
    btnLed: TToolButton;
    cDiaManual: TColorDialog;
    tbColorBar: TToolBar;
    btnGreen: TToolButton;
    btnOrange: TToolButton;
    btnRed: TToolButton;
    ToolButton1: TToolButton;
    btnManual: TToolButton;
    shLed: TShape;
    ImageList1: TImageList;
    Panel1: TPanel;
    gbStandardLogikModules: TGroupBox;
    gbLedModules: TGroupBox;
    udChangeInputCount: TUpDown;
    lbChooseColor: TLabel;
    btnText: TToolButton;
    edTextSize: TEdit;
    udTextSize: TUpDown;
    lbTextSize: TLabel;
    gbOrientation: TGroupBox;
    tbOrientation: TToolBar;
    btnLeft: TToolButton;
    btnRight: TToolButton;
    btnDown: TToolButton;
    btnUp: TToolButton;
    ilOrientation: TImageList;
    procedure btnAndClick(Sender: TObject);
    procedure btnOrClick(Sender: TObject);
    procedure btnXorClick(Sender: TObject);
    procedure btnLedClick(Sender: TObject);
    procedure btnGreenClick(Sender: TObject);
    procedure btnOrangeClick(Sender: TObject);
    procedure btnRedClick(Sender: TObject);
    procedure btnManualClick(Sender: TObject);
    procedure tBarSizeChange(Sender: TObject);
    procedure edSizeChange(Sender: TObject);
    procedure udChangeInputCountChangingEx(Sender: TObject; var AllowChange: Boolean;
      NewValue: Smallint; Direction: TUpDownDirection);
    procedure FormCreate(Sender: TObject);
    procedure btnTextClick(Sender: TObject);
    procedure edTextSizeChange(Sender: TObject);
    procedure btnOrientationClick(Sender: TObject);
  private
    { Private-Deklarationen }
    procedure AllColorButtonsUp;
  public
    procedure Check; override;
    procedure UnCheck; override;
    procedure RefreshForm; override;
    procedure EnableAllButtons(AStatus: boolean);
    procedure InterfaceTextSize(AStatus: boolean);
    procedure SaveSettingsToFile(AFile: string);
    procedure LoadSettingsFromFile(AFile: string);
  end;

var
  FormOptionsStandardModules: TFormOptionsStandardModules;

implementation

{$R *.dfm}

uses Haupt, ObjectList;

var LastAction: TActAction = AddLed;

procedure TFormOptionsStandardModules.Check;
begin
  Form1.aShowOptionsStandardModules.Checked := true;
end;

procedure TFormOptionsStandardModules.UnCheck;
begin
  Form1.aShowOptionsStandardModules.Checked := false;
end;

procedure TFormOptionsStandardModules.AllColorButtonsUp;
begin
  btnGreen.Down := false;
  btnOrange.Down := false;
  btnRed.Down := false;
  btnManual.Down := false;
end;

procedure TFormOptionsStandardModules.btnAndClick(Sender: TObject);
begin
  Form1.SetAction(AddAnd, Sender);
end;

procedure TFormOptionsStandardModules.btnGreenClick(Sender: TObject);
begin
  AllColorButtonsUp;
  btnGreen.Down := true;
  if btnGreen.ImageIndex = 0 then
  begin
    shLed.Brush.Color := clLime;
    ObjectList.gColor := clLime;
  end else begin
    shLed.Brush.Color := clBlack;
    ObjectList.gColor := clBlack;
  end;
end;

procedure TFormOptionsStandardModules.btnLedClick(Sender: TObject);
begin
  if Sender = btnLed then
    Form1.SetAction(AddLed, Sender);
  btnLed.Down := true;
  if LastAction = AddText then
  begin
    btnGreen.ImageIndex := 0;
    btnGreen.Hint := 'grn';
    btnGreenClick(Sender);
  end;
  LastAction := AddLed;
end;

procedure TFormOptionsStandardModules.btnOrientationClick(Sender: TObject);
begin
  if (Sender as TToolButton).Name = 'btnLeft' then
    gOrientation := orLeft
  else if (Sender as TToolButton).Name = 'btnRight' then
      gOrientation := orRight
    else if (Sender as TToolButton).Name = 'btnUp' then
        gOrientation := orUp
      else if (Sender as TToolButton).Name = 'btnDown' then
          gOrientation := orDown
end;

procedure TFormOptionsStandardModules.btnTextClick(Sender: TObject);
begin
  if Sender = btnText then
    Form1.SetAction(AddText, Sender);
  btnText.Down := true;
  if LastAction = AddLed then
  begin
    btnGreen.ImageIndex := 4;
    btnGreen.Hint := 'schwarz';
    btnGreenClick(Sender);
  end;
  InterfaceTextSize(true);
  LastAction := AddText;
end;

procedure TFormOptionsStandardModules.btnManualClick(Sender: TObject);
begin
  AllColorButtonsUp;
  btnManual.Down := true;
  if cDiaManual.Execute then
  begin
    shLed.Brush.Color := cDiaManual.Color;
    ObjectList.gColor := cDiaManual.Color;
  end;
end;

procedure TFormOptionsStandardModules.btnOrangeClick(Sender: TObject);
begin
  AllColorButtonsUp;
  btnOrange.Down := true;
  shLed.Brush.Color := rgb(255,230,0);
  ObjectList.gColor := rgb(255,230,0);
end;

procedure TFormOptionsStandardModules.btnOrClick(Sender: TObject);
begin
  Form1.SetAction(AddOr, Sender);
end;

procedure TFormOptionsStandardModules.btnRedClick(Sender: TObject);
begin
  AllColorButtonsUp;
  btnRed.Down := true;
  shLed.Brush.Color := clRed;
  ObjectList.gColor := clRed;
end;

procedure TFormOptionsStandardModules.btnXorClick(Sender: TObject);
begin
  Form1.SetAction(AddXor, Sender);
end;

procedure TFormOptionsStandardModules.edSizeChange(Sender: TObject);
begin
  ObjectList.GSize := StrToInt(edSize.Text);
end;

procedure TFormOptionsStandardModules.edTextSizeChange(Sender: TObject);
var A: integer;
begin
  if TryStrToInt(edTextSize.Text, A) then
    gTextSize := A;
end;

procedure TFormOptionsStandardModules.FormCreate(Sender: TObject);
var aHint: string;
begin
  ClientHeight := 345;
  NameIndex := 1;

  aHint := 'Eingangsanzahl erhhen: Hotkey ''+''' + #13#10 +
                   'Eingangsanzahl veringern: Hotkey ''#''';

  btnLeft.Down := true;
  tBarSize.Hint := aHint;
  edSize.Hint := aHint;
  udChangeInputCount.Hint := aHint;
end;

procedure TFormOptionsStandardModules.EnableAllButtons(AStatus: Boolean);
begin
  btnAnd.Enabled := AStatus;
  btnOr.Enabled := AStatus;
  btnXor.Enabled := AStatus;
  lbInputCount.Enabled := AStatus;
  edSize.Enabled := AStatus;
  udChangeInputCount.Enabled := AStatus;
  tBarSize.Enabled := AStatus;

  btnLeft.Enabled := AStatus;
  btnRight.Enabled := AStatus;
  btnUp.Enabled := AStatus;
  btnDown.Enabled := AStatus;

  btnLed.Enabled := AStatus;
  btnText.Enabled := AStatus;
  lbChooseColor.Enabled := AStatus;
  btnGreen.Enabled := AStatus;
  btnOrange.Enabled := AStatus;
  btnRed.Enabled := AStatus;
  btnManual.Enabled := AStatus;
  shLed.Visible := AStatus;

  lbTextSize.Enabled := AStatus;
  edTextSize.Enabled := AStatus;
  udTextSize.Enabled := AStatus;

  InterfaceTextSize(AStatus and (gActAction = AddText));
end;

procedure TFormOptionsStandardModules.InterfaceTextSize(AStatus: Boolean);
begin
  if AStatus then
  begin
    tbLedModule.Left := 10;
  end else begin
    tbLedModule.Left := 47;
  end;
  lbTextSize.Visible := AStatus;
  edTextSize.Visible := AStatus;
  udTextSize.Visible := AStatus;
end;

procedure TFormOptionsStandardModules.tBarSizeChange(Sender: TObject);
begin
  edSize.Text := IntToStr(tBarSize.Position);
  tBarSize.Position := tBarSize.Position; 
end;

procedure TFormOptionsStandardModules.udChangeInputCountChangingEx(Sender: TObject;
  var AllowChange: Boolean; NewValue: Smallint; Direction: TUpDownDirection);
begin
  tBarSize.Position := NewValue;
end;    

procedure TFormOptionsStandardModules.RefreshForm;
begin
  edSize.Text := IntToStr(ObjectList.GSize);
  tBarSize.Position := ObjectList.GSize;
  
  tbStandardLogikModules.ButtonHeight := 30;
  tbOrientation.ButtonHeight := 30;
  tbLedModule.ButtonHeight := 30;
  tbColorBar.ButtonHeight := 25;
end;

procedure TFormOptionsStandardModules.SaveSettingsToFile(AFile: string);
var IniFile: TIniFile;
begin
  IniFile := TIniFile.Create(AFile);
  try
    IniFile.WriteInteger('Simulation', 'Size', ObjectList.gSize);
  finally
    IniFile.Free;
  end;
end;

procedure TFormOptionsStandardModules.LoadSettingsFromFile(AFile: string);
var IniFile: TIniFile;
  aSize: integer;
begin
  IniFile := TIniFile.Create(AFile);
  try
    aSize := IniFile.ReadInteger('Simulation', 'Size', ObjectList.gSize);
    if aSize > 10 then              // set Trackbar to max.
      tBarSize.Position := 10;
    ObjectList.gSize := aSize;
    RefreshForm;
  finally
    IniFile.Free;
  end;
end;

end.
