unit RakBinaryStreamData;

{///////////////////////////////////////////////////////////////////////////////
// Version      : 1.0.3
// Beschreibung : Die Unit RakBinaryStreamData stellt Klassen zur Verfgung, mit
                  denen man beliebige Daten in einem Stream speichern und aus
                  einen Stream lesen kann.
                  Diese Klassen sind hauptschlich fr Programme gedacht, die
                  ihre Daten auf der Festplatte speichern wollen. Besonders
                  geeignet sind die Klassen fr Programme, die immer weiter
                  entwickelt werden und deshalb mit der Zeit Daten speichern
                  mssen, an die in der ersten Version noch nicht gedacht wurde.
                  Durch die Art der Speicherung der Daten wird es ermglicht,
                  dass auch alte Programmversionen Dateien von neuen
                  Programmversionen und umgekehrt lesen knnen, so hnlich,
                  wie es z.B. bei Html-Browsern der Fall ist. Ein Browser, der
                  fr die Version 3 von HTML entwickelt wurde kann auch
                  HTML-Dateien der Version 4 laden und darstellen. Die neuen
                  Elemente, die mit der 4. Version von HTML eingefhrt wurden,
                  ignoriert er einfach. Dadurch stimmt die Darstellung zwar
                  nicht ganz, aber er verweigert sich auch nicht.
                  Die Daten werden in einer hierarchischen Struktur, hnlich
                  wie in XML, abgespeichert. Die Daten werden aber nicht wie
                  bei XML in Textform, sondern binr gespeichert. Dadurch
                  bentigen sie weniger Platz als Daten im XML-Format.
                  Auerdem ist es ganz einfach mglich binre Daten zu speichern.
                  Fr Bitmap-, Icon-, Jpeg-Bilder und Metafile-Grafiken
                  gibt es bereits Klassen, die das untersttzen. Fr andere
                  binre Daten, wie z.B. MIDI oder AVI, ist eine Einbindung
                  realtiv einfach, wenn bereits eine Klasse existiert, die
                  das entsprechende Format untersttzt und Methoden hat,
                  mit denen man die Daten in einem Stream abspeichern kann.
                  Ein Beispiel dafr bildet die Klasse TBitmap, die mit
                  Hilfe der Klassen TRakBinaryGraphic und TRakBinaryBitmap
                  in das Framework eingebunden ist.
                  Jede Information, die gespeichert wird, wird zusammen mit einem
                  eindeutigen Namen gespeichert, somit ist es mglich, dass
                  eine alte Programmversion Dateien einer neueren Programmversion
                  anzeigen kann. Das alte Programm fordert nur die Daten an,
                  mit denen es was anfangen kann. Andere Daten werden einfach
                  ignoriert. Damit ein Programm realtiv problemlos mit Dateien
                  von lteren Programmversionen zurecht kommt, muss es einfach
                  nur berprfen, ob die gewnschten Daten vorhanden sind,
                  und nicht hart davon ausgehen, dass die Daten vorhanden sein
                  mssen. O.K. ganz so einfach ist es natrlich nicht, da man
                  ja irgendwie darauf reagieren muss, falls Daten nicht
                  vorhanden sind :-)
                  Man hat auf alle Flle nicht das Problem, dass Programme
                  die Dateien von anderen Versionen nicht lesen knnten. Das
                  Einlesen von Dateien funktioniert immer, vorausgesetzt die
                  Datei selbst ist nicht kaputt. Man muss nach dem Einlesen
                  der Datei prfen, welche Daten zur Verfgung stehen. Da alle
                  Daten mit eindeutigen Namen abgespeichert sind, stellt dies
                  kein Problem dar.

                  Wer sich daran strt, dass die Daten in binrer Form
                  gespeichert sind, dem stellt die Unit
                  RakBinaryStreamDataXML Funktionen zur Verfgung, mit denen man
                  das binre Format in ein XML-Format und zurck konvertieren
                  kann.
// erstellt am  : 10.11.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
  29.1.2002 Neuer Basistyp TRakBinaryShortString
            Da die Unit jetzt zur allgemeinen Verfgung gestellt wird,
            darf kein neuer Basistyp mehr eingefhrt werden!!!!!
  29.6.2002 Neue Methode AddToTag bei der Klasse TRakBinaryPicture
  13.7.2002 Neue Methoden Clear, SaveToClipboardFormat und LoadFromClipboardFormat
            bei der Klasse TRakBinaryStreamData
            Neue Methode Clear bei der Klasse TRakBinaryTag
  14.7.2002 Neues Property AByte der Klasse TRakBinaryBlob
  4.8.2002  Bugfix: TRakBinaryPicture.SetGraphic: IsNull wird jetzt auch gesetzt,
            wenn das Property Empty der Grafik gleich true, da SaveToStream
            einer leeren Grafik zu einer Exception fhrt
  5.10.2202 Klasse TRakBinaryTag: Die zweite AddAttribute-Methode auch zu einer Funktion gemacht
                                  Bugfix: Bei den AttributeAsXXX-Funktionen wird jetzt auch dann
                                          der DefaultValue zurckgegeben, wenn IsNull = true.
  10.1.2003 Bugfix: Implementation der Methoden TRakBinaryWideString.SaveValue
                    und TRakBinaryWideString.LoadValue war vertauscht
            Versionsnummer eingefhrt
  19.7.2003 Neue Klasse TRakBinaryTShiftState
  6.9.2003 Neue Methoden TRakBinaryTag.ReadFromPublishedProperties
  						und TRakBinaryTag.WriteToPublishedProperties, mit den dazugehrigen
              Klassen und Interfaces
  7.9.2003 Neue Klassenmethoden TRakBinaryData.SaveObjectToFile
  						und TRakBinaryData.LoadObjectFromFile
///////////////////////////////////////////////////////////////////////////////}
// Delphi 5 Implementation
//
// Copyright (c) 2001-2002 by Richard Kaspar
// ("http://www.kasparsoft.de")
//
// Definitions:
// - "Package" refers to the collection of files distributed by
//   the Copyright Holder, and derivatives of that collection of
//   files created through textual modification.
// - "Standard Version" refers to such a Package if it has not
//   been modified, or has been modified in accordance with the
//   wishes of the Copyright Holder.
// - "Copyright Holder" is whoever is name in the copyright or
//   copyrights for the package.
// - "You" is you, if you're thinking about copying or distributing
//   this Package.
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Package"), to deal in the Package without restriction,
// including without limitation the rights to use, copy, modify,
// merge, publish, distribute, sublicense, and/or sell copies of the
// Package, and to permit persons to whom the Package is furnished
// to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Package.
//
// You may modify your copy of this Package in any way, provided
// that you insert a prominent notice in each changed file stating
// how and when you changed a file, and provided that you do at
// least one of the following:
//
// a) allow the Copyright Holder to include your modifications in
// the Standard Version of the Package.
//
// b) use the modified Package only within your corporation or
// organization.
//
// c) rename any non standard executables, units, and classes so
// the names do not conflict with standard executables, units, and
// classes, and provide a separate manual page that clearly documents
// how it differs from the standard version.
//
// d) make other distribution arrangements with the Copyright Holder.
//
// The name of the Copyright Holder may not be used to endorse or
// promote products derived from this Package without specific prior
// written permission.
//
// THE PACKAGE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// PACKAGE OR THE USE OR OTHER DEALINGS IN THE PACKAGE.
{$I RAK.INC}

interface

uses
  miscfunc, RakCollection, RakNamespace, RakDestroy,
  Windows, Sysutils, Classes, Controls, Graphics, jpeg, Clipbrd, TypInfo;

type
  TRakBinaryAttribute = class;
  TRakBinaryAttributes = class;
  TRakBinaryTagList = class;
  IRakBinaryTagList = interface;
  IRakBinaryReadFromPropertiesOptions = interface;
  IRakBinaryWriteToPropertiesOptions = interface;

  TRakBinaryDataType = class of TRakBinaryAttribute;
  TRakBinaryLoadResult = (
    blrOK,          // Die Daten konnten geladen werden
    blrErrVersion,  // Es ist zwar das richtige Datenformat, aber in einer Version, die nicht untersttzt wird
    blrErrFormat,    // Der Stream enthlt kein bekanntes Datenformat
    blrErrCorrupt);  // Die Daten im Stream sind defekt
  TRakBinaryLoadClipboardResult = (
    blcrOK,          // In der Zwischenablage befanden sich Daten, die gelesen werden konnten
    blcrEmpty,      // In der Zwischenablage befanden sich keine Daten im erwarteten Format
    blcrErrVersion,  // In der Zwischenablage befanden sich Daten. Es war zwar das richtige Datenformat, aber in einer Version, die nicht untersttzt wird
    blcrErrFormat,    // In der Zwischenablage befanden sich Daten. Der Stream enthlt kein bekanntes Datenformat
    blcrErrCorrupt,   // In der Zwischenablage befanden sich Daten. Die Daten im Stream sind defekt
    blcrErrUnknown);  // Ein unerwarteter Fehler trat auf

  {****************************************************************************
  <class name="TRakBinaryTag" superclass="TRakCollection" public="true" defaultproperty="Tags">
    <description>
      TRakBinaryTag nimmt alle Daten auf, die in einem Stream gespeichert werden
      sollen, bzw. ldt alle seine Daten aus einen Stream.
      Instanzen von TRakBinaryTag werden ab jetzt als Tags bezeichnet.
      Tag hat nichts mit der Zeitangabe Tag (engl. Day) zu tun. Tag wird
      als Bezeichnung verwendet, so wie bei HTML oder XML.
      Ein Tag hlt die Daten in sog. Attributen. Jedes Attribut hat einen Typ, der
      festlegt, wie die Daten des Attributes in einem Stream gespeichert werden.
      Auer den Attributen kann ein Tag beliebig viele Unter-Tags besitzen.
      Somit knnen Daten in einer hierarchischen Struktur angeordnet werden.
      Sowohl die Attribute, als auch die Tags haben Namen. Attribute
      haben einen eindeutigen Namen. Somit kann man Anhand eines Attributnamens
      auf ein Attribut zugreifen (siehe Methode AttributeByName).
      Unter-Tags haben keine eindeutigen Namen. Wenn man auf Tags
      ber den Namen zugreifen will, dann erhlt man eine Liste von Tags
      (siehe Methode FindTagsByName).
      Die Namen von Attributen sind nur innerhalb eines Tags eindeutig.
      Ein Unter-Tag kann also Attribute mit den gleichen Namen haben, wie sein
      ber-Tag.
    </description>
  </class>
  ****************************************************************************}
  TRakBinaryTag = class(TRakCollection)
  private
    FTagName: ShortString;
    FAttributes: TRakBinaryAttributes;
    function GetTags(Idx: integer): TRakBinaryTag;
    procedure SetTagName(const Value: ShortString);
    procedure FillListByName(List: TRakBinaryTagList; Name: ShortString; WithSubTags: boolean);

    procedure SaveToStreamNow(Stream: TStream);
    procedure LoadFromStreamNow(Stream: TStream);
  protected
    procedure DoAssign(Source: TRakCollection); override;
  public
    constructor Create; override;
    destructor Destroy; override;

    // AddTag erzeugt einen neuen Unter-Tag und gibt die Referenz darauf zurck
    function AddTag: TRakBinaryTag; overload;
    function AddTag(TagName: ShortString): TRakBinaryTag; overload;
    // AddAttribut erzeugt ein neues Attribut fr den Tag und versorgt es eventuell gleich mit Daten
    function AddAttribute(DataType: TRakBinaryDataType; Name: ShortString; Value: AnsiString): TRakBinaryAttribute; overload;
    function AddAttribute(DataType: TRakBinaryDataType; Name: ShortString): TRakBinaryAttribute; overload;
    // Mit AttributeByName kann auf Attribute Anhand ihres Namens zugegriffen werden
    function AttributeByName(Name: ShortString): TRakBinaryAttribute;
    function AttributeAsInteger(Name: ShortString; DefaultValue: Int64): Int64;
    function AttributeAsFloat(Name: ShortString; DefaultValue: extended): extended;
    function AttributeAsString(Name: ShortString; DefaultValue: AnsiString): AnsiString;
    function AttributeAsBoolean(Name: ShortString; DefaultValue: boolean): boolean;
    // Mit Clear kann man alle Untertags und Attribute lschen
    procedure Clear; virtual;
    // FindTagsByName gibt eine Liste aller Unter-Tags mit den gewnschten Namen zurck.
    // Es kann festgelegt werden, ob nur unmittelbare Unter-Tags betroffen sind oder der ganze Zweig
    function FindTagsByName(TagName: ShortString; WithSubTags: boolean = false): IRakBinaryTagList;
    function FindFirstTag(TagName: ShortString): TRakBinaryTag;

    procedure ReadFromPublishedProperties(Obj: TObject;
    	Options: IRakBinaryReadFromPropertiesOptions = nil;
      ReadFromClassAttributs: boolean = false);
    procedure WriteToPublishedProperties(Obj: TObject;
    	Options: IRakBinaryWriteToPropertiesOptions = nil);

    // TagName ist der Name des Tags
    property TagName: ShortString read FTagName write SetTagName;
    // Mit Attributes kann auf die Attribute in Listenform zugegriffen werden
    property Attributes: TRakBinaryAttributes read FAttributes;
    // property Count: integer; stellt "Vater"-Klasse TRakCollection zur Verfgung
    // Mit Tags kann auf alle Unter-Tags zugegriffen werden (0 <= Idx < Count)
    property Tags[Idx: integer]: TRakBinaryTag read GetTags; default;
  end;

  {****************************************************************************
  <class name="TRakBinaryStreamData" superclass="TRakBinaryTag" public="true" defaultproperty="">
    <description>
      TRakBinaryStreamData ist die Klasse, an die alle zu speicherden Daten
      bergeben werden mssen. Danach kann man die mit der Methode
      SaveToStream in z.B einen TFileStream schreiben.
      Mit der Methode LoadFromStream kann man alle Daten aus einem Stream
      laden und danach darauf zugreifen.

      Die meisten Properties und Methoden, um auf die Daten zuzugreifen findet
      man in der Vaterklasse TRakBinaryTag.
    </description>
  </class>
  ****************************************************************************}
  TRakBinaryStreamData = class(TRakBinaryTag)
  private
    FApplicationName: ShortString;
    FApplicationVersion: ShortString;
  public
  	// Mit SaveObjectToFile knnen die published Properties eines Objektes in einer Datei gespeichert werden
  	class procedure SaveObjectToFile(Obj: TObject; FileName: string);
  	// Mit LoadObjectFromFile knnen die published Properties eines Objektes aus einer Datei versorgt werden
  	class procedure LoadObjectFromFile(Obj: TObject; FileName: string);

    // SaveToStream schreibt alle Attribute und Unter-Tags in einen Stream
    procedure SaveToStream(Stream: TStream);
    procedure SaveToFile(FileName: string);
    // LoadFromStream ldt Daten aus einen Stream, anhand derer Attribute und Unter-Tags erzeugt werden
    function LoadFromStream(Stream: TStream): TRakBinaryLoadResult;
    function LoadFromFile(FileName: string): TRakBinaryLoadResult;
    // SaveToClipboardFormat legt alle Daten in der Zwischenablage ab
    procedure SaveToClipboardFormat(ClipboardFormat: Word; ClearData: boolean = true;
      ClipboardIsOpen: boolean = false);
    // LoadFromClipboardFormat ldt Daten aus der Zwischenablage, anhand derer Attribute und Unter-Tags erzeugt werden
    function LoadFromClipboardFormat(ClipboardFormat: Word): TRakBinaryLoadClipboardResult;
    // Clear lscht alle Untertags, Attribute, ApplicationName und ApplicationVersion
    procedure Clear; override;

    // ApplicationName kann den Namen der Applikation aufnehmen, falls man diesen mit speichern mchte
    property ApplicationName: ShortString read FApplicationName write FApplicationName;
    // ApplicationVersion kann die Version der Applikation aufnehmen, falls man diese mit speichern mchte
    property ApplicationVersion: ShortString read FApplicationVersion write FApplicationVersion;
  end;

  {****************************************************************************
  <interface name="IRakBinaryTagList" superinterface="IUnknown" public="true">
    <description>
      IRakBinaryTagList wird z.B. von der Methode TRakBinaryTag.FindTagsByName
      benutzt, um die Liste der gefundenen Tags zurckzugeben.
      Da IRakBinaryTagList ein Interface ist, muss man sich nicht
      um die Freigabe des Objektes kmmern. Sobald die letzte Referenz
      auf das Objekt weg ist, Lscht sich das Objekt von selbst.
      Das ist dem Garbage-Collection von Java sehr hnlich.
    </description>
  </interface>
  ****************************************************************************}
  IRakBinaryTagList = interface
    function Count: integer;
    function Items(Idx: integer): TRakBinaryTag;
  end;

  {****************************************************************************
  <record name="TRakBinaryReadFromSimplePropertyDescription" public="true">
    <description>
      Wird von IRakBinaryReadFrom.BeforeReadFromSimpleProperty
      und IRakBinaryReadFrom.AfterReadFromSimpleProperty verwendet
    </description>
  </record>
  ****************************************************************************}
  TRakBinaryReadFromSimplePropertyDescription = record
  	PropertyName: string; // Changing of PropertyName has nor effects
  	AttributName: string; // default: Same as PropertyName; Set to '', if you want to ignore it
    Tag: TRakBinaryTag;
  end;

  {****************************************************************************
  <record name="TRakBinaryReadFromClassPropertyDescription" public="true">
    <description>
      Wird von IRakBinaryReadFrom.BeforeReadFromClassProperty
      und IRakBinaryReadFrom.AfterReadFromClassProperty verwendet
    </description>
  </record>
  ****************************************************************************}
  TRakBinaryReadFromClassPropertyDescription = record
  	PropertyName: string; // Changing of PropertyName has nor effects
  	TagName: string; // default: Same as PropertyName; Set to '', if you want to ignore it
    IgnoreClassAttributs: boolean; // default: false;
    Tag: TRakBinaryTag;
  end;


  {****************************************************************************
  <interface name="IRakBinaryReadFrom" superinterface="IUnknown" public="true">
    <description>
      Klassen, die die IRakBinaryReadFrom implementieren, erhalten bei der Abarbeitung
      von TRakBinaryTag.ReadFromPublishedProperties die Mglichkeit gezielt das
      Speichern seiner Werte zu beeinflussen.
    </description>
  </interface>
  ****************************************************************************}
  IRakBinaryReadFrom = interface
  	['{581CF1B8-43F3-4BA7-B17E-645313B020D5}']
    // BeforeReadFromProperties wird aufgerufen, bevor alle published Properties
    // in Tag bernommen werden.
    // Hier kann man eigene Daten in Tag ablegen
    // Mit DoIt kann man generell unterbinden, dass die published Properties bernommen werden
    // Mit Options kann man das Defaultverhalten gezielt verndern
    procedure BeforeReadFromProperties(Tag: TRakBinaryTag;
    	Options: IRakBinaryReadFromPropertiesOptions;
      var DoIt: boolean); // DoIt is initialized with true
    // AfterReadFromProperties wird aufgerufen, nachdem alle published Properties
    // in Tag bernommen worden sind, vorausgesetzt in BeforeReadFromProperties wurde DoIt nicht
    // auf false gesetzt.
    // Hier kann man noch weiter Daten in Tag ablegen
    procedure AfterReadFromProperties(Tag: TRakBinaryTag);

    // BeforeReadFromSimpleProperty gibt einem die Mglichkeit das Speichern einzelner Properties
    // zu unterbinden oder unter einem anderen Namen zu speichern, um Namenskonflikte zu vermeiden
    procedure BeforeReadFromSimpleProperty(var PropertyDescription: TRakBinaryReadFromSimplePropertyDescription);
    procedure AfterReadFromSimpleProperty(const PropertyDescription: TRakBinaryReadFromSimplePropertyDescription);

    // BeforeReadFromClassProperty gibt einem die Mglichkeit das Speichern einzelner Properties
    // zu unterbinden oder unter einem anderen Namen zu speichern, um Namenskonflikte zu vermeiden
    procedure BeforeReadFromClassProperty(var PropertyDescription: TRakBinaryReadFromClassPropertyDescription);
    procedure AfterReadFromClassProperty(const PropertyDescription: TRakBinaryReadFromClassPropertyDescription);

    // HandleNotSupportedReadProperties wird fr die Properties aufgerufen, die einem Typ haben,
    // der vom Defaultverhalten nicht untersttzt wird.
    procedure HandleNotSupportedReadProperties(PropertyName: ShortString);
  end;

  TRakBinaryReadFrom = class(TInterfacedObject, IRakBinaryReadFrom)
  public
    procedure BeforeReadFromProperties(Tag: TRakBinaryTag; Options: IRakBinaryReadFromPropertiesOptions; var DoIt: boolean); virtual;
    procedure AfterReadFromProperties(Tag: TRakBinaryTag); virtual;
    procedure BeforeReadFromSimpleProperty(var PropertyDescription: TRakBinaryReadFromSimplePropertyDescription); virtual;
    procedure AfterReadFromSimpleProperty(const PropertyDescription: TRakBinaryReadFromSimplePropertyDescription); virtual;
    procedure BeforeReadFromClassProperty(var PropertyDescription: TRakBinaryReadFromClassPropertyDescription); virtual;
    procedure AfterReadFromClassProperty(const PropertyDescription: TRakBinaryReadFromClassPropertyDescription); virtual;
    procedure HandleNotSupportedReadProperties(PropertyName: ShortString); virtual;
  end;

  {****************************************************************************
  <record name="TRakBinaryWriteToSimplePropertyDescription" public="true">
    <description>
      Wird von IRakBinaryWriteTo.BeforeWriteToSimpleProperty
      und IRakBinaryWriteTo.AfterWriteToSimpleProperty verwendet
    </description>
  </record>
  ****************************************************************************}
  TRakBinaryWriteToSimplePropertyDescription = record
  	PropertyName: string; // Changing of PropertyName has nor effects
  	AttributName: string; // default: Same as PropertyName; Set to '', if you want to ignore it
    Tag: TRakBinaryTag;
  end;

  {****************************************************************************
  <record name="TRakBinaryWriteToClassPropertyDescription" public="true">
    <description>
      Wird von IRakBinaryWriteTo.BeforeWriteToClassProperty
      und IRakBinaryWriteTo.AfterWriteToClassProperty verwendet
    </description>
  </record>
  ****************************************************************************}
  TRakBinaryWriteToClassPropertyDescription = record
  	PropertyName: string; // Changing of PropertyName has nor effects
  	TagName: string; // default: Same as PropertyName; Set to '', if you want to ignore it
    Tag: TRakBinaryTag;
  end;


  {****************************************************************************
  <interface name="IRakBinaryWriteTo" superinterface="IUnknown" public="true">
    <description>
      Klassen, die die IRakBinaryWriteTo implementieren, erhalten bei der Abarbeitung
      von TRakBinaryTag.WriteToPublishedProperties die Mglichkeit gezielt das
      Laden seiner Werte zu beeinflussen.
    </description>
  </interface>
  ****************************************************************************}
  IRakBinaryWriteTo = interface
  	['{10A1C612-D138-47D3-946D-0657CA78FFA7}']
    // BeforeWriteToProperties wird aufgerufen, bevor alle published Properties
    // mit den Werten aus Tag versorgt werden.
    // Hier kann man eigene Daten aus Tag auslesen
    // Mit DoIt kann man generell unterbinden, dass die published Properties versorgt werden
    // Mit Options kann man das Defaultverhalten gezielt verndern
    procedure BeforeWriteToProperties(Tag: TRakBinaryTag;
    	Options: IRakBinaryWriteToPropertiesOptions;
      var DoIt: boolean); // DoIt is initialized with true
    // AfterWriteToProperties wird aufgerufen, nachdem alle published Properties
    // mit den Werten aus Tag versorgt wurden, vorausgesetzt in BeforeWriteToProperties wurde DoIt nicht
    // auf false gesetzt.
    // Hier kann man noch eigene Daten aus Tag auslesen
    procedure AfterWriteToProperties(Tag: TRakBinaryTag);

    // BeforeWriteToSimpleProperty gibt einem die Mglichkeit das Setzen einzelner Properties
    // zu unterbinden oder das Laden aus einem Attribut mit einem anderen Namen zu veranlassen
    procedure BeforeWriteToSimpleProperty(var PropertyDescription: TRakBinaryWriteToSimplePropertyDescription);
    // AttributForSimplePropertyNotFound wird aufgerufen, wenn kein Attribut fr ein Property gefunden wurde
    // Hier kann man z.B. dem Property einen Defaultwert verpassen
    procedure AttributForSimplePropertyNotFound(PropertyDescription: TRakBinaryWriteToSimplePropertyDescription);
    procedure AfterWriteToSimpleProperty(const PropertyDescription: TRakBinaryWriteToSimplePropertyDescription);

    // BeforeWriteToSimpleProperty gibt einem die Mglichkeit das Setzen einzelner Properties
    // zu unterbinden oder das Laden aus einem Tag mit einem anderen Namen zu veranlassen
    procedure BeforeWriteToClassProperty(var PropertyDescription: TRakBinaryWriteToClassPropertyDescription);
    // TagForClassPropertyNotFound wird aufgerufen, wenn kein Tag fr ein Property gefunden wurde
    // Hier kann man z.B. dem Property einen Defaultwert verpassen
    procedure TagForClassPropertyNotFound(PropertyDescription: TRakBinaryWriteToClassPropertyDescription);
    procedure AfterWriteToClassProperty(const PropertyDescription: TRakBinaryWriteToClassPropertyDescription);

    // HandleNotSupportedWriteProperties wird fr die Properties aufgerufen, die einem Typ haben,
    // der vom Defaultverhalten nicht untersttzt wird.
    procedure HandleNotSupportedWriteProperties(PropertyName: ShortString);
  end;

  TRakBinaryWriteTo = class(TInterfacedObject, IRakBinaryWriteTo)
  public
    procedure BeforeWriteToProperties(Tag: TRakBinaryTag; Options: IRakBinaryWriteToPropertiesOptions; var DoIt: boolean); virtual;
    procedure AfterWriteToProperties(Tag: TRakBinaryTag); virtual;
    procedure BeforeWriteToSimpleProperty(var PropertyDescription: TRakBinaryWriteToSimplePropertyDescription); virtual;
    procedure AttributForSimplePropertyNotFound(PropertyDescription: TRakBinaryWriteToSimplePropertyDescription); virtual;
    procedure AfterWriteToSimpleProperty(const PropertyDescription: TRakBinaryWriteToSimplePropertyDescription); virtual;
    procedure BeforeWriteToClassProperty(var PropertyDescription: TRakBinaryWriteToClassPropertyDescription); virtual;
    procedure TagForClassPropertyNotFound(PropertyDescription: TRakBinaryWriteToClassPropertyDescription); virtual;
    procedure AfterWriteToClassProperty(const PropertyDescription: TRakBinaryWriteToClassPropertyDescription); virtual;
    procedure HandleNotSupportedWriteProperties(PropertyName: ShortString); virtual;
  end;

  TRakBinaryReadWrite = class(TInterfacedObject, IRakBinaryReadFrom, IRakBinaryWriteTo)
  public
    procedure BeforeReadFromProperties(Tag: TRakBinaryTag; Options: IRakBinaryReadFromPropertiesOptions; var DoIt: boolean); virtual;
    procedure AfterReadFromProperties(Tag: TRakBinaryTag); virtual;
    procedure BeforeReadFromSimpleProperty(var PropertyDescription: TRakBinaryReadFromSimplePropertyDescription); virtual;
    procedure AfterReadFromSimpleProperty(const PropertyDescription: TRakBinaryReadFromSimplePropertyDescription); virtual;
    procedure BeforeReadFromClassProperty(var PropertyDescription: TRakBinaryReadFromClassPropertyDescription); virtual;
    procedure AfterReadFromClassProperty(const PropertyDescription: TRakBinaryReadFromClassPropertyDescription); virtual;
    procedure HandleNotSupportedReadProperties(PropertyName: ShortString); virtual;

    procedure BeforeWriteToProperties(Tag: TRakBinaryTag; Options: IRakBinaryWriteToPropertiesOptions; var DoIt: boolean); virtual;
    procedure AfterWriteToProperties(Tag: TRakBinaryTag); virtual;
    procedure BeforeWriteToSimpleProperty(var PropertyDescription: TRakBinaryWriteToSimplePropertyDescription); virtual;
    procedure AttributForSimplePropertyNotFound(PropertyDescription: TRakBinaryWriteToSimplePropertyDescription); virtual;
    procedure AfterWriteToSimpleProperty(const PropertyDescription: TRakBinaryWriteToSimplePropertyDescription); virtual;
    procedure BeforeWriteToClassProperty(var PropertyDescription: TRakBinaryWriteToClassPropertyDescription); virtual;
    procedure TagForClassPropertyNotFound(PropertyDescription: TRakBinaryWriteToClassPropertyDescription); virtual;
    procedure AfterWriteToClassProperty(const PropertyDescription: TRakBinaryWriteToClassPropertyDescription); virtual;
    procedure HandleNotSupportedWriteProperties(PropertyName: ShortString); virtual;
  end;

  {****************************************************************************
  <interface name="IRakBinaryReadFromPropertiesOptionsItem" superinterface="IUnknown" public="true">
    <description>
      IRakBinaryReadFromPropertiesOptionsItem wird von IRakBinaryReadFromPropertiesOptions verwendet.
      Mit PropertyName wird das Property festgelegt, fr das das Defaultverhalten durchbrochen
      werden soll.
      Falls man nicht mchte, dass eine Exception geschmissen wird, wenn das Objekt berhaupt kein
      solches published Property hat, dann setzt man IgnoreMissing auf true.
      Weitere Beschreibungen findet man bei den abgeleiteten Interfaces
      IRakBinaryReadFromSimpleProperty und IRakBinaryReadFromClassProperty
    </description>
  </interface>
  ****************************************************************************}
  IRakBinaryReadFromPropertiesOptionsItem = interface
  	['{BAC521AF-399A-4AAE-8D51-DDCD45534EFF}']
  	function GetPropertyName: string;
    function GetIgnoreMissing: boolean;
  	procedure SetPropertyName(Value: string);
    procedure SetIgnoreMissing(Value: boolean);

  	property PropertyName: string read GetPropertyName write SetPropertyName;
    property IgnoreMissing: boolean read GetIgnoreMissing write SetIgnoreMissing; // default: false;
  end;

  {****************************************************************************
  <interface name="IRakBinaryReadFromSimpleProperty" superinterface="IRakBinaryReadFromPropertiesOptionsItem" public="true">
    <description>
      IRakBinaryReadFromSimpleProperty legt fest, wie ein bestimmtes
      Property, das einen einfachen Typ also keine Klasse hat, in ein BinaryTag eingehngt wird.
      Wenn man nicht mchte, dass das Property eingehngt wird, dann gibt man
      bei AttributName einen Leerstring an. Ansonsten gibt man hier den Namen des "Attributs" an,
      das fr das Property erzeugt wird.
      Hinweis: Wenn der Name des Attributs dem Namen des Propeties entsprechen soll,
      	dann kann man sich den ganzen Eintrag sparen, da dies das Defaultverhalten ist,
    </description>
  </interface>
  ****************************************************************************}
  IRakBinaryReadFromSimpleProperty = interface(IRakBinaryReadFromPropertiesOptionsItem)
	  ['{E27FBF5D-E0AE-4AA6-8105-BFB502DC1779}']
    function GetAttributName: string;
    procedure SetAttributName(Value: string);

  	property AttributName: string read GetAttributName write SetAttributName;
  end;

  {****************************************************************************
  <interface name="IRakBinaryReadFromClassProperty" superinterface="IRakBinaryReadFromPropertiesOptionsItem" public="true">
    <description>
      IRakBinaryReadFromClassProperty legt fest, wie ein bestimmtes
      Property, das als Typ eine Klasse hat, in ein BinaryTag eingehngt wird.
      Wenn man nicht mchte, dass das Property eingehngt wird, dann gibt man
      bei TagName einen Leerstring an. Ansonsten gibt man hier den Namen des "Untertags" an,
      das fr das Property erzeugt wird. In der Regel wird wohl TagName, den Namen des
      Properties erhalten und somit den gleichen Wert wie
      AttributName (IRakBinaryReadFromPropertiesOptionsItemSimple).
      ACHTUNG: Es wird geprft, ob TagName eindeutig ist und falls nicht eine Exception geworfen.
      Fr "einfache" Properties werden dann in dem "Untertag" wieder die entsprechenden Attribute
      angelegt. Wenn man mchte, dass auch wieder fr dessen "Class"-Properties Untertags
      rekursiv angelegt werden, dann setzt man ReadFromClassAttributs auf true.
      Wenn man auch fr dessen Properties vom Defaultverhalten abweichen mchte, dann kann man
      dies mit Options wieder genau festlegen.
    </description>
  </interface>
  ****************************************************************************}
  IRakBinaryReadFromClassProperty = interface(IRakBinaryReadFromPropertiesOptionsItem)
  	['{D0F3970D-8D9D-405F-8FA4-D717190701EC}']
  	function GetTagName: string;
    function GetReadFromClassAttributs: boolean;
    function GetOptions: IRakBinaryReadFromPropertiesOptions;

  	procedure SetTagName(Value: string);
    procedure SetReadFromClassAttributs(Value: boolean);

  	property TagName: string read GetTagName write SetTagName;
    property ReadFromClassAttributs: boolean read GetReadFromClassAttributs write SetReadFromClassAttributs;
    property Options: IRakBinaryReadFromPropertiesOptions read GetOptions;
  end;

  {****************************************************************************
  <interface name="IRakBinaryReadFromPropertiesOptions" superinterface="IUnknown" public="true">
    <description>
      IRakBinaryReadFromPropertiesOptions wird z.B. von der Methode
      TRakBinaryTag.ReadFromPublishedProperties als Parameter erwartet.
      Damit kann man festlegen, welche Properties wie als Attribute und Untertags
      aufgenommen werden.
      Defaultmig werden fr alle "einfachen" published Properties eines Objektes
      in dem BinaryTag ein Attribut mit dem Namen und dem Wert des Properties angelegt.
      Properties, die als Typ eine Klasse haben, werden als "Untertag" eingehngt
      und darunter dann wieder die Properties des entsprechenden Objektes als Attribute
      oder weitere "Untertags".
      Dieses Standardverhalten, kann fr jedes Property einzeln abgendert werden.
      Fr jedes Property, das sich nicht nach dem Standardverhalten richten soll,
      muss in IRakBinaryReadFromPropertiesOptions ein Element aufgenommen werden. Dazu
      stehen die Methoden AddSimpleProperty und AddClassProperty zur Verfgung.
      Die Beschreibung, was man am Defaultverhalten ndern kann findet man bei den
      Beschreibungen der Schnittstellen IRakBinaryReadFromPropertiesOptionsItemSimple
      und IRakBinaryReadFromClassProperty.
    </description>
  </interface>
  ****************************************************************************}
  IRakBinaryReadFromPropertiesOptions = interface
  	function GetCount: integer;
    function GetItems(Idx: integer): IRakBinaryReadFromPropertiesOptionsItem;

    function AddSimpleProperty: IRakBinaryReadFromSimpleProperty;
    function AddClassProperty: IRakBinaryReadFromClassProperty;

    function ItemByName(Name: ShortString): IRakBinaryReadFromPropertiesOptionsItem;

  	property Count: integer read GetCount;
    property Items[Idx: integer]: IRakBinaryReadFromPropertiesOptionsItem read GetItems; default;
  end;

  {****************************************************************************
  <interface name="IRakBinaryWriteToPropertiesOptionsItem" superinterface="IUnknown" public="true">
    <description>
      IRakBinaryWriteToPropertiesOptionsItem wird von IRakBinaryWriteToPropertiesOptions verwendet.
      Mit PropertyName wird das Property festgelegt, fr das das Defaultverhalten durchbrochen
      werden soll.
      Falls man nicht mchte, dass eine Exception geschmissen wird, wenn das Objekt berhaupt kein
      solches published Property hat, dann setzt man IgnoreMissing auf true.
      Weitere Beschreibungen findet man bei den abgeleiteten Interfaces
      IRakBinaryWriteToSimpleProperty und IRakBinaryWriteToClassProperty
    </description>
  </interface>
  ****************************************************************************}
  IRakBinaryWriteToPropertiesOptionsItem = interface
  	['{30499310-A615-46DE-B441-C27E78E8533A}']
  	function GetPropertyName: string;
    function GetIgnoreMissing: boolean;
  	procedure SetPropertyName(Value: string);
    procedure SetIgnoreMissing(Value: boolean);

  	property PropertyName: string read GetPropertyName write SetPropertyName;
    property IgnoreMissing: boolean read GetIgnoreMissing write SetIgnoreMissing; // default: false;
  end;

  {****************************************************************************
  <interface name="IRakBinaryWriteToSimpleProperty" superinterface="IRakBinaryWriteToPropertiesOptionsItem" public="true">
    <description>
      IRakBinaryWriteToSimpleProperty legt fest, wie ein bestimmtes
      Property, das einen einfachen Typ also keine Klasse hat, aus einem BinaryTag versorgt wird.
      Wenn man nicht mchte, dass der Wert des Properties gendert wird, dann gibt man
      bei AttributName einen Leerstring an. Ansonsten gibt man hier den Namen des "Attributs" an,
      dessen Wert das Property bernehmen soll.
      Hinweis: Wenn der Name des Attributs dem Namen des Propeties entsprechen soll,
      	dann kann man sich den ganzen Eintrag sparen, da dies das Defaultverhalten ist,
    </description>
  </interface>
  ****************************************************************************}
  IRakBinaryWriteToSimpleProperty = interface(IRakBinaryWriteToPropertiesOptionsItem)
	  ['{6540149F-0FC8-4BE6-8E3F-E12295166531}']
    function GetAttributName: string;
    procedure SetAttributName(Value: string);

  	property AttributName: string read GetAttributName write SetAttributName;
  end;

  {****************************************************************************
  <interface name="IRakBinaryWriteToClassProperty" superinterface="IRakBinaryWriteToPropertiesOptionsItem" public="true">
    <description>
      IRakBinaryWriteToClassProperty legt fest, wie ein bestimmtes
      Property, das als Typ eine Klasse hat, aus einem BinaryTag versorgt wird.
      Wenn man nicht mchte, dass das Property verndert wird, dann gibt man
      bei TagName einen Leerstring an. Ansonsten gibt man hier den Namen des "Untertags" an,
      aus dem das Property versorgt werden soll. In der Regel wird wohl TagName, den Namen des
      Properties erhalten und somit den gleichen Wert wie
      AttributName (IRakBinaryWriteToPropertiesOptionsItemm).
      "Einfache" Properties werden dann wieder aus dem entsprechenden Attribut des "Untertags"
      versorgt.
      Wenn man auch fr dessen Properties vom Defaultverhalten abweichen mchte, dann kann man
      dies mit Options wieder genau festlegen.
    </description>
  </interface>
  ****************************************************************************}
  IRakBinaryWriteToClassProperty = interface(IRakBinaryWriteToPropertiesOptionsItem)
  	['{B3BF38A3-D522-43F0-9C41-289F8ED8C058}']
  	function GetTagName: string;
    function GetOptions: IRakBinaryWriteToPropertiesOptions;

  	procedure SetTagName(Value: string);

  	property TagName: string read GetTagName write SetTagName;
    property Options: IRakBinaryWriteToPropertiesOptions read GetOptions;
  end;

  {****************************************************************************
  <interface name="IRakBinaryWriteToPropertiesOptions" superinterface="IUnknown" public="true">
    <description>
      IRakBinaryWriteToPropertiesOptions wird z.B. von der Methode
      TRakBinaryTag.WriteToPublishedProperties als Parameter erwartet.
      Damit kann man festlegen, wie Properties aus Attributen und Untertags
      mit Werten gefllt werden.
      Defaultmig werden alle "einfachen" published Properties eines Objektes
      mit dem Wert eines gleichnamigen Attributs gefllt. Wenn es kein gleichnamiges
      Attribut gibt oder es einen falschen Typ hat, dann wird das Property nicht verndert.
      Properties, die als Typ eine Klasse haben, werden aus gleichnamigen "Untertags" gefllt.

      Dieses Standardverhalten, kann fr jedes Property einzeln abgendert werden.
      Fr jedes Property, das sich nicht nach dem Standardverhalten richten soll,
      muss in IRakBinaryWriteToPropertiesOptions ein Element aufgenommen werden. Dazu
      stehen die Methoden AddSimpleProperty und AddClassProperty zur Verfgung.
      Die Beschreibung, was man am Defaultverhalten ndern kann findet man bei den
      Beschreibungen der Schnittstellen IRakBinaryWriteToPropertiesOptionsItemSimple
      und IRakBinaryWriteClassProperty.
    </description>
  </interface>
  ****************************************************************************}
  IRakBinaryWriteToPropertiesOptions = interface
  	function GetCount: integer;
    function GetItems(Idx: integer): IRakBinaryWriteToPropertiesOptionsItem;

    function AddSimpleProperty: IRakBinaryWriteToSimpleProperty;
    function AddClassProperty: IRakBinaryWriteToClassProperty;

    function ItemByName(Name: ShortString): IRakBinaryWriteToPropertiesOptionsItem;

  	property Count: integer read GetCount;
    property Items[Idx: integer]: IRakBinaryWriteToPropertiesOptionsItem read GetItems; default;
  end;


  {****************************************************************************
  <class name="TRakBinaryAttribute" superclass="TRakKompositionItem" public="true" defaultproperty="">
    <description>
      TRakBinaryAttribute ist die abstrakte Basisklasse fr alle Klassen,
      die ein Attribut eines Tags (TRakBinaryTag) reprsentieren knnen.
      Jedes Attribut hat einen eindeutigen Typ (TRakBinaryDataType). Der Typ
      ergibt sich automatisch aus der Klasse selbst. Es gibt eine Reihe von
      Basistypen, die unmittelbar von TRakBinaryAttribute abgeleitet sind.
      Diese Basistypen sind von jetzt an fr alle Zeiten festgelegt. Es knnen
      keine weiteren Basistypen mehr dazu kommen. Es knnen aber weitere
      Typen von den Basistypen abgeleitet werden.
      Die Basistypen sind:

      TRakBinaryBlob         // Zum Speichern beliebiger binrer Daten
      TRakBinaryBoolean     // Zum Speichern von Werten des Typs boolean (1 Byte)
      TRakBinaryCurrency    // Zum Speichern von Werten des Typs Currency
      TRakBinaryDateTime    // Zum Speichern von Werten des Typs TDateTime
      TRakBinaryReal48      // Zum Speichern von Werten des Typs Real48
      TRakBinarySingle      // Zum Speichern von Werten des Typs Single
      TRakBinaryDouble      // Zum Speichern von Werten des Typs Double
      TRakBinaryExtended    // Zum Speichern von Werten des Typs Extended
      TRakBinaryComp        // Zum Speichern von Werten des Typs Comp
      TRakBinaryByte         // Zum Speichern von Werten des Typs Byte
      TRakBinaryShortint    // Zum Speichern von Werten des Typs Shortint
      TRakBinaryChar         // Zum Speichern von Werten des Typs char
      TRakBinaryWord        // Zum Speichern von Werten des Typs Word
      TRakBinarySmallint    // Zum Speichern von Werten des Typs Smallint
      TRakBinaryLongword     // Zum Speichern von Werten des Typs Longword
      TRakBinaryLongint      // Zum Speichern von Werten des Typs Longint
      TRakBinaryInt64       // Zum Speichern von Werten des Typs Int64
      TRakBinaryShortString  // Zum Speichern von Werten des Typs ShortString
      TRakBinaryAnsiString  // Zum Speichern von Werten des Typs AnsiString
      TRakBinaryWideString  // Zum Speichern von Werten des Typs WideString

      Wenn Sie sich wundern sollten, dass es keine Basistypen fr "interger"
      und "string" gibt, dann liegt das daran, dass "integer" und "string" keine
      festgelegte Gre besitzen. Wie "string" vom Compiler verwendet wird,
      wird durch die Compilerdirektive &H festgelegt, wodurch sich der
      Speicherbedarf von Strings ndert. Zum Speichern von "strings" verwenden
      Sie einfach TRakBinaryAnsiString.
      Wie der Typ "integer" vom Compiler verwendet wird, hngt von der
      Compilerversion ab. Bei Delphi 1 entsprach "integer" noch dem Typ "Shortint".
      Bei Delphi 5 entspricht "integer" dem Typ Longint. Und wer wei, welchem Typ
      "integer" in der Delphi Version 10 entsprechen wird.

      TRakBinaryBlob ist fr abgeleitete Klassen sicherlich am interessantesten
      (z.B. Klasse TRakBinaryGraphic).
      Jeder Datentyp wei von welchem Basistyp er direkt oder indirekt
      abgeleitet ist, siehe Property BaseTypeId.
      Jeder Datentyp hat einen Namen (Property ItemName), der das Attribut
      innerhalb seines besitztenden Tags eindeutig macht. (Der eindeutige Name
      wird mit Hilfe des Propertys NamespaceMember sichergestellt)
      Ein Attribut kann seinen Namen und seinen Wert in einen Stream speichern
      und daraus laden.

      Ausserdem ist jeder Datentyp in der Lage seinen Wert in Stringform
      darzustellen, siehe Property AsWideString. Dadurch wird es mglich,
      die binren Daten in ein Textformat wie z.B. XML umzuwandeln.

      Hinweis: Datentypen, die von Basistypen abgeleitet sind, bentigen
                beim Speichern mehr Speicherplatz als der Basistyp selbst.
                Dies ist so, obwohl sich die Gre der eigentlichen Daten
                gar nicht ndern kann. Das liegt daran, dass noch
                Zusatzinformation abgespeichert werden muss, damit beim Laden
                wieder eine Instanz des richtigen Datentyps erzeugt werden kann.
                Bei groen Datenmengen, wie z.B. bei Bildern fllt diese
                Zusatzinformation nicht ins Gewicht. Bei kleinen Datentypen
                wie z.B. TRakBinaryByte, kann das den Speicherbedarf
                vervielfachen. Deswegen macht es wahrscheinlich nur Sinn
                von TRakBinaryBlob eigene Datentypen abzuleiten.
    </description>
  </class>
  ****************************************************************************}
  TRakBinaryAttribute = class(TRakKompositionItem, IRakNamespaceMember) // TRakCollectionItem
  private
    FIsNull: boolean;
    FNamespaceMember: TRakNamespaceMemberImplementation;

    procedure SetItemName(const Value: ShortString);
    function GetBaseTypeId: Byte;
    procedure SetAsWideString(const Value: WideString);
    function GetAsWideString: WideString;
    procedure SetIsNull(const Value: boolean);

    property NamespaceMember: TRakNamespaceMemberImplementation read FNamespaceMember implements IRakNamespaceMember;
    function GetItemName: ShortString;
    function GetDataTypeAsString: ShortString;
    function GetDataType: TRakBinaryDataType;
    function GetNullString: string;

    procedure SaveValue(Stream: TStream); virtual; abstract;
    procedure LoadValue(Stream: TStream); virtual; abstract;
    procedure SetValueAsWideString(const Value: WideString); virtual; abstract;
    function GetValueAsWideString: WideString; virtual; abstract;
    function GetParent: TRakBinaryAttributes;
  protected
    procedure SetCollection(Value: TRakCollection); override;
  public
    constructor Create; override;
    destructor Destroy; override;

    procedure Assign(Source: TRakCollectionItem); override;

    // BaseIdOf ermittlet fr einen Datentyp, die ID des Basistyps, von dem der Datentyp abgeleitet ist (oder selbst ist)
    class function BaseIdOf(DataType: TRakBinaryDataType): Byte;
    // DataTypeOf ermittlet den Basistyp anhand seiner ID;
    class function DataTypeOf(BaseId: Byte): TRakBinaryDataType;
    // DataTypeOfClassName ermittelt den Datentyp anhand seines Klassennamens
    class function DataTypeOfClassName(Name: ShortString): TRakBinaryDataType;
    // Mit RegisterBinaryDataType mssen alle Datentypen bekannt gemacht werden,
    // die von einem Basistyp abgeleitet sind, damit das Laden aus einem Stream
    // korrekt funktioniert. Sollte ein Typ nicht registriert worden sein,
    // dann wird beim Laden aus einem Stream nicht eine Instanz des Datentyps,
    // sondern eine Instanz des Basistyps erzeugt, von dem der Datentyp abgeleitet ist.
    class procedure RegisterBinaryDataType(DataType: TRakBinaryDataType);

    // SaveToStream speichert den Namen und den Wert des Attributes in einen Stream
    procedure SaveToStream(Stream: TStream);
    // LoadFromStream ldt den Namen und den Wert des Attributes aus einem Stream
    procedure LoadFromStream(Stream: TStream);

    // DataType ist der Datentyp des Attributes. Der Datentyp ist die Klasse, s.a. TObject.ClassType
    property DataType: TRakBinaryDataType read GetDataType;
    // DataTypeAsString ist der Datetyp in string-Form. DataTypeAsString ist der Name der Klasse s.a. TObject.ClassName
    property DataTypeAsString: ShortString read GetDataTypeAsString;
    // BaseTypeId gibt den Basistyp an, von dem der Datentyp abgeleitet ist. Der Basistyp legt fest, wie die Daten in einem Stream gespeichert werden.
    property BaseTypeId: Byte read GetBaseTypeId;

    // ItemName ist der eindeutige Name des Attributes
    property ItemName: ShortString read GetItemName write SetItemName;

    // AsWideString ist der Wert des Attributes in String-Form
    property AsWideString: WideString read GetAsWideString write SetAsWideString;
    // IsNull gibt an, ob das Attribut berhaupt einen definierten Wert hat
    property IsNull: boolean read FIsNull write SetIsNull;

    property Parent: TRakBinaryAttributes read GetParent;
  end;


  {****************************************************************************
  <class name="TRakBinaryAttributes" superclass="TRakCollection" public="true" defaultproperty="Items">
    <description>
      TRakBinaryAttributes wird von einem Tag (TRakBinaryTag)
      zum Verwalten seiner Attribute verwendet
    </description>
  </class>
  ****************************************************************************}
  TRakBinaryAttributes = class(TRakCollection, IRakNamespace)
  private
    FNamespace: TRakNamespaceImplementation;

    function GetItems(Idx: integer): TRakBinaryAttribute;
  protected
    property Namespace: TRakNamespaceImplementation read FNamespace implements IRakNameSpace;
  public
    constructor Create; override;
    destructor Destroy; override;

    // Add erzeugt ein neues Attribut und gibt eine Referenz darauf zurck
    function Add(DataType: TRakBinaryDataType): TRakBinaryAttribute;
    // AttributeByName sucht nach einem Attribut mit einem bestimmten Namen
    function AttributeByName(Name: string): TRakBinaryAttribute;

    // SaveToStream speichert alle Attribute in einem Stream
    procedure SaveToStream(Stream: TStream);
    // LoadFromStream ldt Daten aus einem Stream, anhand denen Attribute erzeugt werden
    procedure LoadFromStream(Stream: TStream);

    // Mit Items kann auf alle Attribute zugegriffen werden
    property Items[Idx: integer]: TRakBinaryAttribute read GetItems; default;
  end;

  {****************************************************************************
  <class name="TRakBinaryAnsiString" superclass="TRakBinaryAttribute" public="true" defaultproperty="">
    <description>
      TRakBinaryAnsiString ist der Basisdatentyp, der fr ein Attribut steht,
      das Daten in einem AnsiString speichert.
    </description>
  </class>
  ****************************************************************************}
  TRakBinaryAnsiString = class(TRakBinaryAttribute)
  private
    FValue: AnsiString;

    procedure SaveValue(Stream: TStream); override;
    procedure LoadValue(Stream: TStream); override;
    procedure SetValueAsWideString(const Value: WideString); override;
    function GetValueAsWideString: WideString; override;
  protected
    procedure SetValue(const Value: AnsiString); virtual;
    function GetValue: AnsiString; virtual;
  public
    class procedure AddToTag(Tag: TRakBinaryTag; Name: ShortString; Value: AnsiString);

    procedure Assign(Source: TRakCollectionItem); override;

    property Value: AnsiString read GetValue write SetValue;
  end;

  {****************************************************************************
  <class name="TRakBinaryShortString" superclass="TRakBinaryAttribute" public="true" defaultproperty="">
    <description>
      TRakBinaryShortString ist der Basisdatentyp, der fr ein Attribut steht,
      das Daten in einem ShortString speichert.
    </description>
  </class>
  ****************************************************************************}
  TRakBinaryShortString = class(TRakBinaryAttribute)
  private
    FValue: ShortString;

    procedure SaveValue(Stream: TStream); override;
    procedure LoadValue(Stream: TStream); override;
    procedure SetValueAsWideString(const Value: WideString); override;
    function GetValueAsWideString: WideString; override;
  protected
    procedure SetValue(const Value: ShortString); virtual;
    function GetValue: ShortString; virtual;
  public
    class procedure AddToTag(Tag: TRakBinaryTag; Name: ShortString; Value: ShortString);

    procedure Assign(Source: TRakCollectionItem); override;

    property Value: ShortString read GetValue write SetValue;
  end;

  {****************************************************************************
  <class name="TRakBinaryWideString" superclass="TRakBinaryAttribute" public="true" defaultproperty="">
    <description>
      TRakBinaryWideString ist der Basisdatentyp, der fr ein Attribut steht,
      das Daten in einem WideString speichert.
    </description>
  </class>
  ****************************************************************************}
  TRakBinaryWideString = class(TRakBinaryAttribute)
  private
    FValue: WideString;

    procedure SaveValue(Stream: TStream); override;
    procedure LoadValue(Stream: TStream); override;
    procedure SetValueAsWideString(const Value: WideString); override;
    function GetValueAsWideString: WideString; override;
  protected
    procedure SetValue(const Value: WideString); virtual;
    function GetValue: WideString; virtual;
  public
    class procedure AddToTag(Tag: TRakBinaryTag; Name: ShortString; Value: WideString);

    procedure Assign(Source: TRakCollectionItem); override;

    property Value: WideString read GetValue write SetValue;
  end;

  {****************************************************************************
  <class name="TRakBinaryByte" superclass="TRakBinaryAttribute" public="true" defaultproperty="">
    <description>
      TRakBinaryByte ist der Basisdatentyp, der fr ein Attribut steht,
      das Daten in einem Byte speichert.
    </description>
  </class>
  ****************************************************************************}
  TRakBinaryByte = class(TRakBinaryAttribute)
  private
    FValue: Byte;

    procedure SaveValue(Stream: TStream); override;
    procedure LoadValue(Stream: TStream); override;
    procedure SetValueAsWideString(const Value: WideString); override;
    function GetValueAsWideString: WideString; override;
  protected
    procedure SetValue(const Value: Byte); virtual;
    function GetValue: Byte; virtual;
  public
    class procedure AddToTag(Tag: TRakBinaryTag; Name: ShortString; Value: Byte);

    procedure Assign(Source: TRakCollectionItem); override;

    property Value: Byte read GetValue write SetValue;
  end;

  {****************************************************************************
  <class name="TRakBinaryShortint" superclass="TRakBinaryAttribute" public="true" defaultproperty="">
    <description>
      TRakBinaryShortint ist der Basisdatentyp, der fr ein Attribut steht,
      das Daten in einem Shortint speichert.
    </description>
  </class>
  ****************************************************************************}
  TRakBinaryShortint = class(TRakBinaryAttribute)
  private
    FValue: Shortint;

    procedure SaveValue(Stream: TStream); override;
    procedure LoadValue(Stream: TStream); override;
    procedure SetValueAsWideString(const Value: WideString); override;
    function GetValueAsWideString: WideString; override;
  protected
    procedure SetValue(const Value: Shortint); virtual;
    function GetValue: Shortint; virtual;
  public
    class procedure AddToTag(Tag: TRakBinaryTag; Name: ShortString; Value: Shortint);

    procedure Assign(Source: TRakCollectionItem); override;

    property Value: Shortint read GetValue write SetValue;
  end;

  {****************************************************************************
  <class name="TRakBinaryChar" superclass="TRakBinaryAttribute" public="true" defaultproperty="">
    <description>
      TRakBinaryChar ist der Basisdatentyp, der fr ein Attribut steht,
      das Daten in einem Char speichert.
    </description>
  </class>
  ****************************************************************************}
  TRakBinaryChar = class(TRakBinaryAttribute)
  private
    FValue: Char;

    procedure SaveValue(Stream: TStream); override;
    procedure LoadValue(Stream: TStream); override;
    procedure SetValueAsWideString(const Value: WideString); override;
    function GetValueAsWideString: WideString; override;
  protected
    procedure SetValue(const Value: Char); virtual;
    function GetValue: Char; virtual;
  public
    class procedure AddToTag(Tag: TRakBinaryTag; Name: ShortString; Value: Char);

    procedure Assign(Source: TRakCollectionItem); override;

    property Value: Char read GetValue write SetValue;
  end;

  {****************************************************************************
  <class name="TRakBinaryWord" superclass="TRakBinaryAttribute" public="true" defaultproperty="">
    <description>
      TRakBinaryWord ist der Basisdatentyp, der fr ein Attribut steht,
      das Daten in einem Word speichert.
    </description>
  </class>
  ****************************************************************************}
  TRakBinaryWord = class(TRakBinaryAttribute)
  private
    FValue: Word;

    procedure SaveValue(Stream: TStream); override;
    procedure LoadValue(Stream: TStream); override;
    procedure SetValueAsWideString(const Value: WideString); override;
    function GetValueAsWideString: WideString; override;
  protected
    procedure SetValue(const Value: Word); virtual;
    function GetValue: Word; virtual;
  public
    class procedure AddToTag(Tag: TRakBinaryTag; Name: ShortString; Value: Word);

    procedure Assign(Source: TRakCollectionItem); override;

    property Value: Word read GetValue write SetValue;
  end;

  {****************************************************************************
  <class name="TRakBinarySmallint" superclass="TRakBinaryAttribute" public="true" defaultproperty="">
    <description>
      TRakBinarySmallint ist der Basisdatentyp, der fr ein Attribut steht,
      das Daten in einem Smallint speichert.
    </description>
  </class>
  ****************************************************************************}
  TRakBinarySmallint = class(TRakBinaryAttribute)
  private
    FValue: Smallint;

    procedure SaveValue(Stream: TStream); override;
    procedure LoadValue(Stream: TStream); override;
    procedure SetValueAsWideString(const Value: WideString); override;
    function GetValueAsWideString: WideString; override;
  protected
    procedure SetValue(const Value: Smallint); virtual;
    function GetValue: Smallint; virtual;
  public
    class procedure AddToTag(Tag: TRakBinaryTag; Name: ShortString; Value: Smallint);

    procedure Assign(Source: TRakCollectionItem); override;

    property Value: Smallint read GetValue write SetValue;
  end;

  {****************************************************************************
  <class name="TRakBinaryLongword" superclass="TRakBinaryAttribute" public="true" defaultproperty="">
    <description>
      TRakBinaryLongword ist der Basisdatentyp, der fr ein Attribut steht,
      das Daten in einem Longword speichert.
    </description>
  </class>
  ****************************************************************************}
  TRakBinaryLongword = class(TRakBinaryAttribute)
  private
    FValue: LongWord;

    procedure SaveValue(Stream: TStream); override;
    procedure LoadValue(Stream: TStream); override;
    procedure SetValueAsWideString(const Value: WideString); override;
    function GetValueAsWideString: WideString; override;
  protected
    procedure SetValue(const Value: LongWord); virtual;
    function GetValue: LongWord; virtual;
  public
    class procedure AddToTag(Tag: TRakBinaryTag; Name: ShortString; Value: LongWord);

    procedure Assign(Source: TRakCollectionItem); override;

    property Value: LongWord read GetValue write SetValue;
  end;

  {****************************************************************************
  <class name="TRakBinaryLongint" superclass="TRakBinaryAttribute" public="true" defaultproperty="">
    <description>
      TRakBinaryLongint ist der Basisdatentyp, der fr ein Attribut steht,
      das Daten in einem Longint speichert.
    </description>
  </class>
  ****************************************************************************}
  TRakBinaryLongint = class(TRakBinaryAttribute)
  private
    FValue: Longint;

    procedure SaveValue(Stream: TStream); override;
    procedure LoadValue(Stream: TStream); override;
    procedure SetValueAsWideString(const Value: WideString); override;
    function GetValueAsWideString: WideString; override;
  protected
    procedure SetValue(const Value: Longint); virtual;
    function GetValue: Longint; virtual;
  public
    class procedure AddToTag(Tag: TRakBinaryTag; Name: ShortString; Value: Longint);

    procedure Assign(Source: TRakCollectionItem); override;

    property Value: Longint read GetValue write SetValue;
  end;

  {****************************************************************************
  <class name="TRakBinaryInt64" superclass="TRakBinaryAttribute" public="true" defaultproperty="">
    <description>
      TRakBinaryInt64 ist der Basisdatentyp, der fr ein Attribut steht,
      das Daten in einem Int64 speichert.
    </description>
  </class>
  ****************************************************************************}
  TRakBinaryInt64 = class(TRakBinaryAttribute)
  private
    FValue: Int64;

    procedure SaveValue(Stream: TStream); override;
    procedure LoadValue(Stream: TStream); override;
    procedure SetValueAsWideString(const Value: WideString); override;
    function GetValueAsWideString: WideString; override;
  protected
    procedure SetValue(const Value: Int64); virtual;
    function GetValue: Int64; virtual;
  public
    class procedure AddToTag(Tag: TRakBinaryTag; Name: ShortString; Value: Int64);

    procedure Assign(Source: TRakCollectionItem); override;

    property Value: Int64 read GetValue write SetValue;
  end;

  {****************************************************************************
  <class name="TRakBinaryExtended" superclass="TRakBinaryAttribute" public="true" defaultproperty="">
    <description>
      TRakBinaryExtended ist der Basisdatentyp, der fr ein Attribut steht,
      das Daten in einem extended speichert.
    </description>
  </class>
  ****************************************************************************}
  TRakBinaryExtended = class(TRakBinaryAttribute)
  private
    FValue: extended;

    procedure SaveValue(Stream: TStream); override;
    procedure LoadValue(Stream: TStream); override;
    procedure SetValueAsWideString(const Value: WideString); override;
    function GetValueAsWideString: WideString; override;
  protected
    procedure SetValue(const Value: extended); virtual;
    function GetValue: extended; virtual;
  public
    class procedure AddToTag(Tag: TRakBinaryTag; Name: ShortString; Value: extended);

    procedure Assign(Source: TRakCollectionItem); override;

    property Value: extended read GetValue write SetValue;
  end;

  {****************************************************************************
  <class name="TRakBinaryReal48" superclass="TRakBinaryAttribute" public="true" defaultproperty="">
    <description>
      TRakBinaryReal48 ist der Basisdatentyp, der fr ein Attribut steht,
      das Daten in einem Real48 speichert.
    </description>
  </class>
  ****************************************************************************}
  TRakBinaryReal48 = class(TRakBinaryAttribute)
  private
    FValue: Real48;

    procedure SaveValue(Stream: TStream); override;
    procedure LoadValue(Stream: TStream); override;
    procedure SetValueAsWideString(const Value: WideString); override;
    function GetValueAsWideString: WideString; override;
  protected
    procedure SetValue(const Value: Real48); virtual;
    function GetValue: Real48; virtual;
  public
    class procedure AddToTag(Tag: TRakBinaryTag; Name: ShortString; Value: Real48);

    procedure Assign(Source: TRakCollectionItem); override;

    property Value: Real48 read GetValue write SetValue;
  end;

  {****************************************************************************
  <class name="TRakBinarySingle" superclass="TRakBinaryAttribute" public="true" defaultproperty="">
    <description>
      TRakBinarySingle ist der Basisdatentyp, der fr ein Attribut steht,
      das Daten in einem Single speichert.
    </description>
  </class>
  ****************************************************************************}
  TRakBinarySingle = class(TRakBinaryAttribute)
  private
    FValue: Single;

    procedure SaveValue(Stream: TStream); override;
    procedure LoadValue(Stream: TStream); override;
    procedure SetValueAsWideString(const Value: WideString); override;
    function GetValueAsWideString: WideString; override;
  protected
    procedure SetValue(const Value: Single); virtual;
    function GetValue: Single; virtual;
  public
    class procedure AddToTag(Tag: TRakBinaryTag; Name: ShortString; Value: Single);

    procedure Assign(Source: TRakCollectionItem); override;

    property Value: Single read GetValue write SetValue;
  end;

  {****************************************************************************
  <class name="TRakBinaryDouble" superclass="TRakBinaryAttribute" public="true" defaultproperty="">
    <description>
      TRakBinaryDouble ist der Basisdatentyp, der fr ein Attribut steht,
      das Daten in einem Double speichert.
    </description>
  </class>
  ****************************************************************************}
  TRakBinaryDouble = class(TRakBinaryAttribute)
  private
    FValue: Double;

    procedure SaveValue(Stream: TStream); override;
    procedure LoadValue(Stream: TStream); override;
    procedure SetValueAsWideString(const Value: WideString); override;
    function GetValueAsWideString: WideString; override;
  protected
    procedure SetValue(const Value: Double); virtual;
    function GetValue: Double; virtual;
  public
    class procedure AddToTag(Tag: TRakBinaryTag; Name: ShortString; Value: Double);

    procedure Assign(Source: TRakCollectionItem); override;

    property Value: Double read GetValue write SetValue;
  end;

  {****************************************************************************
  <class name="TRakBinaryComp" superclass="TRakBinaryAttribute" public="true" defaultproperty="">
    <description>
      TRakBinaryComp ist der Basisdatentyp, der fr ein Attribut steht,
      das Daten in einem Comp speichert.
    </description>
  </class>
  ****************************************************************************}
  TRakBinaryComp = class(TRakBinaryAttribute)
  private
    FValue: Comp;

    procedure SaveValue(Stream: TStream); override;
    procedure LoadValue(Stream: TStream); override;
    procedure SetValueAsWideString(const Value: WideString); override;
    function GetValueAsWideString: WideString; override;
  protected
    procedure SetValue(const Value: Comp); virtual;
    function GetValue: Comp; virtual;
  public
    class procedure AddToTag(Tag: TRakBinaryTag; Name: ShortString; Value: Comp);

    procedure Assign(Source: TRakCollectionItem); override;

    property Value: Comp read GetValue write SetValue;
  end;

  {****************************************************************************
  <class name="TRakBinaryCurrency" superclass="TRakBinaryAttribute" public="true" defaultproperty="">
    <description>
      TRakBinaryCurrency ist der Basisdatentyp, der fr ein Attribut steht,
      das Daten in einem Currency speichert.
    </description>
  </class>
  ****************************************************************************}
  TRakBinaryCurrency = class(TRakBinaryAttribute)
  private
    FValue: Currency;

    procedure SaveValue(Stream: TStream); override;
    procedure LoadValue(Stream: TStream); override;
    procedure SetValueAsWideString(const Value: WideString); override;
    function GetValueAsWideString: WideString; override;
  protected
    procedure SetValue(const Value: Currency); virtual;
    function GetValue: Currency; virtual;
  public
    class procedure AddToTag(Tag: TRakBinaryTag; Name: ShortString; Value: Currency);

    procedure Assign(Source: TRakCollectionItem); override;

    property Value: Currency read GetValue write SetValue;
  end;

  {****************************************************************************
  <class name="TRakBinaryBoolean" superclass="TRakBinaryAttribute" public="true" defaultproperty="">
    <description>
      TRakBinaryBoolean ist der Basisdatentyp, der fr ein Attribut steht,
      das Daten in einem boolean speichert.
    </description>
  </class>
  ****************************************************************************}
  TRakBinaryBoolean = class(TRakBinaryAttribute)
  private
    FValue: boolean;

    procedure SaveValue(Stream: TStream); override;
    procedure LoadValue(Stream: TStream); override;
    procedure SetValueAsWideString(const Value: WideString); override;
    function GetValueAsWideString: WideString; override;
  protected
    procedure SetValue(const Value: boolean); virtual;
    function GetValue: boolean; virtual;
  public
    class procedure AddToTag(Tag: TRakBinaryTag; Name: ShortString; Value: boolean);

    procedure Assign(Source: TRakCollectionItem); override;

    property Value: boolean read GetValue write SetValue;
  end;

  {****************************************************************************
  <class name="TRakBinaryCurrency" superclass="TRakBinaryDateTime" public="true" defaultproperty="">
    <description>
      TRakBinaryDateTime ist der Basisdatentyp, der fr ein Attribut steht,
      das Daten in einem TDateTime speichert.
    </description>
  </class>
  ****************************************************************************}
  TRakBinaryDateTime = class(TRakBinaryAttribute)
  private
    FValue: TDateTime;

    procedure SaveValue(Stream: TStream); override;
    procedure LoadValue(Stream: TStream); override;
    procedure SetValueAsWideString(const Value: WideString); override;
    function GetValueAsWideString: WideString; override;
  protected
    procedure SetValue(const Value: TDateTime); virtual;
    function GetValue: TDateTime; virtual;
  public
    class procedure AddToTag(Tag: TRakBinaryTag; Name: ShortString; Value: TDateTime);

    procedure Assign(Source: TRakCollectionItem); override;

    property Value: TDateTime read GetValue write SetValue;
  end;

  {****************************************************************************
  <class name="TRakBinaryBlob" superclass="TRakBinaryDateTime" public="true" defaultproperty="">
    <description>
      TRakBinaryBlob ist der Basisdatentyp, der fr ein Attribut steht,
      das beliebige binre Daten speichert.
      Von TRakBinaryBlob kann man z.B. Datentypen ableiten, die binre
      Daten eines bestimmten Formates speichern. Diesem abgeleiteten
      Datentyp kann man dann Properties und Methoden geben, welche die
      Arbeit mit diesem bestimmten Format erleichtern.
      Ein Beispiel dafr sind Bilddaten, wie sie die Delphi-Klasse TBitmap
      zur Verfgung stellt (siehe Klasse TRakBinaryBitmap).
      Wenn Sie Klassen von TRakBinaryBlob ableiten, dann vergessen Sie bitte nicht,
      die Klassen mit TRakBinaryAttribut.RegisterBinaryDataType zu registrien.
    </description>
  </class>
  ****************************************************************************}
  TRakBinaryBlob = class(TRakBinaryAttribute)
  private
    procedure SaveValue(Stream: TStream); override;
    procedure LoadValue(Stream: TStream); override;
    procedure SetValueAsWideString(const Value: WideString); override;
    function GetValueAsWideString: WideString; override;
    function GetAByte(Idx: Longword): Byte;
    procedure SetAByte(Idx: Longword; const Value: Byte);
    procedure SetAsBase64(const Value: WideString);
    function GetAsBase64: WideString;
  protected
    // In FMemoryStream werden die binren Daten normalerweise aufbewahrt
    // Abgeleitete Klassen knnen ihre binren Daten auch anderweitig speichern.
    // Dann darf man aber nicht vergessen entweder die Methoden
    // GetPointerForLoading und GetPointerForSaving zu berschreiben oder
    // durch berschreiben der Methoden BeforeLoading, AfterLoading,
    // BeforeSaving und AfterSaving sicher zu stellen, dass whrend des
    // Ladens und Speicherns sich die binren Daten in FMemoryStream befinden
    FMemoryStream: TMemoryStream;

    // GetPointerForLoading kann von abgeleiteten Klassen berschrieben werden,
    // falls diese ihre binren Daten nicht in FMemoryStream gespeichert halten.
    // GetPointerForLoading muss einen Zeiger zurckliefern, der angibt,
    // wohin die zu ladenden binren Daten gespeichert werden sollen.
    // Der Paramter Size gibt an, wieviele Bytes geladen werden, also
    // wie gro der Puffer sein muss
    function GetPointerForLoading(Size: Int64): Pointer; virtual;
    // GetPointerForSaving kann von abgeleiteten Klassen berschrieben werden,
    // falls diese ihre binren Daten nicht in FMemoryStream gespeichert halten.
    // GetPointerForSaving muss einen Zeiger zurckliefern, der angibt,
    // von wo die zu speichernden binren Daten gelesen werden knnen.
    function GetPointerForSaving: Pointer; virtual;

    // BeforeLoading wird aufgerufen, bevor neue binre Daten in den Puffer (FMemoryStream) geladen werden
    procedure BeforeLoading; virtual;
    // AfterLoading wird aufgerufen, nachdem neue binre Daten in den Puffer (FMemoryStream) geladen wurden
    procedure AfterLoading; virtual;
    // BeforeSaving wird aufgerufen, bevor die binre Daten aus dem Puffer (FMemoryStream) gelsen werden
    procedure BeforeSaving; virtual;
    // AfterSaving wird aufgerufen, nachdem die binre Daten aus dem Puffer (FMemoryStream) gelsen wurden
    procedure AfterSaving; virtual;
  public
    constructor Create; override;
    destructor Destroy; override;

    procedure Assign(Source: TRakCollectionItem); override;

    // GetBinarySize gibt an, wieviele Bytes die binren Daten umfassen
    // GetBinarySize muss von abgeleiteten Klassen berschrieben werden,
    // falls diese ihre binren Daten nicht in FMemoryStream halten.
    function GetBinarySize: Longword; virtual;
    // Mit GetAsBinary kann man die binren Daten auslesen
    function GetAsBinary(Puffer: Pointer; Count: integer): integer; virtual;
    // Mit SetAsBinary kann man neue binren Daten schreiben
    procedure SetAsBinary(Puffer: Pointer; Count: integer); virtual;

    property AByte[Idx: Longword]: Byte read GetAByte write SetAByte;
    property AsBase64: WideString read GetAsBase64 write SetAsBase64;
  end;

  {****************************************************************************
  <class name="TRakBinaryPicture" superclass="TRakBinaryBlob" public="true" defaultproperty="">
    <description>
      TRakBinaryPicture ist von dem Basisdatentyp TRakBinaryBlob abgeleitet
      und einen Datentyp dar, welcher alle von der Delphiklasse TGraphic
      abgeleiteten Klassen speicherbar macht.
      TRakBinaryPicture lst die Klasse TRakBinaryGraphic und davon abgeleitete Klassen ab.
    </description>
  </class>
  ****************************************************************************}
  TRakBinaryPicture = class(TRakBinaryBlob)
  private
    FPicture: TPicture;

    procedure SetGraphic(const Value: TGraphic);
    function GetGraphic: TGraphic;
  protected
    FIsSaving: boolean;

    procedure AfterLoading; override;
    procedure BeforeSaving; override;
    procedure AfterSaving; override;
  public
    constructor Create; override;
    destructor Destroy; override;

    class procedure AddToTag(Tag: TRakBinaryTag; Name: ShortString; Value: TGraphic);

    procedure Assign(Source: TRakCollectionItem); override;

    function GetBinarySize: Longword; override;

    class procedure RegisterGraphicClass(AClass: TGraphicClass);

    property Graphic: TGraphic read GetGraphic write SetGraphic;
  end;

  {****************************************************************************
  <class name="TRakBinaryTShiftState" superclass="TRakBinaryBlob" public="true" defaultproperty="">
    <description>
      TRakBinaryTShiftState speichert den Delphityp TShiftState
    </description>
  </class>
  ****************************************************************************}
  TRakBinaryTShiftState = class(TRakBinaryBlob)
  private
    function GetValue: TShiftState;
    procedure SetValue(const Value: TShiftState);
  public
    constructor Create; override;

    class procedure AddToTag(Tag: TRakBinaryTag; Name: ShortString; Value: TShiftState);
    class function ReadValue(Tag: TRakBinaryTag; Name: ShortString;
      DefaultValue: TShiftState): TShiftState;

    property Value: TShiftState read GetValue write SetValue;
  end;

  {****************************************************************************
  <class name="TRakBinaryGraphic" superclass="TRakBinaryBlob" public="true" defaultproperty="">
    <description>
      TRakBinaryGraphic ist von dem Basisdatentyp TRakBinaryBlob abgeleitet
      und einen Datentyp dar, welcher alle von der Delphiklasse TGraphic
      abgeleiteten Klassen speicherbar macht.
    </description>
  </class>
  ****************************************************************************}
  TRakBinaryGraphic = class(TRakBinaryBlob)
  private
    FGraphic: TGraphic;

    procedure SetGraphic(const Value: TGraphic);
  protected
    FIsSaving: boolean;

    procedure AfterLoading; override;
    procedure BeforeSaving; override;
    procedure AfterSaving; override;

    function CreateGraphic: TGraphic; virtual; abstract;
  public
    constructor Create; override;
    destructor Destroy; override;

    procedure Assign(Source: TRakCollectionItem); override;

    function GetBinarySize: Longword; override;

    property Graphic: TGraphic read FGraphic write SetGraphic;
  end;

  {****************************************************************************
  <class name="TRakBinaryBitmap" superclass="TRakBinaryGraphic" public="true" defaultproperty="">
    <description>
      TRakBinaryBitmap ist ein Datentyp speziell fr die Delphiklasse TBitmap
    </description>
  </class>
  ****************************************************************************}
  TRakBinaryBitmap = class(TRakBinaryGraphic)
  private
    procedure SetValue(const Value: TBitmap);
    function GetValue: TBitmap;
  protected
    function CreateGraphic: TGraphic; override;
  public
    constructor Create; override;
    //ACHTUNG: Wenn man auf Value lesend zugreift, dann mit folgendem Code: MeinBitmap.Assign(BinaryBitmap.Value); Folgender Code ist gefhrlich: MeinBitmap := BinaryBitmap.Value;
    property Value: TBitmap read GetValue write SetValue;
  end;

  {****************************************************************************
  <class name="TRakBinaryIcon" superclass="TRakBinaryGraphic" public="true" defaultproperty="">
    <description>
      TRakBinaryIcon ist ein Datentyp speziell fr die Delphiklasse TIcon
    </description>
  </class>
  ****************************************************************************}
  TRakBinaryIcon = class(TRakBinaryGraphic)
  private
    procedure SetValue(const Value: TIcon);
    function GetValue: TIcon;
  protected
    function CreateGraphic: TGraphic; override;
  public
    constructor Create; override;
    property Value: TIcon read GetValue write SetValue;
  end;

  {****************************************************************************
  <class name="TRakBinaryMetafile" superclass="TRakBinaryGraphic" public="true" defaultproperty="">
    <description>
      TRakBinaryMetafile ist ein Datentyp speziell fr die Delphiklasse TMetafile
    </description>
  </class>
  ****************************************************************************}
  TRakBinaryMetafile = class(TRakBinaryGraphic)
  private
    procedure SetValue(const Value: TMetafile);
    function GetValue: TMetafile;
  protected
    function CreateGraphic: TGraphic; override;
  public
    constructor Create; override;
    property Value: TMetafile read GetValue write SetValue;
  end;

  {****************************************************************************
  <class name="TRakBinaryJPEG" superclass="TRakBinaryGraphic" public="true" defaultproperty="">
    <description>
      TRakBinaryJPEG ist ein Datentyp speziell fr die Delphiklasse TJPEGImage
    </description>
  </class>
  ****************************************************************************}
  TRakBinaryJPEG = class(TRakBinaryGraphic)
  private
    procedure SetValue(const Value: TJPEGImage);
    function GetValue: TJPEGImage;
  protected
    function CreateGraphic: TGraphic; override;
  public
    constructor Create; override;
    property Value: TJPEGImage read GetValue write SetValue;
  end;

  {****************************************************************************
  <class name="TRakBinaryTagList" superclass="TInterfacedObject" public="true" defaultproperty="">
    <description>
      TRakBinaryTagList ist eine Implementation des Interfaces IRakBinaryTagList
    </description>
  </class>
  ****************************************************************************}
  TRakBinaryTagList = class(TInterfacedObject, IRakBinaryTagList)
  private
    FList: TList;
  public
    constructor Create;
    destructor Destroy; override;

    procedure Add(Tag: TRakBinaryTag);

    function Count: integer;
    function Items(Idx: integer): TRakBinaryTag;
  end;

  {****************************************************************************
  <class name="TRakBinaryReadFromPropertiesOptions" superclass="TInterfacedObject" public="true" defaultproperty="Items">
    <description>
      siehe Beschreigung zum Interface IRakBinaryReadFromPropertiesOptions
    </description>
  </class>
  ****************************************************************************}
  TRakBinaryReadFromPropertiesOptions = class(TInterfacedObject, IRakBinaryReadFromPropertiesOptions)
  private
  	FItems: TInterfaceList;

  	function GetCount: integer;
    function GetItems(Idx: integer): IRakBinaryReadFromPropertiesOptionsItem;
  public
  	constructor Create;
    destructor Destroy; override;

    function AddSimpleProperty: IRakBinaryReadFromSimpleProperty;
    function AddClassProperty: IRakBinaryReadFromClassProperty;

    function ItemByName(Name: ShortString): IRakBinaryReadFromPropertiesOptionsItem;

  	property Count: integer read GetCount;
    property Items[Idx: integer]: IRakBinaryReadFromPropertiesOptionsItem read GetItems;
  end;

  {****************************************************************************
  <class name="TRakBinaryWriteToPropertiesOptions" superclass="TInterfacedObject" public="true" defaultproperty="Items">
    <description>
      siehe Beschreigung zum Interface IRakBinaryWriteToPropertiesOptions
    </description>
  </class>
  ****************************************************************************}
  TRakBinaryWriteToPropertiesOptions = class(TInterfacedObject, IRakBinaryWriteToPropertiesOptions)
  private
  	FItems: TInterfaceList;

  	function GetCount: integer;
    function GetItems(Idx: integer): IRakBinaryWriteToPropertiesOptionsItem;
  public
  	constructor Create;
    destructor Destroy; override;

    function AddSimpleProperty: IRakBinaryWriteToSimpleProperty;
    function AddClassProperty: IRakBinaryWriteToClassProperty;

    function ItemByName(Name: ShortString): IRakBinaryWriteToPropertiesOptionsItem;

  	property Count: integer read GetCount;
    property Items[Idx: integer]: IRakBinaryWriteToPropertiesOptionsItem read GetItems;
  end;



implementation

uses Contnrs;

const
  DataTypeTable: array[1..20] of TRakBinaryDataType = (
    TRakBinaryBlob, TRakBinaryBoolean, TRakBinaryCurrency,
    TRakBinaryDateTime, TRakBinaryReal48, TRakBinarySingle,
    TRakBinaryDouble, TRakBinaryExtended, TRakBinaryComp,
    TRakBinaryByte, TRakBinaryShortint, TRakBinaryChar,
    TRakBinaryWord, TRakBinarySmallint, TRakBinaryLongword,
    TRakBinaryLongint, TRakBinaryInt64, TRakBinaryAnsiString,
    TRakBinaryWideString, TRakBinaryShortString);

  XX: WideString = '+-0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';

type
  TRakBinaryReadFromPropertiesOptionsItem = class(TInterfacedObject, IRakBinaryReadFromPropertiesOptionsItem)
  private
  	FPropertyName: string;
    FIgnoreMissing: boolean;

  	function GetPropertyName: string;
    function GetIgnoreMissing: boolean;
  	procedure SetPropertyName(Value: string);
    procedure SetIgnoreMissing(Value: boolean);
  end;

  TRakBinaryReadFromSimpleProperty = class(TRakBinaryReadFromPropertiesOptionsItem, IRakBinaryReadFromSimpleProperty)
  private
  	FAttributName: string;

    function GetAttributName: string;
    procedure SetAttributName(Value: string);
  end;

  TRakBinaryReadFromClassProperty = class(TRakBinaryReadFromPropertiesOptionsItem, IRakBinaryReadFromClassProperty)
  private
  	FTagName: string;
    FReadFromClassAttributs: boolean;
    FOptions: IRakBinaryReadFromPropertiesOptions;

  	function GetTagName: string;
    function GetReadFromClassAttributs: boolean;
    function GetOptions: IRakBinaryReadFromPropertiesOptions;

  	procedure SetTagName(Value: string);
    procedure SetReadFromClassAttributs(Value: boolean);
  public
  	constructor Create;
  end;

  TRakBinaryWriteToPropertiesOptionsItem = class(TInterfacedObject, IRakBinaryWriteToPropertiesOptionsItem)
  private
  	FPropertyName: string;
    FIgnoreMissing: boolean;

  	function GetPropertyName: string;
    function GetIgnoreMissing: boolean;
  	procedure SetPropertyName(Value: string);
    procedure SetIgnoreMissing(Value: boolean);
  end;

  TRakBinaryWriteToSimpleProperty = class(TRakBinaryWriteToPropertiesOptionsItem, IRakBinaryWriteToSimpleProperty)
  private
  	FAttributName: string;

    function GetAttributName: string;
    procedure SetAttributName(Value: string);
  end;

  TRakBinaryWriteToClassProperty = class(TRakBinaryWriteToPropertiesOptionsItem, IRakBinaryWriteToClassProperty)
  private
  	FTagName: string;
    FOptions: IRakBinaryWriteToPropertiesOptions;

  	function GetTagName: string;
    function GetOptions: IRakBinaryWriteToPropertiesOptions;

  	procedure SetTagName(Value: string);
  public
  	constructor Create;
  end;

var
  RegisteredDataTypes: TClassList;
  RegisteredGraphicClasses: TClassList;

function ReadAnsiString(Stream: TStream): AnsiString;
var
  Len: Longword;
begin
  Stream.Read(Len, SizeOf(Len));
  SetLength(Result, Len);
  Stream.Read(PChar(Result)^, Len);
end;

function ReadShortString(Stream: TStream): ShortString;
var
  Len: byte;
  str: AnsiString;
begin
  Stream.Read(Len, SizeOf(Len));
  SetLength(str, Len);
  Stream.Read(PChar(str)^, Len);
  Result := str;
end;

procedure WriteAnsiString(Stream: TStream;
  str: AnsiString);
var
  Len: Longword;
begin
  Len := length(str);
  Stream.Write(Len, SizeOf(Len));
  Stream.Write(PChar(str)^, Len);
end;

procedure WriteShortString(Stream: TStream;
  str: ShortString);
var
  s: AnsiString;
  Len: Byte;
begin
  Len := length(str);
  s := str;
  Stream.Write(Len, SizeOf(Len));
  Stream.Write(PChar(s)^, Len);
end;



{ TRakBinaryAttribute }


constructor TRakBinaryAttribute.Create;
begin
  inherited;
  FIsNull := true;
  FNamespaceMember := TRakNamespaceMemberImplementation.Create;
end;

class function TRakBinaryAttribute.DataTypeOf(BaseId: Byte): TRakBinaryDataType;
begin
  Result := nil;
  if IsBetween(BaseId, 1, length(DataTypeTable)) then
    Result := DataTypeTable[BaseId];
end;

destructor TRakBinaryAttribute.Destroy;
begin
  //Hier darf nicht mit FreeTObject gearbeitet werden
  FNamespaceMember.Free;
  FNamespaceMember := nil;

  inherited;
end;

function TRakBinaryAttribute.GetAsWideString: WideString;
begin
  if IsNull then
    Result := GetNullString
  else
    Result := GetValueAsWideString;
end;

function TRakBinaryAttribute.GetBaseTypeId: Byte;
begin
  Result := BaseIdOf(TRakBinaryDataType(self.ClassType));
end;

function TRakBinaryAttribute.GetDataType: TRakBinaryDataType;
begin
  Result := TRakBinaryDataType(ClassType);
end;

function TRakBinaryAttribute.GetDataTypeAsString: ShortString;
begin
  Result := ClassName;
end;

function TRakBinaryAttribute.GetItemName: ShortString;
begin
  Result := NamespaceMember.IDName;
end;

function TRakBinaryAttribute.GetNullString: string;
begin
  Result := '';
end;

class function TRakBinaryAttribute.BaseIdOf(DataType: TRakBinaryDataType): Byte;
var
  i: integer;
begin
  Result := 0;
  for i := 1 to length(DataTypeTable) do
  begin
    if DataType.InheritsFrom(DataTypeTable[i]) then
    begin
      Result := i;
      break;
    end;
  end;
end;

procedure TRakBinaryAttribute.LoadFromStream(Stream: TStream);
var
  bNull: boolean;
begin
  ItemName := ReadShortString(Stream);
  Stream.Read(bNull, SizeOf(bNull));
  IsNull := bNull;
  if not IsNull then
    LoadValue(Stream);
end;

procedure TRakBinaryAttribute.SaveToStream(Stream: TStream);
var
  bNull: boolean;
begin
  WriteShortString(Stream, ItemName);
  bNull := IsNull;
  Stream.Write(bNull, SizeOf(bNull));
  if not IsNull then
    SaveValue(Stream);
end;

{procedure TRakBinaryAttribute.SaveValue(Stream: TStream);
var
  Anz: longint;
  Puffer: PChar;
  Value: TValueType;
begin
  {case DataType of
     bdtInteger: Writer.WriteInteger(AsInteger);
    bdtFloat: Writer.WriteFloat(AsFloat);
    bdtString: Writer.WriteString(AsString);
    bdtBoolean: Writer.WriteBoolean(AsBoolean);
    bdtDate: Writer.WriteInteger(trunc(AsDate));
    bdtDateTime: Writer.WriteDate(AsDateTime);
    bdtBinary:
      begin
        Value := vaBinary;
        Writer.Write(Value, SizeOf(Value));

        Anz := GetBinarySize;
        Writer.write(Anz, SizeOf(Anz));
        Puffer := StrAlloc(Anz);
        try
          GetAsBinary(Puffer, Anz);
          Writer.Write(Puffer^, Anz);
        finally
          StrDispose(Puffer);
        end;
      end;
    {bdtInt64: ColItem := BasicAdd(TRakBinaryBlob);
    bdtCurrency: ColItem := BasicAdd(TRakBinaryBlob);
    bdtWideString: ColItem := BasicAdd(TRakBinaryBlob);
    bdtPicture: ColItem := BasicAdd(TRakBinaryBlob);
    bdtAudio: ColItem := BasicAdd(TRakBinaryBlob);
    bdtMovie: ColItem := BasicAdd(TRakBinaryBlob);
  end;
end;  }

procedure TRakBinaryAttribute.SetAsWideString(const Value: WideString);
begin
  if Value = GetNullString then
    IsNull := true
  else
    SetValueAsWideString(Value);
end;

procedure TRakBinaryAttribute.SetCollection(Value: TRakCollection);
begin
  inherited;
  if NameSpaceMember <> nil then
  begin
    if Value is TRakBinaryAttributes then
      NameSpaceMember.INamespace := TRakBinaryAttributes(Value)
    else
      NameSpaceMember.INamespace := nil;
  end;
end;

procedure TRakBinaryAttribute.SetIsNull(const Value: boolean);
begin
  FIsNull := Value;
end;

procedure TRakBinaryAttribute.SetItemName(const Value: ShortString);
begin
  NamespaceMember.IDName := Value;
end;

class function TRakBinaryAttribute.DataTypeOfClassName(
  Name: ShortString): TRakBinaryDataType;
var
  i: integer;
begin
  Result := nil;
  for i := 0 to RegisteredDataTypes.Count - 1 do
  begin
    if RegisteredDataTypes[i].ClassName = Name then
    begin
      Result := TRakBinaryDataType(RegistereddataTypes[i]);
      break;
    end;
  end;
end;

class procedure TRakBinaryAttribute.RegisterBinaryDataType(
  DataType: TRakBinaryDataType);
begin
  RegisteredDataTypes.Add(DataType);
end;



function TRakBinaryAttribute.GetParent: TRakBinaryAttributes;
begin
  Result := TRakBinaryAttributes(Collection);
end;

procedure TRakBinaryAttribute.Assign(Source: TRakCollectionItem);
var
  Src: TRakBinaryAttribute;
begin
  inherited;
  if Source is TRakBinaryAttribute then
  begin
    Src := TRakBinaryAttribute(Source);
    FIsNull := Src.FIsNull;
    ItemName := Src.ItemName;
  end;
end;

{ TRakBinaryByte }

class procedure TRakBinaryByte.AddToTag(Tag: TRakBinaryTag;
  Name: ShortString; Value: Byte);
var
  Attr: TRakBinaryByte;
begin
  Attr := TRakBinaryByte(Tag.AddAttribute(TRakBinaryByte, Name));
  Attr.Value := Value;
end;

procedure TRakBinaryByte.Assign(Source: TRakCollectionItem);
var
  Src: TRakBinaryByte;
begin
  inherited;
  if Source is TRakBinaryByte then
  begin
    Src := TRakBinaryByte(Source);
    FValue := Src.FValue;
  end;
end;

function TRakBinaryByte.GetValue: Byte;
begin
  Result := FValue;
end;

function TRakBinaryByte.GetValueAsWideString: WideString;
begin
  Result := IntToStr(FValue);
end;


procedure TRakBinaryByte.LoadValue(Stream: TStream);
var
  Wert: Byte;
begin
  Stream.Read(Wert, SizeOf(Wert));
  Value := Wert;
end;

procedure TRakBinaryByte.SaveValue(Stream: TStream);
var
  Wert: Byte;
begin
  Wert := Value;
  Stream.Write(Wert, SizeOf(Wert));
end;

procedure TRakBinaryByte.SetValue(const Value: Byte);
begin
  FValue := Value;
  IsNull := false;
end;

procedure TRakBinaryByte.SetValueAsWideString(const Value: WideString);
begin
  self.Value := StrToInt(Value);
end;

{ TRakBinaryTag }

function TRakBinaryTag.AddAttribute(DataType: TRakBinaryDataType;
  Name: ShortString; Value: AnsiString): TRakBinaryAttribute;
begin
  Result := Attributes.Add(DataType);
  Result.ItemName := Name;
  Result.AsWideString := Value;
end;

function TRakBinaryTag.AddAttribute(DataType: TRakBinaryDataType;
  Name: ShortString): TRakBinaryAttribute;
begin
  Result := Attributes.Add(DataType);
  Result.ItemName := Name;
end;

function TRakBinaryTag.AddTag: TRakBinaryTag;
begin
  Result := TRakBinaryTag(BasicAdd(TRakBinaryTag));
end;

function TRakBinaryTag.AddTag(TagName: ShortString): TRakBinaryTag;
begin
  Result := AddTag;
  Result.TagName := TagName;
end;

function TRakBinaryTag.AttributeAsBoolean(Name: ShortString;
  DefaultValue: boolean): boolean;
var
  Attr: TRakBinaryAttribute;
begin
  Result := DefaultValue;

  Attr := AttributeByName(Name);
  if (Attr is TRakBinaryBoolean) and not Attr.IsNull then
    Result := TRakBinaryBoolean(Attr).Value;
end;

function TRakBinaryTag.AttributeAsFloat(Name: ShortString;
  DefaultValue: extended): extended;
var
  Attr: TRakBinaryAttribute;
begin
  Result := DefaultValue;

  Attr := AttributeByName(Name);
  if (Attr <> nil) and not Attr.IsNull then
  begin
    if Attr is TRakBinaryExtended then
      Result := TRakBinaryExtended(Attr).Value
    else if Attr is TRakBinaryDouble then
      Result := TRakBinaryDouble(Attr).Value
    else if Attr is TRakBinarySingle then
      Result := TRakBinarySingle(Attr).Value
    else if Attr is TRakBinaryReal48 then
      Result := TRakBinaryReal48(Attr).Value;
  end;
end;

function TRakBinaryTag.AttributeAsInteger(Name: ShortString;
  DefaultValue: Int64): Int64;
var
  Attr: TRakBinaryAttribute;
begin
  Result := DefaultValue;

  Attr := AttributeByName(Name);
  if (Attr <> nil) and not Attr.IsNull then
  begin
    if Attr is TRakBinaryLongint then
      Result := TRakBinaryLongint(Attr).Value
    else if Attr is TRakBinarySmallint then
      Result := TRakBinarySmallint(Attr).Value
    else if Attr is TRakBinaryWord then
      Result := TRakBinaryWord(Attr).Value
    else if Attr is TRakBinaryByte then
      Result := TRakBinaryByte(Attr).Value
    else if Attr is TRakBinaryShortint then
      Result := TRakBinaryShortint(Attr).Value
    else if Attr is TRakBinaryInt64 then
      Result := TRakBinaryInt64(Attr).Value
    else if Attr is TRakBinaryLongword then
      Result := TRakBinaryLongword(Attr).Value
  end;
end;

function TRakBinaryTag.AttributeAsString(Name: ShortString;
  DefaultValue: AnsiString): AnsiString;
var
  Attr: TRakBinaryAttribute;
begin
  Result := DefaultValue;

  Attr := AttributeByName(Name);
  if (Attr <> nil) and not Attr.IsNull then
  begin
    if Attr is TRakBinaryAnsiString then
      Result := TRakBinaryAnsiString(Attr).Value
    else if Attr is TRakBinaryWideString then
      Result := TRakBinaryWideString(Attr).Value
    else if Attr is TRakBinaryShortString then
      Result := TRakBinaryShortString(Attr).Value;
  end;
end;

function TRakBinaryTag.AttributeByName(Name: ShortString): TRakBinaryAttribute;
begin
  Result := Attributes.AttributeByName(Name);
end;

procedure TRakBinaryTag.Clear;
begin
  FreeAll;
  Attributes.FreeAll;
end;

constructor TRakBinaryTag.Create;
begin
  inherited;
  FAttributes := TRakBinaryAttributes.Create;
end;

destructor TRakBinaryTag.Destroy;
begin
  FreeTObject(FAttributes);

  inherited;
end;

procedure TRakBinaryTag.DoAssign(Source: TRakCollection);
var
  Src: TRakBinaryTag;
begin
  inherited;
  if Source is TRakBinaryTag then
  begin
    Src := TRakBinaryTag(Source);
    FTagName := Src.FTagName;
    FAttributes.Assign(Src.FAttributes);
  end;
end;

procedure TRakBinaryTag.FillListByName(List: TRakBinaryTagList;
  Name: ShortString; WithSubTags: boolean);
var
  i: integer;
begin
  for i := 0 to Count - 1 do
  begin
    if Tags[i].TagName = Name then
      List.Add(Tags[i]);
    if WithSubTags then
      Tags[i].FillListByName(List, Name, true);
  end;
end;

function TRakBinaryTag.FindFirstTag(TagName: ShortString): TRakBinaryTag;
var
  i: integer;
begin
  Result := nil;
  for i := 0 to Count - 1 do
  begin
    if Tags[i].TagName = TagName then
    begin
      Result := Tags[i];
      break;
    end;
  end;
end;

function TRakBinaryTag.FindTagsByName(TagName: ShortString; WithSubTags: boolean): IRakBinaryTagList;
var
  List: TRakBinaryTagList;
begin
  List := TRakBinaryTagList.Create;
  Result := List;
  FillListByName(List, TagName, WithSubTags);
end;

function TRakBinaryTag.GetTags(Idx: integer): TRakBinaryTag;
begin
  Result := TRakBinaryTag(inherited Items[Idx]);
end;

procedure TRakBinaryTag.LoadFromStreamNow(Stream: TStream);
var
  Anz, i: Longword;
begin
  TagName := ReadShortString(Stream);
  Attributes.LoadFromStream(Stream);
  FreeAll;
  Stream.Read(Anz, SizeOf(Anz));
  for i := 1 to Anz do
    AddTag.LoadFromStreamNow(Stream);
end;

procedure TRakBinaryTag.ReadFromPublishedProperties(Obj: TObject;
  Options: IRakBinaryReadFromPropertiesOptions; ReadFromClassAttributs: boolean);
var
  i, iProps: Integer;
  Props: PPropList;
  TypeData: PTypeData;
  PropInfo: PPropInfo;
  TypeInfo: PTypeInfo;
  Item: IRakBinaryReadFromPropertiesOptionsItem;
  ItemName: ShortString;
  ClassItem: IRakBinaryReadFromClassProperty;
  SubOptions: IRakBinaryReadFromPropertiesOptions;
  rfca: boolean;
  Intf: IRakBinaryReadFrom;
  ClassRec: TRakBinaryReadFromClassPropertyDescription;
  SimpleRec: TRakBinaryReadFromSimplePropertyDescription;
  DoIt: boolean;
begin
	if (Obj <> nil) and (Obj.ClassInfo <> nil) then
  begin //if the object was compiled with {$M+}
    Obj.GetInterface(IRakBinaryReadFrom, Intf);
    DoIt := true;
    if Intf <> nil then
    begin
    	if Options = nil then
      	Options := TRakBinaryReadFromPropertiesOptions.Create;

    	Intf.BeforeReadFromProperties(self, Options, DoIt);
    end;

    if DoIt then
    begin
    	try
        TypeData := GetTypeData(Obj.ClassInfo);
        if (TypeData <> nil) and (TypeData^.PropCount > 0) then
        begin
          GetMem(Props, TypeData^.PropCount * sizeof(Pointer));
          try
            GetPropInfos(Obj.ClassInfo, Props);
            // First we are looking for missing properties
            if Options <> nil then
            begin
              for iProps := 0 to Options.Count - 1 do
              begin
                Item := Options[iProps];
                if not Item.IgnoreMissing then
                begin
                  if GetPropInfo(Obj, Item.PropertyName) = nil then
                    raise Exception.Create('Property "' + Item.PropertyName + '" mising in class "'+ Obj.ClassName + '"');
                end;
              end;
            end;

            // All required properties where found
            // Creating a attribute or subtag for every property
            for i := 0 to TypeData^.PropCount - 1 do
            begin
              PropInfo := Props^[i];
              TypeInfo := PropInfo^.PropType^;
              ItemName := PropInfo^.Name;
              case TypeInfo^.Kind of
                tkInteger, tkChar, tkEnumeration, tkSet, tkWChar,
                tkFloat, tkString, tkInt64, tkLString, tkWString:
                  begin // Simple-Type
                    if Options <> nil then
                    begin // if some properties need special handling
                      Item := Options.ItemByName(PropInfo^.Name);
                      if Item <> nil then
                      begin // if this property needs special handling
                        if not IsIntf(Item, IRakBinaryReadFromSimpleProperty) then
                          raise Exception.Create('Property "' + PropInfo^.Name + '" is a simple property');
                        ItemName := (Item as IRakBinaryReadFromSimpleProperty).AttributName;
                      end;
                    end;

                  	if Intf <> nil then
                    begin
                    	SimpleRec.AttributName := ItemName;
                      SimpleRec.PropertyName := PropInfo^.Name;
                      SimpleRec.Tag := self;
                      Intf.BeforeReadFromSimpleProperty(SimpleRec);
                      ItemName := SimpleRec.AttributName;
                    end;
                    if ItemName <> '' then
                    begin // This property has to be handled
                    	try
                        case TypeInfo^.Kind of
                          tkInteger: TRakBinaryLongint.AddToTag(self, ItemName, GetOrdProp(Obj, PropInfo^.Name));
                          tkChar: TRakBinaryChar.AddToTag(self, ItemName, char(GetOrdProp(Obj, PropInfo^.Name)));
                          tkEnumeration: TRakBinaryAnsiString.AddToTag(self, ItemName, GetEnumProp(Obj, PropInfo^.Name));
                          tkSet: TRakBinaryAnsiString.AddToTag(self, ItemName, GetSetProp(Obj, PropInfo^.Name));
                          tkWChar: TRakBinarySmallint.AddToTag(self, ItemName, GetOrdProp(Obj, PropInfo^.Name));
                          tkFloat: TRakBinaryExtended.AddToTag(self, ItemName, GetFloatProp(Obj, PropInfo^.Name));
                          tkString: TRakBinaryShortString.AddToTag(self, ItemName, GetStrProp(Obj, PropInfo^.Name));
                          tkLString: TRakBinaryAnsiString.AddToTag(self, ItemName, GetStrProp(Obj, PropInfo^.Name));
                          tkWString: TRakBinaryAnsiString.AddToTag(self, ItemName, GetStrProp(Obj, PropInfo^.Name));
                          tkInt64: TRakBinaryInt64.AddToTag(self, ItemName, GetInt64Prop(Obj, PropInfo^.Name));
                        end;
                      finally
                        if Intf <> nil then
                          Intf.AfterReadFromSimpleProperty(SimpleRec);
                      end;
                    end;
                  end;
                tkClass:
                  begin // Class-Type
                    rfca := ReadFromClassAttributs;
                    SubOptions := nil;
                    if not ReadFromClassAttributs then
                      ItemName := '';

                    if Options <> nil then
                    begin // if some properties need special handling
                      Item := Options.ItemByName(PropInfo^.Name);
                      if Item <> nil then
                      begin // if this property needs special handling
                        if not IsIntf(Item, IRakBinaryReadFromClassProperty) then
                          raise Exception.Create('Property "' + PropInfo^.Name + '" is a class property');
                        ItemName := (Item as IRakBinaryReadFromClassProperty).TagName;
                        ClassItem := Item as IRakBinaryReadFromClassProperty;
                        SubOptions := ClassItem.Options;
                        rfca := ClassItem.ReadFromClassAttributs;
                      end;
                    end;

	                 	if Intf <> nil then
                    begin
                    	ClassRec.TagName := ItemName;
                      ClassRec.PropertyName := PropInfo^.Name;
                      ClassRec.Tag := self;
                      Intf.BeforeReadFromClassProperty(ClassRec);
                      ItemName := ClassRec.TagName;
                    end;

                    if ItemName <> '' then
                    begin // This property has to be handled
                    	try
		                    ItemName := UpperCase(ItemName);
                        if FindFirstTag(ItemName) <> nil then
                          raise Exception.Create('Tagname for property "' + PropInfo^.Name + '" has to be unique');

                        AddTag(ItemName).ReadFromPublishedProperties(GetObjectProp(Obj, PropInfo^.Name), SubOptions, rfca)
                      finally
			                 	if Intf <> nil then
    		                  Intf.AfterReadFromClassProperty(ClassRec);
                      end;
                    end;
                  end;
                else  // Not supported
                	begin
	                 	if Intf <> nil then
                    	Intf.HandleNotSupportedReadProperties(PropInfo^.Name);
                  end;
              end;
            end;
          finally
            Freemem(Props);
          end;
        end;
      finally
        if Intf <> nil then
        begin
          if Options = nil then
            Options := TRakBinaryReadFromPropertiesOptions.Create;

          Intf.AfterReadFromProperties(self);
        end;
      end;
    end;
  end;
end;

procedure TRakBinaryTag.SaveToStreamNow(Stream: TStream);
var
  Anz, i: Longword;
begin
  WriteShortString(Stream, TagName);
  Attributes.SaveToStream(Stream);
  Anz := Count;
  Stream.Write(Anz, SizeOf(Anz));
  if Anz > 0 then
  begin
    for i := 0 to Anz -1 do
      Tags[i].SaveToStreamNow(Stream);
  end;
end;

procedure TRakBinaryTag.SetTagName(const Value: ShortString);
begin
  FTagName := Value;
end;

procedure TRakBinaryTag.WriteToPublishedProperties(Obj: TObject;
  Options: IRakBinaryWriteToPropertiesOptions);
var
  i, iProps: Integer;
  Props: PPropList;
  TypeData: PTypeData;
  PropInfo: PPropInfo;
  TypeInfo: PTypeInfo;
  Item: IRakBinaryWriteToPropertiesOptionsItem;
  ItemName: ShortString;
  Attr: TRakBinaryAttribute;
  SubTag: TRakBinaryTag;
  SubOptions: IRakBinaryWriteToPropertiesOptions;
  Intf: IRakBinaryWriteTo;
  ClassRec: TRakBinaryWriteToClassPropertyDescription;
  SimpleRec: TRakBinaryWriteToSimplePropertyDescription;
  DoIt: boolean;
begin
	if (Obj <> nil) and (Obj.ClassInfo <> nil) then
  begin //if the object was compiled with {$M+}
    Obj.GetInterface(IRakBinaryWriteTo, Intf);
    DoIt := true;
    if Intf <> nil then
    begin
      if Options = nil then
        Options := TRakBinaryWriteToPropertiesOptions.Create;

      Intf.BeforeWriteToProperties(self, Options, DoIt);
    end;

    if DoIt then
    begin
      try
        TypeData := GetTypeData(Obj.ClassInfo);
        if (TypeData <> nil) and (TypeData^.PropCount > 0) then
        begin
          GetMem(Props, TypeData^.PropCount * sizeof(Pointer));
          try
            GetPropInfos(Obj.ClassInfo, Props);
            // First we are looking for missing properties
            if Options <> nil then
            begin
              for iProps := 0 to Options.Count - 1 do
              begin
                Item := Options[iProps];
                if not Item.IgnoreMissing then
                begin
                  if GetPropInfo(Obj, Item.PropertyName) = nil then
                    raise Exception.Create('Property "' + Item.PropertyName + '" mising in class "'+ Obj.ClassName + '"');
                end;
              end;
            end;

            // All required properties where found
            // Searching the attribute or subtag for every property
            for i := 0 to TypeData^.PropCount - 1 do
            begin
              PropInfo := Props^[i];
              TypeInfo := PropInfo^.PropType^;
              ItemName := PropInfo^.Name;
              case TypeInfo^.Kind of
                tkInteger, tkChar, tkEnumeration, tkSet, tkWChar,
                tkFloat, tkString, tkInt64, tkLString, tkWString:
                  begin // Simple-Type
                    if Options <> nil then
                    begin // if some properties need special handling
                      Item := Options.ItemByName(PropInfo^.Name);
                      if Item <> nil then
                      begin // if this property needs special handling
                        if not IsIntf(Item, IRakBinaryWriteToSimpleProperty) then
                          raise Exception.Create('Property "' + PropInfo^.Name + '" is a simple property');
                        ItemName := (Item as IRakBinaryWriteToSimpleProperty).AttributName;
                      end;
                    end;

                    if Intf <> nil then
                    begin
                      SimpleRec.AttributName := ItemName;
                      SimpleRec.PropertyName := PropInfo^.Name;
                      SimpleRec.Tag := self;
                      Intf.BeforeWriteToSimpleProperty(SimpleRec);
                      ItemName := SimpleRec.AttributName;
                    end;

                    if ItemName <> '' then
                    begin // This property has to be handled
                      try
                        Attr := AttributeByName(ItemName);
                        if (Attr <> nil) and not Attr.IsNull then
                        begin // We found a value for the property in this tag
                          case TypeInfo^.Kind of
                            tkInteger:
                              begin
                                if Attr is TRakBinaryLongint then
                                  SetOrdProp(Obj, PropInfo^.Name, TRakBinaryLongint(Attr).Value);
                              end;
                            tkChar:
                              begin
                                if Attr is TRakBinaryChar then
                                  SetOrdProp(Obj, PropInfo^.Name, ord(TRakBinaryChar(Attr).Value));
                              end;
                            tkEnumeration:
                              begin
                                if Attr is TRakBinaryAnsiString then
                                  SetEnumProp(Obj, PropInfo^.Name, TRakBinaryAnsiString(Attr).Value);
                              end;
                            tkSet:
                              begin
                                if Attr is TRakBinaryAnsiString then
                                  SetSetProp(Obj, PropInfo^.Name, TRakBinaryAnsiString(Attr).Value);
                              end;
                            tkWChar:
                              begin
                                if Attr is TRakBinarySmallint then
                                  SetOrdProp(Obj, PropInfo^.Name, TRakBinarySmallint(Attr).Value);
                              end;
                            tkFloat:
                              begin
                                if Attr is TRakBinaryExtended then
                                  SetFloatProp(Obj, PropInfo^.Name, TRakBinaryExtended(Attr).Value);
                              end;
                            tkString:
                              begin
                                if Attr is TRakBinaryShortString then
                                  SetStrProp(Obj, PropInfo^.Name, TRakBinaryShortString(Attr).Value);
                              end;
                            tkLString:
                              begin
                                if Attr is TRakBinaryAnsiString then
                                  SetStrProp(Obj, PropInfo^.Name, TRakBinaryAnsiString(Attr).Value);
                              end;
                            tkWString:
                              begin
                                if Attr is TRakBinaryAnsiString then
                                  SetStrProp(Obj, PropInfo^.Name, TRakBinaryAnsiString(Attr).Value);
                              end;
                            tkInt64:
                              begin
                                if Attr is TRakBinaryLongint then
                                  SetInt64Prop(Obj, PropInfo^.Name, TRakBinaryInt64(Attr).Value);
                              end;
                          end;
                        end;
                      finally
                        if Intf <> nil then
                          Intf.AfterWriteToSimpleProperty(SimpleRec);
                      end;
                    end;
                  end;
                tkClass:
                  begin // Class-Type
                    SubOptions := nil;

                    if Options <> nil then
                    begin // if some properties need special handling
                      Item := Options.ItemByName(PropInfo^.Name);
                      if Item <> nil then
                      begin // if this property needs special handling
                        if not IsIntf(Item, IRakBinaryWriteToClassProperty) then
                          raise Exception.Create('Property "' + PropInfo^.Name + '" is a class property');
                        ItemName := (Item as IRakBinaryWriteToClassProperty).TagName;
                        SubOptions := (Item as IRakBinaryWriteToClassProperty).Options;
                      end;
                    end;

                    if Intf <> nil then
                    begin
                      ClassRec.TagName := ItemName;
                      ClassRec.PropertyName := PropInfo^.Name;
                      ClassRec.Tag := self;
                      Intf.BeforeWriteToClassProperty(ClassRec);
                      ItemName := ClassRec.TagName;
                    end;

                    if ItemName <> '' then
                    begin // This property has to be handled
                      try
                        ItemName := UpperCase(ItemName);
                        SubTag := FindFirstTag(ItemName);
                        if SubTag <> nil then
                          SubTag.WriteToPublishedProperties(GetObjectProp(Obj, PropInfo^.Name), SubOptions)
                      finally
                        if Intf <> nil then
                          Intf.AfterWriteToClassProperty(ClassRec);
                      end;
                    end;
                  end;
                else  // Not supported
                  begin
                    if Intf <> nil then
                      Intf.HandleNotSupportedWriteProperties(PropInfo^.Name);
                  end;
              end;
            end;
          finally
            Freemem(Props);
          end;
        end;
      finally
        if Intf <> nil then
        begin
          if Options = nil then
            Options := TRakBinaryWriteToPropertiesOptions.Create;

          Intf.AfterWriteToProperties(self);
        end;
      end;
    end;
  end;
end;

{ TRakBinaryAttributes }

function TRakBinaryAttributes.Add(DataType: TRakBinaryDataType): TRakBinaryAttribute;
begin
  Result := TRakBinaryAttribute(BasicAdd(DataType));
end;

function TRakBinaryAttributes.AttributeByName(
  Name: string): TRakBinaryAttribute;
var
  i: integer;
begin
  Result := nil;
  for i := 0 to Count - 1 do
  begin
    if Items[i].ItemName = Name then
    begin
      Result := Items[i];
      break;
    end;
  end;
end;

constructor TRakBinaryAttributes.Create;
begin
  inherited;
  FNamespace := TRakNamespaceImplementation.Create;
end;

destructor TRakBinaryAttributes.Destroy;
begin
  // Hier darf nicht mit FreeTObject gearbeitet werden
  FNamespace.Free;
  FNamespace := nil;

  inherited;
end;

function TRakBinaryAttributes.GetItems(Idx: integer): TRakBinaryAttribute;
begin
  Result := TRakBinaryAttribute(inherited Items[Idx]);
end;

procedure TRakBinaryAttributes.LoadFromStream(Stream: TStream);
var
  Anz, i: LongWord;
  BaseTypeId: Byte;
  Item: TRakBinaryAttribute;
  DataType: TRakBinaryDataType;
begin
  FreeAll;
  Stream.Read(Anz, SizeOf(Anz));
  if Anz > 0 then
  begin
    for i := 0 to Anz - 1 do
    begin
      Stream.Read(BaseTypeId, SizeOf(BaseTypeId));
      if BaseTypeId = 0 then
      begin
        Stream.Read(BaseTypeId, SizeOf(BaseTypeId));
        DataType := TRakBinaryAttribute.DataTypeOfClassName(ReadShortString(Stream));
        if DataType = nil then
          Item := Add(TRakBinaryAttribute.DataTypeOf(BaseTypeId))
        else
          Item := Add(DataType)
      end
      else
        Item := Add(TRakBinaryAttribute.DataTypeOf(BaseTypeId));

      Item.LoadFromStream(Stream);
    end;
  end;
end;

procedure TRakBinaryAttributes.SaveToStream(Stream: TStream);
var
  Anz, i: LongWord;
  BaseTypeId: Byte;
  Item: TRakBinaryAttribute;
begin
  Anz := Count;
  Stream.Write(Anz, SizeOf(Anz));
  if Anz > 0 then
  begin
    for i := 0 to Anz - 1 do
    begin
      Item := Items[i];
      if Item.ClassParent = TRakBinaryAttribute then
      begin
        BaseTypeId := Item.BaseTypeId;
        Stream.Write(BaseTypeId, SizeOf(BaseTypeId));
      end
      else
      begin
        if TRakBinaryAttribute.DataTypeOfClassName(Item.DataTypeAsString) = nil then
          raise Exception.Create('Die Klasse "' + Item.DataTypeAsString + '" muss mit TRakBinaryAttribute.RegisterBinaryDataType registriert werden.');

        BaseTypeId := 0;
        Stream.Write(BaseTypeId, SizeOf(BaseTypeId));

        BaseTypeId := Item.BaseTypeId;
        Stream.Write(BaseTypeId, SizeOf(BaseTypeId));

        WriteShortString(Stream, Item.DataTypeAsString);
      end;

      Item.SaveToStream(Stream);
    end;
  end;
end;

{ TRakBinaryExtended }

class procedure TRakBinaryExtended.AddToTag(Tag: TRakBinaryTag;
  Name: ShortString; Value: extended);
var
  Attr: TRakBinaryExtended;
begin
  Attr := TRakBinaryExtended(Tag.AddAttribute(TRakBinaryExtended, Name));
  Attr.Value := Value;
end;

procedure TRakBinaryExtended.Assign(Source: TRakCollectionItem);
var
  Src: TRakBinaryExtended;
begin
  inherited;
  if Source is TRakBinaryExtended then
  begin
    Src := TRakBinaryExtended(Source);
    FValue := Src.FValue;
  end;
end;

function TRakBinaryExtended.GetValue: extended;
begin
  Result := FValue;
end;

function TRakBinaryExtended.GetValueAsWideString: WideString;
begin
  Result := FloatToStr(Value);
end;

procedure TRakBinaryExtended.LoadValue(Stream: TStream);
var
  Wert: extended;
begin
  Stream.Read(Wert, SizeOf(Wert));
  Value := Wert;
end;

procedure TRakBinaryExtended.SaveValue(Stream: TStream);
var
  Wert: extended;
begin
  Wert := Value;
  Stream.Write(Wert, SizeOf(Wert));
end;

procedure TRakBinaryExtended.SetValue(const Value: extended);
begin
  FValue := Value;
  IsNull := false;
end;

procedure TRakBinaryExtended.SetValueAsWideString(const Value: WideString);
begin
  self.Value := InputToFloat(Value, FValue);
end;

{ TRakBinaryBoolean }

class procedure TRakBinaryBoolean.AddToTag(Tag: TRakBinaryTag;
  Name: ShortString; Value: boolean);
var
  Attr: TRakBinaryBoolean;
begin
  Attr := TRakBinaryBoolean(Tag.AddAttribute(TRakBinaryBoolean, Name));
  Attr.Value := Value;
end;

procedure TRakBinaryBoolean.Assign(Source: TRakCollectionItem);
var
  Src: TRakBinaryBoolean;
begin
  inherited;
  if Source is TRakBinaryBoolean then
  begin
    Src := TRakBinaryBoolean(Source);
    FValue := Src.FValue;
  end;
end;

function TRakBinaryBoolean.GetValue: boolean;
begin
  Result := FValue;
end;

function TRakBinaryBoolean.GetValueAsWideString: WideString;
begin
  Result := Decide(Value, 'Ja', 'Nein');
end;

procedure TRakBinaryBoolean.LoadValue(Stream: TStream);
var
  Wert: boolean;
begin
  Stream.Read(Wert, SizeOf(Wert));
  Value := Wert;
end;

procedure TRakBinaryBoolean.SaveValue(Stream: TStream);
var
  Wert: boolean;
begin
  Wert := Value;
  Stream.Write(Wert, SizeOf(Wert));
end;

procedure TRakBinaryBoolean.SetValue(const Value: boolean);
begin
  FValue := Value;
  IsNull := false;
end;

procedure TRakBinaryBoolean.SetValueAsWideString(const Value: WideString);
begin
   self.Value := StrToBool(Value);
end;

{ TRakBinaryDateTime }

class procedure TRakBinaryDateTime.AddToTag(Tag: TRakBinaryTag;
  Name: ShortString; Value: TDateTime);
var
  Attr: TRakBinaryDateTime;
begin
  Attr := TRakBinaryDateTime(Tag.AddAttribute(TRakBinaryDateTime, Name));
  Attr.Value := Value;
end;

procedure TRakBinaryDateTime.Assign(Source: TRakCollectionItem);
var
  Src: TRakBinaryDateTime;
begin
  inherited;
  if Source is TRakBinaryDateTime then
  begin
    Src := TRakBinaryDateTime(Source);
    FValue := Src.FValue;
  end;
end;

function TRakBinaryDateTime.GetValue: TDateTime;
begin
  Result := FValue;
end;

function TRakBinaryDateTime.GetValueAsWideString: WideString;
begin
  Result := DateTimeToStr(Value);
end;

procedure TRakBinaryDateTime.LoadValue(Stream: TStream);
var
  Wert: Double;
begin
  Stream.Read(Wert, SizeOf(Wert));
  Value := Wert;
end;

procedure TRakBinaryDateTime.SaveValue(Stream: TStream);
var
  Wert: double;
begin
  Wert := Value;
  Stream.Write(Wert, SizeOf(Wert));
end;

procedure TRakBinaryDateTime.SetValue(const Value: TDateTime);
begin
  FValue := Value;
  IsNull := false;
end;

procedure TRakBinaryDateTime.SetValueAsWideString(const Value: WideString);
begin
  self.Value := StrToDateTime(Value);
end;

{ TRakBinaryAnsiString }

class procedure TRakBinaryAnsiString.AddToTag(Tag: TRakBinaryTag;
  Name: ShortString; Value: AnsiString);
var
  Attr: TRakBinaryAnsiString;
begin
  Attr := TRakBinaryAnsiString(Tag.AddAttribute(TRakBinaryAnsiString, Name));
  Attr.Value := Value;
end;

procedure TRakBinaryAnsiString.Assign(Source: TRakCollectionItem);
var
  Src: TRakBinaryAnsiString;
begin
  inherited;
  if Source is TRakBinaryAnsiString then
  begin
    Src := TRakBinaryAnsiString(Source);
    FValue := Src.FValue;
  end;
end;

function TRakBinaryAnsiString.GetValue: AnsiString;
begin
  Result := FValue;
end;

function TRakBinaryAnsiString.GetValueAsWideString: WideString;
begin
  Result := Value;
end;

procedure TRakBinaryAnsiString.LoadValue(Stream: TStream);
begin
  Value := ReadAnsiString(Stream);
end;

procedure TRakBinaryAnsiString.SaveValue(Stream: TStream);
begin
  WriteAnsiString(Stream, Value);
end;

procedure TRakBinaryAnsiString.SetValue(const Value: AnsiString);
begin
  FValue := Value;
  IsNull := false;
end;

procedure TRakBinaryAnsiString.SetValueAsWideString(const Value: WideString);
begin
  self.Value := Value;
end;

{ TRakBinaryShortString }

class procedure TRakBinaryShortString.AddToTag(Tag: TRakBinaryTag; Name,
  Value: ShortString);
var
  Attr: TRakBinaryShortString;
begin
  Attr := TRakBinaryShortString(Tag.AddAttribute(TRakBinaryShortString, Name));
  Attr.Value := Value;
end;

procedure TRakBinaryShortString.Assign(Source: TRakCollectionItem);
var
  Src: TRakBinaryShortString;
begin
  inherited;
  if Source is TRakBinaryShortString then
  begin
    Src := TRakBinaryShortString(Source);
    FValue := Src.FValue;
  end;
end;

function TRakBinaryShortString.GetValue: ShortString;
begin
  Result := FValue;
end;

function TRakBinaryShortString.GetValueAsWideString: WideString;
begin
  Result := Value;
end;

procedure TRakBinaryShortString.LoadValue(Stream: TStream);
begin
  Value := ReadShortString(Stream);
end;

procedure TRakBinaryShortString.SaveValue(Stream: TStream);
begin
  WriteShortString(Stream, Value);
end;

procedure TRakBinaryShortString.SetValue(const Value: ShortString);
begin
  FValue := Value;
  IsNull := false;
end;

procedure TRakBinaryShortString.SetValueAsWideString(
  const Value: WideString);
begin
  self.Value := Value;
end;


{ TRakBinaryBlob }

procedure TRakBinaryBlob.AfterLoading;
begin
end;

procedure TRakBinaryBlob.AfterSaving;
begin
end;

procedure TRakBinaryBlob.Assign(Source: TRakCollectionItem);
var
  Src: TRakBinaryBlob;
begin
  inherited;
  if Source is TRakBinaryBlob then
  begin
    Src := TRakBinaryBlob(Source);
    FMemoryStream.Size := Src.FMemoryStream.Size;
    MoveMemory(FMemoryStream.Memory, Src.FMemoryStream.Memory, FMemoryStream.Size);
  end;
end;

procedure TRakBinaryBlob.BeforeLoading;
begin
end;

procedure TRakBinaryBlob.BeforeSaving;
begin
end;

constructor TRakBinaryBlob.Create;
begin
  inherited;

  FMemoryStream := TMemoryStream.Create;
end;

destructor TRakBinaryBlob.Destroy;
begin
  FreeTObject(FMemoryStream);

  inherited;
end;

function TRakBinaryBlob.GetAsBinary(Puffer: Pointer;
  Count: integer): integer;
begin
  BeforeSaving;

  if Longword(Count) <= GetBinarySize then
  begin
    Result := GetBinarySize;
    MoveMemory(Puffer, GetPointerForSaving, GetBinarySize);
  end
  else
    Result := -GetBinarySize;

  AfterSaving;
end;


function TRakBinaryBlob.GetBinarySize: Longword;
begin
  Result := FMemoryStream.Size;
end;

function TRakBinaryBlob.GetAByte(Idx: Longword): Byte;
var
  Stream: TMemoryStream;
  Len: Longword;
begin
  Len := GetBinarySize;
  if Idx >= Len then
    raise Exception.Create('Index out of range');

  Stream := TMemoryStream.Create;
  try
    Stream.Size := Len;
    GetAsBinary(Stream.Memory, Len);
    Result := ord((PChar(Stream.Memory) + Idx)^);
  finally
    Stream.Free;
  end;
end;

function TRakBinaryBlob.GetPointerForLoading(Size: Int64): Pointer;
begin
  FMemoryStream.Size := Size;
  Result := FMemoryStream.Memory;
end;

function TRakBinaryBlob.GetPointerForSaving: Pointer;
begin
  Result := FMemoryStream.Memory;
end;

function TRakBinaryBlob.GetValueAsWideString: WideString;
var
  i, t: integer;
  Null: Byte;
  iByte: Byte;
  P: PChar;
begin
  BeforeSaving;

  SetLength(Result, GetBinarySize * 2);
  P := GetPointerForSaving;
  Null := ord('A');
  t := 1;
  for i := 1 to GetBinarySize do
  begin
    iByte := Byte(P^);
    Result[t] := WideChar(iByte div 16 + Null);
    Result[t+1] := WideChar(iByte mod 16 + Null);
    inc(t, 2);
    inc(P);
  end;

  AfterSaving;
end;

procedure TRakBinaryBlob.LoadValue(Stream: TStream);
var
  Anz: Int64;
begin
  BeforeLoading;
  Stream.Read(Anz, SizeOf(Anz));
  Stream.Read(GetPointerForLoading(Anz)^, Anz);
  AfterLoading;
end;

procedure TRakBinaryBlob.SaveValue(Stream: TStream);
var
  Anz: Int64;
begin
  BeforeSaving;

  Anz := GetBinarySize;
  Stream.Write(Anz, SizeOf(Anz));
  Stream.Write(GetPointerForSaving^, Anz);

  AfterSaving;
end;

procedure TRakBinaryBlob.SetAsBinary(Puffer: Pointer;
  Count: integer);
begin
  BeforeLoading;
  MoveMemory(GetPointerForLoading(Count), Puffer, Count);
  AfterLoading;
end;

procedure TRakBinaryBlob.SetAByte(Idx: Longword; const Value: Byte);
var
  Stream: TMemoryStream;
  Len: Longword;
  AByte: PByte;
begin
  Len := GetBinarySize;
  if Idx >= Len then
    raise Exception.Create('Index out of range');

  Stream := TMemoryStream.Create;
  try
    Stream.Size := Len;
    GetAsBinary(Stream.Memory, Len);
    AByte := PByte(PChar(Stream.Memory) + Idx);
    AByte^ := Value;
    SetAsBinary(Stream.Memory, Len);
  finally
    Stream.Free;
  end;
end;

procedure TRakBinaryBlob.SetValueAsWideString(const Value: WideString);
var
  i: integer;
  Null: Byte;
  iByte: Byte;
  P: PChar;
begin
  BeforeLoading;

  P := GetPointerForLoading(length(Value) div 2);
  Null := ord('A');
  i := 1;
  while i < length(Value) do
  begin
    iByte := (Byte(Value[i]) - Null) * 16 + (Byte(Value[i+1]) - Null);
    Byte(P^) := iByte;
    inc(P);
    inc(i, 2);
  end;

  AfterLoading;
end;

procedure TRakBinaryBlob.SetAsBase64(const Value: WideString);
  procedure Degrease4To3(Src1, Src2, Src3, Src4: WideChar; Dst: PChar);
    function CharToByte(c: char): Byte;
    begin
      case c of
        'a'..'z': Result := Ord(c) - Ord('a') + 38;
        'A'..'Z': Result := Ord(c) - Ord('A') + 12;
        '0'..'9': Result := Ord(c) - Ord('0') + 2;
        '+': Result := 0;
        '-': Result := 1;
        else Result := 0;
      end;
    end;
  var
    Byte1, Byte2, Byte3, Byte4: Byte;
  begin
    Byte1 := CharToByte(char(Src1));
    Byte2 := CharToByte(char(Src2));
    Byte3 := CharToByte(char(Src3));
    Byte4 := CharToByte(char(Src4));
    Dst^ := char((Byte1 shl 2) + (Byte2 shr 4));
    (Dst + 1)^ := char((Byte2 shl 4) + (Byte3 shr 2));
    (Dst + 2)^ := char((Byte3 shl 6) + Byte4);
  end;
var
  i, t: integer;
  P: PChar;
  LenB, LenT: Longword;
  Rest: Byte;
  Puffer3: array[0..2] of char;
begin
  Rest := 0;
  LenT := 0;
  LenB := 0;
  if (Length(Value) - 1) mod 4 = 0 then
  begin
    Rest := ord(char(Value[1])) - ord('0');
    if Rest < 3 then
    begin
      LenT := (Length(Value) - 1) div 4;
      LenB := LenT * 3;
      if Rest > 0 then
      begin
        dec(LenT);
        LenB := LenB - 3 + Rest;
      end;
    end;
  end;

  BeforeLoading;

  P := GetPointerForLoading(LenB);
  t := 2;
  for i := 1 to LenT do
  begin
    Degrease4To3(Value[t], Value[t+1], Value[t+2], Value[t+3], P);
    inc(P, 3);
    inc(t, 4);
  end;
  if Rest > 0 then
  begin
    Degrease4To3(Value[t], Value[t+1], Value[t+2], Value[t+3], @Puffer3[0]);
    for I := 0 to Rest - 1 do
      P^ := Puffer3[I];
  end;

  AfterLoading;
end;

function TRakBinaryBlob.GetAsBase64: WideString;
  procedure Expand3To4(Src: PChar; var Dst1, Dst2, Dst3, Dst4: WideChar);
  var
    Byte1, Byte2, Byte3: Byte;
  begin
    Byte1 := Ord(Src^);
    Byte2 := Ord((Src + 1)^);
    Byte3 := Ord((Src + 2)^);
    Dst1 := XX[(Byte1 shr 2) + 1];
    Dst2 := XX[((Byte1 shl 4) and 63) + (Byte2 shr 4) + 1];
    Dst3 := XX[((Byte2 shl 2) and 63) + (Byte3 shr 6) + 1];
    Dst4 := XX[(Byte3 and 63) + 1];
  end;
var
  i, t: integer;
  P: PChar;
  Len, LenB, LenT: Longword;
  Rest: Byte;
  Puffer3: array[0..2] of char;
begin
  BeforeSaving;

  // Die Anzahl der Bytes ermitteln, die konvertiert werden mssen
  Len := GetBinarySize;
  // Es werden je drei Bytes zusammen konvertiert
  // Die Anzahl der vollstndigen 3ByteBlocks ermitteln
  LenT := Len div 3;
  // Die Gre des bentigten Speichers ermitteln
  // Jeder 3ByteBlock braucht nach dem Konvertieren vier Bytes Platz
  // Ein unvollstndiger 3ByteBlock bentigt auch vier Bytes
  Rest := Len mod 3;
  if Rest = 0 then
    LenB := LenT * 4
  else
    LenB := LenT * 4 + 4;

  // Den Speicher fr die Rckgabe anfordern
  SetLength(Result, LenB + 1);

  // Den Zeiger auf das erste zu konvertierende Byte besorgen
  P := GetPointerForSaving;
  // Zuerst wird gespeichert, wie gro der unvollstndige 3ByteBlock ist
  Result[1] := WideChar(ord('0') + Rest);
  // t legt fest, welches Zeichen als nchstes versorgt wird
  t := 2;
  // Alle vollstndigen 3ByteBlocks konvertieren
  for i := 1 to LenT do
  begin
    Expand3To4(P, Result[t], Result[t+1], Result[t+2], Result[t+3]);
    inc(t, 4);
    inc(P, 3);
  end;
  if Rest > 0 then
  begin // Wenn es einen unvollstndigen 3ByteBlock gibt
    // Die restlichen Bytes in einen drei Byte groen Puffer kopieren,
    // damit es beim Konvertieren nicht zu einer Zugriffsverletzung kommt
    for i := 0 to Rest - 1 do
      Puffer3[i] := (P+i)^;
    Expand3To4(@Puffer3[0], Result[t], Result[t+1], Result[t+2], Result[t+3]);
  end;

  AfterSaving;
end;

{ TRakBinaryInt64 }

class procedure TRakBinaryInt64.AddToTag(Tag: TRakBinaryTag;
  Name: ShortString; Value: Int64);
var
  Attr: TRakBinaryInt64;
begin
  Attr := TRakBinaryInt64(Tag.AddAttribute(TRakBinaryInt64, Name));
  Attr.Value := Value;
end;

procedure TRakBinaryInt64.Assign(Source: TRakCollectionItem);
var
  Src: TRakBinaryInt64;
begin
  inherited;
  if Source is TRakBinaryInt64 then
  begin
    Src := TRakBinaryInt64(Source);
    FValue := Src.FValue;
  end;
end;

function TRakBinaryInt64.GetValue: Int64;
begin
  Result := FValue;
end;

function TRakBinaryInt64.GetValueAsWideString: WideString;
begin
  Result := IntToStr(Value);
end;

procedure TRakBinaryInt64.LoadValue(Stream: TStream);
var
  Wert: Int64;
begin
  Stream.Read(Wert, SizeOf(Wert));
  Value := Wert;
end;

procedure TRakBinaryInt64.SaveValue(Stream: TStream);
var
  Wert: Int64;
begin
  Wert := Value;
  Stream.Write(Wert, SizeOf(Wert));
end;

procedure TRakBinaryInt64.SetValue(const Value: Int64);
begin
  FValue := Value;
  IsNull := false;
end;

procedure TRakBinaryInt64.SetValueAsWideString(const Value: WideString);
begin
  self.Value := StrToInt64(Value);
end;


{ TRakBinaryCurrency }

class procedure TRakBinaryCurrency.AddToTag(Tag: TRakBinaryTag;
  Name: ShortString; Value: Currency);
var
  Attr: TRakBinaryCurrency;
begin
  Attr := TRakBinaryCurrency(Tag.AddAttribute(TRakBinaryCurrency, Name));
  Attr.Value := Value;
end;

procedure TRakBinaryCurrency.Assign(Source: TRakCollectionItem);
var
  Src: TRakBinaryCurrency;
begin
  inherited;
  if Source is TRakBinaryCurrency then
  begin
    Src := TRakBinaryCurrency(Source);
    FValue := Src.FValue;
  end;
end;

function TRakBinaryCurrency.GetValue: Currency;
begin
  Result := FValue;
end;

function TRakBinaryCurrency.GetValueAsWideString: WideString;
begin
  Result := CurrToStr(Value);
end;

procedure TRakBinaryCurrency.LoadValue(Stream: TStream);
var
  Wert: Currency;
begin
  Stream.Read(Wert, SizeOf(Wert));
  Value := Wert;
end;

procedure TRakBinaryCurrency.SaveValue(Stream: TStream);
var
  Wert: Currency;
begin
  Wert := Value;
  Stream.Write(Wert, SizeOf(Wert));
end;

procedure TRakBinaryCurrency.SetValue(const Value: Currency);
begin
  FValue := Value;
  IsNull := false;
end;

procedure TRakBinaryCurrency.SetValueAsWideString(const Value: WideString);
begin
  self.Value := StrToCurr(Value);
end;

{ TRakBinaryWideString }

class procedure TRakBinaryWideString.AddToTag(Tag: TRakBinaryTag;
  Name: ShortString; Value: WideString);
var
  Attr: TRakBinaryWideString;
begin
  Attr := TRakBinaryWideString(Tag.AddAttribute(TRakBinaryWideString, Name));
  Attr.Value := Value;
end;

procedure TRakBinaryWideString.Assign(Source: TRakCollectionItem);
var
  Src: TRakBinaryWideString;
begin
  inherited;
  if Source is TRakBinaryWideString then
  begin
    Src := TRakBinaryWideString(Source);
    FValue := Src.FValue;
  end;
end;

function TRakBinaryWideString.GetValue: WideString;
begin
  Result := FValue;
end;

function TRakBinaryWideString.GetValueAsWideString: WideString;
begin
  Result := Value;
end;

procedure TRakBinaryWideString.LoadValue(Stream: TStream);
var
  Len: Longword;
  str: WideString;
begin
  Stream.Read(Len, SizeOf(Len));
  SetLength(str, Len);
  Stream.Read(Pointer(str)^, Len * 2);
  Value := str;
end;

procedure TRakBinaryWideString.SaveValue(Stream: TStream);
var
  Len: Longword;
  str: WideString;
begin
  str := Value;
  Len := Length(str);
  Stream.Write(Len, SizeOf(Len));
  Stream.Write(Pointer(str)^, Len * 2);
end;

procedure TRakBinaryWideString.SetValue(const Value: WideString);
begin
  FValue := Value;
  IsNull := false;
end;

procedure TRakBinaryWideString.SetValueAsWideString(const Value: WideString);
begin
  self.Value := Value;
end;

{ TRakBinaryBitmap }

constructor TRakBinaryBitmap.Create;
begin
  inherited;
  Graphic := TBitmap.Create;
end;

function TRakBinaryBitmap.CreateGraphic: TGraphic;
begin
  Result := TBitmap.Create;
end;

function TRakBinaryBitmap.GetValue: TBitmap;
begin
  Result := TBitmap(Graphic);
end;

procedure TRakBinaryBitmap.SetValue(const Value: TBitmap);
begin
  Graphic.Assign(Value);
  IsNull := false;
end;

{ TRakBinaryIcon }

constructor TRakBinaryIcon.Create;
begin
  inherited;
  Graphic := TIcon.Create;
end;

function TRakBinaryIcon.CreateGraphic: TGraphic;
begin
  Result := TIcon.Create;
end;

function TRakBinaryIcon.GetValue: TIcon;
begin
  Result := TIcon(Graphic);
end;

procedure TRakBinaryIcon.SetValue(const Value: TIcon);
begin
  Graphic.Assign(Value);
  IsNull := false;
end;

{ TRakBinaryGraphic }

procedure TRakBinaryGraphic.AfterLoading;
begin
  if Graphic <> nil then
    Graphic.LoadFromStream(FMemoryStream);
  FMemoryStream.Clear;
end;

procedure TRakBinaryGraphic.AfterSaving;
begin
  FIsSaving := false;
  FMemoryStream.Clear;
end;

procedure TRakBinaryGraphic.Assign(Source: TRakCollectionItem);
var
  Src: TRakBinaryGraphic;
begin
  inherited;
  if Source is TRakBinaryGraphic then
  begin
    Src := TRakBinaryGraphic(Source);
    FGraphic.Assign(Src.FGraphic);
  end;
end;

procedure TRakBinaryGraphic.BeforeSaving;
begin
  if Graphic <> nil then
    Graphic.SaveToStream(FMemoryStream);

  FIsSaving := true;
end;

constructor TRakBinaryGraphic.Create;
begin
  inherited;

  FGraphic := CreateGraphic;
end;

destructor TRakBinaryGraphic.Destroy;
begin
  FreeTObject(FGraphic);
  inherited
end;

function TRakBinaryGraphic.GetBinarySize: Longword;
begin
  if not FIsSaving then
  begin
    FMemoryStream.Clear;
    Graphic.SaveToStream(FMemoryStream);
  end;

  Result := FMemoryStream.Size;

  if not FIsSaving then
    FMemoryStream.Clear;
end;

procedure TRakBinaryGraphic.SetGraphic(const Value: TGraphic);
begin
  if FGraphic <> nil then
  begin
    FGraphic.Assign(Value);
    IsNull := false;
  end;
end;

{ TRakBinaryMetafile }

constructor TRakBinaryMetafile.Create;
begin
  inherited;
  Graphic := TMetafile.Create;
end;

function TRakBinaryMetafile.CreateGraphic: TGraphic;
begin
  Result := TMetafile.Create;
end;

function TRakBinaryMetafile.GetValue: TMetafile;
begin
  Result := TMetafile(Graphic);
end;

procedure TRakBinaryMetafile.SetValue(const Value: TMetafile);
begin
  Graphic.Assign(Value);
  IsNull := false;
end;

{ TRakBinaryJPEG }

constructor TRakBinaryJPEG.Create;
begin
  inherited;
  Graphic := TJPEGImage.Create;
end;

function TRakBinaryJPEG.CreateGraphic: TGraphic;
begin
  Result := TJPEGImage.Create;
end;

function TRakBinaryJPEG.GetValue: TJPEGImage;
begin
  Result := TJPEGImage(Graphic);
end;

procedure TRakBinaryJPEG.SetValue(const Value: TJPEGImage);
begin
  Graphic.Assign(Value);
  IsNull := false;
end;

{ TRakBinaryTagList }

procedure TRakBinaryTagList.Add(Tag: TRakBinaryTag);
begin
  FList.Add(Tag);
end;

function TRakBinaryTagList.Count: integer;
begin
  Result := FList.Count;
end;



constructor TRakBinaryTagList.Create;
begin
  inherited;
  FList := TList.Create;
end;

destructor TRakBinaryTagList.Destroy;
begin
  FreeTObject(FList);

  inherited;
end;

function TRakBinaryTagList.Items(Idx: integer): TRakBinaryTag;
begin
  Result := TRakBinaryTag(FList[Idx]);
end;

{ TRakBinaryShortint }

class procedure TRakBinaryShortint.AddToTag(Tag: TRakBinaryTag;
  Name: ShortString; Value: Shortint);
var
  Attr: TRakBinaryShortint;
begin
  Attr := TRakBinaryShortint(Tag.AddAttribute(TRakBinaryShortint, Name));
  Attr.Value := Value;
end;

procedure TRakBinaryShortint.Assign(Source: TRakCollectionItem);
var
  Src: TRakBinaryShortint;
begin
  inherited;
  if Source is TRakBinaryShortint then
  begin
    Src := TRakBinaryShortint(Source);
    FValue := Src.FValue;
  end;
end;

function TRakBinaryShortint.GetValue: Shortint;
begin
  Result := FValue;
end;

function TRakBinaryShortint.GetValueAsWideString: WideString;
begin
  Result := IntToStr(FValue);
end;

procedure TRakBinaryShortint.LoadValue(Stream: TStream);
var
  Wert: Shortint;
begin
  Stream.Read(Wert, SizeOf(Wert));
  Value := Wert;
end;

procedure TRakBinaryShortint.SaveValue(Stream: TStream);
var
  Wert: Shortint;
begin
  Wert := Value;
  Stream.Write(Wert, SizeOf(Wert));
end;

procedure TRakBinaryShortint.SetValue(const Value: Shortint);
begin
  FValue := Value;
  IsNull := false;
end;

procedure TRakBinaryShortint.SetValueAsWideString(const Value: WideString);
begin
  self.Value := StrToInt(Value);
end;

{ TRakBinaryChar }

class procedure TRakBinaryChar.AddToTag(Tag: TRakBinaryTag;
  Name: ShortString; Value: Char);
var
  Attr: TRakBinaryChar;
begin
  Attr := TRakBinaryChar(Tag.AddAttribute(TRakBinaryChar, Name));
  Attr.Value := Value;
end;

procedure TRakBinaryChar.Assign(Source: TRakCollectionItem);
var
  Src: TRakBinaryChar;
begin
  inherited;
  if Source is TRakBinaryChar then
  begin
    Src := TRakBinaryChar(Source);
    FValue := Src.FValue;
  end;
end;

function TRakBinaryChar.GetValue: Char;
begin
  Result := FValue;
end;

function TRakBinaryChar.GetValueAsWideString: WideString;
begin
  Result := WideChar(FValue);
end;

procedure TRakBinaryChar.LoadValue(Stream: TStream);
var
  Wert: char;
begin
  Stream.Read(Wert, SizeOf(Wert));
  Value := Wert;
end;

procedure TRakBinaryChar.SaveValue(Stream: TStream);
var
  Wert: char;
begin
  Wert := Value;
  Stream.Write(Wert, SizeOf(Wert));
end;

procedure TRakBinaryChar.SetValue(const Value: Char);
begin
  FValue := Value;
  IsNull := false;
end;

procedure TRakBinaryChar.SetValueAsWideString(const Value: WideString);
begin
  self.Value := char(Value[1]);
end;

{ TRakBinaryWord }

class procedure TRakBinaryWord.AddToTag(Tag: TRakBinaryTag;
  Name: ShortString; Value: Word);
var
  Attr: TRakBinaryWord;
begin
  Attr := TRakBinaryWord(Tag.AddAttribute(TRakBinaryWord, Name));
  Attr.Value := Value;
end;

procedure TRakBinaryWord.Assign(Source: TRakCollectionItem);
var
  Src: TRakBinaryWord;
begin
  inherited;
  if Source is TRakBinaryWord then
  begin
    Src := TRakBinaryWord(Source);
    FValue := Src.FValue;
  end;
end;

function TRakBinaryWord.GetValue: Word;
begin
  Result := FValue;
end;

function TRakBinaryWord.GetValueAsWideString: WideString;
begin
  Result := IntToStr(FValue);
end;

procedure TRakBinaryWord.LoadValue(Stream: TStream);
var
  Wert: Word;
begin
  Stream.Read(Wert, SizeOf(Wert));
  Value := Wert;
end;

procedure TRakBinaryWord.SaveValue(Stream: TStream);
var
  Wert: word;
begin
  Wert := Value;
  Stream.Write(Wert, SizeOf(Wert));
end;

procedure TRakBinaryWord.SetValue(const Value: Word);
begin
  FValue := Value;
  IsNull := false;
end;

procedure TRakBinaryWord.SetValueAsWideString(const Value: WideString);
begin
  self.Value := StrToInt(Value);
end;

{ TRakBinarySmallint }

class procedure TRakBinarySmallint.AddToTag(Tag: TRakBinaryTag;
  Name: ShortString; Value: Smallint);
var
  Attr: TRakBinarySmallint;
begin
  Attr := TRakBinarySmallint(Tag.AddAttribute(TRakBinarySmallint, Name));
  Attr.Value := Value;
end;

procedure TRakBinarySmallint.Assign(Source: TRakCollectionItem);
var
  Src: TRakBinarySmallint;
begin
  inherited;
  if Source is TRakBinarySmallint then
  begin
    Src := TRakBinarySmallint(Source);
    FValue := Src.FValue;
  end;
end;

function TRakBinarySmallint.GetValue: Smallint;
begin
  Result := FValue;
end;

function TRakBinarySmallint.GetValueAsWideString: WideString;
begin
  Result := IntToStr(FValue);
end;

procedure TRakBinarySmallint.LoadValue(Stream: TStream);
var
  Wert: Smallint;
begin
  Stream.Read(Wert, SizeOf(Wert));
  Value := Wert;
end;

procedure TRakBinarySmallint.SaveValue(Stream: TStream);
var
  Wert: Smallint;
begin
  Wert := Value;
  Stream.Write(Wert, SizeOf(Wert));
end;

procedure TRakBinarySmallint.SetValue(const Value: Smallint);
begin
  FValue := Value;
  IsNull := false;
end;

procedure TRakBinarySmallint.SetValueAsWideString(const Value: WideString);
begin
  self.Value := StrToInt(Value);
end;

{ TRakBinaryLongword }

class procedure TRakBinaryLongword.AddToTag(Tag: TRakBinaryTag;
  Name: ShortString; Value: LongWord);
var
  Attr: TRakBinaryLongword;
begin
  Attr := TRakBinaryLongword(Tag.AddAttribute(TRakBinaryLongword, Name));
  Attr.Value := Value;
end;

procedure TRakBinaryLongword.Assign(Source: TRakCollectionItem);
var
  Src: TRakBinaryLongword;
begin
  inherited;
  if Source is TRakBinaryLongword then
  begin
    Src := TRakBinaryLongword(Source);
    FValue := Src.FValue;
  end;
end;

function TRakBinaryLongword.GetValue: LongWord;
begin
  Result := FValue;
end;

function TRakBinaryLongword.GetValueAsWideString: WideString;
begin
  Result := IntToStr(FValue);
end;

procedure TRakBinaryLongword.LoadValue(Stream: TStream);
var
  Wert: Longword;
begin
  Stream.Read(Wert, SizeOf(Wert));
  Value := Wert;
end;

procedure TRakBinaryLongword.SaveValue(Stream: TStream);
var
  Wert: Longword;
begin
  Wert := Value;
  Stream.Write(Wert, SizeOf(Wert));
end;

procedure TRakBinaryLongword.SetValue(const Value: LongWord);
begin
  FValue := Value;
  IsNull := false;
end;

procedure TRakBinaryLongword.SetValueAsWideString(const Value: WideString);
begin
  self.Value := StrToInt(Value);
end;

{ TRakBinaryLongint }

class procedure TRakBinaryLongint.AddToTag(Tag: TRakBinaryTag;
  Name: ShortString; Value: Integer);
var
  Attr: TRakBinaryLongint;
begin
  Attr := TRakBinaryLongint(Tag.AddAttribute(TRakBinaryLongint, Name));
  Attr.Value := Value;
end;

procedure TRakBinaryLongint.Assign(Source: TRakCollectionItem);
var
  Src: TRakBinaryLongint;
begin
  inherited;
  if Source is TRakBinaryLongint then
  begin
    Src := TRakBinaryLongint(Source);
    FValue := Src.FValue;
  end;
end;

function TRakBinaryLongint.GetValue: Longint;
begin
  Result := FValue;
end;

function TRakBinaryLongint.GetValueAsWideString: WideString;
begin
  Result := IntToStr(FValue);
end;

procedure TRakBinaryLongint.LoadValue(Stream: TStream);
var
  Wert: Longint;
begin
  Stream.Read(Wert, SizeOf(Wert));
  Value := Wert;
end;

procedure TRakBinaryLongint.SaveValue(Stream: TStream);
var
  Wert: Longint;
begin
  Wert := Value;
  Stream.Write(Wert, SizeOf(Wert));
end;

procedure TRakBinaryLongint.SetValue(const Value: Integer);
begin
  FValue := Value;
  IsNull := false;
end;

procedure TRakBinaryLongint.SetValueAsWideString(const Value: WideString);
begin
  self.Value := StrToInt(Value);
end;

{ TRakBinaryReal48 }

class procedure TRakBinaryReal48.AddToTag(Tag: TRakBinaryTag;
  Name: ShortString; Value: Real48);
var
  Attr: TRakBinaryReal48;
begin
  Attr := TRakBinaryReal48(Tag.AddAttribute(TRakBinaryReal48, Name));
  Attr.Value := Value;
end;

procedure TRakBinaryReal48.Assign(Source: TRakCollectionItem);
var
  Src: TRakBinaryReal48;
begin
  inherited;
  if Source is TRakBinaryReal48 then
  begin
    Src := TRakBinaryReal48(Source);
    FValue := Src.FValue;
  end;
end;

function TRakBinaryReal48.GetValue: Real48;
begin
  Result := FValue;
end;

function TRakBinaryReal48.GetValueAsWideString: WideString;
begin
  Result := FloatToStr(Value);
end;

procedure TRakBinaryReal48.LoadValue(Stream: TStream);
var
  Wert: extended;
begin
  Stream.Read(Wert, SizeOf(Wert));
  Value := Wert;
end;

procedure TRakBinaryReal48.SaveValue(Stream: TStream);
var
  Wert: extended;
begin
  Wert := Value;
  Stream.Write(Wert, SizeOf(Wert));
end;

procedure TRakBinaryReal48.SetValue(const Value: Real48);
begin
  FValue := Value;
  IsNull := false;
end;

procedure TRakBinaryReal48.SetValueAsWideString(const Value: WideString);
begin
  self.Value := InputToFloat(Value, FValue);
end;

{ TRakBinarySingle }

class procedure TRakBinarySingle.AddToTag(Tag: TRakBinaryTag;
  Name: ShortString; Value: Single);
var
  Attr: TRakBinarySingle;
begin
  Attr := TRakBinarySingle(Tag.AddAttribute(TRakBinarySingle, Name));
  Attr.Value := Value;
end;

procedure TRakBinarySingle.Assign(Source: TRakCollectionItem);
var
  Src: TRakBinarySingle;
begin
  inherited;
  if Source is TRakBinarySingle then
  begin
    Src := TRakBinarySingle(Source);
    FValue := Src.FValue;
  end;
end;

function TRakBinarySingle.GetValue: Single;
begin
  Result := FValue;
end;

function TRakBinarySingle.GetValueAsWideString: WideString;
begin
  Result := FloatToStr(Value);
end;

procedure TRakBinarySingle.LoadValue(Stream: TStream);
var
  Wert: extended;
begin
  Stream.Read(Wert, SizeOf(Wert));
  Value := Wert;
end;

procedure TRakBinarySingle.SaveValue(Stream: TStream);
var
  Wert: extended;
begin
  Wert := Value;
  Stream.Write(Wert, SizeOf(Wert));
end;

procedure TRakBinarySingle.SetValue(const Value: Single);
begin
  FValue := Value;
  IsNull := false;
end;

procedure TRakBinarySingle.SetValueAsWideString(const Value: WideString);
begin
  self.Value := InputToFloat(Value, FValue);
end;

{ TRakBinaryDouble }

class procedure TRakBinaryDouble.AddToTag(Tag: TRakBinaryTag;
  Name: ShortString; Value: Double);
var
  Attr: TRakBinaryDouble;
begin
  Attr := TRakBinaryDouble(Tag.AddAttribute(TRakBinaryDouble, Name));
  Attr.Value := Value;
end;

procedure TRakBinaryDouble.Assign(Source: TRakCollectionItem);
var
  Src: TRakBinaryDouble;
begin
  inherited;
  if Source is TRakBinaryDouble then
  begin
    Src := TRakBinaryDouble(Source);
    FValue := Src.FValue;
  end;
end;

function TRakBinaryDouble.GetValue: Double;
begin
  Result := FValue;
end;

function TRakBinaryDouble.GetValueAsWideString: WideString;
begin
  Result := FloatToStr(Value);
end;

procedure TRakBinaryDouble.LoadValue(Stream: TStream);
var
  Wert: extended;
begin
  Stream.Read(Wert, SizeOf(Wert));
  Value := Wert;
end;

procedure TRakBinaryDouble.SaveValue(Stream: TStream);
var
  Wert: extended;
begin
  Wert := Value;
  Stream.Write(Wert, SizeOf(Wert));
end;

procedure TRakBinaryDouble.SetValue(const Value: Double);
begin
  FValue := Value;
  IsNull := false;
end;

procedure TRakBinaryDouble.SetValueAsWideString(const Value: WideString);
begin
  self.Value := InputToFloat(Value, FValue);
end;

{ TRakBinaryComp }

class procedure TRakBinaryComp.AddToTag(Tag: TRakBinaryTag;
  Name: ShortString; Value: Comp);
var
  Attr: TRakBinaryComp;
begin
  Attr := TRakBinaryComp(Tag.AddAttribute(TRakBinaryComp, Name));
  Attr.Value := Value;
end;

procedure TRakBinaryComp.Assign(Source: TRakCollectionItem);
var
  Src: TRakBinaryComp;
begin
  inherited;
  if Source is TRakBinaryComp then
  begin
    Src := TRakBinaryComp(Source);
    FValue := Src.FValue;
  end;
end;

function TRakBinaryComp.GetValue: Comp;
begin
  Result := FValue;
end;

function TRakBinaryComp.GetValueAsWideString: WideString;
begin
  Result := FloatToStr(Value);
end;

procedure TRakBinaryComp.LoadValue(Stream: TStream);
var
  Wert: extended;
begin
  Stream.Read(Wert, SizeOf(Wert));
  Value := Wert;
end;

procedure TRakBinaryComp.SaveValue(Stream: TStream);
var
  Wert: extended;
begin
  Wert := Value;
  Stream.Write(Wert, SizeOf(Wert));
end;

procedure TRakBinaryComp.SetValue(const Value: Comp);
begin
  FValue := Value;
  IsNull := false;
end;

procedure TRakBinaryComp.SetValueAsWideString(const Value: WideString);
begin
  self.Value := InputToFloat(Value, FValue);
end;

{ TRakBinaryStreamData }

procedure TRakBinaryStreamData.Clear;
begin
  inherited;
  ApplicationName := '';
  ApplicationVersion := '';
end;

function TRakBinaryStreamData.LoadFromClipboardFormat(
  ClipboardFormat: Word): TRakBinaryLoadClipboardResult;
var
  Stream: TMemoryStream;
  MemHandle: THandle;
  MemBlock: Pointer;
  ASize: integer;
begin
  try
    Result := blcrEmpty;

    Clear;

    Clipboard.Open;
    try
      if Clipboard.HasFormat(ClipboardFormat) then
      begin // Wenn in der Zwischenablage etwas in richtigen Format vorliegt
        Result := blcrErrUnknown;
        // Eine Referenz auf die Daten in der Zwischenablage anfordern
        MemHandle := Clipboard.GetAsHandle(ClipboardFormat);
        if MemHandle <> 0 then
        begin
          // Die Gre (Anzahl der Bytes) feststellen
          ASize := GlobalSize(MemHandle);
          Stream := nil;
          try
            Stream := TMemoryStream.Create;

            // Den Inhalt der Zwischenablage fixieren
            MemBlock := GlobalLock(MemHandle);
            try
              // Die Daten in den Stream kopieren
              Stream.Write(MemBlock^, ASize);
            finally
              GlobalUnlock(MemHandle);
            end;

            // Den Inhalt des Streams in das Zwischenformat umwandeln
            Stream.Seek(0, soFromBeginning);
            case LoadFromStream(Stream) of
              blrOK: Result := blcrOK;
              blrErrVersion: Result := blcrErrVersion;
              blrErrFormat: Result := blcrErrFormat;
              blrErrCorrupt: Result := blcrErrCorrupt;
            end;
          finally
            Stream.Free;
          end;
        end;
      end;
    finally
      Clipboard.Close;
    end;
  except
    Result := blcrErrUnknown;
  end;
end;

function TRakBinaryStreamData.LoadFromFile(FileName: string): TRakBinaryLoadResult;
var
  Stream: TFileStream;
begin
  Stream := TFileStream.Create(FileName, fmOpenRead);
  try
    Result := LoadFromStream(Stream);
  finally
    Stream.Free;
  end;
end;

function TRakBinaryStreamData.LoadFromStream(
  Stream: TStream): TRakBinaryLoadResult;
var
  Version: Word;
  str: ShortString;
begin
  Result := blrErrFormat;
  try
    str := ReadShortString(Stream);
    if str = 'TRakBinaryStreamData' then
    begin
      try
        Stream.Read(Version, SizeOf(Version));
        if Version <> 1 then
          Result := blrErrVersion
        else
        begin
          Result := blrOK;
          ApplicationName := ReadShortString(Stream);
          ApplicationVersion := ReadShortString(Stream);
          LoadFromStreamNow(Stream);
        end;
      except
        Result := blrErrCorrupt;
      end;
    end;
  except
  end;
end;

class procedure TRakBinaryStreamData.LoadObjectFromFile(Obj: TObject;
  FileName: string);
var
	Data: TRakBinaryStreamData;
begin
	Data := TRakBinaryStreamData.Create;
  try
  	Data.LoadFromFile(FileName);
		Data.WriteToPublishedProperties(Obj);
  finally
    Data.Free;
  end;
end;

class procedure TRakBinaryStreamData.SaveObjectToFile(Obj: TObject;
  FileName: string);
var
	Data: TRakBinaryStreamData;
begin
	Data := TRakBinaryStreamData.Create;
  try
		Data.ReadFromPublishedProperties(Obj, nil, true);
    Data.SaveToFile(FileName);
  finally
    Data.Free;
  end;
end;

procedure TRakBinaryStreamData.SaveToClipboardFormat(ClipboardFormat: Word;
  ClearData: boolean; ClipboardIsOpen: boolean);
var
  Stream: TMemoryStream;
  MemHandle: THandle;
  MemBlock: Pointer;
  CloseIt: boolean;
begin
  CloseIt := false;
  Stream := nil;
  try
    // Einen Memory-Stream erzeugen, in den wir alle Daten des Kontaktes ablegen
    Stream := TMemoryStream.Create;

    // Das Zwischenformat in den Stream schreiben
    SaveToStream(Stream);

    if ClearData then
      Clear;

    // Speicher fr die Zwischenablage anfordern
    MemHandle := GlobalAlloc(GMEM_DDESHARE, Stream.Size);
    try
      // Den Speicher fixieren
      MemBlock := GlobalLock(MemHandle);
      try
        // Den Inhalt des Streams in den Speicher kopieren
        Stream.Seek(0, soFromBeginning);
        Stream.Read(MemBlock^, Stream.Size);
      finally
        // Die Fixierung wieder aufheben
        GlobalUnlock(MemHandle);
      end;
      // Den Speicher an die Zwischenablage bergeben und als das richtigen Format kennzeichnen
      if not ClipboardIsOpen then
      begin
        Clipboard.Open;
        CloseIt := true;
      end;

      Clipboard.SetAsHandle(ClipboardFormat, MemHandle);
    except
      GlobalFree(MemHandle);
      raise;
    end;
  finally
    if CloseIt then
      Clipboard.Close;

    Stream.Free;
  end;
end;

procedure TRakBinaryStreamData.SaveToFile(FileName: string);
var
  Stream: TFileStream;
begin
  Stream := TFileStream.Create(FileName, fmCreate);
  try
    SaveToStream(Stream);
  finally
    Stream.Free;
  end;
end;

procedure TRakBinaryStreamData.SaveToStream(Stream: TStream);
var
  Version: Word;
begin
  WriteShortString(Stream, 'TRakBinaryStreamData');
  Version := 1;
  Stream.Write(Version, SizeOf(Version));
  WriteShortString(Stream, ApplicationName);
  WriteShortString(Stream, ApplicationVersion);
  SaveToStreamNow(Stream);
end;


{ TRakBinaryPicture }

class procedure TRakBinaryPicture.AddToTag(Tag: TRakBinaryTag;
  Name: ShortString; Value: TGraphic);
var
  Attr: TRakBinaryPicture;
begin
  Attr := TRakBinaryPicture(Tag.AddAttribute(TRakBinaryPicture, Name));
  Attr.Graphic := Value;
end;

procedure TRakBinaryPicture.AfterLoading;
var
  KlassenName: string;
  TmpGraphic: TGraphic;
  i: integer;
begin
  KlassenName := ReadShortString(FMemoryStream);

  Graphic := nil;
  for i := 0 to RegisteredGraphicClasses.Count - 1 do
  begin
    if CompareText(KlassenName, RegisteredGraphicClasses[i].ClassName) = 0 then
    begin
      TmpGraphic := TGraphicClass(RegisteredGraphicClasses[i]).Create;
      try
        TmpGraphic.LoadFromStream(FMemoryStream);
        Graphic := TmpGraphic;
      finally
        TmpGraphic.Free;
      end;
      break;
    end;
  end;

  FMemoryStream.Clear;
end;

procedure TRakBinaryPicture.AfterSaving;
begin
  FIsSaving := false;
  FMemoryStream.Clear;
end;

procedure TRakBinaryPicture.Assign(Source: TRakCollectionItem);
var
  Src: TRakBinaryPicture;
begin
  inherited;
  if Source is TRakBinaryPicture then
  begin
    Src := TRakBinaryPicture(Source);
    FPicture.Assign(Src.FPicture);
  end;
end;

procedure TRakBinaryPicture.BeforeSaving;
var
  KlassenName: string;
begin
  if Graphic = nil then
    KlassenName := ''
  else
    KlassenName := Graphic.ClassName;

  WriteShortString(FMemoryStream, KlassenName);

  if Graphic <> nil then
    Graphic.SaveToStream(FMemoryStream);

  FIsSaving := true;
end;

constructor TRakBinaryPicture.Create;
begin
  inherited;
  FPicture := TPicture.Create;
end;

destructor TRakBinaryPicture.Destroy;
begin
  FPicture.Free;
  inherited;
end;

function TRakBinaryPicture.GetBinarySize: Longword;
begin
  if not FIsSaving then
  begin
    FMemoryStream.Clear;
    Graphic.SaveToStream(FMemoryStream);
  end;

  Result := FMemoryStream.Size;

  if not FIsSaving then
    FMemoryStream.Clear;
end;

function TRakBinaryPicture.GetGraphic: TGraphic;
begin
  Result := FPicture.Graphic;
end;

class procedure TRakBinaryPicture.RegisterGraphicClass(
  AClass: TGraphicClass);
begin
  RegisteredGraphicClasses.Add(AClass);
end;

procedure TRakBinaryPicture.SetGraphic(const Value: TGraphic);
begin
  if Value <> nil then
  begin
    if RegisteredGraphicClasses.IndexOf(Value.ClassType) = -1 then
      raise Exception.Create('Die Klasse "' + Value.ClassName + '" muss mit TRakBinaryPicture.RegisterGraphicClass registriert werden.');
  end;

  FPicture.Graphic := Value;
  IsNull := (Value = nil) or Value.Empty;
end;

{ TRakBinaryTShiftState }

class procedure TRakBinaryTShiftState.AddToTag(Tag: TRakBinaryTag;
  Name: ShortString; Value: TShiftState);
var
  Attr: TRakBinaryTShiftState;
begin
  Attr := TRakBinaryTShiftState(Tag.AddAttribute(TRakBinaryTShiftState, Name));
  Attr.Value := Value;
end;

constructor TRakBinaryTShiftState.Create;
begin
  inherited;
  FMemoryStream.Size := sizeof(TShiftState);
end;

function TRakBinaryTShiftState.GetValue: TShiftState;
begin
  CopyMemory(@Result, FMemoryStream.Memory, sizeof(TShiftState));
end;

class function TRakBinaryTShiftState.ReadValue(Tag: TRakBinaryTag;
  Name: ShortString; DefaultValue: TShiftState): TShiftState;
var
  Attr: TRakBinaryAttribute;
begin
  Result := DefaultValue;

  Attr := Tag.AttributeByName(Name);
  if (Attr is TRakBinaryTShiftState) and not Attr.IsNull then
    Result := TRakBinaryTShiftState(Attr).Value;
end;

procedure TRakBinaryTShiftState.SetValue(const Value: TShiftState);
begin
  CopyMemory(FMemoryStream.Memory, @Value, sizeof(TShiftState));
  IsNull := false;
end;

{ TRakBinaryReadFromPropertiesOptions }

function TRakBinaryReadFromPropertiesOptions.AddClassProperty: IRakBinaryReadFromClassProperty;
begin
	Result := TRakBinaryReadFromClassProperty.Create;
  FItems.Add(Result);
end;

function TRakBinaryReadFromPropertiesOptions.AddSimpleProperty: IRakBinaryReadFromSimpleProperty;
begin
	Result := TRakBinaryReadFromSimpleProperty.Create;
  FItems.Add(Result);
end;

constructor TRakBinaryReadFromPropertiesOptions.Create;
begin
	inherited;

  FItems := TInterfaceList.Create;
end;

destructor TRakBinaryReadFromPropertiesOptions.Destroy;
begin
	FItems.Free;

  inherited;
end;

function TRakBinaryReadFromPropertiesOptions.GetCount: integer;
begin
	Result := FItems.Count;
end;

function TRakBinaryReadFromPropertiesOptions.GetItems(
  Idx: integer): IRakBinaryReadFromPropertiesOptionsItem;
begin
	Result := FItems[Idx] as IRakBinaryReadFromPropertiesOptionsItem;
end;

function TRakBinaryReadFromPropertiesOptions.ItemByName(
  Name: ShortString): IRakBinaryReadFromPropertiesOptionsItem;
var
  i: integer;
begin
	Result := nil;
  for i := 0 to Count - 1 do
  begin
  	if CompareText(Items[i].PropertyName, Name) = 0 then
    begin
      Result := Items[i];
      break;
    end;
  end;
end;

{ TRakBinaryWriteToPropertiesOptions }

function TRakBinaryWriteToPropertiesOptions.AddClassProperty: IRakBinaryWriteToClassProperty;
begin
	Result := TRakBinaryWriteToClassProperty.Create;
  FItems.Add(Result);
end;

function TRakBinaryWriteToPropertiesOptions.AddSimpleProperty: IRakBinaryWriteToSimpleProperty;
begin
	Result := TRakBinaryWriteToSimpleProperty.Create;
  FItems.Add(Result);
end;

constructor TRakBinaryWriteToPropertiesOptions.Create;
begin
	inherited;
  FItems := TInterfaceList.Create;
end;

destructor TRakBinaryWriteToPropertiesOptions.Destroy;
begin
	FItems.Free;
  inherited;
end;

function TRakBinaryWriteToPropertiesOptions.GetCount: integer;
begin
	Result := FItems.Count;
end;

function TRakBinaryWriteToPropertiesOptions.GetItems(
  Idx: integer): IRakBinaryWriteToPropertiesOptionsItem;
begin
	Result := FItems[Idx] as IRakBinaryWriteToPropertiesOptionsItem;
end;

function TRakBinaryWriteToPropertiesOptions.ItemByName(
  Name: ShortString): IRakBinaryWriteToPropertiesOptionsItem;
var
  i: integer;
begin
	Result := nil;
  for i := 0 to Count - 1 do
  begin
  	if CompareText(Items[i].PropertyName, Name) = 0 then
    begin
      Result := Items[i];
      break;
    end;
  end;
end;

{ TRakBinaryReadFromPropertiesOptionsItem }

function TRakBinaryReadFromPropertiesOptionsItem.GetIgnoreMissing: boolean;
begin
	Result := FIgnoreMissing;
end;

function TRakBinaryReadFromPropertiesOptionsItem.GetPropertyName: string;
begin
	Result := FPropertyName;
end;

procedure TRakBinaryReadFromPropertiesOptionsItem.SetIgnoreMissing(
  Value: boolean);
begin
	FIgnoreMissing := Value;
end;

procedure TRakBinaryReadFromPropertiesOptionsItem.SetPropertyName(
  Value: string);
begin
	FPropertyName := Value;
end;

{ TRakBinaryReadFromSimpleProperty }

function TRakBinaryReadFromSimpleProperty.GetAttributName: string;
begin
	Result := FAttributName;
end;

procedure TRakBinaryReadFromSimpleProperty.SetAttributName(Value: string);
begin
	FAttributName := Value;
end;

{ TRakBinaryReadFromClassProperty }

constructor TRakBinaryReadFromClassProperty.Create;
begin
	inherited;
  FOptions := TRakBinaryReadFromPropertiesOptions.Create;
end;

function TRakBinaryReadFromClassProperty.GetOptions: IRakBinaryReadFromPropertiesOptions;
begin
	Result := FOptions;
end;

function TRakBinaryReadFromClassProperty.GetReadFromClassAttributs: boolean;
begin
  Result := FReadFromClassAttributs;
end;

function TRakBinaryReadFromClassProperty.GetTagName: string;
begin
  Result := FTagName;
end;

procedure TRakBinaryReadFromClassProperty.SetReadFromClassAttributs(
  Value: boolean);
begin
	FReadFromClassAttributs := Value;
end;

procedure TRakBinaryReadFromClassProperty.SetTagName(Value: string);
begin
	FTagName := Value;
end;

{ TRakBinaryWriteToPropertiesOptionsItem }

function TRakBinaryWriteToPropertiesOptionsItem.GetIgnoreMissing: boolean;
begin
	Result := FIgnoreMissing;
end;

function TRakBinaryWriteToPropertiesOptionsItem.GetPropertyName: string;
begin
	Result := FPropertyName;
end;

procedure TRakBinaryWriteToPropertiesOptionsItem.SetIgnoreMissing(
  Value: boolean);
begin
	FIgnoreMissing := Value;
end;

procedure TRakBinaryWriteToPropertiesOptionsItem.SetPropertyName(
  Value: string);
begin
	FPropertyName := Value;
end;

{ TRakBinaryWriteToSimpleProperty }

function TRakBinaryWriteToSimpleProperty.GetAttributName: string;
begin
	Result := FAttributName;
end;

procedure TRakBinaryWriteToSimpleProperty.SetAttributName(Value: string);
begin
  FAttributName := Value;
end;

{ TRakBinaryWriteToClassProperty }

constructor TRakBinaryWriteToClassProperty.Create;
begin
	inherited;
  FOptions := TRakBinaryWriteToPropertiesOptions.Create;
end;

function TRakBinaryWriteToClassProperty.GetOptions: IRakBinaryWriteToPropertiesOptions;
begin
	Result := FOptions;
end;

function TRakBinaryWriteToClassProperty.GetTagName: string;
begin
  Result := FTagName;
end;

procedure TRakBinaryWriteToClassProperty.SetTagName(Value: string);
begin
	FTagName := Value;
end;

{ TRakBinaryReadFrom }

procedure TRakBinaryReadFrom.AfterReadFromClassProperty(
  const PropertyDescription: TRakBinaryReadFromClassPropertyDescription);
begin
end;

procedure TRakBinaryReadFrom.AfterReadFromProperties(Tag: TRakBinaryTag);
begin
end;

procedure TRakBinaryReadFrom.AfterReadFromSimpleProperty(
  const PropertyDescription: TRakBinaryReadFromSimplePropertyDescription);
begin
end;

procedure TRakBinaryReadFrom.BeforeReadFromClassProperty(
  var PropertyDescription: TRakBinaryReadFromClassPropertyDescription);
begin
end;

procedure TRakBinaryReadFrom.BeforeReadFromProperties(Tag: TRakBinaryTag;
  Options: IRakBinaryReadFromPropertiesOptions; var DoIt: boolean);
begin
end;

procedure TRakBinaryReadFrom.BeforeReadFromSimpleProperty(
  var PropertyDescription: TRakBinaryReadFromSimplePropertyDescription);
begin
end;

procedure TRakBinaryReadFrom.HandleNotSupportedReadProperties(
  PropertyName: ShortString);
begin
end;

{ TRakBinaryWriteTo }

procedure TRakBinaryWriteTo.AfterWriteToClassProperty(
  const PropertyDescription: TRakBinaryWriteToClassPropertyDescription);
begin
end;

procedure TRakBinaryWriteTo.AfterWriteToProperties(Tag: TRakBinaryTag);
begin
end;

procedure TRakBinaryWriteTo.AfterWriteToSimpleProperty(
  const PropertyDescription: TRakBinaryWriteToSimplePropertyDescription);
begin
end;

procedure TRakBinaryWriteTo.AttributForSimplePropertyNotFound(
  PropertyDescription: TRakBinaryWriteToSimplePropertyDescription);
begin
end;

procedure TRakBinaryWriteTo.BeforeWriteToClassProperty(
  var PropertyDescription: TRakBinaryWriteToClassPropertyDescription);
begin
end;

procedure TRakBinaryWriteTo.BeforeWriteToProperties(Tag: TRakBinaryTag;
  Options: IRakBinaryWriteToPropertiesOptions; var DoIt: boolean);
begin
end;

procedure TRakBinaryWriteTo.BeforeWriteToSimpleProperty(
  var PropertyDescription: TRakBinaryWriteToSimplePropertyDescription);
begin
end;

procedure TRakBinaryWriteTo.HandleNotSupportedWriteProperties(
  PropertyName: ShortString);
begin
end;

procedure TRakBinaryWriteTo.TagForClassPropertyNotFound(
  PropertyDescription: TRakBinaryWriteToClassPropertyDescription);
begin
end;

{ TRakBinaryReadWrite }

procedure TRakBinaryReadWrite.AfterReadFromClassProperty(
  const PropertyDescription: TRakBinaryReadFromClassPropertyDescription);
begin
end;

procedure TRakBinaryReadWrite.AfterReadFromProperties(Tag: TRakBinaryTag);
begin
end;

procedure TRakBinaryReadWrite.AfterReadFromSimpleProperty(
  const PropertyDescription: TRakBinaryReadFromSimplePropertyDescription);
begin
end;

procedure TRakBinaryReadWrite.AfterWriteToClassProperty(
  const PropertyDescription: TRakBinaryWriteToClassPropertyDescription);
begin
end;

procedure TRakBinaryReadWrite.AfterWriteToProperties(Tag: TRakBinaryTag);
begin
end;

procedure TRakBinaryReadWrite.AfterWriteToSimpleProperty(
  const PropertyDescription: TRakBinaryWriteToSimplePropertyDescription);
begin
end;

procedure TRakBinaryReadWrite.AttributForSimplePropertyNotFound(
  PropertyDescription: TRakBinaryWriteToSimplePropertyDescription);
begin
end;

procedure TRakBinaryReadWrite.BeforeReadFromClassProperty(
  var PropertyDescription: TRakBinaryReadFromClassPropertyDescription);
begin
end;

procedure TRakBinaryReadWrite.BeforeReadFromProperties(Tag: TRakBinaryTag;
  Options: IRakBinaryReadFromPropertiesOptions; var DoIt: boolean);
begin
end;

procedure TRakBinaryReadWrite.BeforeReadFromSimpleProperty(
  var PropertyDescription: TRakBinaryReadFromSimplePropertyDescription);
begin
end;

procedure TRakBinaryReadWrite.BeforeWriteToClassProperty(
  var PropertyDescription: TRakBinaryWriteToClassPropertyDescription);
begin
end;

procedure TRakBinaryReadWrite.BeforeWriteToProperties(Tag: TRakBinaryTag;
  Options: IRakBinaryWriteToPropertiesOptions; var DoIt: boolean);
begin
end;

procedure TRakBinaryReadWrite.BeforeWriteToSimpleProperty(
  var PropertyDescription: TRakBinaryWriteToSimplePropertyDescription);
begin
end;

procedure TRakBinaryReadWrite.HandleNotSupportedReadProperties(
  PropertyName: ShortString);
begin
end;

procedure TRakBinaryReadWrite.HandleNotSupportedWriteProperties(
  PropertyName: ShortString);
begin
end;

procedure TRakBinaryReadWrite.TagForClassPropertyNotFound(
  PropertyDescription: TRakBinaryWriteToClassPropertyDescription);
begin
end;

initialization
  RegisteredDataTypes := TClassList.Create;
  RegisteredGraphicClasses := TClassList.Create;

  TRakBinaryAttribute.RegisterBinaryDataType(TRakBinaryPicture);
  TRakBinaryAttribute.RegisterBinaryDataType(TRakBinaryBitmap);
  TRakBinaryAttribute.RegisterBinaryDataType(TRakBinaryIcon);
  TRakBinaryAttribute.RegisterBinaryDataType(TRakBinaryMetafile);
  TRakBinaryAttribute.RegisterBinaryDataType(TRakBinaryJPEG);
  TRakBinaryAttribute.RegisterBinaryDataType(TRakBinaryTShiftState);

  TRakBinaryPicture.RegisterGraphicClass(TBitmap);
  TRakBinaryPicture.RegisterGraphicClass(TIcon);
  TRakBinaryPicture.RegisterGraphicClass(TMetafile);
  TRakBinaryPicture.RegisterGraphicClass(TJpegImage);

finalization
  RegisteredDataTypes.Free;
  RegisteredGraphicClasses.Free;

end.
