unit RakCollection;

{///////////////////////////////////////////////////////////////////////////////
// Beschreibung : Die unit RakCollection stellt Klassen zur Verfgung, die
                  sehr stark an die Klassen TCollection und TCollectioItem
                  von Delphi erinnern. Im Gegensatz zu TCollection
                  kann TRakCollection sich selbst aufnehmen. Auerdem
                  bietet TRakCollection abgeleiteten Klassen eine
                  Unzahl von Methoden, mit denn sie in das Standardverhalten
                  eingriefen knnen.
// erstellt am  : 1.9.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
  6.10.2001:  Neue Methode GetKomposition in TRakCollectionItem
              Neue Klasse TRakKompositionItem
  22.11.2001: TRakCollection ist nicht mehr von TInterfacedObject sonder von TRakInterfacedObject abgeleitet
  24.08.2002: TRakCollection.FreeAll ruft Changed nur noch auf, wenn wirklich etwas gelscht wurde
  29.05.2003: Bugfix in TRakCollection.DoAssign: nil-Elelemnte wurden nicht bercksichtigt
              TRakCollection.CompareItems arbeitet jetzt mit TRakCollectionItem.Compare
              TRakCollectionItem.Compare schmeit keine Exception mehr
///////////////////////////////////////////////////////////////////////////////}
// Delphi 5 Implementation
//
// Copyright (c) 2001-2002 by Richard Kaspar
// ("http://www.kasparsoft.de")
//
// Definitions:
// - "Package" refers to the collection of files distributed by
//   the Copyright Holder, and derivatives of that collection of
//   files created through textual modification.
// - "Standard Version" refers to such a Package if it has not
//   been modified, or has been modified in accordance with the
//   wishes of the Copyright Holder.
// - "Copyright Holder" is whoever is name in the copyright or
//   copyrights for the package.
// - "You" is you, if you're thinking about copying or distributing
//   this Package.
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Package"), to deal in the Package without restriction,
// including without limitation the rights to use, copy, modify,
// merge, publish, distribute, sublicense, and/or sell copies of the
// Package, and to permit persons to whom the Package is furnished
// to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Package.
//
// You may modify your copy of this Package in any way, provided
// that you insert a prominent notice in each changed file stating
// how and when you changed a file, and provided that you do at
// least one of the following:
//
// a) allow the Copyright Holder to include your modifications in
// the Standard Version of the Package.
//
// b) use the modified Package only within your corporation or
// organization.
//
// c) rename any non standard executables, units, and classes so
// the names do not conflict with standard executables, units, and
// classes, and provide a separate manual page that clearly documents
// how it differs from the standard version.
//
// d) make other distribution arrangements with the Copyright Holder.
//
// The name of the Copyright Holder may not be used to endorse or
// promote products derived from this Package without specific prior
// written permission.
//
// THE PACKAGE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// PACKAGE OR THE USE OR OTHER DEALINGS IN THE PACKAGE.
{$I RAK.INC}

interface

uses Classes, SysUtils, RakDestroy, MiscFunc;

type
  TRakCollection = class;
  TRakBaumZweig = class;
  TRakItemMoveMode = (mmAuto, mmBefore, mmAfter, mmChild);
  IRakCollection = interface;

  IRakCollectionItem = interface(IRakOwned)
    procedure IAssign(Source: IRakCollectionItem);
    function ICompare(WithItem: IRakCollectionItem): integer;
  end;

  IRakCollection = interface(IRakOwner)
  //public
    function IAddItem(NewItem: IRakCollectionItem): boolean;
    function IDeleteItem(DelItem: IRakCollectionItem): boolean;
    function IDeleteAt(Idx: integer): IRakCollectionItem;
    function IFreeItem(DelItem: IRakCollectionItem): boolean;
    function FreeAt(Idx: integer): boolean;

    function IIndexOfItem(Item: IRakCollectionItem): integer;
  end;

  {****************************************************************************
  <class name="TRakCollectionItem" superclass="TPersistent" public="true" defaultproperty="">
    <description>
      TRakCollectionItem ist die Basisklasse fr alle Objekte, die in einen
      Kontainer der Klasse TRakCollection aufgenommen werden knnen.<br>
      TRakCollectionItem und TRakCollection haben die gleiche Funktionsweise
      wie die Delphi-Klassen TCollectionItem und TCollection, mit dem Unterschied,
      dass die Rak-Klassen einiges mehr knnen.
    </description>

    <variable name="FCollection" type="TRakCollection" access="protected" isClassVariable="false">
      <description>
        Die <link class="TRakCollection">Collection</link>,die der Eigentmer
        dieses Items ist.
      </description>
    </variable>

    <variable name="FIgnoreChange" type="integer" access="protected" isClassVariable="false">
      <description>
        In FIgnoreChange wird mitgezhlt, wie oft das Property IgnoreChange
        gesetzt und wieder gelscht wurde.
      </description>
    </variable>

    <property name="Collection" type="TRakCollection " access="public" isArray="false" range="">
      <description>
        Die <link class="TRakCollection">Collection</link>,die der Eigentmer
        dieses Items ist.
      </description>
    </property>

    <property name="IgnoreChange" type="boolean" access="public" isArray="false" range="">
      <description>
        IgnoreChange bestimmt, ob die Collection benachrichtigt wird,
        wenn sich die Daten des Items gendert haben. Wenn abgeleitete Klassen
        ihre Daten ndern, dann sollten diese danach die Methode Changed
        aufrufen.<br>
        IgnoreChange kann fters auf true gesetzt werden. Erst wenn es genauso
        oft auf false gesetzt worden ist, nimmt es tatschlich wieder den Wert
        false an.
      </description>
    </property>

    <property name="Index" type="integer" access="public" isArray="false" range="">
      <description>
        Index bestimmt die Position des Items innerhalb der Collection.
        Das erste Item hat den Index-Wert 0.
        Durch Zuweisen eines Wertes kann man also die Position des Items
        verschieben.
      </description>
    </property>

    <method name="SetCollection(Value: TRakCollection)">
      <shortdescription>
        SetCollection ist die write-Methode des Properties Collection.<br>
        SetCollection sorgt dafr, dass die Collection, zu der das Item
        vorher gehrt hat, wei, dass es nicht mehr Besitzer des Items ist.
        Genauso wird die neue Collection informiert.<br>
        Diese Methode kann von abgleiteten Klassen berschrieben werden,
        wenn noch weitere Aktionen durchgefhrt werden mssen beim
        Umhngen eines Items.
      </shortdescription>
    </method>

    <method name="GetIgnoreChange">
      <shortdescription>
        read-Methode des Properties IgnoreChange.<br>
      </shortdescription>
    </method>

    <method name="SetIgnoreChange(Value: boolean)">
      <shortdescription>
        write-Methode des Properties IgnoreChange.<br>
      </shortdescription>
    </method>

    <method name="GetIndex">
      <shortdescription>
        read-Methode des Properties Index.<br>
      </shortdescription>
    </method>

    <method name="SetIndex(Value: integer)">
      <shortdescription>
        write-Methode des Properties Index.<br>
        Diese Methode kann von abgleiteten Klassen berschrieben werden,
        wenn noch weitere Aktionen durchgefhrt werden mssen beim
        ndern der Position eines Items.<br>
        ACHTUNG: SetIndex fhrt eine Verschiebung eines Items durch.
        Damit das Item auf eine andere Position verschoben werden kann, muss
        mind. ein weiteres Item seine Position ndern. Bei den Items, die Platz
        machen mssen, wird nicht dessen SetIndex aufgerufen. Es wird aber nach
        dem Verschieben von allen Items, die die Position gewechselt haben,
        die Methode Moved aufgerufen. Wenn ein Item etwas machen soll,
        wenn es verschoben wurde, dann sollte man besser Moved berschreiben.<br>
        SetIndex knnte man z.B. dann berschreiben, wenn man unterbinden will,
        dass das Item eine bestimmte Position einnehmen kann und es statt dessen
        einen Platz vorher ansiedeln will.
      </shortdescription>
    </method>

    <method name="GetKomposition">
      <shortdescription>
        GetKomposition kann von abgeleiteten Klassen berschrieben werden,
        damit das Item sich einer "Komposition" anschlieen kann. Sobald
        ein Objekt einer Komposition gelscht wird, werden alle Objekte
        der Komposition gelscht.
      </shortdescription>
    </method>

    <method name="Moved(PreviousItem: TRakCollectionItem)">
      <shortdescription>
        Wird aufgerufen, nachdem sich die Position des Items gendert hat.<br>
        PreviousItem ist das Item, das vorher an dieser Position stand.<br>
        siehe auch Methode <link class="TRakColletionItem" meth="SetIndex">SetIndex</link>.
      </shortdescription>
    </method>

    <method name="Assign(Source: TPersistent)">
      <shortdescription>
        Assign sorgt dafr, dass dieses Item die Daten von Source bernimmt.<br>
        Da TRakCollectionItem nur Zustandsdaten aber keine persistenten Daten
        hat, macht dessen Assign gar nichts. Abgeleitete Klassen mssen
        dafr sorgen, dass hier dessen persistenten Daten kopiert werden.
      </shortdescription>
    </method>

    <method name="Changed(Who: TRakCollectionItem)">
      <shortdescription>
        Changed sollte immer dann aufgerufen werden, wenn sich was gendert
        hat. Who ist das Item, bei dem sich etwas gendert hat.<br>
        Changed von TRakCollectionItem macht nicht mehr als dass es die
        Changed-Methode seiner Collection aufruft, falls dies nicht durch das
        Property IgnoreChange unterbunden wird.
      </shortdescription>
    </method>

    <method name="Compare(WithItem: TRakCollectionItem)">
      <shortdescription>
        Compare sollte von abgeleiteten Klassen berschrieben werden und die
        persistenten Daten der beiden Items (self und WithItem) vergleichen.<br>
        Sollten die Daten gleich sein, dann muss 0 zurckgegeben werden.
        Sollte die Daten auf "kleiner" und "grer" vergleichbar sein, dann sollten
        man einen Wert kleiner 0 bei "kleiner" und ein Wert grer 0
        bei "grer" zurckgegeben.
      </shortdescription>
    </method>

    <method name="MoveTo(Item: TRakCollectionItem; Mode: TRakItemMoveMode)">
      <shortdescription>
        MoveTo bietet sehr vielseitige Mglichkeiten ein Item zu
        verschieben. Whrend man mit dem Property Index nur die Position
        innheralb seiner Collection verschieben kann, kann man mit MoveTo auch
        zwischen zwei Collectionen verschieben. Wenn man zwischen zwei
        Collectionen verschiebt, dann kann man ber den Eingangsparameter Mode
        ganz einfach die relative Lage zu einem bestimmten Item angeben.<br>
        MoveTo drfte besonders ntztlich sein, wenn man ein DragAndDrop in
        einem Baum anbieten will.<br> Man kann alle Verschiebungen auch ber
        die Properties Collection und Index vornehmen. Dann kann es aber sein,
        dass man viel Code schreiben muss, damit das Item wirklich auf
        der richtigen Position landet.<br>
        Der Eingangsparameter "Item" bestimmt grob, wohin dieses Item verschoben wird.
        Dieses Item landet in der Nhe von "Item". Wo genau, hngt davon ab,
        was an "Mode" bergeben wird.<br>
        mmAuto: Das Item kommt in die gleiche Collection wie "Item".
                Wenn es in einer anderen Collection als "Item" war oder
                hinter "Item" platziert war, dann wird es unmittelbar
                vor "Item" gestellt. Ansonsten wird es unmittelbar hinter
                "Item" gestellt.<br>
        mmBefore: Das Item kommt in die gleiche Collection wie "Item".
                  Es wird unmittelbar vor "Item" gestellt.<br>
        mmAfter: Das Item kommt in die gleiche Collection wie "Item".
                  Es wird unmittelbar hinter "Item" gestellt.<br>
        mmChild: "Item" wird zur Collection von diesem Item, wobei es an die
                  letzte Stelle gestellt wird. Das funktioniert natrlich nur,
                  wenn "Item" vom Typ TRakCollection ist. Sollte dies nicht
                  der Fall sein, dann wird nicht verschoben.<br>
        <br>
        Solte "Item" = nil sein, dann wird es bei mmBefore an die erste
        Stelle und ansonsten an letzte Stelle verschoben.
      </shortdescription>
    </method>
  </class>
  ****************************************************************************}
  TRakCollectionItem = class(TRakInterfacedObject, IRakCollectionItem)
  public // IRakCollectionItem
    procedure IAssign(Source: IRakCollectionItem);
    function ICompare(WithItem: IRakCollectionItem): integer;

    function IGetOwner: IRakOwner;
    procedure ISetOwner(Value: IRakOwner);
  protected
    FCollection: TRakCollection;
    FIgnoreChange: integer;

    procedure SetCollection(Value: TRakCollection); virtual;
    function GetIgnoreChange: boolean; virtual;
    procedure SetIgnoreChange(Value: boolean); virtual;
    function GetIndex: integer; virtual;
    procedure SetIndex(Value: integer); virtual;

    function GetKomposition: IRakKomposition; virtual;

    procedure Moved(PreviousItem: TRakCollectionItem); virtual;

    property Collection: TRakCollection read FCollection write SetCollection;
  public
    constructor Create; virtual;
    destructor Destroy; override;

    procedure Assign(Source: TRakCollectionItem); overload; virtual;
    procedure Changed(Who: TRakCollectionItem); virtual;
    function Compare(WithItem: TRakCollectionItem): integer; virtual;
    procedure MoveTo(Item: TRakCollectionItem;
      Mode: TRakItemMoveMode);

    property IgnoreChange: boolean read GetIgnoreChange write SetIgnoreChange;
    property Index: integer read GetIndex write SetIndex;
  end;

  TRakCollectionItemClass = class of TRakCollectionItem;

TRakCollectionStates = (csFreeAll, csAssign, csAdd, csSetItem,
  csInitialising, csRestoring, csSaving, csDestroying,
  csElse0, csElse1, csElse2, csElse3, csElse4, csElse5, csElse6, csElse7,
  csElse8, csElse9);
TRakCollectionState = set of TRakCollectionStates;
TRakCompareItemsEvent = procedure(Sender: TObject;
  Item1, Item2: TRakCollectionItem; var CompResult: integer) of object;
TRakColMovedEvent = procedure(Sender: TObject;
  MovedItem, PreviousItem: TRakCollectionItem) of object;

  {****************************************************************************
  <class name="TRakCollection" superclass="TRakCollectionItem" public="true" defaultproperty="Items">
    <description>
      TRakCollection ist eine Kontainerklasse, die Instanzen von TRakCollectionItem
      in sich aufnehmen kann. TRakCollection ist dann der Besitzer dieser Objekte.
      Da TRakCollection von TRakCollectionItem abgeleitet ist, kann TRakCollection
      also auch Instanzen von sich selbst aufnehmen. Somit kann eine Baumstruktur
      aufgebaut werden.<br>
    </description>

    <variable name="FList" type="TList" access="protected" isClassVariable="false">
      <description>
        FList hlt die Zeiger auf die Elemente.
      </description>
    </variable>

    <variable name="FSorted" type="boolean" access="protected" isClassVariable="false">
      <description>
        FSorted legt fest, ob die Elemente sortiert sind.
      </description>
    </variable>

    <variable name="FSorted" type="boolean" access="protected" isClassVariable="false">
      <description>
        FSorted legt fest, ob die Elemente sortiert sind.
      </description>
    </variable>

    <property name="Count" type="integer" access="public" isArray="false" range="">
      <description>
        Die Anzahl der Elemente, die der Kontainer anthlt
      </description>
    </property>

    <property name="Items" type="TRakCollectionItem" access="public" isArray="true" range="0..Count-1">
      <description>
        Die einzelnen Elemente. Auf das erste Element kann mit den Index 0
        zugegriffen werden.
      </description>
    </property>

    <property name="LastItem" type="TRakCollectionItem" access="public" isArray="false" range="">
      <description>
        Das letzte Element. Die entspricht dem Property Items[Count-1]. Sollte
        es keine Elemente geben, dann wird nil zurckgegeben.
      </description>
    </property>

    <property name="Sorted" type="boolean" access="public" isArray="false" range="">
      <description>
        Legt fest, ob die Elemente sortiert werden.
      </description>
    </property>

    <property name="State" type="TRakCollectionState" access="public" isArray="false" range="">
      <description>
        Gibt den Status des Kontainers zurck.
      </description>
    </property>

    <event name="OnCompareItems" type="TRakCompareItemsEvent" access="public">
      <description>
        OnCompareItems wird beim Sortieren der Elemente ausgelst und ermglicht
        es den Benutzer des Kontainers selbst festzulgen, wie sortiert wird.
      </event>
    </property>

    <event name="OnMoved" type="TRakColMovedEvent" access="public">
      <description>
        OnMoved wird fr jedes Element aufgerufen, dass seinen Index verndert hat.
      </event>
    </property>

    <method name="RemoveItem(Item: TRakCollectionItem)">
      <shortdescription>
        RemoveItem nimmt das Element "Item" aus der Liste heraus. Das Element
        wird hier nicht gelscht. Bevor es aus der Liste herausgenommen wird,
        wird die Methode CanDeleteItem aufgerufen. Dadurch knnen abgeleitete
        Klassen verhindern, dass das Element entfernt wird. Nach dem Entfernen
        wird die Methode ItemDeleted aufgerufen.
      </shortdescription>
    </method>

    <method name="StateAdd(Status: TRakCollectionStates)">
      <shortdescription>
        Mit StateAdd kann ein weiterer Status gesetzt werden.
      </shortdescription>
    </method>

    <method name="CanAddItem(NewItem: TRakCollectionItem)">
      <shortdescription>
        CanAddItem wird aufgerufen bevor "NewItem" als neues Element aufgenommen
        wird. Hier knnen abgeleitete Klassen verhindern, dass bestimmte Elemente
        aufgenommen werden knnen.
      </shortdescription>
    </method>

    <method name="CanDeleteItem(Item: TRakCollectionItem)">
      <shortdescription>
        CanDeleteItem wird aufgerufen bevor "Item" aus der Liste der Elemente
        herausgenommen wird.
      </shortdescription>
    </method>

    <method name="ItemAdded(NewItem: TRakCollectionItem)">
      <shortdescription>
        ItemAdded wird aufgerufen nachdem ein neues Element hinzugekommen ist.
      </shortdescription>
    </method>

    <method name="ItemDeleted(DelItem: TRakCollectionItem)">
      <shortdescription>
        ItemDeleted wird aufgerufen nachdem ein Element aus der Liste
        entfernt wurde.
      </shortdescription>
    </method>

    <method name="AppendItem">
      <shortdescription>
        AppendItem wird aufgerufen, wenn durch das Setzen des Properties Count
        sich die Anzahl der Elemente vergrern soll. Fr jedes neue Element,
        das hinzukommen soll wird AppendItem einmal aufgerufen.<br>
        Defaultmig fgt AppendItem ein nil-Element dazu.
      </shortdescription>
    </method>

    <method name="BasicAdd(TheClass: TRakCollectionItemClass)">
      <shortdescription>
        BasicAdd ist die Methode, mit der ein neues Element erzeugt werden
        sollte. TheClass ist die Klasse des neu zu erzeugenden Elements.
        BasicAdd erzeugt eine Instanz, trgt sie in die Liste ein und gibt eine
        Referenz auf die Instanz zurck.<br>
        Whrend BasicAdd abgearbeitet wird, ist der Status "csAdd" gesetzt.
      </shortdescription>
    </method>

    <method name="DoAssign(Source: TRakCollection)">
      <shortdescription>
        DoAssign ist die Methode, die abgeleitete Klassen berschreiben sollten,
        um die persitenten Daten einer anderen Collection zu bernehmen.
      </shortdescription>
    </method>

    <method name="DoDestroy">
      <shortdescription>
        DoDestroy sollte an Stelle von Destroy berschrieben werden, um
        die Aufrumarbeiten durchzufhren.
      </shortdescription>
    </method>

    <method name="CompareItems(Item1, Item2: Pointer)">
      <shortdescription>
        CompareItems wird immer wieder aufgerufen, wenn sich die Collection
        sortiert. CompareItems sollte einen Wert kleiner als 0 zurckgeben, wenn
        Item1 kleiner als Item2 ist, 0, wenn Item1 gleich Item2 ist, grer 0,
        wenn Item1 grer als 0 ist.
      </shortdescription>
    </method>

    <method name="FindTreeItem">
      <shortdescription>
        Mit FindTreeItem kann man den Baum, in der diese Collection ein Zweig
        ist, nach einem bestimmten Zweig suchen lassen.<br>
        Die Methode IsSearched legt fest, welcher Zweig der gesuchte ist.
      </shortdescription>
    </method>

    <method name="GetAnker">
      <shortdescription>
        GetAnker liefert den Wurzelzweig des Baumes zurck, in dem diese
        Collection ein Zweig ist.
      </shortdescription>
    </method>

    <method name="GetCount">
      <shortdescription>
        die read-Meghode des Properties Count.
      </shortdescription>
    </method>

    <method name="SetCount(Value: integer)">
      <shortdescription>
        die write-Meghode des Properties Count.
      </shortdescription>
    </method>

    <method name="GetItems(Index: Integer)">
      <shortdescription>
        die read-Meghode des Properties Items.
      </shortdescription>
    </method>

    <method name="SetItems(Index: Integer; Value: TRakCollectionItem)">
      <shortdescription>
        die write-Meghode des Properties Items.
      </shortdescription>
    </method>

    <method name="IsSearched(Item: TRakCollection)">
      <shortdescription>
        IsSearched muss von abgeleiteten Klassen berschrieben werden, wenn diese
        den Aufruf der Methode FindTreeItem untersttzen mchten.
      </shortdescription>
    </method>

    <method name="SetSorted(Value: boolean)">
      <shortdescription>
        write-Methode des Properties Sorted
      </shortdescription>
    </method>

    <method name="Add(TheClass: TRakCollectionItemClass)">
      <shortdescription>
        Mit Add kann dem Kontainer ein ganz bestimmte Sorte von Element
        hinzugefgt werden.
      </shortdescription>
    </method>

    <method name="Add(TheClass: TRakCollectionItemClass)">
      <shortdescription>
        Mit AddItem kann dem Kontainer ein Element hinzugefgt werden, das
        bereits auerhalb des Kontainers existiert.
      </shortdescription>
    </method>

    <method name="AddNil">
      <shortdescription>
        Mit AddNil kann dem Kontainer ein nil-Element hinzugefgt werden.
      </shortdescription>
    </method>

    <method name="Assign(Source: TPersistent)">
      <shortdescription>
        Mit Assign kann ein Kontainer zu der Kopie eines anderen Kontainers
        gemacht werden.
      </shortdescription>
    </method>

    <method name="CheckIdx(Idx: integer)">
      <shortdescription>
        Mit CheckIdx kann geprft werden, ob Idx ein gltiger Index fr
        das Property Items ist.
      </shortdescription>
    </method>

    <method name="CorrectIdx(var Idx: integer)">
      <shortdescription>
        Mit CorrectIdx kann dafr gesorgt werden werden, dass Idx
        ein gltiger Index fr das Property Items ist, ausser es gibt
        keine Elemente. Dann wird Idx zu -1.
      </shortdescription>
    </method>

    <method name="DeleteAt(Idx: integer)">
      <shortdescription>
        Mit DeleteAt kann man ein Element aus einer Collection herausnehmen,
        ohne dass das Element zerstrt wird.
      </shortdescription>
    </method>

    <method name="FreeAll">
      <shortdescription>
        Mit FreeAll kann man alle Element der Collection zerstren.
      </shortdescription>
    </method>

    <method name="MoveBlockTo(BlockStart, BlockEnd, Dest: integer; Mode: TRakItemMoveMode)">
      <shortdescription>
        Mit MoveBlockTo kann man gleich mehrere Elemente auf einmal verschieben.
        Wenn man nur ein Element verschieben will, macht man das am
        einfachsten, indem man dem Index-Property des Elements einen neuen
        Wert zuweist. Wenn man mehrere Elemente auf einmal verschieben will,
        ist der Aufruf von MoveBlockTo effizienter als jedes Element einzeln zu
        verschieben.<br>
        BlockStart und BlockEnd legen fest, welche Elemente verschoben werden.
        Dest legt grob fest, wohin die die Items verschoben werden. Mode
        legt genau fest, wohin die Items verschoben werden.<br>
        mmAuto,mmChild: Wenn das erste Element hinter Dest liegt, dann
                        wird der Block unmittelbar vor Dest platziert.
                        Wenn das letzte Element vor Dest liegt, dann
                        wird der Block unmittelbar hinter Dest platziert.
                        Sonst wird nichts verschoben.
        mmBefore: Wenn das erste Element hinter Dest liegt, dann
                  wird der Block unmittelbar vor Dest platziert.
                  Sonst wird nichts verschoben.
        mmAfter:  Wenn das letzte Element vor Dest liegt, dann
                  wird der Block unmittelbar hinter Dest platziert.
                  Sonst wird nichts verschoben.
      </shortdescription>
    </method>

    <method name="Pack">
      <shortdescription>
        Mit Pack knnen alle nil-Elemente entfernt werden.
      </shortdescription>
    </method>

    <method name="Sort">
      <shortdescription>
        Mit Sort kann eine Sortierung der Elemente auch durchgefhrt werden,
        selbst wenn das Property Sorted = false.
      </shortdescription>
    </method>
  </class>
  ****************************************************************************}
  TRakCollection = class(TRakCollectionItem, IRakCollection)
  private
    FState: TRakCollectionState;

    FOnCompareItems: TRakCompareItemsEvent;
    FOnMoved: TRakColMovedEvent;
    function GetLastItem: TRakCollectionItem;

    procedure IPlugOn(NewItem: IRakOwned);
    procedure IPlugOff(DelItem: IRakOwned);

    function IAddItem(NewItem: IRakCollectionItem): boolean;

    function IDeleteItem(DelItem: IRakCollectionItem): boolean;
    function IDeleteAt(Idx: integer): IRakCollectionItem;

    function IFreeItem(DelItem: IRakCollectionItem): boolean;

    function IIndexOfItem(Item: IRakCollectionItem): integer;

    function IGetItem(Idx: integer): IRakOwned;
  protected
    FList: TList;
    FSorted: boolean;

    procedure DoDestroy; virtual;

    procedure StateAdd(Status: TRakCollectionStates); virtual;
    procedure StateDelete(Status: TRakCollectionStates); virtual;

    function BasicAdd(TheClass: TRakCollectionItemClass): TRakCollectionItem;
    procedure AddItem(NewItem: TRakCollectionItem; CanAvoid: boolean); overload;
    procedure RemoveItem(Item: TRakCollectionItem; CanAvoid: boolean); overload;
    procedure RemoveItem(Idx: integer; CanAvoid: boolean); overload;

{$IFDEF VORDELPHI2005}
    function CanAddItem(TheClass: TRakCollectionItemClass): boolean; overload; virtual;
{$ELSE}
    function CanAddItemClass(TheClass: TRakCollectionItemClass): boolean; overload; virtual;
{$ENDIF}
    function CanAddItem(NewItem: TRakCollectionItem): boolean; overload; virtual;
    function CanDeleteItem(Item: TRakCollectionItem): boolean; virtual;

    procedure ItemAdd(NewItem: TRakCollectionItem); virtual;
    procedure ItemAdded(NewItem: TRakCollectionItem); virtual;
    procedure ItemDelete(DelItem: TRakCollectionItem); virtual;
    procedure ItemDeleted(DelItem: TRakCollectionItem; OldIdx: integer); virtual;

    procedure AppendItem; virtual;
    procedure DoAssign(Source: TRakCollection); virtual;
    function CompareItems(Item1, Item2: Pointer): integer; virtual;
    function FindTreeItem: TRakCollection;
    function GetAnker: TRakCollection;
    function GetCount: integer; virtual;
    procedure SetCount(Value: integer); virtual;
    function GetItems(Index: Integer): TRakCollectionItem;
    procedure SetItems(Index: Integer; Value: TRakCollectionItem);
    function IsSearched(Item: TRakCollection): boolean; virtual;
    procedure SetSorted(Value: boolean); virtual;
  public
    constructor Create; override;
    destructor Destroy; override;

    function Add(TheClass: TRakCollectionItemClass): TRakCollectionItem;
    procedure AddItem(NewItem: TRakCollectionItem); overload; virtual;
    procedure AddNil; virtual;
    function IsBranchNode(Node: TRakCollectionItem): boolean; virtual;
    function CreateItem(TheClass: TRakCollectionItemClass): TRakCollectionItem; virtual;
    procedure AddCollection(Col: TRakCollection); virtual;
    procedure Assign(Source: TRakCollectionItem); override;
    function CheckIdx(Idx: integer): boolean; virtual;
    procedure CorrectIdx(var Idx: integer); virtual;
    function DeleteAt(Idx: integer): TRakCollectionItem; 
    function FreeAt(Idx: integer): boolean; virtual;
    procedure FreeAll; virtual;
    procedure MoveBlockTo(BlockStart, BlockEnd, Dest: integer;
      Mode: TRakItemMoveMode); virtual;
    procedure Pack; virtual;
    procedure Sort; virtual;

    property Count: integer read GetCount write SetCount;
    property Items[Index: Integer]: TRakCollectionItem read GetItems
      write SetItems; default;
    property LastItem: TRakCollectionItem read GetLastItem;
    property Sorted: boolean read FSorted write SetSorted;
    property State: TRakCollectionState read FState;

    property OnCompareItems: TRakCompareItemsEvent read FOnCompareItems
      write FOnCompareItems;
    property OnMoved: TRakColMovedEvent read FOnMoved write FOnMoved;
  end;


{*******************************************************************************
** Klasse       : TRakKompositionItem
** Autor        : Richard Kaspar
** Beschreibung :
*******************************************************************************}
  TRakKompositionItem = class(TRakCollectionItem)
  private
    FKomposition: IRakKomposition;
    procedure SetKomposition(const Value: IRakKomposition);
  protected
    function GetKomposition: IRakKomposition; override;
  public
    property Komposition: IRakKomposition read GetKomposition write SetKomposition;
  end;

{*******************************************************************************
** Klasse       : TRakChildCollection
** Autor        : Richard Kaspar
** Beschreibung :
*******************************************************************************}
  TRakChildCollection = class(TRakCollection, IRakDestroyOwner)
  private
    FDestroyOwner: TRakDestroyOwner;
  protected
    property DestroyOwner: TRakDestroyOwner read FDestroyOwner implements IRakDestroyOwner;
  public
    constructor Create; override;
    destructor Destroy; override;
  end;

{*******************************************************************************
** Klasse       : TRakChildCollectionItem
** Autor        : Richard Kaspar
** Beschreibung :
*******************************************************************************}
  TRakChildCollectionItem = class(TRakCollectionItem, IRakDestroyOwner)
  private
    FDestroyOwner: TRakDestroyOwner;
  protected
    procedure DoDestroy; virtual;
    property DestroyOwner: TRakDestroyOwner read FDestroyOwner implements IRakDestroyOwner;
  public
    constructor Create; override;
    destructor Destroy; override;
  end;

  {****************************************************************************
  <class name="TRakIntegerItem" superclass="TRakCollectionItem" public="true" defaultproperty="">
    <description>
      TRakIntegerItem wird von der Klasse TRakIntegerCollection verwendet.
    </description>

    <property name="Wert" type="integer" access="public" isArray="false" range="">
      <description>
        Der Integer-Wert.
      </description>
    </property>
  </class>
  ****************************************************************************}
  TRakIntegerItem = class(TRakCollectionItem)
  protected
    FWert: integer;

    procedure SetWert(Value: integer); virtual;
  public
    procedure Assign(Source: TRakCollectionItem); override;

    property Wert: integer read FWert write SetWert;
  end;

  {****************************************************************************
  <class name="TRakIntegerCollection" superclass="TRakCollection" public="true" defaultproperty="">
    <description>
      TRakIntegerCollection kann als eine Art dynamisches Array fr Integerwerte
      verwendet werden. Seit Delphi aber dynamische Arrays untersttzt drfte
      diese Klasse relativ berfssig sein.
    </description>
  </class>
  ****************************************************************************}
  TRakIntegerCollection = class(TRakCollection)
  private
    function GetItems(Index: Integer): integer;
    procedure SetItems(Index: Integer; Value: integer);
  protected
    procedure AppendItem; override;
    function CompareItems(Item1, Item2: Pointer): integer; override;
  public
    procedure Add(Value: integer);
    procedure FreeItem(Index: integer);

    function IndexOf(Zahl: integer): integer;

    procedure Move(FromIndex, ToIndex: integer);

    property Items[Index: Integer]: integer read GetItems
      write SetItems; default;
  end;


{*******************************************************************************
** Klasse       : TRakIDPaarItem
** Autor        : Richard Kaspar
** Beschreibung :
*******************************************************************************}
  TRakIDPaarItem = class(TRakCollectionItem)
  private
    FID1: integer;
    FID2: integer;
    procedure SetID1(const Value: integer);
    procedure SetID2(const Value: integer);
  public
    procedure Assign(Source: TRakCollectionItem); override;

    property ID1: integer read FID1 write SetID1;
    property ID2: integer read FID2 write SetID2;
  end;

{*******************************************************************************
** Klasse       : TRakIDPaarCol
** Autor        : Richard Kaspar
** Beschreibung :
*******************************************************************************}
  TRakIDPaarCol = class(TRakCollection)
  private
    function GetItems(Idx: Integer): TRakIDPaarItem;
  public
    function Add: TRakIDPaarItem;

    property Items[Idx: Integer]: TRakIDPaarItem read GetItems; default;
  end;

{*******************************************************************************
** Klasse       : TRakObjectPaarItem
** Autor        : Richard Kaspar
** Beschreibung :
*******************************************************************************}
  TRakObjectPaarItem = class(TRakCollectionItem)
  private
    FObject1: TObject;
    FObject2: TObject;
    procedure SetObject1(const Value: TObject);
    procedure SetObject2(const Value: TObject);
  public
    procedure Assign(Source: TRakCollectionItem); override;

    property Object1: TObject read FObject1 write SetObject1;
    property Object2: TObject read FObject2 write SetObject2;
  end;

{*******************************************************************************
** Klasse       : TRakObjectPaarCol
** Autor        : Richard Kaspar
** Beschreibung :
*******************************************************************************}
  TRakObjectPaarCol = class(TRakCollection)
  private
    function GetItems(Idx: Integer): TRakObjectPaarItem;
  public
    function Add: TRakObjectPaarItem; overload;
    function Add(Obj1, Obj2: TObject): TRakObjectPaarItem; overload;

    property Items[Idx: Integer]: TRakObjectPaarItem read GetItems; default;
  end;


  {****************************************************************************
  <class name="TRakBaumBlaetter" superclass="TRakCollection" public="true" defaultproperty="Items">
    <description>
      TRakBaumBlaetter wird von TRakBaumZweig verwendet, um die
      unmittelbaren Bltter des Baumes zu verwalten.<br>
      Als Bltter knnen abgeleitete Klassen von TRakCollectionItem verwendet werden.
      Diese Klassen sollten die Methode Compare berschreiben.
    </description>

    <variable name="FZweig" type="TRakBaumZweig" access="protected" isClassVariable="false">
      <description>
        Der Zweig dessen unmittelbaren Bltter TRakBaumBlaetter verwaltet.
      </description>
    </variable>

    <property name="Zweig" type="TRakBaumZweig" access="public" isArray="true" range="0..Count-1">
      <description>
        Der Zweig dessen unmittelbaren Bltter TRakBaumBlaetter verwaltet.
      </description>
    </property>

    <method name="CanAddItem(NewItem: TRakCollectionItem)">
      <shortdescription>
        CanAddItem unterbindet, dass nil-Bltter aufgenommen werden
        und fragt seinen Zweig, ob das Blatt aufgenommen werden darf.
      </shortdescription>
    </method>

    <method name="ItemAdded(Item: TRakCollectionItem)">
      <shortdescription>
        ItemAdded gibt dem Zweig bescheid, dass ein neues Blatt hinzugekommen ist
      </shortdescription>
    </method>

    <method name="ItemDeleted(Item: TRakCollectionItem)">
      <shortdescription>
        ItemDeleted gibt dem Zweig bescheid, dass ein Blatt gelscht wurde
      </shortdescription>
    </method>

    <method name="ExistsEqualBlatt(Blatt: TRakCollectionItem)">
      <shortdescription>
        Mit ExistsEqualBlatt kann man feststellen, ob es mindestens ein Blatt
        gibt, dass die gleichen Daten speichert.<br>
        ACHTUNG: Als Bltter knnen nur Klassen verwendet werden, die von
        TRakCollectionItem abgeleitet sind. Diese Klassen mssen die Methode
        Compare so berschreiben, dass sie berprft, ob zwei Bltter
        die gleichen Daten speichern. Wenn man ExistsEqualBlatt nicht
        von Hand aufruft, dann muss man das allerdings nur dann wirklich
        gemachen, wenn die Funktion DoppelteBlaetter von TRakBaumZweig
        so berschrieben wurde, dass sie true zurckgibt.
      </shortdescription>
    </method>
  </class>
  ****************************************************************************}
  TRakBaumBlaetter = class(TRakCollection)
  protected
    FZweig: TRakBaumZweig;

    function CanAddItem(NewItem: TRakCollectionItem): boolean; override;
    procedure ItemAdded(Item: TRakCollectionItem); override;
    procedure ItemDeleted(Item: TRakCollectionItem; OldIdx: integer); override;
  public
    destructor Destroy; override;

    function ExistsEqualBlatt(Blatt: TRakCollectionItem): boolean; virtual;

    property Zweig: TRakBaumZweig read FZweig;
  end;

  {****************************************************************************
  <class name="TRakBaumZweig" superclass="TRakCollection" public="true" defaultproperty="Zweige">
    <description>
      TRakBaumZweig ist eine abstrakte Klasse.<br>
      TRakBaumZweig stellt einen Zweig eines Baumes dar. Falls es der
      Wurzelzweig ist, ist der Zweig praktisch der komplette Baum.<br>
      Ein Zweig kann Unterzweige haben und gleichzeitig auch unmittelbare Bltter.
      Ein Zweig sollte eigentlich nie ein Blatt sein, also weder Unterzweige noch
      eigene Bltter haben, es wird aber nicht erzwungen.<br>
      Um auf die Unterzweige zuzugreifen, knnen die Properties Count und Zweige
      verwendet werden. Mit dem Property Blaetter kann man auf die unmittelbaren
      Bltter zugreifen. Unmittelbare Bltter sind die eigenen Bltter und nicht
      etwa auch die Bltter der Unterzweige. Fr die Verwaltung der Bltter wird
      eine Instanz von TRakBaumBlaetter oder einer davon abgeleiteten Klasse
      verwendet.
    </description>

    <variable name="FBlaetter" type="TRakBaumBlaetter" access="protected" isClassVariable="false">
      <description>
        Verwaltet die unmittelbaren Bltter des Zweiges.
      </description>
    </variable>

    <variable name="FBlaetterList" type="TList" access="protected" isClassVariable="false">
      <description>
        FBlaetterList enthlt Referenzen auf alle Bltter des Zweiges, auch
        auf die Bltter der Unterzweige.
      </description>
    </variable>

    <variable name="FOrderList" type="TList" access="protected" isClassVariable="false">
      <description>
        FOrderList hlt Referenzen auf alle unmittelbaren Bltter und alle
        unmittelbaren Unterzweige des Zweiges.
      </description>
    </variable>

    <property name="Blaetter" type="TRakBaumBlaetter" access="public" isArray="false" range="">
      <description>
        Blaetter ist der Kontainer fr alle unmittelbaren Bltter des Zweiges
      </description>
    </property>

    <property name="OrderCount" type="integer" access="public" isArray="false" range="">
      <description>
        OrderCount ist die Summe aller unmittelbaren Bltter und aller
        unmittelbaren Unterzweige und gibt an, wieviele Elemente ber
        das Property OrderItems erreichbar sind.<br>
        OrderCount knnte man sich selbst errechnen: OrderCount = Count + Blaetter.Count
      </description>
    </property>

    <property name="OrderItems" type="TRakCollectionItem" access="public" isArray="true" range="0..OrderCount-1">
      <description>
        OrderItems liefert entweder die Referenz auf ein unmittelbares Blatt
        oder auf einen unmittelbaren Unterzweig zurck. Mit Hilfe der
        Order-Methoden kann die Reihenfolge bestimmt werden.<br>
        Mit Hilfe des Properties "Index" eines Unterzweiges, kann man
        die Reihenfolge des Properties "Zeige" verndern. Dies hat allerdings
        keine Auswirkung auf die Reihenfolge von OrderItems. Das gleiche
        gilt fr das Index-Property der Bltter.
      </description>
    </property>

    <property name="Parent" type="TRakBaumZweig" access="public" isArray="false" range="">
      <description>
        Parent ist der Zweig, von dem dieser Zweig ein Unterzweig ist.
        Falls Zweig der Wurzelzweig ist, dann liefert Parent nil zurck.
      </description>
    </property>

    <property name="Zweige" type="TRakBaumZweig" access="public" isArray="true" range="0..Count-1">
      <description>
        Die Unterzweige des Zweigs. Das Property Count gibt an, wieviele
        Unterzweige es gibt.
      </description>
    </property>

    <method name="BlattAdded(Blatt: TRakCollectionItem)">
      <shortdescription>
        BlattAdded wird aufgerufen, wenn der Zweig oder einer seiner
        Unterzweige ein neues Blatt bekommen hat.
      </shortdescription>
    </method>

    <method name="BlattDeleted(Blatt: TRakCollectionItem)">
      <shortdescription>
        BlattAdded wird aufgerufen, wenn der Zweig oder einer seiner
        Unterzweige ein Blatt verloren hat.
      </shortdescription>
    </method>

    <method name="CanAddBlatt(NewBlatt: TRakCollectionItem)">
      <shortdescription>
        CanAddBlatt verhindert, dass doppelte Bltter aufgenommen
        werden, wenn die Funktion DoppelteBlaetter false zurckgibt.
        Das Ganze funktioniert allerdings nur, wenn die Klasse fr die Bltter
        die Methode Compare von TRakCollctionItem berschrieben hat.
      </shortdescription>
    </method>

    <method name="CreateFBlaetter">
      <shortdescription>
        CreateFBlaetter ist abstrakt.<br>
        CreateFBlaetter soll das Objekt erzeugen, das fr die
        Verwaltung von den unmittelbaren Blttern des Zweiges zustndig ist.<br>
        Die Klasse fr dieses Objekt muss TRakBaumBlaetter oder eine davon
        abgeleitete sein.
      </shortdescription>
    </method>

    <method name="DoppelteBlaetter">
      <shortdescription>
        Die Funktion DoppelteBlaetter bestimmt, ob der Baum Bltter enthalten
        darf, die die gleichen Daten speichern.<br>
        DoppelteBlaetter gibt standardmig true zurck.
      </shortdescription>
    </method>

    <method name="ExistsEqualBlatt(Blatt: TRakCollectionItem)">
      <shortdescription>
        Mit ExistsEqualBlatt kann man feststellen, ob es mindestens ein Blatt
        in diesem oder einem seiner Unterzweige gibt, das die gleichen Daten
        speichert.<br>
        ACHTUNG: Als Bltter knnen nur Klassen verwendet werden, die von
        TRakCollectionItem abgeleitet sind. Diese Klassen mssen die Methode
        Compare so berschreiben, dass sie berprft, ob zwei Bltter
        die gleichen Daten speichern. Wenn man ExistsEqualBlatt nicht
        von Hand aufruft, dann muss man das allerdings nur dann wirklich
        machen, wenn die Funktion DoppelteBlaetter
        so berschrieben wurde, dass sie false zurckgibt.
      </shortdescription>
    </method>

    <method name="ItemAdded(NewItem: TRakCollectionItem)">
      <shortdescription>
        ItemAdded sorgt dafr, dass FOrderList korrekt gefllt ist.
      </shortdescription>
    </method>

    <method name="ItemDeleted(DelItem: TRakCollectionItem)">
      <shortdescription>
        ItemDeleted sorgt dafr, dass FOrderList korrekt gefllt ist.
      </shortdescription>
    </method>

    <method name="OrderExchange(OrderItem1, OrderItem2: TRakCollectionItem)">
      <shortdescription>
        OrderExchange tauscht die Position von zwei
        <link class="TRakBaumZweig" prop="OrderItems">OrderItems</link>.
      </shortdescription>
    </method>

    <method name="OrderExchange(Idx1, Idx2: integer)">
      <shortdescription>
        OrderExchange tauscht die Position von zwei
        <link class="TRakBaumZweig" prop="OrderItems">OrderItems</link>
        anhand ihrer derzeitigen Position. Erstes Element: Idx = 0
      </shortdescription>
    </method>

    <method name="OrderFirst">
      <shortdescription>
        OrderFirst gibt das erste
        <link class="TRakBaumZweig" prop="OrderItems">OrderItem</link>
        zurck. Wenn es berhaupt keine OrderItems gibt, dann wird nil
        zurckgegeben. Das Erste entspricht dem Property OrderItems[0].
      </shortdescription>
    </method>

    <method name="OrderIdx(OrderItem: TRakCollectionItem)">
      <shortdescription>
        OrderIdx gibt die Position des bergebenen
        <link class="TRakBaumZweig" prop="OrderItems">OrderItem</link>
        zurck. Wenn es das OrderItem nicht gibt, dann wird -1 zurckgegeben.
        Beim ersten OrderItem wird 0 zurckgegeben.
      </shortdescription>
    </method>

    <method name="OrderMove(ThisOrderItem, ToOrderItem: TRakCollectionItem)">
      <shortdescription>
        OrderMove verschiebt das
        <link class="TRakBaumZweig" prop="OrderItems">OrderItem</link>
        ThisOrderItem an die Position des OrderItems ToOrderItem. Ob
        ToOrderItem danach vor oder hinter ThisOrderItem steht, hngt davon ab,
        wie vorher die relative Position war.<br>
        Beispiele:<br>
        "Item0, ThisOrderItem, Item2, ToOrderItem" wird zu "Item0, Item2, ToOrderItem, ThisOrderItem"<br>
        "Item0, ToOrderItem, Item2, ThisOrderItem" wird zu "Item0, ThisOrderItem, ToOrderItem, Item2"<br>
      </shortdescription>
    </method>

    <method name="OrderMove(OrderItem: TRakCollectionItem; ToIdx: integer)">
      <shortdescription>
        siehe <link class="TRakBaumZweig" meth="OrderMove(ThisOrderItem, ToOrderItem: TRakCollectionItem)">
        OrderMove(ThisOrderItem, ToOrderItem: TRakCollectionItem)</link>
      </shortdescription>
    </method>

    <method name="OrderMove(FromIdx, ToIdx: integer)">
      <shortdescription>
        siehe <link class="TRakBaumZweig" meth="OrderMove(ThisOrderItem, ToOrderItem: TRakCollectionItem)">
        OrderMove(ThisOrderItem, ToOrderItem: TRakCollectionItem)</link>
      </shortdescription>
    </method>

    <method name="OrderLast">
      <shortdescription>
        OrderFirst gibt das letzte
        <link class="TRakBaumZweig" prop="OrderItems">OrderItem</link>
        zurck. Wenn es berhaupt keine OrderItems gibt, dann wird nil
        zurckgegeben. Das Letzte entspricht dem Property OrderItems[OrderCount-1].
      </shortdescription>
    </method>
  </class>
  ****************************************************************************}
  TRakBaumZweig = class(TRakCollection)
  private
    function GetZweige(Idx: integer): TRakBaumZweig;
    function GetParent: TRakBaumZweig;
    function GetOrderCount: integer;
    function GetOrderItems(Idx: integer): TRakCollectionItem;
  protected
    FBlaetter: TRakBaumBlaetter;
    FBlaetterList: TList;
    FOrderList: TList;

    procedure BlattAdded(Blatt: TRakCollectionItem); virtual;
    procedure BlattDeleted(Blatt: TRakCollectionItem); virtual;
    procedure OwnBlattAdded(Blatt: TRakCollectionItem); virtual;
    procedure OwnBlattDeleted(Blatt: TRakCollectionItem); virtual;
    function CanAddBlatt(NewBlatt: TRakCollectionItem): boolean; virtual;
    function CreateFBlaetter: TRakBaumBlaetter; virtual; abstract;
    procedure DoDestroy; override;
    function DoppelteBlaetter: boolean; virtual;
    function ExistsEqualBlatt(Blatt: TRakCollectionItem): boolean; virtual;
    procedure ItemAdded(NewItem: TRakCollectionItem); override;
    procedure ItemDeleted(DelItem: TRakCollectionItem; OldIdx: integer); override;
  public
    constructor Create; override;

    procedure OrderExchange(OrderItem1, OrderItem2: TRakCollectionItem); overload; virtual;
    procedure OrderExchange(Idx1, Idx2: integer); overload; virtual;
    function OrderFirst: TRakCollectionItem; virtual;
    function OrderIdx(OrderItem: TRakCollectionItem): integer; virtual;
    procedure OrderMove(ThisOrderItem, ToOrderItem: TRakCollectionItem); overload; virtual;
    procedure OrderMove(OrderItem: TRakCollectionItem; ToIdx: integer); overload; virtual;
    procedure OrderMove(FromIdx, ToIdx: integer); overload; virtual;
    function OrderLast: TRakCollectionItem; virtual;

    function IsOwnBlatt(Blatt: TRakCollectionItem): boolean;

    property Blaetter: TRakBaumBlaetter read FBlaetter;
    property OrderCount: integer read GetOrderCount;
    property OrderItems[Idx: integer]: TRakCollectionItem read GetOrderItems;
    property Parent: TRakBaumZweig read GetParent;
    property Zweige[Idx: integer]: TRakBaumZweig read GetZweige; default;
  end;


implementation

{ TRakCollectionItem }

constructor TRakCollectionItem.Create;
begin
  inherited Create;

  FCollection := nil;
  FIgnoreChange := 0;
end;

destructor TRakCollectionItem.Destroy;
var
  Komposition: IRakKomposition;
begin
  Komposition := GetKomposition;
  if Komposition <> nil then
    Komposition.MemberDestroying(self);

  Collection := nil;

  inherited Destroy;
end;

procedure TRakCollectionItem.Assign(Source: TRakCollectionItem);
begin
end;

procedure TRakCollectionItem.Changed(Who: TRakCollectionItem);
begin
  if not IgnoreChange and Assigned(FCollection) then
    FCollection.Changed(Who);
end;

procedure TRakCollectionItem.MoveTo(Item: TRakCollectionItem;
  Mode: TRakItemMoveMode);
begin
  if Item = nil then
  begin
    if  Collection <> nil then
    begin
      case Mode of
      mmBefore:
        Index := 0;
      mmAfter, mmAuto:
        Index := Collection.Count - 1;
      end;
    end;
  end
  else if Mode = mmChild then
  begin
    if Item is TRakCollection then
    begin
      Collection := TRakCollection(Item);
    end;
  end
  else if Item.Collection <> nil then
  begin
    if Item.Collection <> Collection then
    begin
      Collection := Item.Collection;
    end;

    if Item.Collection = Collection then
    begin
      case Mode of
      mmAuto:
        Index := Item.Index;
      mmBefore:
        if Index > Item.Index then
          Index := Item.Index
        else if Index < Item.Index then
          Index := Item.Index - 1;
      mmAfter:
        if Index > Item.Index then
          Index := Item.Index + 1
        else if Index < Item.Index then
          Index := Item.Index;
      end;
    end;
  end;
end;

function TRakCollectionItem.GetIgnoreChange: boolean;
begin
  Result := FIgnoreChange <> 0;
end;

procedure TRakCollectionItem.SetIgnoreChange(Value: boolean);
begin
  if Value then
  begin
    if FIgnoreChange = -1 then
      raise Exception.Create('FIgnoreChange-Puffer ist voll');

    inc(FIgnoreChange)
  end
  else if FIgnoreChange <> 0 then
    dec(FIgnoreChange);
end;

procedure TRakCollectionItem.SetCollection(Value: TRakCollection);
var
  CanAdd: boolean;
begin
  if Value <> FCollection then
  begin
    CanAdd := true;
    if (Value <> nil) and (Value.FList.IndexOf(self) = -1) then
      CanAdd := Value.CanAddItem(self);

    if CanAdd then
    begin
      if Assigned(FCollection) then
        FCollection.RemoveItem(self, false);

      FCollection := Value;
      
      if Value <> nil then
      begin
        if Value.FList.IndexOf(self) = -1 then
          Value.AddItem(self, false);
      end;
    end;
  end;
end;

function TRakCollectionItem.GetIndex: integer;
begin
  if Assigned(FCollection) then
  begin
    Result := FCollection.FList.IndexOf(self);
  end
  else
  begin
    Result := -1;
  end;
end;

procedure TRakCollectionItem.SetIndex(Value: integer);
var
  i: integer;
  MoveList: TRakObjectPaarCol;
begin
  if Assigned(FCollection) then
  begin // Wenn das Item berhaupt zu einer Collection gehrt
    i := Index;
    if i <> Value then
    begin // Wenn sich die Position wirklich verndert
      if (Value < 0) or (Value >= FCollection.Count) then
        raise Exception.Create('Ungltiger Index');

      if i >= 0 then
      begin // Wenn das Item in der Collection verzeichnet ist
        // In MoveList wird gespeichert, welches Item fr welches andere Item Platz machen musste.
        MoveList := TRakObjectPaarCol.Create;
        try
          MoveList.Add(self, TObject(FCollection.FList[Value]));
          if i < Value then
          begin // Wenn das Item nach hinten verschoben werden soll
            while i < Value do
            begin
              MoveList.Add(TObject(FCollection.FList[i+1]), TObject(FCollection.FList[i]));
              FCollection.FList[i] := FCollection.FList[i+1];
              inc(i);
            end;
            FCollection.FList[i] := self;
          end
          else if i > Value then
          begin // Wenn das Item nach vorne verschoben werden soll
            while i > Value do
            begin
              MoveList.Add(TObject(FCollection.FList[i-1]), TObject(FCollection.FList[i]));
              FCollection.FList[i] := FCollection.FList[i-1];
              dec(i);
            end;
            FCollection.FList[i] := self;
          end;

          // Jetzt den Items und externen Benutzern die Mglichkeit geben,
          // auf die Verschiebung zu reagieren
          for i := 0 to MoveList.Count - 1 do
            TRakCollectionItem(MoveList[i].Object1).Moved(TRakCollectionItem(MoveList[i].Object2));
        finally
          MoveList.Free;
        end;
      end
      else
      begin // Wenn das Item in der Collection nicht verzeichnet ist
        FCollection := nil;
        raise Exception.Create('Verbindung zwischen Collection und Item ist abgerissen');
      end;
    end;
  end;
end;

procedure TRakCollectionItem.Moved(PreviousItem: TRakCollectionItem);
begin
  Changed(self);
end;

function TRakCollectionItem.Compare(WithItem: TRakCollectionItem): integer;
begin
  Result := 0;
end;

procedure TRakCollectionItem.IAssign(Source: IRakCollectionItem);
var
  Obj: TObject;
begin
  Obj := Source.IGetObject;
  if Obj is TRakCollectionItem then
    Assign(TRakCollectionItem(Obj));
end;

function TRakCollectionItem.ICompare(
  WithItem: IRakCollectionItem): integer;
var
  Obj: TObject;
begin
  Result := 0;
  Obj := WithItem.IGetObject;
  if Obj is TRakCollectionItem then
    Result := Compare(TRakCollectionItem(WithItem));
end;

function TRakCollectionItem.IGetOwner: IRakOwner;
begin
  Result := IRakCollection(Collection);
end;

procedure TRakCollectionItem.ISetOwner(Value: IRakOwner);
var
  Obj: TObject;
begin
  Obj := Value.IGetObject;
  if Obj is TRakCollection then
    Collection := TRakCollection(Value.IGetObject);
end;

function TRakCollectionItem.GetKomposition: IRakKomposition;
begin
  Result := nil;
end;

{ TRakCollection }


constructor TRakCollection.Create;
begin
  inherited Create;

  FList := TList.Create;
  FState := [];
  FSorted := false;
end;

destructor TRakCollection.Destroy;
begin
  StateAdd(csDestroying);

  DoDestroy;

  inherited Destroy;
end;

procedure TRakCollection.DoDestroy;
begin
  FreeAll;
  FList.Free;
  FList := nil;
end;

function TRakCollection.GetLastItem: TRakCollectionItem;
begin
  if Count = 0 then
    Result := nil
  else
    Result := Items[Count - 1];
end;

procedure TRakCollection.StateAdd(Status: TRakCollectionStates);
begin
  FState := FState + [Status];
end;

procedure TRakCollection.StateDelete(Status: TRakCollectionStates);
begin
  FState := FState - [Status];
end;

procedure TRakCollection.AppendItem;
begin
  AddNil;
end;

function TRakCollection.BasicAdd(TheClass: TRakCollectionItemClass): TRakCollectionItem;
begin
  Result := nil;
{$IFDEF VORDELPHI2005}
  if CanAddItem(TheClass) then
{$ELSE}
  if CanAddItemClass(TheClass) then
{$ENDIF}
  begin
    Result := TheClass.Create;
    AddItem(Result, false);
  end;
end;

function TRakCollection.CompareItems(Item1, Item2: Pointer): integer;
begin
  if (Item1 = nil) and (Item2 = nil) then
    Result := 0
  else if Item1 = nil then
    Result := -1
  else if Item2 = nil then
    Result := 1
  else
    Result := TRakCollectionItem(Item1).Compare(TRakCollectionItem(Item2));
end;

function TRakCollection.FindTreeItem: TRakCollection;
  function Search(Item: TRakCollection): TRakCollection;
  var
    i: integer;
  begin
    Result := nil;
    if IsSearched(Item) then
      Result := Item
    else
    begin
      i := 0;
      while (Result = nil) and (i < Item.Count) do
      begin
        Result := Search(Item[i] as TRakCollection);
        inc(i);
      end;
    end;
  end;

begin
   Result := Search(GetAnker);
end;

function TRakCollection.GetAnker: TRakCollection;
begin
  if Collection = nil then
    Result := self
  else
    Result := Collection.GetAnker;
end;

function TRakCollection.GetCount: integer;
begin
  if FList = nil then
    Result := 0
  else
    Result := FList.Count;
end;

procedure TRakCollection.SetCount(Value: integer);
var
  i: integer;
begin
  if Value >= 0 then
  begin
    if Value < Count then
    begin
      for i := Count - 1 downto Value do
        Items[i].Free;
    end
    else if Value > Count then
    begin
      for i := Count + 1 to Value do
        AppendItem;
    end;
  end;
end;

function TRakCollection.GetItems(Index: Integer): TRakCollectionItem;
begin
  Result := FList[Index];
end;

function TRakCollection.IsSearched(Item: TRakCollection): boolean;
begin
  Result := false;
end;

procedure TRakCollection.SetSorted(Value: boolean);
begin
  if Value <> FSorted then
  begin
    FSorted := Value;
    if Sorted then
      Sort;
  end;
end;

procedure TRakCollection.SetItems(Index: Integer; Value: TRakCollectionItem);
var
  lclChanged: boolean;
  Item: TRakCollectionItem;
begin
  if Items[Index] <> Value then
  begin
    lclChanged := false;
    StateAdd(csSetItem);
    try
      Item := Items[Index];
      AddItem(Value, true);
      if Value.Collection = self then
      begin
        RemoveItem(Index, true);
        if Item.Collection = self then
          RemoveItem(Count-1, false)
        else
        begin
          lclChanged := true;
          Value.Index := Index;
        end;
      end;
    finally
      StateDelete(csSetItem);
    end;

    if lclChanged then
      Changed(self);
  end;
end;

function TRakCollection.Add(TheClass: TRakCollectionItemClass): TRakCollectionItem;
begin
  Result := BasicAdd(TheClass);
end;

procedure TRakCollection.AddItem(NewItem: TRakCollectionItem; CanAvoid: boolean);
begin
  if not CanAvoid or CanAddItem(NewItem) then
  begin
    ItemAdd(NewItem);
    StateAdd(csAdd);
    try
      FList.Add(NewItem);
      NewItem.Collection := self;
    finally
      StateDelete(csAdd);
    end;
    ItemAdded(NewItem);
  end;
end;

procedure TRakCollection.AddItem(NewItem: TRakCollectionItem);
begin
  AddItem(NewItem, true);
end;

procedure TRakCollection.AddNil;
begin
  if CanAddItem(nil) then
  begin
    StateAdd(csAdd);
    try
      FList.Add(nil);
    finally
      StateDelete(csAdd);
    end;
    ItemAdded(nil);
  end;
end;

procedure TRakCollection.Assign(Source: TRakCollectionItem);
begin
  if Source is TRakCollection then
  begin
    StateAdd(csAssign);
    try
      inherited Assign(Source);

      DoAssign(TRakCollection(Source));
    finally
      StateDelete(csAssign);
    end;
    Changed(self);
  end;
end;

procedure TRakCollection.DoAssign(Source: TRakCollection);
var
  i: integer;
  Item: TRakCollectionItem;
begin
  FreeAll;
  for i := 0 to Source.Count - 1 do
  begin
    if Source[i] = nil then
      AddNil
    else
    begin
      Item := Add(TRakCollectionItemClass(Source[i].ClassType));
      Item.Assign(Source[i]);
    end;
  end;
end;

function TRakCollection.DeleteAt(Idx: integer): TRakCollectionItem;
begin
  if (Idx < 0) or (Idx >= Count) then
    Result := nil
  else
  begin
    Result := Items[Idx];
    RemoveItem(Idx, true);
    if Result.Collection = self then
      Result := nil;
  end;
end;

procedure TRakCollection.FreeAll;
var
  i: integer;
begin
  if Count > 0 then
  begin
    StateAdd(csFreeAll);
    try
      for i := Count-1 downto 0 do
      begin
        if i < Count then
        begin
          if Items[i] = nil then
            RemoveItem(i, true)
          else
            Items[i].Free;
        end;
      end;
    finally
      StateDelete(csFreeAll);
    end;
    Changed(self);
  end;
end;


function lCompareItems(Item1, Item2: Pointer): integer; far;
var
  CompCol: TRakCollection;
begin
  CompCol := TRakCollectionItem(Item1).Collection;

  if Assigned(CompCol.FOnCompareItems) then
    CompCol.FOnCompareItems(CompCol, TRakCollectionItem(Item1),
      TRakCollectionItem(Item2), Result)
  else
    Result := CompCol.CompareItems(Item1, Item2);
end;

procedure TRakCollection.Pack;
var
  i, t: integer;
  TmpItem: TRakCollectionItem;
begin
  FList.Pack;
  for i := Count - 1 downto 1 do
  begin
    TmpItem := Items[i];
    t := i - 1;
    while t >= 0 do
    begin
      if CompareItems(TmpItem, Items[t]) = 0 then
      begin
        t := 0;
        TmpItem.Free;
      end;
      dec(t);
    end;
  end;
end;

procedure TRakCollection.Sort;
begin
  FList.Sort(lCompareItems);
end;


procedure TRakCollection.RemoveItem(Item: TRakCollectionItem; CanAvoid: boolean);
begin
  if Item.Collection = self then
    RemoveItem(Item.Index, CanAvoid);
end;

procedure TRakCollection.RemoveItem(Idx: integer; CanAvoid: boolean);
var
  Item: TRakCollectionItem;
begin
  Item := Items[Idx];
  if not CanAvoid or CanDeleteItem(Item) then
  begin
    ItemDelete(Item);
    FList.Delete(Idx);
    ItemDeleted(Item, Idx);
  end;
end;

procedure TRakCollection.ItemAdd(NewItem: TRakCollectionItem);
begin
end;

procedure TRakCollection.ItemAdded(NewItem: TRakCollectionItem);
begin
  Changed(self);
end;

procedure TRakCollection.ItemDeleted(DelItem: TRakCollectionItem; OldIdx: integer);
begin
  Changed(self);
end;

procedure TRakCollection.ItemDelete(DelItem: TRakCollectionItem);
begin
end;

function TRakCollection.CanAddItem(NewItem: TRakCollectionItem): boolean;
begin
  Result := true;
end;

{$IFDEF VORDELPHI2005}
function TRakCollection.CanAddItem(
  TheClass: TRakCollectionItemClass): boolean;
{$ELSE}
function TRakCollection.CanAddItemClass(
  TheClass: TRakCollectionItemClass): boolean;
{$ENDIF}
begin
  Result := true;
end;

function TRakCollection.CanDeleteItem(Item: TRakCollectionItem): boolean;
begin
  Result := true;
end;

procedure TRakCollection.AddCollection(Col: TRakCollection);
var
  i: integer;
  Item: TRakCollectionItem;
begin
  for i := 0 to Col.Count - 1 do
  begin
    if Col[i] = nil then
      AddNil
    else
    begin
      Item := CreateItem(TRakCollectionItemClass(Col[i].ClassType));
      if Item <> nil then
        Item.Assign(Col[i]);
    end;
  end;
end;

function TRakCollection.CreateItem(TheClass: TRakCollectionItemClass): TRakCollectionItem;
begin
  Result := BasicAdd(TheClass);
end;


function TRakCollection.IsBranchNode(Node: TRakCollectionItem): boolean;
var
  i: integer;
  Item: TRakCollectionItem;
begin
  Result := Node = self;
  if not Result then
  begin
    for i := 0 to Count - 1 do
    begin
      Item := Items[i];
      if Item is TRakCollection then
      begin
        Result := TRakCollection(Item).IsBranchNode(Node);
        if Result then
          break;
      end;
    end;
  end;
end;

function TRakCollection.IAddItem(NewItem: IRakCollectionItem): boolean;
var
  Obj: TObject;
begin
  Result := false;
  Obj := NewItem.IGetObject;
  if (Obj is TRakCollectionItem) then
  begin
    AddItem(TRakCollectionItem(Obj));
    Result := TRakCollectionItem(Obj).Collection = self;
  end;
end;

function TRakCollection.IDeleteAt(Idx: integer): IRakCollectionItem;
begin
  Result := DeleteAt(Idx);
end;

function TRakCollection.IDeleteItem(DelItem: IRakCollectionItem): boolean;
var
  Obj: TObject;
begin
  Result := false;
  Obj := DelItem.IGetObject;
  if (Obj is TRakCollectionItem) then
  begin
    if TRakCollectionItem(Obj).Collection = self then
    begin
      TRakCollectionItem(Obj).Collection := nil;
      Result := true;
    end;
  end;
end;

function TRakCollection.FreeAt(Idx: integer): boolean;
begin
  Result := (0 <= Idx) and (Idx < Count);
  if Result then
    Items[Idx].Free;
end;

function TRakCollection.IFreeItem(DelItem: IRakCollectionItem): boolean;
var
  Obj: TObject;
begin
  Result := false;
  Obj := DelItem.IGetObject;
  if (Obj is TRakCollectionItem) then
  begin
    TRakCollectionItem(Obj).Free;
    Result := true;
  end;
end;

function TRakCollection.IGetItem(Idx: integer): IRakOwned;
begin
  Result := IRakCollectionItem(Items[Idx]);
end;

function TRakCollection.IIndexOfItem(Item: IRakCollectionItem): integer;
var
  Obj: TObject;
begin
  Result := -1;
  Obj := Item.IGetObject;
  if (Obj is TRakCollectionItem) then
    Result := TRakCollectionItem(Obj).Index;
end;

procedure TRakCollection.IPlugOff(DelItem: IRakOwned);
var
  Obj: TObject;
begin
  Obj := DelItem.IGetObject;
  if (Obj is TRakCollectionItem) then
    RemoveItem(TRakCollectionItem(Obj), false);
end;

procedure TRakCollection.IPlugOn(NewItem: IRakOwned);
var
  Obj: TObject;
begin
  Obj := NewItem.IGetObject;
  if (Obj is TRakCollectionItem) then
    AddItem(TRakCollectionItem(Obj), false);
end;

{ TRakIntegerItem }

procedure TRakIntegerItem.Assign(Source: TRakCollectionItem);
begin
  inherited Assign(Source);

  Wert := TRakIntegerItem(Source).Wert;
end;

procedure TRakIntegerItem.SetWert(Value: integer);
begin
  FWert := Value;
  Changed(self);
  if (Collection <> nil) and (Collection.Sorted) then
    Collection.Sort;
end;

{ TRakIntegerCollection }

function TRakIntegerCollection.GetItems(Index: Integer): integer;
begin
  Result := TRakIntegerItem(inherited GetItems(Index)).Wert;
end;

procedure TRakIntegerCollection.SetItems(Index: Integer; Value: integer);
begin
  TRakIntegerItem(inherited GetItems(Index)).Wert := Value;
end;

procedure TRakIntegerCollection.Add(Value: integer);
begin
  TRakIntegerItem(inherited Add(TRakIntegerItem)).Wert := Value;
end;

procedure TRakIntegerCollection.AppendItem;
begin
  Add(0);
end;

function TRakIntegerCollection.CompareItems(Item1, Item2: Pointer): integer;
begin
  Result := TRakIntegerItem(Item1).Wert - TRakIntegerItem(Item2).Wert;
end;

procedure TRakIntegerCollection.Move(FromIndex, ToIndex: integer);
begin
  inherited Items[FromIndex].Index := ToIndex;
end;

function TRakIntegerCollection.IndexOf(Zahl: integer): integer;
var
  i: integer;
begin
  Result := -1;
  i := 0;
  while (Result = -1) and (i < Count) do
  begin
    if Items[i] = Zahl then
      Result := i;
    inc(i);
  end;
end;

procedure TRakIntegerCollection.FreeItem(Index: integer);
begin
  inherited Items[Index].Free;
end;

{ TRakChildCollection }

constructor TRakChildCollection.Create;
begin
  inherited;

  FDestroyOwner := TRakDestroyOwner.Create(self, self);
end;

destructor TRakChildCollection.Destroy;
var
  LDestroyOwner: TRakDestroyOwner;
begin
  DestroyOwner.CheckDestroy;
  LDestroyOwner := DestroyOwner;
  inherited;
  LDestroyOwner.TheChildIsDestroyed;
  LDestroyOwner.Free;
end;

{ TRakChildCollectionItem }

constructor TRakChildCollectionItem.Create;
begin
  inherited;

  FDestroyOwner := TRakDestroyOwner.Create(self, self);
end;

destructor TRakChildCollectionItem.Destroy;
begin
  DestroyOwner.CheckDestroy;
  DoDestroy;
  DestroyOwner.Free;
  inherited;
end;

procedure TRakChildCollectionItem.DoDestroy;
begin
end;

procedure TRakCollection.MoveBlockTo(BlockStart, BlockEnd, Dest: integer;
  Mode: TRakItemMoveMode);
var
  AList: TList;
  i: integer;
  Start, Ende, DestIdx, SrcIdx: integer;
  MoveList: TRakObjectPaarCol;
  MoveItem: TRakObjectPaarItem;
begin
  if Count > 1 then
  begin //Falls es berhaupt etwas zum verschieben geben kann
    // Sicherstellen, dass BlockStart kleinergleich BlockEnd ist
    KleinerGroesser(BlockStart, BlockEnd);
    // Sicherstellen, dass es korrekte Indizes sind
    CorrectIdx(BlockStart);
    CorrectIdx(BlockEnd);
    //Nicht untersttzte Verschiebe-Modi auf mmAuto umstellen
    if not (Mode in [mmBefore, mmAfter]) then
      Mode := mmAuto;
    // Sicherstellen, dass Dest ein korrekter Index ist
    if Dest < 0 then
    begin
      Dest := 0;
      Mode := mmBefore;
    end
    else if Dest >= Count then
    begin
      Dest := Count - 1;
      Mode := mmAfter;
    end;

    //Die Referenzen auf den zu verschiebenden Block werden in AList gesichert
    AList := TList.Create;
    //In MoveList wird fr jedes verschobenen Item gespeichert, welches Item
    //vorher auf dem Platz war
    MoveList := TRakObjectPaarCol.Create;
    try
      if Dest < BlockStart then
      begin //Wenn nach vorne verschoben werden muss
        if Mode = mmAuto then
          Mode := mmBefore;

        if Mode = mmBefore then
          Start := Dest
        else
          Start := Dest + 1;

        if Start < BlockStart then
        begin // Wenn wirklich verschoben werden muss
          DestIdx := Start;
          for i := BlockStart to BlockEnd do
          begin
            AList.Add(Items[i]);
            MoveItem := MoveList.Add;
            MoveItem.Object1 := Items[i];
            MoveItem.Object2 := Items[DestIdx];
            inc(DestIdx);
          end;

          Ende := BlockStart - 1;
          DestIdx := BlockEnd;
          // Die Elemente verschieben, die nicht zum Block gehoeren
          for i := Ende downto Start do
          begin
            MoveItem := MoveList.Add;
            MoveItem.Object1 := FList[i];
            MoveItem.Object2 := FList[DestIdx];
            FList[DestIdx] := FList[i];
            dec(DestIdx);
          end;
          //Die Referenzen des Blockes wieder aufnehmen
          SrcIdx := 0;
          for i := Start to Start + AList.Count - 1 do
          begin
            FList[i] := AList[SrcIdx];
            inc(SrcIdx);
          end;
        end;
      end
      else if Dest > BlockEnd then
      begin //Wenn nach hinten verschoben werden muss
        if Mode = mmAuto then
          Mode := mmAfter;

        if Mode = mmAfter then
          Ende := Dest
        else
          Ende := Dest - 1;

        if Ende > BlockEnd then
        begin // Wenn wirklich verschoben werden muss
          DestIdx := Ende - BlockEnd + BlockStart;
          for i := BlockStart to BlockEnd do
          begin
            AList.Add(Items[i]);
            MoveItem := MoveList.Add;
            MoveItem.Object1 := Items[i];
            MoveItem.Object2 := Items[DestIdx];
            inc(DestIdx);
          end;

          Start := BlockEnd + 1;
          DestIdx := BlockStart;
          // Die Elemente verschieben, die nicht zum Block gehoeren
          for i := Start to Ende do
          begin
            MoveItem := MoveList.Add;
            MoveItem.Object1 := FList[i];
            MoveItem.Object2 := FList[DestIdx];
            FList[DestIdx] := FList[i];
            inc(DestIdx);
          end;
          //Die Referenzen des Blockes wieder aufnehmen
          SrcIdx := 0;
          DestIdx := Ende - BlockEnd + BlockStart;
          for i := DestIdx to DestIdx + AList.Count - 1 do
          begin
            FList[i] := AList[SrcIdx];
            inc(SrcIdx);
          end;
        end;
      end;

      // Jetzt den Items und externen Benutzern die Mglichkeit geben,
      // auf die Verschiebung zu reagieren
      for i := 0 to MoveList.Count - 1 do
      begin
        TRakCollectionItem(MoveList[i].Object1).Moved(TRakCollectionItem(MoveList[i].Object2));
      end;
      if Assigned(FOnMoved) then
      begin
        for i := 0 to MoveList.Count - 1 do
        begin
          FOnMoved(self, TRakCollectionItem(MoveList[i].Object1),
            TRakCollectionItem(MoveList[i].Object2));
        end;
      end;
    finally
      AList.Free;
      MoveList.Free;
    end;
  end;
end;

procedure TRakCollection.CorrectIdx(var Idx: integer);
begin
  if Idx >= Count then
    Idx := Count - 1
  else if Idx < 0 then
    Idx := 0;
end;

function TRakCollection.CheckIdx(Idx: integer): boolean;
begin
  Result := (Idx >= 0) and (Idx < Count);
end;

{ TRakIDPaarItem }

procedure TRakIDPaarItem.Assign(Source: TRakCollectionItem);
var
  Src: TRakIDPaarItem;
begin
  inherited;
  Src := TRakIDPaarItem(Source);
  FID1 := Src.FID1;
  FID2 := Src.FID2;
end;

procedure TRakIDPaarItem.SetID1(const Value: integer);
begin
  FID1 := Value;
end;

procedure TRakIDPaarItem.SetID2(const Value: integer);
begin
  FID2 := Value;
end;

{ TRakIDPaarCol }

function TRakIDPaarCol.Add: TRakIDPaarItem;
begin
  Result := TRakIDPaarItem(BasicAdd(TRakIDPaarItem));
end;

function TRakIDPaarCol.GetItems(Idx: Integer): TRakIDPaarItem;
begin
  Result := TRakIDPaarItem(inherited Items[Idx]);
end;

{ TRakObjectPaarItem }

procedure TRakObjectPaarItem.Assign(Source: TRakCollectionItem);
var
  Src: TRakObjectPaarItem;
begin
  inherited;
  Src := TRakObjectPaarItem(Source);
  FObject1 := Src.FObject1;
  FObject2 := Src.FObject2;
end;

procedure TRakObjectPaarItem.SetObject1(const Value: TObject);
begin
  FObject1 := Value;
end;

procedure TRakObjectPaarItem.SetObject2(const Value: TObject);
begin
  FObject2 := Value;
end;

{ TRakObjectPaarCol }

function TRakObjectPaarCol.Add: TRakObjectPaarItem;
begin
  Result := TRakObjectPaarItem(BasicAdd(TRakObjectPaarItem));
end;

function TRakObjectPaarCol.Add(Obj1, Obj2: TObject): TRakObjectPaarItem;
begin
  Result := Add;
  Result.Object1 := Obj1;
  Result.Object2 := Obj2;
end;

function TRakObjectPaarCol.GetItems(Idx: Integer): TRakObjectPaarItem;
begin
  Result := TRakObjectPaarItem(inherited Items[Idx]);
end;



{ TRakBaumZweig }

procedure TRakBaumZweig.BlattAdded(Blatt: TRakCollectionItem);
begin
  FBlaetterList.Add(Blatt);
  if Parent <> nil then
    Parent.BlattAdded(Blatt);
end;

procedure TRakBaumZweig.BlattDeleted(Blatt: TRakCollectionItem);
begin
  FBlaetterList.Remove(Blatt);
  if Parent <> nil then
    Parent.BlattDeleted(Blatt);
end;

function TRakBaumZweig.CanAddBlatt(NewBlatt: TRakCollectionItem): boolean;
begin
  Result := DoppelteBlaetter;
  if not Result then
  begin
    if Parent = nil then
      Result := ExistsEqualBlatt(NewBlatt)
    else
      Result := Parent.CanAddBlatt(NewBlatt);
  end;
end;

constructor TRakBaumZweig.Create;
begin
  inherited;

  FBlaetter := CreateFBlaetter;
  FBlaetterList := TList.Create;
  FOrderList := TList.Create;
end;

procedure TRakBaumZweig.DoDestroy;
var
  Obj: TObject;
begin
  Obj := FBlaetter;
  FBlaetter.FZweig := nil;
  FBlaetter := nil;
  Obj.Free;

  Obj := FBlaetterList;
  FBlaetterList := nil;
  Obj.Free;

  Obj := FOrderList;
  FOrderList := nil;
  Obj.Free;

  inherited;
end;

function TRakBaumZweig.DoppelteBlaetter: boolean;
begin
  Result := true;
end;

function TRakBaumZweig.ExistsEqualBlatt(
  Blatt: TRakCollectionItem): boolean;
var
  i: integer;
begin
  Result := Blaetter.ExistsEqualBlatt(Blatt);
  if not Result then
  begin
    i := 0;
    while (i < Count) and not Result do
    begin
      Result := Zweige[i].ExistsEqualBlatt(Blatt);
      inc(i);
    end;
  end;
end;

function TRakBaumZweig.GetOrderCount: integer;
begin
  Result := FOrderList.Count;
end;

function TRakBaumZweig.GetOrderItems(Idx: integer): TRakCollectionItem;
begin
  Result := TRakCollectionItem(FOrderList[Idx]);
end;

function TRakBaumZweig.GetParent: TRakBaumZweig;
begin
  Result := TRakBaumZweig(Collection);
end;

function TRakBaumZweig.GetZweige(Idx: integer): TRakBaumZweig;
begin
  Result := TRakBaumZweig(inherited Items[Idx]);
end;

function TRakBaumZweig.IsOwnBlatt(Blatt: TRakCollectionItem): boolean;
begin
  Result := Blatt.Collection = Blaetter;
end;

procedure TRakBaumZweig.ItemAdded(NewItem: TRakCollectionItem);
begin
  inherited;

  if FOrderList.IndexOf(NewItem) = -1 then
    FOrderList.Add(NewItem);
end;

procedure TRakBaumZweig.ItemDeleted(DelItem: TRakCollectionItem; OldIdx: integer);
begin
  inherited;

  if FOrderList <> nil then
     FOrderList.Remove(DelItem);
end;

procedure TRakBaumZweig.OrderExchange(OrderItem1,
  OrderItem2: TRakCollectionItem);
begin
  OrderExchange(OrderIdx(OrderItem1), OrderIdx(OrderItem2));
end;

procedure TRakBaumZweig.OrderExchange(Idx1, Idx2: integer);
begin
  FOrderList.Exchange(Idx1, Idx2);
end;

function TRakBaumZweig.OrderFirst: TRakCollectionItem;
begin
  if OrderCount = 0 then
    Result := nil
  else
    Result := OrderItems[0];
end;

function TRakBaumZweig.OrderIdx(OrderItem: TRakCollectionItem): integer;
var
  i: integer;
begin
  Result := -1;
  
  for i := 0 to OrderCount - 1 do
  begin
    if FOrderList[i] = OrderItem then
    begin
      Result := i;
      break;
    end;
  end;
end;

function TRakBaumZweig.OrderLast: TRakCollectionItem;
begin
  if OrderCount = 0 then
    Result := nil
  else
    Result := OrderItems[OrderCount - 1];
end;

procedure TRakBaumZweig.OrderMove(ThisOrderItem,
  ToOrderItem: TRakCollectionItem);
begin
  OrderMove(OrderIdx(ThisOrderItem), OrderIdx(ToOrderItem));
end;

procedure TRakBaumZweig.OrderMove(FromIdx, ToIdx: integer);
begin
  FOrderList.Move(FromIdx, ToIdx);
end;

procedure TRakBaumZweig.OrderMove(OrderItem: TRakCollectionItem;
  ToIdx: integer);
begin
  OrderMove(OrderIdx(OrderItem), ToIdx);
end;

procedure TRakBaumZweig.OwnBlattAdded(Blatt: TRakCollectionItem);
begin
  if IsOwnBlatt(Blatt) then
     FOrderList.Add(Blatt);

  BlattAdded(Blatt);
end;

procedure TRakBaumZweig.OwnBlattDeleted(Blatt: TRakCollectionItem);
begin
  FOrderList.Remove(Blatt);
  BlattDeleted(Blatt);
end;

{ TRakBaumBlaetter }

function TRakBaumBlaetter.CanAddItem(NewItem: TRakCollectionItem): boolean;
begin
  Result := NewItem <> nil;
  if Result then
    Result := Zweig.CanAddBlatt(NewItem);
end;

destructor TRakBaumBlaetter.Destroy;
begin
  if (FZweig <> nil) and (FZweig.FBlaetter = self) then
  begin
    FZweig.FBlaetter := nil;
    FZweig.Free;
  end;

  inherited;
end;

function TRakBaumBlaetter.ExistsEqualBlatt(
  Blatt: TRakCollectionItem): boolean;
var
  i: integer;
begin
  Result := false;
  i := 0;
  while not Result and (i < Count) do
  begin
    Result := Items[i].Compare(Blatt) = 0;
    inc(i);
  end;
end;

procedure TRakBaumBlaetter.ItemAdded(Item: TRakCollectionItem);
begin
  if FZweig <> nil then
    FZweig.OwnBlattAdded(Item);
end;

procedure TRakBaumBlaetter.ItemDeleted(Item: TRakCollectionItem; OldIdx: integer);
begin
  if FZweig <> nil then
    FZweig.OwnBlattDeleted(Item);
end;

{ TRakKompositionItem }

function TRakKompositionItem.GetKomposition: IRakKomposition;
begin
  Result := FKomposition;
end;


procedure TRakKompositionItem.SetKomposition(
  const Value: IRakKomposition);
begin
  if FKomposition = nil then
  begin
    FKomposition := Value;
    FKomposition.AddMember(self);
  end
  else
    raise Exception.Create('TRakKompositionItem.SetKomposition darf nur einmal aufgerufen werden.');
end;

end.

