unit RakNamespace;
///////////////////////////////////////////////////////////////////////////////}
// Delphi 5 Implementation
//
// Copyright (c) 2000 by Richard Kaspar
// ("http://www.kasparsoft.de")
//
// Definitions:
// - "Package" refers to the collection of files distributed by
//   the Copyright Holder, and derivatives of that collection of
//   files created through textual modification.
// - "Standard Version" refers to such a Package if it has not
//   been modified, or has been modified in accordance with the
//   wishes of the Copyright Holder.
// - "Copyright Holder" is whoever is name in the copyright or
//   copyrights for the package.
// - "You" is you, if you're thinking about copying or distributing
//   this Package.
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Package"), to deal in the Package without restriction,
// including without limitation the rights to use, copy, modify,
// merge, publish, distribute, sublicense, and/or sell copies of the
// Package, and to permit persons to whom the Package is furnished
// to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Package.
//
// You may modify your copy of this Package in any way, provided
// that you insert a prominent notice in each changed file stating
// how and when you changed a file, and provided that you do at
// least one of the following:
//
// a) allow the Copyright Holder to include your modifications in
// the Standard Version of the Package.
//
// b) use the modified Package only within your corporation or
// organization.
//
// c) rename any non standard executables, units, and classes so
// the names do not conflict with standard executables, units, and
// classes, and provide a separate manual page that clearly documents
// how it differs from the standard version.
//
// d) make other distribution arrangements with the Copyright Holder.
//
// The name of the Copyright Holder may not be used to endorse or
// promote products derived from this Package without specific prior
// written permission.
//
// THE PACKAGE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// PACKAGE OR THE USE OR OTHER DEALINGS IN THE PACKAGE.
{$I RAK.INC}

interface

uses
  miscfunc, RakCollection, RakDestroy,
  sysutils, classes;

type
  TRakNamespace = class;
  TRakNamespaceMember = class;
  IRakNamespace = interface;


{////////////////////////////////////////////////////////////////////////////////
// Interface    : IRakNamespaceMember
// Beschreibung : IRakNamespaceMember ist die Schnittstelle fr alle
                  Objekte, die innerhalb eines Namensraums einen eindeutigen
                  Namen haben mssen.
// erstellt am  : 30.9.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////}
  IRakNamespaceMember = interface(IRakObject)
  // protected
    procedure SetIDNameVar(NewIDName: string);
  // public
    function GetIDName: string;
    function SetIDName(WantedName: string; OnlyExact: boolean = false): boolean;
    function GetCompleteName: string;

    function IGetNamespace: IRakNamespace;
    procedure ISetNamespace(Value: IRakNamespace);
  end;

{////////////////////////////////////////////////////////////////////////////////
// Interface    : IRakNamespace
// Beschreibung : IRakNamespace ist die Schnittstelle fr Namensrume.
                  Objekte, die in einen Namensraum aufgenommen werden
                  haben einen Namen, mit dem sie innerhalb des Namensraumes
                  eindeutig identifiziert werden knnen.
// erstellt am  : 30.9.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
  2.6.2002 Neue Methode IMemberByCompleteName
////////////////////////////////////////////////////////////////////////////////}
  IRakNamespace = interface(IRakObject)
  //protected
    function ISetMemberIDName(Member: IRakNamespaceMember; WantedName: string;
      OnlyExact: boolean = false): string;
    procedure IPlugOn(NewMember: IRakNamespaceMember);
    procedure IPlugOff(DelMember: IRakNamespaceMember);
  //public
    function GetSeparaterChar: char;
    function GetPath: string;

    function IMemberByIDName(IDName: string): IRakNamespaceMember;
    function IMemberByCompleteName(CompleteName: string): IRakNamespaceMember;
    function IGetMember(Idx: integer): IRakNamespaceMember;
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakNamespaceMemberImplementation
// Beschreibung : TRakNamespaceMemberImplementation ist eine Klasse, welche
                  die Schnittstelle IRakNamespaceMember implementiert.
// erstellt am  : 30.9.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////}
  TRakNamespaceMemberImplementation = class(TRakInterfacedObject, IRakNamespaceMember)
  protected
    FIDName: string;
    FNamespace: IRakNamespace;

    procedure WriteIDName(Value: string); virtual;

    procedure SeparatNamespace; virtual;
    procedure ConnectNamespace; virtual;
  public // IRakNamespaceMember
    procedure SetIDNameVar(NewIDName: string); virtual;
    function GetIDName: string; virtual;
    function SetIDName(WantedName: string; OnlyExact: boolean = false): boolean; virtual;
    function GetCompleteName: string; virtual;

    function IGetNamespace: IRakNamespace; virtual;
    procedure ISetNamespace(Value: IRakNamespace); virtual;
  public
    destructor Destroy; override;

    property IDName: string read GetIDName write WriteIDName;
    property CompleteName: string read GetCompleteName;
    property INamespace: IRakNamespace read IGetNamespace write ISetNamespace;
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakNamespaceImplementation
// Beschreibung : TRakNamespaceImplementation ist eine Klasse, welche
                  die Schnittstelle IRakNamespace implementiert.
                  TRakNamespaceImplementation kann Instanzen von
                  TRakNamespaceMemberImplementation in sich aufnehmen und stellt
                  sicher, dass jedes Objekt einen eindeutigen IDName hat.
                  TRakNamespaceImplementation wird nicht der Besitzer
                  von den Objekten, gibt sie also auch nicht frei, wenn
                  er sich selbst zerstrt.
                  TRakNamespaceImplementation sorgt dafr, dass das Property
                  IDName der Objekte keinen Leerstring enthlt und dass IDName
                  das Zeichen "GetSeparaterChar" nicht enthlt. Ansonsten
                  darf IDName alle Zeichen enthalten. Abgeleitete Klassen
                  knnen dies durch berschreiben der Methode CheckIDNameSyntax
                  ndern.
// erstellt am  : 30.9.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
    11.11.2001: Bugfix: Beim zerstren hat er sich nihct von seinen Mitgliedern getrennt
    2.6.2002 Neue Methode IMemberByCompleteName
////////////////////////////////////////////////////////////////////////////////}
  TRakNamespaceImplementation = class(TRakNamespaceMemberImplementation, IRakNamespace)
  protected
    FMembers: TInterfaceList;

    procedure PlugOn(NewMember: IRakNamespaceMember); virtual;
    procedure PlugOff(DelMember: IRakNamespaceMember); virtual;
    function ISetIDName(Member: IRakNamespaceMember; WantedName: string;
      OnlyExact: boolean): string; virtual;

    function CheckIDNameSyntax(var WantedName: string;
      OnlyExact: boolean): boolean; virtual;

    function GetCount: integer; virtual;
  public // IRakNamespace
    function ISetMemberIDName(Member: IRakNamespaceMember; WantedName: string;
      OnlyExact: boolean = false): string; virtual;
    procedure IPlugOn(NewMember: IRakNamespaceMember); virtual;
    procedure IPlugOff(DelMember: IRakNamespaceMember); virtual;
    function GetSeparaterChar: char; virtual;
    function GetPath: string; virtual;

    function IMemberByIDName(IDName: string): IRakNamespaceMember; virtual;
    function IMemberByCompleteName(CompleteName: string): IRakNamespaceMember; virtual;
    function IGetMember(Idx: integer): IRakNamespaceMember; virtual;
  public
    constructor Create;
    destructor Destroy; override;

    function AddMember(NewMember: IRakNamespaceMember): boolean;
    function DeleteMember(DelMember: IRakNamespaceMember): boolean;
    function IndexOfMember(Member: IRakNamespaceMember): integer;
    procedure PlugOffAll;

    property Count: integer read GetCount;
    property Members[Idx: integer]: IRakNamespaceMember read IGetMember;
  end;


  // Die Klassen ab hier sind noch Baustellen

  TRakNsCollectionItem = class(TRakCollectionItem)
  private
    FOwner: TRakNamespaceMember;
  public
    destructor Destroy; override;

    property Owner: TRakNamespaceMember read FOwner;
  end;

  TRakNsCollection = class(TRakCollection)
  private
    FOwner: TRakNamespace;
  protected
    procedure DoDestroy; override;
  public
    property Owner: TRakNamespace read FOwner;
  end;



  TRakNamespaceMember = class(TRakNamespaceMemberImplementation)
  private
    FColItem: TRakNsCollectionItem;
    Komposition: IRakKomposition;
  public
    constructor Create;
    destructor Destroy; override;
  end;

  TRakNamespace = class(TRakNamespaceImplementation)
  private
    FCol: TRakNsCollection;
    Komposition: IRakKomposition;
  public
    constructor Create;
    destructor Destroy; override;
  end;

implementation

{ TRakNamespaceMemberImplementation }

procedure TRakNamespaceMemberImplementation.ConnectNamespace;
begin
  if Assigned(FNamespace) then
    FNamespace.IPlugOn(self);
end;

destructor TRakNamespaceMemberImplementation.Destroy;
begin
  INamespace := nil;

  inherited;
end;

function TRakNamespaceMemberImplementation.GetCompleteName: string;
begin
  if INamespace <> nil then
    Result := INamespace.GetPath + INamespace.GetSeparaterChar + IDName
  else
    Result := IDName;
end;

function TRakNamespaceMemberImplementation.GetIDName: string;
begin
  Result := FIDName;
end;

function TRakNamespaceMemberImplementation.IGetNamespace: IRakNamespace;
begin
  Result := FNamespace;
end;

procedure TRakNamespaceMemberImplementation.SeparatNamespace;
begin
  if Assigned(FNamespace) then
    FNamespace.IPlugOff(self);
end;

function TRakNamespaceMemberImplementation.SetIDName(WantedName: string;
  OnlyExact: boolean): boolean;
begin
  if Assigned(FNamespace) then
  begin
    FNamespace.ISetMemberIDName(self, WantedName, OnlyExact);
    Result := IDName = WantedName;
  end
  else
  begin
    FIDName := WantedName;
    Result := true;
  end;
end;

procedure TRakNamespaceMemberImplementation.SetIDNameVar(
  NewIDName: string);
begin
  FIDName := NewIDName;
end;

procedure TRakNamespaceMemberImplementation.ISetNamespace(
  Value: IRakNamespace);
begin
  if Value <> FNamespace then
  begin
    SeparatNamespace;

    FNamespace := Value;

    ConnectNamespace;
  end;
end;

procedure TRakNamespaceMemberImplementation.WriteIDName(Value: string);
begin
  SetIDName(Value, true);
end;


{ TRakNamespaceImplementation }

function TRakNamespaceImplementation.AddMember(
  NewMember: IRakNamespaceMember): boolean;
begin
  Result := false;
  if (NewMember <> nil) and (IndexOfMember(NewMember) = -1) then
    NewMember.ISetNamespace(self);
end;

function TRakNamespaceImplementation.CheckIdNameSyntax(
  var WantedName: string; OnlyExact: boolean): boolean;
var
  Separater: char;
begin
  Separater := GetSeparaterChar;
  Result := Pos(Separater, WantedName) = 0;
  if not Result and not OnlyExact then
  begin // Wenn im Namen das Zeichen enthalten ist, mit dem Namespaces getrennt werden, aber wir den Namen verndern drfen
    // Dann schmeissen wir die Trennzeichen aus dem Namen raus
    WantedName := StringReplace(WantedName, Separater, '', [rfReplaceAll]);
    Result := true;
  end;

  if Result and (WantedName = '') then
  begin // Ein IDName darf kein Leerstring sein
    Result := not OnlyExact;
    if Result then
    begin // Wenn wir den Namen verndern drfen
      if Separater = '_' then
        WantedName := 'A'
      else
        WantedName := '_';
    end;
  end;
end;

constructor TRakNamespaceImplementation.Create;
begin
  inherited;

  FMembers := TInterfaceList.Create;
end;

function TRakNamespaceImplementation.DeleteMember(
  DelMember: IRakNamespaceMember): boolean;
begin
  Result := IndexOfMember(DelMember) >= 0;
  if Result then
    DelMember.ISetNamespace(nil);
end;

destructor TRakNamespaceImplementation.Destroy;
begin
  PlugOffAll;
  FreeTObject(FMembers);

  inherited;
end;

function TRakNamespaceImplementation.GetCount: integer;
begin
  Result := FMembers.Count;
end;

function TRakNamespaceImplementation.IGetMember(
  Idx: integer): IRakNamespaceMember;
begin
  Result := IRakNamespaceMember(FMembers[Idx]);
end;

function TRakNamespaceImplementation.GetSeparaterChar: char;
begin
  Result := '.';
end;

function TRakNamespaceImplementation.IndexOfMember(
  Member: IRakNamespaceMember): integer;
begin
  Result := FMembers.IndexOf(Member);
end;

function TRakNamespaceImplementation.IMemberByIDName(
  IDName: string): IRakNamespaceMember;
var
  i: integer;
begin
  Result := nil;
  for i := 0 to Count - 1 do
  begin
    if Members[i].GetIDName = IDName then
    begin
      Result := Members[i];
      break;
    end;
  end;
end;

procedure TRakNamespaceImplementation.PlugOff(
  DelMember: IRakNamespaceMember);
begin
  FMembers.Remove(DelMember);
end;

procedure TRakNamespaceImplementation.PlugOn(
  NewMember: IRakNamespaceMember);
begin
  if Assigned(NewMember) and (IndexOfMember(NewMember) = -1) then
    FMembers.Add(NewMember)
end;

function TRakNamespaceImplementation.ISetIDName(
  Member: IRakNamespaceMember; WantedName: string;
  OnlyExact: boolean): string;
var
  Counter: integer;
  lclMember: IRakNamespaceMember;
  TryName: string;
begin
  Result := '';
  if CheckIdNameSyntax(WantedName, OnlyExact) then
  begin
    lclMember := IMemberByIDName(WantedName);
    if lclMember = Member then
      Result := WantedName
    else if lclMember = nil then
    begin
      Member.SetIDNameVar(WantedName);
      Result := WantedName;
    end
    else if not OnlyExact then
    begin
      Counter := 1;
      repeat
        inc(Counter);
        TryName := WantedName + IntToStr(Counter);
      until IMemberByIDName(TryName) = nil;
      Member.SetIDNameVar(TryName);
      Result := TryName;
    end;
  end;
end;

function TRakNamespaceImplementation.ISetMemberIDName(
  Member: IRakNamespaceMember; WantedName: string;
  OnlyExact: boolean): string;
begin
  Result := '';
  if IndexOfMember(Member) >= 0 then
    Result := ISetIDName(Member, WantedName, OnlyExact);
end;

function TRakNamespaceImplementation.GetPath: string;
begin
  Result := GetCompleteName;
end;

procedure TRakNamespaceImplementation.IPlugOff(
  DelMember: IRakNamespaceMember);
begin
  FMembers.Remove(DelMember);
end;

procedure TRakNamespaceImplementation.IPlugOn(
  NewMember: IRakNamespaceMember);
begin
  if FMembers.IndexOf(NewMember) = -1 then
  begin
    ISetIDName(NewMember, NewMember.GetIDName, false);
    FMembers.Add(NewMember);
  end;
end;


procedure TRakNamespaceImplementation.PlugOffAll;
begin
  while Count > 0 do
    DeleteMember(Members[FMembers.Count-1]);
end;

function TRakNamespaceImplementation.IMemberByCompleteName(
  CompleteName: string): IRakNamespaceMember;
var
  i: integer;
begin
  Result := nil;
  for i := 0 to Count - 1 do
  begin
    if Members[i].GetCompleteName = CompleteName then
    begin
      Result := Members[i];
      break;
    end;
  end;
end;

{ TRakNsCollectionItem }

destructor TRakNsCollectionItem.Destroy;
begin
  Owner.Komposition.MemberDestroying(self);
  inherited;
end;

{ TRakNsCollection }

procedure TRakNsCollection.DoDestroy;
begin
  Owner.Komposition.MemberDestroying(self);
  inherited;
end;

{ TRakNamespaceMember }

constructor TRakNamespaceMember.Create;
begin
  inherited;
  Komposition := TRakKomposition.Create(self);
  FColItem := TRakNsCollectionItem.Create;
  FColItem.FOwner := self;
  Komposition.AddMember(FColItem);
end;

destructor TRakNamespaceMember.Destroy;
begin
  Komposition.MemberDestroying(self);
  inherited;
end;

{ TRakNamespace }

constructor TRakNamespace.Create;
begin
  inherited;
  Komposition := TRakKomposition.Create(self);
  FCol := TRakNsCollection.Create;
  FCol.FOwner := self;
  Komposition.AddMember(FCol);
end;

destructor TRakNamespace.Destroy;
begin
  Komposition.MemberDestroying(self);
  inherited;
end;



initialization

end.
