unit misctype;

{///////////////////////////////////////////////////////////////////////////////
// Beschreibung :  Die Unit misctype enthlt einige Typen, die u.a.
                  von der Unit miscfunc verwendet werden
// erstellt am  : ???
////////////////////////////////////////////////////////////////////////////////
// nderungen:
    21.9.2001 Neue Klassen TNewFileWriter und TFileReader
///////////////////////////////////////////////////////////////////////////////}
// Delphi 5 Implementation
//
// Copyright (c) 2000 by Richard Kaspar
// ("http://www.kasparsoft.de")
//
// Definitions:
// - "Package" refers to the collection of files distributed by
//   the Copyright Holder, and derivatives of that collection of
//   files created through textual modification.
// - "Standard Version" refers to such a Package if it has not
//   been modified, or has been modified in accordance with the
//   wishes of the Copyright Holder.
// - "Copyright Holder" is whoever is name in the copyright or
//   copyrights for the package.
// - "You" is you, if you're thinking about copying or distributing
//   this Package.
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Package"), to deal in the Package without restriction,
// including without limitation the rights to use, copy, modify,
// merge, publish, distribute, sublicense, and/or sell copies of the
// Package, and to permit persons to whom the Package is furnished
// to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Package.
//
// You may modify your copy of this Package in any way, provided
// that you insert a prominent notice in each changed file stating
// how and when you changed a file, and provided that you do at
// least one of the following:
//
// a) allow the Copyright Holder to include your modifications in
// the Standard Version of the Package.
//
// b) use the modified Package only within your corporation or
// organization.
//
// c) rename any non standard executables, units, and classes so
// the names do not conflict with standard executables, units, and
// classes, and provide a separate manual page that clearly documents
// how it differs from the standard version.
//
// d) make other distribution arrangements with the Copyright Holder.
//
// The name of the Copyright Holder may not be used to endorse or
// promote products derived from this Package without specific prior
// written permission.
//
// THE PACKAGE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// PACKAGE OR THE USE OR OTHER DEALINGS IN THE PACKAGE.
{$I RAK.INC}

interface

uses classes, sysutils, windows, contnrs;

type
  TPolygon = array of TPoint;

  THorzAlign = (haNone, haLeft, haCenter, haRight);
  TVertAlign = (vaNone, vaTop, vaCenter, vaBottom);
  TCorner = (cTopLeft, cTopRight, cBottomRight, cBottomLeft);
  TRakXDirection = (xdLeft, xdNone, xdRight);
  TRakYDirection = (ydUp, ydNone, ydDown);

  TExtRect = record
    Top, Left, Right, Bottom: extended;
  end;

  TExtLine = record
    X1,X2,Y1,Y2: extended;
  end;

  TExtLineArr = array of TExtLine;

  TExtPoint = record
    X, Y: extended;
  end;

  TExtPointArr = array of TExtPoint;
  TExtPolygon = TExtPointArr;

  TZeichenmenge = set of char;

  TRakPaintStyle = (psNT, psXP);

  EInvalidParams = class(Exception);
  EPropertyReadonly = class(Exception);
  EDatenInkonsistent = class(Exception);

  TAnalizeLinePointResultState = (
    alprsPointTouch,      // Die Linie ist nur ein Punkt. Beide Punkte haben die selbe Position; Faktor und Winkel sind undefiniert
    alprsPointNoTouch,    // Die Linie ist nur ein Punkt. Die Punkte haben nicht die selbe Position; Faktor und Winkel sind undefiniert
    alprsStart,            // Der Punkt ist der Startpunkt der Linie; (Faktor = 0; Abstand = 0)
    alprsEnd,              // Der Punkt ist der Endpunkt der Linie; (Faktor = 1; Abstand = 0)
    alprsTouchIn,          // Der Punkt liegt auf der Linie; (0 < Faktor < 1; Abstand = 0)
    alprsTouchOut,        // Der Punkt liegt auf der verlngerten Linie (Faktor < 0 or Faktor > 0; Abstand = 0)
    alprsNoTouch);        // Der Punkt liegt nicht auf der Linie (Abstand > 0)

  TAnalizeLinePointResult = record
    State: TAnalizeLinePointResultState;
    IsTouching: boolean; // IsTouching := not(State in [alprsPointNoTouch, alprsNoTouch])
    IsTouchingInside: boolean; // IsTouchingInside := State in [alprsPointTouch, alprsTouchIn, alprsStart, alprsEnd]
    Abstand: extended; // Der Abstand von dem Punkt zu der Linie, bei senkrechter Projektion des Punktes auf die Linie; (Abstand >= 0)
    Faktor: extended;  // StartpunktLnie + RichtungsvektorLinie * Faktor ergibt den Projektionspunkt auf der Linie
    Winkel: extended; // Der Winkel zw. den Linien (StartpunktLinie, EndpunktLinie) und (StartpunktLinie, Punkt); (-pi <= Winkel <= pi)
  end;

  TAnalizeLineLineResult = record
    LinesAreParallel: boolean; //Die beiden Linien verlaufen parallel zueinander; Line1Data und Line2Data enthalten keine definierten Werte
                              // LinesAreParallel ist immer false, wenn mindestens eine dr Linien in Wirklichkeit ein Punkt ist
    IsTouching: boolean; // Es gibt genau einen gemeinsamen Punkt beider Linien; Genauere Informationen enthalten Line1Data und Line2Data
    // Da die Linien in Wirklichkeit auch Punkte sein knnen, kann es vorkommen, dass IsTouching = false und LinesAreParallel = false
    // Wenn beide Linien tatschlich Linien sind, dann ist IsTouching = not LinesAreParallel
    Abstand: extended; // Der Abstand zwischen den beiden Linien, wenn LinesAreParallel
    Point: TExtPoint;  // Der Schnittpunkt der beiden Linien, wenn not LinesAreParallel and Line1Data.IsTouching

    Line1Data: TAnalizeLinePointResult; // Ist nur versorgt, wenn LinesAreParallel = false
    Line2Data: TAnalizeLinePointResult; // Ist nur versorgt, wenn LinesAreParallel = false
  end;


  TRakLineCutPointState = (
    lcpsNone,        // Es gibt keinen Schnittpunkt; Point darf nicht ausgewertet werden
    lcpsOut,        // DerSchnittpunkt existiert nur, wenn man die Linie unendlich verlngert
    lcpsPoint,      // Die Linie ist in Wirklichkeit ein Punkt und liegt auf der anderen Linie
    lcpsLineStart,  // Der Schnittpunkt liegt genau am Anfang der Linie
    lcpsLineEnd,    // Der Schnittpunkt liegt genau am Ende der Linie
    lcpsIn);        // Der Schnittpunkt liegt innerhalb der Linie

  TRakLineCutPoint = record
    State: TRakLineCutPointState;
    Point: TExtPoint;
  end;

  TAnalizeLineRectResultState = (
    alrrsOut,           // Die Linie liegt komplett auerhalb des Rechtecks
    alrrsIn,             // Die Linie liegt komplett innnerhalb des Rechtecks
    alrrsCut,           // Die Linie schneidet das Rechteck
    alrrsTouch);        // Die Linie berhrt das Rechteck

  TAnalizeLineRectResult = record
    CutPoint1: TRakLineCutPoint; // Der erste Schnittpunkt der Linie mit dem Rechteck
    CutPoint2: TRakLineCutPoint; // Der zweite Schnittpunkt der Linie mit dem Rechteck
    State: TAnalizeLineRectResultState; // Gibt die relative Position der Linie zu dem Rechteck an
  end;


  TRakClassList = class(TClassList)
  public
    function ClassByName(ClassName: string): TClass;
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TNewFileWriter
// Beschreibung : TNewFileWriter erzeugt eine neue Datei, bzw. berschreibt
                  die Datei, falls vorhanden.
// erstellt am  : 26.9.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////}
  TNewFileWriter = class(TWriter)
  protected
    FStream: TFileStream;
  public
    constructor Create(FileName: string);
    destructor Destroy; override;

    property Stream: TFileStream read FStream;
  end;

{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TFileReader
// Beschreibung : Mit TFileReader kann man eine Datei auslesen, die mit
                  TNewFileWriter erzeugt wurde
// erstellt am  : 26.9.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////}
  TFileReader = class(TReader)
  protected
    FStream: TFileStream;
  public
    constructor Create(FileName: string);
    destructor Destroy; override;

    property Stream: TFileStream read FStream;
  end;


implementation


{ TNewFileWriter }

constructor TNewFileWriter.Create(FileName: string);
var
  Stream: TFileStream;
begin
  Stream := TFileStream.Create(FileName, fmCreate);
  inherited Create(Stream, 1024);
  FStream := Stream;
end;

destructor TNewFileWriter.Destroy;
var
  Stream: TFileStream;
begin
  Stream := FStream;
  inherited;
  Stream.Free;
end;

{ TFileReader }

constructor TFileReader.Create(FileName: string);
var
  Stream: TFileStream;
begin
  Stream := TFileStream.Create(FileName, fmOpenRead);
  inherited Create(Stream, 1024);
  FStream := Stream;
end;

destructor TFileReader.Destroy;
var
  Stream: TFileStream;
begin
  Stream := FStream;
  inherited;
  Stream.Free;
end;


{ TRakClassList }

function TRakClassList.ClassByName(ClassName: string): TClass;
var
  i: integer;
begin
  Result := nil;
  if ClassName <> '' then
  begin
    for i := 0 to Count - 1 do
    begin
      if CompareText(Items[i].ClassName, ClassName) = 0 then
      begin
        Result := Items[i];
        break;
      end;
    end;
  end;
end;

end.














