﻿(*    RefreshBox;
ObjectList.pas
Copyright (C) 2006  Christian Brugger <brugger.christian(AT)web.de>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version. This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
details. You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA 02110, USA
*)
unit ObjectList;

interface

uses
  Windows, Contnrs, ZLib, Graphics, Classes, SysUtils, RakBinaryStreamData,
  miscfunc, ComCtrls, ExtCtrls, Controls, Forms, Math, dialogs;

                                          
type
  TActAction = (None, AddLine, AddLineConnector, AddCouplingDiode, AddAnd, AddOr, AddXor, AddSwitch,
                AddDelayModule, AddLed, AddText, AddInverter, AddHalfAdder, AddFullAdder, AddFlipFlop,
                AddDecoderNoClock, AddDecoderClock, AddShiftingRegister, AddClockGenerator, AddIc,
                SetNegationPoint, PowerOn, Remove);
  TOrientation = (orLeft, orRight, orUp, orDown);

type
  TSetStatusStackTyp = record
    Position: TPoint;
    Status: boolean;
  end;

type
  TElementList = class(TObjectList)
  private

  public
    procedure SaveToBinaryTag(Tag: TRakBinaryTag);
    procedure LoadFromBinaryTag(Tag: TRakBinaryTag; AOwner: TObject);
  end;

  TElementBox = class(TObject)            // Haupt
  private
    FLineList: TElementList;
    FModuleList: TElementList;
    FLineCouplingDiodeList: TElementList;
    FLineConnectorList: TElementList;
    FSetStatusStack: TList;
    FCanvas: TCanvas;
    FVResolution: TPoint;
    FVStartPoint: TPoint;
    FOwner: TObject;
    FSimulationPointer: TObject;
    FClockGenerator: TObject;
    function ModuleCollision(AObject: TObject): boolean;
    function LineCollision(AStartPoint, AEndPoint: TPoint): boolean;
    function FindStatusModule(APosition: TPoint; ACount: integer):TObject;
    function FindConnector(APosition: TPoint):TObject;
    function FindCouplingDiode(APosition: TPoint):boolean;
    function FindCouplingDiodeO(APosition: TPoint):TObject;
    function FindLinePoint(APosition: TPoint; ACount: integer):TObject;
    function FindLineNewLineIsCompleteIn(AStartPoint, AEndPoint: TPoint): TObject;
    function FindPointInLine(AStartPoint, AEndPoint: TPoint): TPoint;
    function FindLineThroughPoint(APoint: TPoint): TObject;
    function LineCount(APosition: TPoint): integer;
    function FindLineExtension(AStartPoint, AEndPoint: TPoint): integer;
    function FindLineCompleteInNewLine(AStartPoint, AEndPoint: TPoint): TObject;
    function AreLinesCrossing(APosition: TPoint): boolean;
    function CountLinesSameHorizontal(APosition: TPoint; AHorizontal: boolean): integer;
    procedure SetConnectingLinesBetweenTwoConnectors(ALogicModule: TObject);
	public
    constructor Create(AOwner: TObject; const ACanvas: TCanvas; AResolution: TPoint);
    destructor Destroy; override;
    property Canvas: TCanvas read FCanvas write FCanvas;
    property Owner: TObject read FOwner write FOwner;
    property VResolution: TPoint read FVResolution write FVResolution;
    property VStartPoint: TPoint read FVStartPoint write FVStartPoint;
    procedure SaveToBinaryTag(Tag: TRakBinaryTag);
    procedure LoadFromBinaryTag(Tag: TRakBinaryTag);
    procedure Paint;
    procedure RefreshLineConnectorList;
    procedure AddLine(AStartPoint, AEndPoint: TPoint);
    procedure AddDirectLine(AStartPoint, AEndPoint: TPoint);
    procedure AddLineConnector(APosition: TPoint);
    procedure SetCouplingDiode(APosition: TPoint);
    procedure AddAndModule(APosition: TPoint; AOrientation: TOrientation = orLeft; ASize: integer = 0);
    procedure AddOrModule(APosition: TPoint; AOrientation: TOrientation = orLeft; ASize: integer = 0);
    procedure AddXorModule(APosition: TPoint; AOrientation: TOrientation = orLeft; ASize: integer = 0);
    procedure AddSwitchModule(APosition: TPoint);
    procedure AddLedModule(APosition: TPoint; AColor: TColor);
    procedure AddTextModule(APosition: TPoint; ACaption: string; ASize: integer = 8; AColor: TColor = clBlack);
    procedure AddIcModule(APosition: TPoint; AIndex: integer);
    procedure AddInverter(APosition: TPoint; AOrientation: TOrientation = orLeft);
    procedure AddDelayModule(APosition: TPoint; AOrientation: TOrientation = orLeft);
    procedure AddHalfAdder(APosition: TPoint; AOrientation: TOrientation = orLeft);
    procedure AddFullAdder(APosition: TPoint; AOrientation: TOrientation = orLeft);
    procedure AddFlipFlop(APosition: TPoint; AOrientation: TOrientation = orLeft);
    procedure AddDecoderNoClock(APosition: TPoint; AOrientation: TOrientation = orLeft);
    procedure AddDecoderClock(APosition: TPoint; AOrientation: TOrientation = orLeft);
    procedure AddShiftingRegister(APosition: TPoint; AOrientation: TOrientation = orLeft);
    procedure AddClockGenerator(APosition: TPoint; AOrientation: TOrientation = orLeft);
    procedure SetNegationPoint(APosition: TPoint);
    procedure SetLineConnector(APosition: TPoint);
    procedure RemoveModule(APosition: TPoint);
    procedure RemoveLine(AStartPoint, AEndPoint: TPoint);
    function ItemsStatus(APosition: TPoint; var AError: integer): boolean;
    procedure SetStatusByClick(APosition: TPoint);
    procedure SetItemsStatus(APosition: TPoint; AStatus: boolean);
    procedure ExecuteSetItemsStatusBuffer(var Changed: boolean);
    procedure SetItemsStatusExecute(APosition: TPoint; AStatus: boolean; var Changed: boolean);
    procedure DivideLines(APosition: TPoint);
    procedure LinkLines(APosition: TPoint);
    procedure CrossLinkLines(APosition: TPoint);
    procedure SwitchOn;
    procedure SwitchOff;
    procedure ExecuteIteration;
    function FindClockGenerator: TObject;
	end;

  TSimulationTab = class(TTabSheet)
  private
    FElementBox: TElementBox;
    FPaintBox: TPaintBox;
    FClick1, FClick2, FClick3: TPoint;
    FIsMouseDown: boolean;
    FHorizontal: boolean;    // for OnMouseMove, OnMouseUp
    FResolution: TPoint;
    FVResolution: TPoint;
    FVStartPoint: TPoint;
    FOwner: TObject;
    FPageControl: TComponent;
    FScrollBox: TScrollBox;
    procedure RefreshScrollBox;
    procedure SetResolution(AResolution: TPoint);
    procedure RefreshBox;
    procedure CatchPoint(var APoint: TPoint);
    procedure FPaintBoxPaint(Sender: TObject);
    procedure FPaintBoxMouseDown(Sender: TObject; Button: TMouseButton;
                                  Shift: TShiftState; X, Y: Integer);
    procedure FPaintBoxMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure FPaintBoxMouseUp(Sender: TObject; Button: TMouseButton;
                                  Shift: TShiftState; X, Y: Integer);
    procedure FPaintBoxMouseLeave(Sender: TObject);
  public
    constructor Create(APageControl: TComponent; AName: string; AOwner: TObject); reintroduce;
    destructor Destroy; override;
    property Resolution: TPoint read FResolution write SetResolution;
    property Owner: TObject read FOwner write FOwner;
    procedure SaveToBinaryTag(Tag: TRakBinaryTag);
    procedure LoadFromBinaryTag(Tag: TRakBinaryTag);
    procedure Paint;
  end;

  TSimulation = class(TObject)
  private
    FPageControl: TPageControl;
    FMainSimulationTab: TSimulationTab;
    FICTabList: TElementList;
    FIcSampleList: TElementList;
    FIcLinkPointer: TObject;
    FClockGeneratorExists: boolean;
    function GetResolution: TPoint;
    procedure SetResolution(AResolution: TPoint);
    function GetMainBox: TElementBox;
    procedure SetMainBox(AElementBox: TElementBox);
    procedure SetClockGeneratorInterval(AInterval: word);
    procedure FPageControlChanging(Sender: TObject; var AllowChange: Boolean);
  public
    constructor Create(APageControl: TPageControl);
    destructor Destroy; override;
    property ClockGeneratorExists: boolean read FClockGeneratorExists write FClockGeneratorExists;
    property Resolution: TPoint read GetResolution write SetResolution;
    property MainBox: TElementBox read GetMainBox write SetMainBox;
    property ClockGeneratorInterval: word write SetClockGeneratorInterval;
    function SaveToFile(AFileName: string): boolean;
    function LoadFromFile(AFileName: string; IsForced: boolean): string;
    procedure SaveToBinaryTag(Tag: TRakBinaryTag);
    procedure LoadFromBinaryTag(Tag: TRakBinaryTag);
    procedure Paint;
    procedure SwitchOn;
    procedure SwitchOff;
    procedure AddIC(AName, ACaption: string; AIcSamplePointer: TObject);
    procedure DeleteActiveIc;
    function IsActiveIc: boolean;
    function AddIcSample(AHeight, AWidth: integer; ACaption: string; ACaptionHorizontal: boolean):TObject;
    procedure StartClockGenerator(AInterval: word; AClockCount: integer);
    procedure StopClockGenerator;
  end;

  TIcTab = class(TTabSheet)
  private
    FPageControl: TPageControl;
    FSimulation: TSimulation;
    FOwner: TObject;
    FName, FCaption: string;
    function GetResolution: TPoint;
    procedure SetResolution(AResolution: TPoint);
  public
    constructor Create(APageControl: TComponent; AName: string; ACaption: string; AOwner: TObject; AIcSamplePointer: TObject); reintroduce;
    destructor Destroy; override;
    procedure SaveToBinaryTag(Tag: TRakBinaryTag);
    procedure LoadFromBinaryTag(Tag: TRakBinaryTag);
    property Resolution: TPoint read GetResolution write SetResolution;
    procedure AddIc(AName, ACaption: string; AIcSamplePointer: TObject);
    procedure DeleteActiveIc;
    function AddIcSample(AHeight, AWidth: integer; ACaption: string; ACaptionHorizontal: boolean):TObject;
  end;

  TElement = class (TObject)
  private

  public
    procedure SaveToBinaryTag(Tag: TRakBinaryTag); virtual; abstract;
    procedure LoadFromBinaryTag(Tag: TRakBinaryTag); virtual; abstract;
  end;

    TLine = class(TElement)
    private
      FStartPoint, FEndPoint: TPoint;
      FStatus: boolean;
      FOwner: TObject;
      FHorizontal: boolean;
    public
      constructor Create(AStartPoint, AEndPoint: TPoint; AOwner: TObject);
      property StartPoint: TPoint read FStartPoint write FStartPoint;
      property EndPoint: TPoint read FEndPoint write FEndPoint;
      property Status: boolean read FStatus write FStatus;
      property Horizontal: boolean read FHorizontal write FHorizontal;
      property Owner: TObject read FOwner write FOwner;
      procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
      procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
      procedure Paint(ACanvas: TCanvas; AVStartPoint: TPoint);
    end;

    TConnector = class(TElement)
    private
      FPosition: TPoint;
      FIsInput: boolean;
      FOrientation: TOrientation;
      FOwner: TObject;
      FStatus: boolean;
      FTempStatus: boolean;
      FNegatived: boolean;
      FCaption: string;
      FDenyNegativationChange, DenyConnection: boolean;
      procedure SetStatus(AStatus: boolean);
      procedure SetNegation(AStatus: boolean);
    public
      constructor Create(APosition: TPoint; AOwner: TObject);
      property Position: TPoint read FPosition write FPosition;
      property IsInput: boolean read FIsInput write FIsInput;
      property Orientation: TOrientation read FOrientation write FOrientation;
      property Owner: TObject read FOwner write FOwner;
      property Status: boolean read FStatus write SetStatus;
      property Negatived: boolean read FNegatived write SetNegation;
      property Caption: string read FCaption write FCaption;
      property DenyNegativationChange: boolean read FDenyNegativationChange write FDenyNegativationChange;
      procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
      procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
      procedure Paint(ACanvas: TCanvas; AVStartPoint: TPoint);
      procedure RefreshStatus;
    end;

    TLineConnector = class(TElement)
    private
      FPosition: TPoint;
      FOwner: TObject;
    public
      constructor Create(APosition: TPoint; AOwner: TObject);
      property Position: TPoint read FPosition write FPosition;
      property Owner: TObject read FOwner write FOwner;
      procedure Paint(ACanvas: TCanvas; AVStartPoint: TPoint);
      procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
      procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
    end;

    TCouplingDiode = class(TElement)
    private
      FPosition: TPoint;
      FOwner: TObject;
    public
      constructor Create(APosition: TPoint; AOwner: TObject);
      property Position: TPoint read FPosition write FPosition;
      property Owner: TObject read FOwner write FOwner;
      procedure Paint(ACanvas: TCanvas; AVStartPoint: TPoint);
      procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
      procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
      procedure CheckStatus(var Changed: boolean);
    end;

    TIcLink = class(TElement)
    private
      FIcModule: TObject;
      FIcTab: TObject;
      //FIcConnectorList: TElementList;
      FOwner: TObject;
    public
      constructor Create(AOwner: TObject);
      destructor Destroy; override;
      procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
      procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
    end;

    TModule = class(TElement)
    private
      FPosition: TPoint;
      FHeight: integer;
      FWidth: integer;
      FOwner: TObject;
      procedure SetHeight(AHeight: integer);
      procedure SetWidth(AWidth: integer);
    public
      constructor Create(APosition: TPoint; AOwner: TObject);
      property Position: TPoint read FPosition write FPosition;
      property Height: integer read FHeight write SetHeight;
      property Width: integer read FWidth write SetWidth;
      property Owner: TObject read FOwner write FOwner;
      procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
      procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
      procedure Paint(ACanvas: TCanvas; AVStartPoint: TPoint); virtual; abstract;
    end;

      TLogicModule = class(TModule)
      private
        FInputConnectorList: TElementList;
        FOutputConnectorList: TElementList;
        FOrientation: TOrientation;
        FCaption: string;
      public
        constructor Create(APosition: TPoint; AOrientation: TOrientation; AOwner: TObject);
        destructor Destroy; override;
        property Caption: string read FCaption write FCaption;
        property Orientation: TOrientation read FOrientation write FOrientation;
        procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
        procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
        procedure Paint(ACanvas: TCanvas; AVStartPoint: TPoint); override;
        procedure AddInputConnector(APosition: TPoint; AOrientation: TOrientation; AOwner: TObject);
        procedure AddOutputConnector(APosition: TPoint; AOrientation: TOrientation; AOwner: TObject);
        procedure ChangedInputStatus(var Changed: boolean); virtual; abstract;
        procedure UpdateStatus;
      end;

        TStandardModule = class(TLogicModule)
        private
          FSize: integer;
        public
          constructor Create(APosition: TPoint; ASize: integer; AOrientation: TOrientation; AOwner: TObject);
          procedure Paint(ACanvas: TCanvas; AVStartPoint: TPoint); override;
          procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
          procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
        end;

          TAndModule = class(TStandardModule)
          private

          public
            constructor Create(APosition: TPoint; ASize: integer; AOrientation: TOrientation; AOwner: TObject);
            procedure ChangedInputStatus(var Changed: boolean); override;
            procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
          end;

          TOrModule = class(TStandardModule)
          private

          public
            constructor Create(APosition: TPoint; ASize: integer; AOrientation: TOrientation; AOwner: TObject);
            procedure ChangedInputStatus(var Changed: boolean); override;
            procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
          end;

          TXorModule = class(TStandardModule)
          private

          public
            constructor Create(APosition: TPoint; ASize: integer; AOrientation: TOrientation; AOwner: TObject);
            procedure ChangedInputStatus(var Changed: boolean); override;
            procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
          end;

        TInverterModule = class(TlogicModule)
        private

        public
          constructor Create(APosition: TPoint; AOrientation: TOrientation; AOwner: TObject);
          procedure Paint(ACanvas: TCanvas; AVStartPoint: TPoint); override;
          procedure ChangedInputStatus(var Changed: boolean); override;
          procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
        end;

        TDelayModule = class(TLogicModule)
        private

        public
          constructor Create(APosition: TPoint; AOrientation: TOrientation; AOwner: TObject);
          procedure Paint(ACanvas: TCanvas; AVStartPoint: TPoint); override;
          procedure ChangedInputStatus(var Changed: boolean); override;
          procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
        end;

        THalfAdderModule = class(TLogicModule)
        private

        public
          constructor Create(APosition: TPoint; AOrientation: TOrientation; AOwner: TObject);
          procedure Paint(ACanvas: TCanvas; AVStartPoint: TPoint); override;
          procedure ChangedInputStatus(var Changed: boolean); override;
          procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
        end;

        TFullAdderModule = class(TLogicModule)
        private

        public
          constructor Create(APosition: TPoint; AOrientation: TOrientation; AOwner: TObject);
          procedure Paint(ACanvas: TCanvas; AVStartPoint: TPoint); override;
          procedure ChangedInputStatus(var Changed: boolean); override;
          procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
        end;

        TFlipFlopModule = class(TLogicModule)
        private
          FInputMaster: Record
            J, C, K: boolean;
          end;
        public
          constructor Create(APosition: TPoint; AOrientation: TOrientation; AOwner: TObject);
          procedure Paint(ACanvas: TCanvas; AVStartPoint: TPoint); override;
          procedure ChangedInputStatus(var Changed: boolean); override;
          procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
          procedure Reset;
          procedure SwapOutput;
        end;

        TDecoderModule = class(TLogicModule)
        private

        public
          constructor Create(APosition: TPoint; AOrientation: TOrientation; AOwner: TObject);
          procedure Paint(ACanvas: TCanvas; AVStartPoint: TPoint); override;
          
        end;

          TDecoderNoClockModule = class(TDecoderModule)
          private
            
          public
            procedure ChangedInputStatus(var Changed: boolean); override;
            procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
          end;

          TDecoderClockModule = class(TDecoderModule)
          private
            FInputMaster: Record
              Clock: boolean;
              OutputSum: integer;
            end;
          public
            constructor Create(APosition: TPoint; AOrientation: TOrientation; AOwner: TObject);
            procedure Paint(ACanvas: TCanvas; AVStartPoint: TPoint); override;
            procedure ChangedInputStatus(var Changed: boolean); override;
            procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
            procedure Reset;
          end;

        TShiftingRegisterModule = class(TLogicModule)
        private
          FInputMaster: Record
            Inp0, Inp2, Clock: boolean;
          end;
        public
          constructor Create(APosition: TPoint; AOrientation: TOrientation; AOwner: TObject);
          procedure Paint(ACanvas: TCanvas; AVStartPoint: TPoint); override;
          procedure ChangedInputStatus(var Changed: boolean); override;
          procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
          procedure Reset;
        end;

        TClockGeneratorModule = class(TLogicModule)
        private
          tiDelay: TTimer;
          FClockCount: integer;
          procedure tiDelayTimer(Sender: TObject);
        public
          constructor Create(APosition: TPoint; AOrientation: TOrientation; AOwner: TObject);
          destructor Destroy; override;
          property ClockCount: integer read FClockCount write FClockCount;
          procedure Paint(ACanvas: TCanvas; AVStartPoint: TPoint); override;
          procedure ChangedInputStatus(var Changed: boolean); override;
          procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
          procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
          procedure Reset;
        end;

        TIc = class(TLogicModule)
        private
          FName: string;
          FCaptionHorizontal: boolean;
        public
          property CaptionHorizontal: boolean read FCaptionHorizontal write FCaptionHorizontal;
          procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
          procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
        end;

          TIcModule = class(TIc)
          private
            FIcSamplePointer: TObject;
            FIndex: integer;
          public
            constructor Create(APosition: TPoint; AOwner: TObject);
            destructor Destroy; override;
            procedure Paint(ACanvas: TCanvas; AVStartPoint: TPoint); override;
            procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
            procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
            procedure ChangedInputStatus(var Changed: boolean); override;
          end;

          TIcSample = class(TIc)
          private
            FIcLinkList: TElementList;
            FMainBox: TObject;
            function FindIcTabByName(AName: string):TObject;                                            
          public                                                                                     // AOwner = Simulation
            constructor Create(AHeight, AWidth: integer; ACaption: string; ACaptionHorizontal: boolean; AOwner: TObject); overload;
            constructor Create(AHeight, AWidth: integer; ACaption: string; ACaptionHorizontal: boolean; AOwner: TObject; Load: boolean); overload;
            destructor Destroy; override;
            procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
            procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
            procedure ChangedInputStatus(var Changed: boolean); override;
            function CreateModule(APosition: TPoint; AElementBox: TObject):TIcModule;
            procedure AddInputConnector(ARelHeight: integer; ACaption: string);
            procedure AddOutputConnector(ARelHeight: integer; ACaption: string);
            procedure AddModule(AModule: TIcModule);
            procedure RemoveModule(AIndex: integer);
            procedure MatchIcData(AIndex: integer);
          end;

      TStatusModule = class(TModule)
      private
        FStatus: boolean;
      public
        constructor Create(APosition: TPoint; AOwner: TObject);
        destructor Destroy; override;
        property Status: boolean read FStatus write FStatus;
      end;

        TSwitchModule = class(TStatusModule)
        private

        public
          procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
          procedure Paint(ACanvas: TCanvas; AVStartPoint: TPoint); override;
        end;

        TLedModule = class(TStatusModule)
        private
          FColor: TColor;
        public
          constructor Create(APosition: TPoint; AColor: TColor; AOwner: TObject);
          property Color: TColor read FColor write FColor;
          procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
          procedure Paint(ACanvas: TCanvas; AVStartPoint: TPoint); override;
          procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
        end;

      TTextModule = class(TModule)
      private
        FCaption: string;
        FColor: TColor;
        FSize: integer;
      public
        constructor Create(APosition: TPoint; ACaption: string; ASize: integer; AColor: TColor; AOwner: TObject);
        property Caption: string read FCaption write FCaption;
        property Color: TColor read FColor write FColor;
        property Size: integer read FSize write FSize;
        procedure Paint(ACanvas: TCanvas; AVStartPoint: TPoint); override;
        procedure SaveToBinaryTag(Tag: TRakBinaryTag); override;
        procedure LoadFromBinaryTag(Tag: TRakBinaryTag); override;
      end;


var
  gAbort:       boolean = false;
  gPaintGrid:   boolean = true;
  gHasChanged:  boolean = false;
  gSlowRePaint: boolean = false;
  gCompress:    boolean = true;
  gAutoRePaint: boolean = false;

  gOrientation: TOrientation = orLeft;

  gDistance:    integer = 17;
  gSize:        integer = 2;
  gIcIndex:     integer = -1;
  gColor:       integer = clLime;
  gTextSize:    integer = 8;

  gTextCaption: string = ''; 

  gActAction:   TActAction = None;
  gBmp:         TBitMap = Nil;




implementation              {implementation}

uses Haupt, OptionsClockGenerator, DynTextEdit;

var GIterating: boolean;

function CompareStreams(const A, B : TStream): Boolean;
const
   BUFSIZE = 8192;
var
   bufA, bufB : string;
   r : Integer;
begin
   if A.Size <> B.Size then
      Result := False
   else
   begin
      A.Position := 0;
      B.Position := 0;

      SetLength(bufA, BUFSIZE);
      SetLength(bufB, BUFSIZE);

      while A.Position < A.Size do
      begin
         r := A.Size - A.Position;
         // da die Properties [TStream].Size & .Position ab Delphi 6
         // als Int64 deklariert sind, könnte ein Überlauf im Integer "r" auftreten.
         // dies wird berücksichtigt durch "r<=0"
         if (r > BUFSIZE) or (r<=0) then
            r := BUFSIZE;
         A.ReadBuffer(bufA[1], r);
         B.ReadBuffer(bufB[1], r);

         if not CompareMem(PChar(bufA), PChar(bufB), r) then
         begin
            Result := False;
            Exit;
         end;
      end;
      Result := True;
   end;
end;

{ Compress a stream }
function CompressStream(inpStream, outStream: TStream): boolean;
var
  InpBuf, OutBuf: Pointer;
  InpBytes, OutBytes: Integer;
begin
  InpBuf := nil;
  OutBuf := nil;
  try
    GetMem(InpBuf, inpStream.Size);
    inpStream.Position := 0;
    InpBytes := inpStream.Read(InpBuf^, inpStream.Size);
    CompressBuf(InpBuf, InpBytes, OutBuf, OutBytes);
    outStream.Write(OutBuf^, OutBytes);
  finally
    if InpBuf <> nil then FreeMem(InpBuf);
    if OutBuf <> nil then FreeMem(OutBuf);
  end;
  inpStream.Position := 0;
  outStream.Position := 0;
  if outStream.Size < inpStream.Size then
    Result := true
  else Result := false;    
end;


{ Decompress a stream }
function DecompressStream(inpStream, outStream: TStream): boolean;
var
  InpBuf, OutBuf: Pointer;
  OutBytes, sz: Integer;
  Head: Word;
begin
  Result := false;
  InpBuf := nil;
  OutBuf := nil;
  sz     := inpStream.Size - inpStream.Position;
  if sz > 0 then
  try
    inpStream.Read(Head,sizeof(Head));
    if Head = $DA78 then
    begin
      inpStream.Position := 0;
      Getmem(InpBuf, sz);
      inpStream.Read(InpBuf^, sz);
      DecompressBuf(InpBuf, sz, 0, OutBuf, OutBytes);
      outStream.Write(OutBuf^, OutBytes);
      Result := true;
    end;
  finally
    if InpBuf <> nil then FreeMem(InpBuf);
    if OutBuf <> nil then FreeMem(OutBuf);
    end;
  inpStream.Position := 0;
  outStream.Position := 0;
end;

  {TElementList}

  procedure TElementList.SaveToBinaryTag(Tag: TRakBinaryTag);
  var I: integer;
  begin
   for I := 0 to Count - 1 do
    if Items[I] is TElement then
      (Items[I] as TElement).SaveToBinaryTag(Tag.AddTag('Element'));
  end;

  procedure TElementList.LoadFromBinaryTag(Tag: TRakBinaryTag; AOwner: TObject);
  var AElement: TElement;
      TagList: IRakBinaryTagList;
      I: integer;
      AName: string;
  begin
    Clear;
    TagList := Tag.FindTagsByName('Element');
    for I := 0 to TagList.Count - 1 do
    begin
      AName := TagList.Items(I).AttributeAsString('Name','');
      if AName =
      'TLine' then begin
					AElement := TLine.Create(point(0,0),point(0,0),AOwner);
					(AElement as TLine).LoadFromBinaryTag(TagList.Items(I));
					Add(AElement);
				end else if AName =
      'TCouplingDiode' then begin
					AElement := TCouplingDiode.Create(point(0,0),AOwner);
					(AElement as TCouplingDiode).LoadFromBinaryTag(TagList.Items(I));
					Add(AElement);
				end else if AName =
      'TConnector' then begin
					AElement := TConnector.Create(point(0,0), AOwner);
					AElement.LoadFromBinaryTag(TagList.Items(I));
					Add(AElement);
				end else if AName =
      'TAndModule' then begin
					AElement := TAndModule.Create(point(0,0), 0, orLeft, AOwner);
					AElement.LoadFromBinaryTag(TagList.Items(I));
					Add(AElement);
				end else if AName =
      'TOrModule' then begin
					AElement := TOrModule.Create(point(0,0), 0, orLeft, AOwner);
					AElement.LoadFromBinaryTag(TagList.Items(I));
					Add(AElement);
				end else if AName =
      'TXorModule' then begin
					AElement := TXorModule.Create(point(0,0), 0, orLeft, AOwner);
					AElement.LoadFromBinaryTag(TagList.Items(I));
					Add(AElement);
				end else if AName =
      'TSwitchModule' then begin
					AElement := TSwitchModule.Create(point(0,0), AOwner);
					AElement.LoadFromBinaryTag(TagList.Items(I));
					Add(AElement);
				end else if AName =
      'TLedModule' then begin
					AElement := TLedModule.Create(point(0,0), clnone, AOwner);
					AElement.LoadFromBinaryTag(TagList.Items(I));
					Add(AElement);
				end else if AName =
      'TTextModule' then begin
					AElement := TTextModule.Create(point(0,0), '', 0, clnone, AOwner);
					AElement.LoadFromBinaryTag(TagList.Items(I));
					Add(AElement);
				end else if AName =
      'TInverterModule' then begin
          AElement := TInverterModule.Create(point(0,0), orLeft, AOwner);
          AElement.LoadFromBinaryTag(TagList.Items(I));
          Add(AElement);
        end else if AName =
      'TDelayModule' then begin
          AElement := TDelayModule.Create(point(0,0), orLeft, AOwner);
          AElement.LoadFromBinaryTag(TagList.Items(I));
          Add(AElement);
        end else if AName =
      'THalfAdderModule' then begin
          AElement := THalfAdderModule.Create(point(0,0), orLeft, AOwner);
          AElement.LoadFromBinaryTag(TagList.Items(I));
          Add(AElement);
        end else if AName =
      'TFullAdderModule' then begin
          AElement := TFullAdderModule.Create(point(0,0), orLeft, AOwner);
          AElement.LoadFromBinaryTag(TagList.Items(I));
          Add(AElement);
        end else if AName =
      'TFlipFlopModule' then begin
          AElement := TFlipFlopModule.Create(point(0,0), orLeft, AOwner);
          AElement.LoadFromBinaryTag(TagList.Items(I));
          Add(AElement);
        end else if AName =
      'TDecoderNoClockModule' then begin
          AElement := TDecoderNoClockModule.Create(point(0,0), orLeft, AOwner);
          AElement.LoadFromBinaryTag(TagList.Items(I));
          Add(AElement);
        end else if AName =
      'TDecoderClockModule' then begin
          AElement := TDecoderClockModule.Create(point(0,0), orLeft, AOwner);
          AElement.LoadFromBinaryTag(TagList.Items(I));
          Add(AElement);
        end else if AName =
      'TShiftingRegisterModule' then begin
          AElement := TShiftingRegisterModule.Create(point(0,0), orLeft, AOwner);
          AElement.LoadFromBinaryTag(TagList.Items(I));
          Add(AElement);
        end else if AName =
      'TClockGeneratorModule' then begin
          AElement := TClockGeneratorModule.Create(point(0,0), orLeft, AOwner);
          AElement.LoadFromBinaryTag(TagList.Items(I));
          Add(AElement);
        end else if AName =
      'TIcSample' then begin
          AElement := TIcSample.Create(0, 0, '', true, AOwner, true);
          AElement.LoadFromBinaryTag(TagList.Items(I));
          Add(AElement);
				end else if AName =
      'TIcLink' then begin
          AElement := TIcLink.Create(AOwner);
          AElement.LoadFromBinaryTag(TagList.Items(I));
          Add(AElement);
				end;
    end;
  end;

  {TElementBox}

  function TElementBox.ModuleCollision(AObject: TObject): boolean;

    function IsPointOnBorder(AModule: TModule; APoint: TPoint): boolean;
    begin
      Result := false;
      if (((APoint.X = AModule.Position.X) or (APoint.X = AModule.Position.X + AModule.Width))
        and (APoint.Y >= AModule.Position.Y) and (APoint.Y <= AModule.Position.Y + AModule.Height))
        or (((APoint.Y = AModule.Position.Y) or (APoint.Y = AModule.Position.Y + AModule.Height))
        and (APoint.X >= AModule.Position.X) and (APoint.X <= AModule.Position.X + AModule.Width))
      then Result := true;
    end;

    function FindConnectorAtModule(AModule: TLogicModule; APoint: TPoint):TObject;
    var I: integer;
        Found: boolean;
    begin
      Result := Nil;
      Found := false;

      I := AModule.FInputConnectorList.Count - 1;
      while not Found and (I > -1) do
      begin
        with AModule.FInputConnectorList[I] as TConnector do
          if (Position.X = APoint.X) and (Position.Y = APoint.Y) then
          begin
            Result := AModule.FInputConnectorList[I];
            Found := true;
          end;
        dec(I);
      end;

      I := AModule.FOutputConnectorList.Count - 1;
      while not Found and (I > -1) do
      begin
        with AModule.FOutputConnectorList[I] as TConnector do
          if (Position.X = APoint.X) and (Position.Y = APoint.Y) then
          begin
            Result := AModule.FOutputConnectorList[I];
            Found := true;
          end;
        dec(I);
      end;
     { for I := 0 to AModule.FInputConnectorList.Count - 1 do
        with AModule.FInputConnectorList[I] as TConnector do
          if (Position.X = APoint.X) and (Position.Y = APoint.Y) then
            Result := true;
      for I := 0 to AModule.FOutputConnectorList.Count - 1 do
        with AModule.FOutputConnectorList[I] as TConnector do
          if (Position.X = APoint.X) and (Position.Y = APoint.Y) then
            Result := true;   }
    end;

{  var I: integer;
      ModuleN, ModuleI: TModule;
      LineI: TLine;
  begin
    Result := false;
    ModuleN := AObject as TModule;

    for I := 0 to FModuleList.Count - 1 do      // BodyCollision for all
    begin
      ModuleI := FModuleList[i] as TModule;
      if (((ModuleN.Position.X >= ModuleI.Position.X)      // TopLeft Corner
        and (ModuleN.Position.X <= ModuleI.Position.X + ModuleI.Width)
        and (ModuleN.Position.Y >= ModuleI.Position.Y)
        and (ModuleN.Position.Y <= ModuleI.Position.Y + ModuleI.Height))
        or ((ModuleN.Position.X + ModuleN.Width >= ModuleI.Position.X)  // BottonRight Corner
        and (ModuleN.Position.X + ModuleN.Width <= ModuleI.Position.X + ModuleI.Width)
        and (ModuleN.Position.Y + ModuleN.Height >= ModuleI.Position.Y)
        and (ModuleN.Position.Y + ModuleN.Height <= ModuleI.Position.Y + ModuleI.Height)))

        or (((ModuleI.Position.X >= ModuleN.Position.X)      // TopLeft Corner
        and (ModuleI.Position.X <= ModuleN.Position.X + ModuleN.Width)
        and (ModuleI.Position.Y >= ModuleN.Position.Y)
        and (ModuleI.Position.Y <= ModuleN.Position.Y + ModuleN.Height))
        or ((ModuleI.Position.X + ModuleI.Width >= ModuleN.Position.X)  // BottonRight Corner
        and (ModuleI.Position.X + ModuleI.Width <= ModuleN.Position.X + ModuleN.Width)
        and (ModuleI.Position.Y + ModuleI.Height >= ModuleN.Position.Y)
        and (ModuleI.Position.Y + ModuleI.Height <= ModuleN.Position.Y + ModuleN.Height)))
      then Result := true;
    end;

    if AObject is TLogicModule then             // LineCollision only for LogicModules
      for I := 0 to FLineList.Count - 1 do
      begin
        LineI := FLineList[I] as TLine;
        if LineI.Horizontal then               // Horizontal
        begin
          if IsPointOnBorder(ModuleN, LineI.StartPoint) then
            if not ConnectorExists(AObject as TLogicModule, LineI.StartPoint) then
              Result := true;
          if IsPointOnBorder(ModuleN, LineI.EndPoint) then
            if not ConnectorExists(AObject as TLogicModule, LineI.EndPoint) then
              Result := true;
        end else begin                         // Vertical
          if (LineI.StartPoint.X >= ModuleN.Position.X)
            and (LineI.StartPoint.X <= ModuleN.Position.X + ModuleN.Width)
            and (LineI.StartPoint.Y <= ModuleN.Position.Y + ModuleN.Height)
            and (LineI.EndPoint.Y >= ModuleN.Position.Y)
          then Result := true;

        end;
      end;          }

  var I, XI, YI, XII, YII: integer;
    AConnector: TObject;
  begin
    Result := false;
    with AObject as TModule do
      for XI := Position.X to Position.X + Width do
        for YI := Position.Y to Position.Y + Height do
        begin
          for I := 0 to FModuleList.Count - 1 do          // other Modules
            with FModuleList[i] as TModule do
              for XII := Position.X to Position.X + Width do
                for YII := Position.Y to Position.Y + Height do
                  if (XI = XII) and (YI  = YII) then
                    Result := true;
          if not (AObject is TStatusModule) then         // StatusModules no LineCollision
            if AObject is TLogicModule then
              for I := 0 to FLineList.Count - 1 do          // Lines
                with FLineList[I] as TLine do
                  for XII := StartPoint.X to EndPoint.X do
                    for YII := StartPoint.Y to EndPoint.Y do
                      if (XI = XII) and (YI = YII) then
                      begin
                        AConnector := FindConnectorAtModule(AObject as TLogicModule,point(XII, YII));
                        if AConnector = Nil then
                          Result := true
                        else if (AConnector as TConnector).Orientation in [orLeft, orRight] then
                          if not Horizontal then
                            Result := true
                          else
                        else if Horizontal then
                            Result := true;
                      end;
                             
      end;
  end;

  function TElementBox.LineCollision(AStartPoint: TPoint; AEndPoint: TPoint): boolean;
  var XI, YI, XII, YII, I: integer;
    AConnector: TObject;
  begin
    Result := false;
    for XI := AStartPoint.X to AEndPoint.X do
      for YI := AStartPoint.Y to AEndPoint.Y do
        for I := 0 to FModuleList.Count - 1 do
          with FModuleList[I] as TModule do
            for XII := Position.X to Position.X + Width do
              for YII := Position.Y to Position.Y + Height do
                if (XI = XII) and (YI = YII) then
                  if not (FModuleList[I] is TStatusModule)  then
                  begin
                    AConnector := FindConnector(point(XI,YI));
                    if AConnector = Nil then   // Connector Found?
                      Result := true
                    else if (AConnector as TConnector).DenyConnection or   // Connection Denied?
                      not (((AEndPoint.X = XII) and (AEndPoint.Y = YII))   // Connector Start- or EndPoint?
                      or ((AStartPoint.X = XII) and (AStartPoint.Y = YII))) then
                        Result := true
                      else if ((AConnector as TConnector).Orientation in [orLeft, orRight]) then
                        if AStartPoint.X = AEndPoint.X then  // vertical
                          Result := true
                        else
                      else if AStartPoint.Y = AEndPoint.Y then  // horizontal
                        Result := true;
                  end else if (FModuleList[I] is TSwitchModule) then             //switch
                    if not ((((AStartPoint.X = XII) and (AStartPoint.Y = YII))
                      or ((AEndPoint.X = XII) and (AEndPoint.Y = YII)))
                      and (LineCount(point(XII, YII)) < 1)) then
                      Result := true

  end;

  function TElementBox.FindStatusModule(APosition: TPoint; ACount: integer): TObject;
  var I, ItemCount: integer;
      Found: boolean;
  begin
    Result := Nil;
    Found := false;
    ItemCount := 0;
    I := 0;
    while not Found and (I < FModuleList.Count) do begin
      if FModuleList[I] is TStatusModule then
        if ((FModuleList[I] as TStatusModule).Position.X = APosition.X)
          and ((FModuleList[I] as TStatusModule).Position.Y = APosition.Y) then
        begin
          inc(ItemCount);
          if ACount = ItemCount then
          begin
            Result := FModuleList[i];
            Found := true;
          end;
        end;
      inc(I);
    end;
  end;

  function TElementBox.FindConnector(APosition: TPoint):TObject;
  var I, II: integer;
      Found: boolean;
  begin
    Result := Nil;
    Found := false;
    I := 0;
    while not Found and (I < FModuleList.Count) do        // Objectloop
    begin
      if FModuleList[i] is TLogicModule then
        with (FModuleList[i] as TLogicModule) do
        begin
          II := 0;
          while not Found and (II < FInputConnectorList.Count) do  // Input
          begin
            if ((FInputConnectorList[II] as TConnector).Position.X = APosition.X)
              and ((FInputConnectorList[II] as TConnector).Position.Y = APosition.Y) then
            begin
              Result := FInputConnectorList[II];
              Found := true;
            end;
            inc(II);
          end;
          II := 0;
          while not Found and (II < FOutputConnectorList.Count) do // Output
          begin
            if ((FOutputConnectorList[II] as TConnector).Position.X = APosition.X)
              and ((FOutputConnectorList[II] as TConnector).Position.Y = APosition.Y) then
            begin
              Result := FOutputConnectorList[II];
              Found := true;
            end;
            inc(II);
          end;
        end;
      inc(I);
    end;
  end;

  function TElementBox.FindCouplingDiode(APosition: TPoint):boolean;
  begin
    if FindCouplingDiodeO(APosition) = Nil then
      Result := false
    else Result := true;
  end;

  function TElementBox.FindCouplingDiodeO(APosition: TPoint):TObject;
  var I: integer;
  begin
    Result := Nil;
    for I := 0 to FLineCouplingDiodeList.Count - 1 do
      if ((FLineCouplingDiodeList[I] as TCouplingDiode).Position.X = APosition.X)
        and ((FLineCouplingDiodeList[I] as TCouplingDiode).Position.Y = APosition.Y) then
        Result := FLineCouplingDiodeList[I];
  end;

  function TElementBox.FindLinePoint(APosition: TPoint; ACount: integer):TObject;
  var I, ItemCount: integer;
      Found: boolean;
  begin
    Result := Nil;
    if ACount < 5 then
    begin
      Found := false;
			ItemCount := 0;
			I := 0;
			while not Found and (I < FLineList.Count) and (ItemCount < 4) do
			begin
				with FLineList[I] as TLine do
					if ((StartPoint.X = APosition.X) and (StartPoint.Y = APosition.Y))
						or ((EndPoint.X = APosition.X) and (EndPoint.Y = APosition.Y)) then
					begin
						inc(ItemCount);
						if ACount = ItemCount then
						begin
							Result := FLineList[I];
							Found := true;
						end;
					end;
				inc(I);
			end;
    end;
  end;

  function TElementBox.FindLineNewLineIsCompleteIn(AStartPoint: TPoint; AEndPoint: TPoint): TObject;
  (* searches a line, the new line (given with AStart- and AEndPoint) is complete in    *)
  (* the endpoints are included!                                                        *)
  (* Result is the Line, wich includes the new line                                     *)
  var Horizontal, Found: boolean;
    I: integer;
  begin
    Result := Nil;
    Found := false;
    Horizontal := AStartPoint.Y = AEndPoint.Y;
    I := FLineList.Count - 1;
    while not Found and (I > -1) do
    begin
      if ((FLineList[I] as TLine).Horizontal = Horizontal) and (
        (AStartPoint.X >= (FLineList[I] as TLine).StartPoint.X)
        and (AStartPoint.Y >= (FLineList[I] as TLine).StartPoint.Y)
        and (AEndPoint.X <= (FLineList[I] as TLine).EndPoint.X)
        and (AEndPoint.Y <= (FLineList[I] as TLine).EndPoint.Y) ) then
      begin
        Result := FLineList[I];
        Found := true;
      end;
      dec(I);
    end;

  {var I, XI, YI: integer;
    IsStartPointInLine, IsEndPointInLine,
  begin
    Result := Nil;
    for I := 0 to FLineList.Count - 1 do
    begin
      IsStartPointInLine := false;
      IsEndPointInLine := false;
      for XI := (FLineList[I] as TLine).StartPoint.X to (FLineList[I] as TLine).EndPoint.X do
        for YI := (FLineList[I] as TLine).StartPoint.Y to (FLineList[I] as TLine).EndPoint.Y do
        begin
          if (AStartPoint.X  = XI) and (AStartPoint.Y = YI) then
            IsStartPointInLine := true;
          if (AEndPoint.X  = XI) and (AEndPoint.Y = YI) then
            IsEndPointInLine := true;
        end;
      if IsStartPointInLine and IsEndPointInLine then
        Result := FLineList[I];
    end;   }
  end;

  function TElementBox.FindPointInLine(AStartPoint, AEndPoint: TPoint): TPoint;
  (* This function searches for points (StartPoints or EndPoints of other Lines *)
  (* or TStatusModules), which would be in the new Line                         *)
  (* Endpoints are excludet!                                                    *)
  (* Result is the coordiante of the Point, wich is in the new line             *)
  var I: integer;
    Found, Horizontal: boolean;
  begin
    Result := point(-1,-1);
    Found := false;
    Horizontal := AStartPoint.Y = AEndPoint.Y;

    I := FLineList.Count - 1;
    while not Found and (I > -1) do
    begin
      if Horizontal then                // new line is horizontal
        if (AStartPoint.Y = (FLineList[I] as TLine).StartPoint.Y)         // -+-----
          and ((FLineList[I] as TLine).StartPoint.X > AStartPoint.X)      //  |
          and ((FLineList[I] as TLine).StartPoint.X < AEndPoint.X) then   //  |
          Result := (FLineList[I] as TLine).StartPoint
        else if (AStartPoint.Y = (FLineList[I] as TLine).EndPoint.Y)
          and ((FLineList[I] as TLine).EndPoint.X > AStartPoint.X)        //     |
          and ((FLineList[I] as TLine).EndPoint.X < AEndPoint.X) then     //     |
          Result := (FLineList[I] as TLine).EndPoint                      // ----+--
        else                            // new line is vertical
      else if (AStartPoint.X = (FLineList[I] as TLine).StartPoint.X)      //  |
        and ((FLineList[I] as TLine).StartPoint.Y > AStartPoint.Y)        //  +-----
        and ((FLineList[I] as TLine).StartPoint.Y < AEndPoint.Y) then     //  |
        Result := (FLineList[I] as TLine).StartPoint
      else if (AStartPoint.X = (FLineList[I] as TLine).EndPoint.X)
        and ((FLineList[I] as TLine).EndPoint.Y > AStartPoint.Y)          //       |
        and ((FLineList[I] as TLine).EndPoint.Y < AEndPoint.Y) then       //  -----+
        Result := (FLineList[I] as TLine).EndPoint;                       //       |
      dec(I);
    end;

    I := FModuleList.Count - 1;
    while not Found and (I > -1) do
    begin                                 
      if FModuleList[I] is TStatusModule then
        if Horizontal then              // new line is horizontal
          if ((FModuleList[I] as TStatusModule).Position.Y = AStartPoint.Y)       //
            and ((FModuleList[I] as TStatusModule).Position.X > AStartPoint.X)    // ---+---
            and ((FModuleList[I] as TStatusModule).Position.X < AEndPoint.X) then //
            Result := (FModuleList[I] as TStatusModule).Position
          else                          // new line is vertical
        else if ((FModuleList[I] as TStatusModule).Position.X = AStartPoint.X)
          and ((FModuleList[I] as TStatusModule).Position.Y > AStartPoint.Y)      //  |
          and ((FModuleList[I] as TStatusModule).Position.Y < AEndPoint.Y) then   //  +
          Result := (FModuleList[I] as TStatusModule).Position;                   //  |
      dec(I);
    end;

  end;
  {var I, XI, YI: integer;
  begin
    Result := point(-1,-1);
    for I := 0 to FLineList.Count - 1 do
      for XI := AStartPoint.X to AEndPoint.X do
        for YI := AStartPoint.Y to AEndPoint.Y do
          if not (((AStartPoint.X = XI) and (AStartPoint.Y = YI))
              or ((XI = AEndPoint.X) and (YI = AEndPoint.Y))) then
          begin                        // no Start or Endpoints
            if ((FLineList[I] as TLine).StartPoint.X = XI)
                and ((FLineList[I] as TLine).StartPoint.Y = YI) then
              Result := (FLineList[I] as TLine).StartPoint;
            if ((FLineList[I] as TLine).EndPoint.X = XI)
                and ((FLineList[I] as TLine).EndPoint.Y = YI) then
              Result := (FLineList[I] as TLine).EndPoint;
          end;
    for I := 0 to FModuleList.Count - 1 do
      for XI := AStartPoint.X to AEndPoint.X do
        for YI := AStartPoint.Y to AEndPoint.Y do
          if not (((AStartPoint.X = XI) and (AStartPoint.Y = YI))
              or ((XI = AEndPoint.X) and (YI = AEndPoint.Y))) then
          begin                       // no Start or Endpoints
            if FModuleList[I] is TStatusModule then
              with FModuleList[I] as TStatusModule do
                if (Position.X = XI) and (Position.Y = YI) then
                  Result := Position;
          end;
  end;                            }

  function TElementBox.FindLineThroughPoint(APoint: TPoint): TObject;
  (* This function is searching a line, on which the point (APoint) lies   *)
  (* Result is a pointer on that line                                      *)
  var I: integer;
    Found: boolean;
  begin
    Result := Nil;
    Found := false;
    I :=  0;
    while not Found and (I < FLineList.Count) do
    begin
      if (FLineList[I] as TLine).Horizontal then     // horizontal
        if ((FLineList[I] as TLine).StartPoint.Y = APoint.Y)        //
          and (APoint.X > (FLineList[I] as TLine).StartPoint.X)     // --+----
          and (APoint.X < (FLineList[I] as TLine).EndPoint.X) then  //
          Result := FLineList[I]                     
        else                                         // vertical
      else if ((FLineList[I] as TLine).StartPoint.X = APoint.X)     //   |
        and (APoint.Y > (FLineList[I] as TLine).StartPoint.Y)       //   +
        and (APoint.Y < (FLineList[I] as TLine).EndPoint.Y) then    //   |
        Result := FLineList[I];        
      inc(I);
    end;
  end;
 { var I, XI, YI: integer;
  begin
    Result := Nil;
    for I := 0 to FLineList.Count - 1 do
      with FLineList[I] as TLine do
        for XI := StartPoint.X to EndPoint.X do
          for YI := StartPoint.Y to EndPoint.Y do
            if not (((StartPoint.X = XI) and (StartPoint.Y = YI))
                or ((XI = EndPoint.X) and (YI = EndPoint.Y))) then
              if (XI = APoint.X) and (YI = APoint.Y) then
                Result := FLineList[I];
  end;           }

  function TElementBox.LineCount(APosition: TPoint): integer;
  var I: integer;
  begin
    Result := 0;
    for I := 0 to FLineList.Count - 1 do
      with FLineList[I] as TLine do
        if ((StartPoint.X = APosition.X) and (StartPoint.Y = APosition.Y))
            or ((EndPoint.X = APosition.X) and (EndPoint.Y = APosition.Y)) then
          inc(Result);
  end;

  function TElementBox.FindLineCompleteInNewLine(AStartPoint: TPoint; AEndPoint: TPoint): TObject;
  var I, XI, YI: integer;
      IsStartPointInLine, IsEndPointInLine: boolean;
  begin
    Result := Nil;
    for I := 0 to FLineList.Count - 1 do
    begin
      IsStartPointInLine := false;
      IsEndPointInLine := false;
      for XI := AStartPoint.X to AEndPoint.X do
        for YI := AStartPoint.Y to AEndPoint.Y do
        begin
          if ((FLineList[I] as TLine).StartPoint.X  = XI) and ((FLineList[I] as TLine).StartPoint.Y = YI) then
            IsStartPointInLine := true;
          if ((FLineList[I] as TLine).EndPoint.X  = XI) and ((FLineList[I] as TLine).EndPoint.Y = YI) then
            IsEndPointInLine := true;
        end;
      if IsStartPointInLine and IsEndPointInLine then
        Result := FLineList[I];
    end;
  end;

  function TElementBox.FindLineExtension(AStartPoint: TPoint; AEndPoint: TPoint): integer;
  var I: integer;
      AHorizontal: boolean;
  begin
    Result := -1;
    AHorizontal := AStartPoint.Y = AEndPoint.Y;
    
    for I := 0 to FLineList.Count - 1 do
      with FLineList[I] as TLine do
        if ((StartPoint.X = AEndPoint.X) and (StartPoint.Y = AEndPoint.Y)
            and (Horizontal = AHorizontal) and (LineCount(AEndPoint) = 1)
            and (FindStatusModule(AEndPoint, 1) = Nil))
            or
            ((EndPoint.X = AStartPoint.X) and (EndPoint.Y = AStartPoint.Y)
             and (Horizontal = AHorizontal) and (LineCount(AStartPoint) = 1)
            and (FindStatusModule(AStartPoint, 1) = Nil)) then
          Result := I;
  end;

  function TElementBox.AreLinesCrossing(APosition: TPoint): boolean;
  var Count, I, XI, YI: integer;
  begin
    Result := false;
    Count := 0;
    for I := 0 to FLineList.Count - 1 do
      with FLineList[I] as TLine do
        for XI := StartPoint.X to EndPoint.X do
          for YI := StartPoint.Y to EndPoint.Y do
            if not (((XI = StartPoint.X) and (YI = StartPoint.Y)) or ((XI = EndPoint.X) and (YI = EndPoint.Y))) then
              if (XI = APosition.X) and (YI = APosition.Y) then
                inc(Count);
    if Count > 1 then
      Result := true;
  end;

  function TElementBox.CountLinesSameHorizontal(APosition: TPoint; AHorizontal: Boolean): integer;
  (* counts Lines at APoint with same AHorizontal                     *)
  var I: integer;
  begin
    Result := 0;
    I := 1;
    while FindLinePoint(APosition, I) <> Nil do
    begin
      if (FindLinePoint(APosition, I) as TLine).Horizontal = AHorizontal then
        inc(Result);
      inc(I);
    end;
  end;

  procedure TElementBox.SetConnectingLinesBetweenTwoConnectors(ALogicModule: TObject);

    procedure CheckConnector(AConnector: TConnector);
    var nConnector: TConnector;
    begin
      with AConnector do
        case Orientation of
          orLeft: begin
            nConnector := FindConnector(point(Position.X - 1, Position.Y)) as TConnector;
            if (nConnector <> Nil) and (IsInput xor nConnector.IsInput) and (nConnector.Orientation = orRight) then
              AddLine(point(Position.X - 1, Position.Y), Position);
          end;
          orRight: begin
            nConnector := FindConnector(point(Position.X + 1, Position.Y)) as TConnector;
            if (nConnector <> Nil) and (IsInput xor nConnector.IsInput) and (nConnector.Orientation = orLeft) then
              AddLine(Position, point(Position.X + 1, Position.Y));
          end;
          orUp: begin
            nConnector := FindConnector(point(Position.X, Position.Y - 1)) as TConnector;
            if (nConnector <> Nil) and (IsInput xor nConnector.IsInput) and (nConnector.Orientation = orDown) then
              AddLine(point(Position.X, Position.Y - 1), Position);
          end;
          orDown: begin
            nConnector := FindConnector(point(Position.X, Position.Y + 1)) as TConnector;
            if (nConnector <> Nil) and (IsInput xor nConnector.IsInput) and (nConnector.Orientation = orUp) then
              AddLine(Position, point(Position.X, Position.Y + 1));
          end;
        end; // of case
    end;

  var I: integer;
  begin
    with ALogicModule as TLogicModule do
    begin
      for I := 0 to FInputConnectorList.Count - 1 do
        CheckConnector(FInputConnectorList[I] as TConnector);
      for I := 0 to FOutputConnectorList.Count - 1 do
        CheckConnector(FOutputConnectorList[I] as TConnector);
    end;
  end;
                   {public}

  constructor TElementBox.Create(AOwner: TObject;const ACanvas: TCanvas; AResolution: TPoint);
  begin
    inherited Create;
    FLineList := TElementList.Create;
    FModuleList := TElementList.Create;
    FLineCouplingDiodeList := TElementList.Create;
    FLineConnectorList := TElementList.Create;
    FSetStatusStack := TList.Create;
    FClockGenerator := Nil;
    Owner := AOwner;
    Canvas := ACanvas;
    VResolution := AResolution;
  end;

  destructor TElementBox.Destroy;
  begin
    FLineList.Free;
    FModuleList.Free;
    FLineCouplingDiodeList.Free;
    FLineConnectorList.Free;
    FSetStatusStack.Free;
    inherited Destroy;
  end;

  procedure TElementBox.SaveToBinaryTag(Tag: TRakBinaryTag);
  begin
    FModuleList.SaveToBinaryTag(Tag.AddTag('ModuleList'));           
    FLineList.SaveToBinaryTag(Tag.AddTag('LineList'));
    FLineCouplingDiodeList.SaveToBinaryTag(Tag.AddTag('LineCouplingDiodeList'));
  end;

  procedure TElementBox.LoadFromBinaryTag(Tag: TRakBinaryTag);
  var LTag: TRakBinaryTag;
  begin
    LTag := Tag.FindFirstTag('ModuleList');                 // Modules
    if LTag <> nil then
      FModuleList.LoadFromBinaryTag(LTag, self);

    LTag := Tag.FindFirstTag('LineList');                   // Lines
    if LTag <> nil then
      FLineList.LoadFromBinaryTag(LTag, self);

    LTag := Tag.FindFirstTag('LineCouplingDiodeList');      // CouplingDiodes
    if LTag <> nil then
      FLineCouplingDiodeList.LoadFromBinaryTag(LTag, self);

    RefreshLineConnectorList;                               // LineConnectors
  end;

  procedure TElementBox.Paint;
  var i, ii: integer;
      TmpRest: TPoint;
  begin
    if gPaintGrid then
    begin
      TmpRest := point(VStartPoint.X mod gDistance, VStartPoint.Y mod gDistance);
      for i:= 0 to (VResolution.x) div gDistance do             //Grid
        for ii:= 0 to (VResolution.y) div gDistance do
        begin
          Canvas.MoveTo(i*gDistance-TmpRest.X,ii*gDistance-TmpRest.Y);
          Canvas.LineTo(i*gDistance+1-TmpRest.X,ii*gDistance+1-TmpRest.Y);
        end;
    end;
    if VStartPoint.X = 0 then    // overpaint first gridpoints
    begin
      Canvas.Pen.Color := clWhite;
      Canvas.MoveTo(0,0);
      Canvas.LineTo(0,VResolution.Y + 1);
      Canvas.Pen.Color := clBlack;
    end;
    if VStartPoint.Y = 0 then
    begin
      Canvas.Pen.Color := clWhite;
      Canvas.MoveTo(0,0);
      Canvas.LineTo(VResolution.X + 1,0);
      Canvas.Pen.Color := clBlack;
    end;


    for I := 0 to FLineList.Count - 1 do                  //Lines
      (FLineList[I] as TLine).paint(Canvas, FVStartPoint);
    for I := 0 to FLineConnectorList.Count - 1 do         //LineConnectors
      (FLineConnectorList[I] as TLineConnector).Paint(Canvas, FVStartPoint);
    for I := 0 to FLineCouplingDiodeList.Count - 1 do     // CouplingDiode
      (FLineCouplingDiodeList[I] as TCouplingDiode).Paint(Canvas, FVStartPoint);
    for i := 0 to FModuleList.Count - 1 do                //Modules
      if not (FModuleList[I] is TTextModule) then
        (FModuleList[i] as TModule).Paint(Canvas, FVStartPoint);
    for i := 0 to FModuleList.Count - 1 do                  //TextModules
      if FModuleList[I] is TTextModule then
        (FModuleList[I] as TTextModule).Paint(Canvas, FVStartPoint);



    Canvas.Pen.Color := clBlack;                                //Border 
    Canvas.MoveTo(0, VResolution.Y - 1);
    Canvas.LineTo(VResolution.X - 1, VResolution.Y - 1);
    Canvas.LineTo(VResolution.X - 1, -1);
  end;   

  procedure TElementBox.RefreshLineConnectorList;
  // Removes all Line Connectors
  // Adds Line Connectors at right position (LineCount > 2)
  // Removes CouplingDiodes at wrong positions (LineCount < 3)
  var I: integer;
      ALineConnector: TLineConnector;

    function FindLineConnector(APosition: TPoint): boolean;
    var I: integer;
    begin
      Result := false;
      for I := 0 to FLineConnectorList.Count - 1 do
        with FLineConnectorList[I] as TLineConnector do
          if (APosition.X = Position.X) and (APosition.Y = Position.Y) then
            Result := true;
    end;

  begin
    FLineConnectorList.Clear;                // line Connectors
    for I := 0 to FLineList.Count - 1 do
      with FLineList[I] as TLine do
      begin
        if not FindCouplingDiode(StartPoint) and not FindLineConnector(StartPoint) and (LineCount(StartPoint) > 2) then
        begin
          ALineConnector := TLineConnector.Create(StartPoint,self);
          FLineConnectorList.Add(ALineConnector);
        end;
        if not FindCouplingDiode(EndPoint) and not FindLineConnector(EndPoint) and (LineCount(EndPoint) > 2) then
        begin
          ALineConnector := TLineConnector.Create(EndPoint,self);
          FLineConnectorList.Add(ALineConnector);
        end;
      end;

    I := FLineCouplingDiodeList.Count - 1;   // Coupling Diode
    while I > -1 do
    begin
      if LineCount((FLineCouplingDiodeList[I] as TCouplingDiode).Position) < 3 then
        FLineCouplingDiodeList.Delete(I);
      dec(I);
    end;
  end;

  procedure TElementBox.AddLine(AStartPoint: TPoint; AEndPoint: TPoint);
  var ALine: TLine;
      APoint: TPoint;
      AHorizontal: boolean;
      TEndPoint: TPoint;
  begin
    if not LineCollision(AStartPoint, AEndPoint) then
    begin
      if AStartPoint.X = AEndPoint.X then
        AHorizontal := false
      else
        AHorizontal := true;
      if FindLineNewLineIsCompleteIn(AStartPoint,AEndPoint) = Nil then
        if FindLineExtension(AStartPoint, AEndPoint) = -1 then       // ToDo
        begin
          APoint := FindPointInLine(AStartPoint, AEndPoint);
          if not ((APoint.X = -1) and (APoint.Y = -1)) then
          begin
            AddLine(AStartPoint, APoint);
            AddLine(APoint, AEndPoint);
          end
          else
          begin
            ALine := TLine.Create(AStartPoint, AEndPoint, self);
            FLineList.Add(ALine);
          end;
          if FindLineThroughPoint(AStartPoint) <> Nil then
            with FindLineThroughPoint(AStartPoint) as TLine do
              if Horizontal <> AHorizontal then begin
                TEndPoint := EndPoint;
                EndPoint := AStartPoint;
                AddLine(AStartPoint,TEndPoint);
              end;
          if FindLineThroughPoint(AEndPoint) <> Nil then
            with FindLineThroughPoint(AEndPoint) as TLine do
              if Horizontal <> AHorizontal then begin
                TEndPoint := EndPoint;
                EndPoint := AEndPoint;
                AddLine(AEndPoint,TEndPoint);
              end;
        end
        else
        begin
          ALine := FLineList[FindLineExtension(AStartPoint, AEndPoint)] as TLine;
          if (ALine.StartPoint.X = AEndPoint.X) and (ALine.StartPoint.Y = AEndPoint.Y) then
          begin
            APoint := ALine.EndPoint;
            FLineList.Delete(FindLineExtension(AStartPoint, AEndPoint));
            AddLine(AStartPoint,APoint)
          end
          else
          begin
            APoint := ALine.StartPoint;
            FLineList.Delete(FindLineExtension(AStartPoint, AEndPoint));
            AddLine(APoint, AEndPoint);
          end;
        end;
      RefreshLineConnectorList;
    end;
  end;

  procedure TElementBox.AddDirectLine(AStartPoint: TPoint; AEndPoint: TPoint);
  begin
    FLineList.Add(TLine.Create(AStartPoint, AEndPoint, self));
  end;

  procedure TElementBox.AddLineConnector(APosition: TPoint);
  var ACouplingDiode: TObject;
  begin
    ACouplingDiode := FindCouplingDiodeO(APosition);
    if ACouplingDiode = Nil then
      if AreLinesCrossing(APosition) then
      begin
        DivideLines(APosition);
        DivideLines(APosition);
        RefreshLineConnectorList;
      end else
    else begin
      FLineCouplingDiodeList.Remove(ACouplingDiode);
      RefreshLineConnectorList;
    end;
  end;

  procedure TElementBox.SetCouplingDiode(APosition: TPoint);
  var AObject: TCouplingDiode;
    Found: boolean;
    I: integer;
  begin
    Found := false;
    I := FLineCouplingDiodeList.Count - 1;
    while I > - 1 do
    begin
      if ((FLineCouplingDiodeList[I] as TCouplingDiode).Position.X = APosition.X)
        and ((FLineCouplingDiodeList[I] as TCouplingDiode).Position.Y = APosition.Y) then
      begin
        FLineCouplingDiodeList.Delete(I);
        CrossLinkLines(APosition);
        RefreshLineConnectorList;
        Found := true;
      end;
      dec(I);
    end;


    if not Found then
    begin
      AObject := TCouplingDiode.Create(APosition, self);

      if AreLinesCrossing(APosition) then
      begin
        DivideLines(APosition);
        DivideLines(APosition);
      end;

      if LineCount(APosition) > 2 then
      begin
        FLineCouplingDiodeList.Add(AObject);
        RefreshLineConnectorList;
      end else
        AObject.Free;
      end;
  end;

  procedure TElementBox.AddAndModule(APosition: TPoint; AOrientation: TOrientation = orLeft; ASize: integer = 0);
  var AObject: TAndModule;
  begin
    if ASize = 0 then ASize := gSize;
    AObject := TAndModule.Create(APosition, ASize, AOrientation, self);
    if not ModuleCollision(AObject) then
    begin
      FModuleList.Add(AObject);
      SetConnectingLinesBetweenTwoConnectors(AObject);
    end else
      AObject.Free;
  end;

  procedure TElementBox.AddOrModule(APosition: TPoint; AOrientation: TOrientation = orLeft; ASize: integer = 0);
  var AObject: TOrModule;
  begin
    if ASize = 0 then ASize := gSize;
    AObject := TOrModule.Create(APosition, ASize, AOrientation, self);
    if not ModuleCollision(AObject) then
    begin
      FModuleList.Add(AObject);
      SetConnectingLinesBetweenTwoConnectors(AObject);
    end else
      AObject.Free;
  end;

  procedure TElementBox.AddXorModule(APosition: TPoint; AOrientation: TOrientation = orLeft; ASize: integer = 0);
  var AObject: TXorModule;
  begin
    if ASize = 0 then ASize := gSize;
    AObject := TXorModule.Create(APosition, ASize, AOrientation, self);
    if not ModuleCollision(AObject) then
    begin
      FModuleList.Add(AObject);
      SetConnectingLinesBetweenTwoConnectors(AObject);
    end else
      AObject.Free;
  end;

  procedure TElementBox.AddSwitchModule(APosition: TPoint);
  var AObject: TSwitchModule;
  begin
    if (LineCount(APosition) < 2) and (FindLineThroughPoint(APosition) = Nil) then
    begin
      AObject := TSwitchModule.Create(APosition, self);
      if not ModuleCollision(AObject) then
        FModuleList.Add(AObject)
      else
        AObject.Free;
    end;
  end;

  procedure TElementBox.AddLedModule(APosition: TPoint; AColor: TColor);
  var AObject: TLedModule;
  begin
    AObject := TLedModule.Create(APosition, AColor, self);
    if not ModuleCollision(AObject) then
      FModuleList.Add(AObject)
    else
      AObject.Free;
  end;

  procedure TElementBox.AddTextModule(APosition: TPoint; ACaption: string; ASize: integer = 8; AColor: TColor = 0);
  var AObject: TTextModule;
  begin
    AObject := TTextModule.Create(APosition, ACaption, ASize, AColor, self);
    if not ModuleCollision(AObject) then
      FModuleList.Add(AObject)
    else
      AObject.Free;
  end;

  procedure TElementBox.AddInverter(APosition: TPoint; AOrientation: TOrientation = orLeft);
  var AObject: TInverterModule;
  begin
    AObject := TInverterModule.Create(APosition, AOrientation, self);
    if not ModuleCollision(AObject) then
    begin
      if AOrientation in [orLeft, orRight] then
        RemoveLine(APosition, point(APosition.X + 1, APosition.Y))
      else RemoveLine(APosition, point(APosition.X, APosition.Y+1));
      FModuleList.Add(AObject);
      SetConnectingLinesBetweenTwoConnectors(AObject);
    end else
      AObject.Free;
  end;    

  procedure TElementBox.AddDelayModule(APosition: TPoint; AOrientation: TOrientation = orLeft);
  var AObject: TDelayModule;
  begin
    AObject := TDelayModule.Create(APosition, AOrientation, self);
    if not ModuleCollision(AObject) then
    begin
      if AOrientation in [orLeft, orRight] then
        RemoveLine(APosition, point(APosition.X + 1, APosition.Y))
      else RemoveLine(APosition, point(APosition.X, APosition.Y+1));
      FModuleList.Add(AObject);
      SetConnectingLinesBetweenTwoConnectors(AObject);
    end else
      AObject.Free;
  end;

  procedure TElementBox.AddHalfAdder(APosition: TPoint; AOrientation: TOrientation = orLeft);
  var AObject: THalfAdderModule;
  begin
    AObject := THalfAdderModule.Create(APosition, AOrientation, self);
    if not ModuleCollision(AObject) then
    begin
      FModuleList.Add(AObject);
      SetConnectingLinesBetweenTwoConnectors(AObject);
    end else
      AObject.Free;
  end;

  procedure TElementBox.AddFullAdder(APosition: TPoint; AOrientation: TOrientation = orLeft);
  var AObject: TFullAdderModule;
  begin
    AObject := TFullAdderModule.Create(APosition, AOrientation, self);
    if not ModuleCollision(AObject) then
    begin
      FModuleList.Add(AObject);
      SetConnectingLinesBetweenTwoConnectors(AObject);
    end else
      AObject.Free;
  end;

  procedure TElementBox.AddFlipFlop(APosition: TPoint; AOrientation: TOrientation = orLeft);
  var AObject: TFlipFlopModule;
  begin
    AObject := TFlipFlopModule.Create(APosition, AOrientation, self);
    if not ModuleCollision(AObject) then
    begin
      FModuleList.Add(AObject);
      SetConnectingLinesBetweenTwoConnectors(AObject);
    end else
      AObject.Free;
  end;

  procedure TElementBox.AddDecoderNoClock(APosition: TPoint; AOrientation: TOrientation = orLeft);
  var AObject: TDecoderNoClockModule;
  begin
    AObject := TDecoderNoClockModule.Create(APosition, AOrientation, self);
    if not ModuleCollision(AObject) then
    begin
      FModuleList.Add(AObject);
      SetConnectingLinesBetweenTwoConnectors(AObject);
    end else
      AObject.Free;
  end;

  procedure TElementBox.AddDecoderClock(APosition: TPoint; AOrientation: TOrientation = orLeft);
  var AObject: TDecoderClockModule;
  begin
    AObject := TDecoderClockModule.Create(APosition, AOrientation, self);
    if not ModuleCollision(AObject) then
    begin
      FModuleList.Add(AObject);
      SetConnectingLinesBetweenTwoConnectors(AObject);
    end else
      AObject.Free;
  end;

  procedure TElementBox.AddShiftingRegister(APosition: TPoint; AOrientation: TOrientation = orLeft);
  var AObject: TShiftingRegisterModule;
  begin
    AObject := TShiftingRegisterModule.Create(APosition, AOrientation, self);
    if not ModuleCollision(AObject) then
    begin
      FModuleList.Add(AObject);
      SetConnectingLinesBetweenTwoConnectors(AObject);
    end else
      AObject.Free;
  end;

  procedure TElementBox.AddClockGenerator(APosition: TPoint; AOrientation: TOrientation = orLeft);
  var AObject: TClockGeneratorModule;
  begin
    if FClockGenerator = Nil then
    begin
      AObject := TClockGeneratorModule.Create(APosition, AOrientation, self);
      if not ModuleCollision(AObject) then
      begin
        FModuleList.Add(AObject);
        SetConnectingLinesBetweenTwoConnectors(AObject);
        FClockGenerator := AObject;
        Form1.Simulation.ClockGeneratorExists := true;
        Form1.RefreshForms;
        gHasChanged := true;
      end else
        AObject.Free;
    end
  end;

  procedure TElementBox.AddIcModule(APosition: TPoint; AIndex: integer);
  var AObject: TIcModule;
  begin
    AObject := ((FSimulationPointer as TSimulation).FIcSampleList[gIcIndex] as TIcSample).CreateModule(APosition, self);
    if not ModuleCollision(AObject) then
    begin
      FModuleList.Add(AObject);
      ((FSimulationPointer as TSimulation).FIcSampleList[gIcIndex] as TIcSample).AddModule(AObject);
    end else
      AObject.Free;
  end;

  procedure TElementBox.SetNegationPoint(APosition: TPoint);
  var AConnector: TObject;
  begin
    AConnector := FindConnector(APosition);
    if not (AConnector = Nil) then
      (AConnector as TConnector).Negatived := not (AConnector as TConnector).Negatived;
  end;

  procedure TElementBox.SetLineConnector(APosition: TPoint);
  var I: integer;
      Found: boolean;
      ACouplingDiode: TObject;
  begin
    Found := false;
    I := 0;
    while I < FLineConnectorList.Count do               // delete
    begin
      with FLineConnectorList[I] as TLineConnector do
        if (Position.X = APosition.X) and (Position.Y = APosition.Y) then
        begin
          Found := true;
          FLineConnectorList.Delete(I);
          CrossLinkLines(APosition);
          RefreshLineConnectorList;
        end;
      inc(I);
    end;                                                // set
    if not Found then
    begin
      ACouplingDiode := FindCouplingDiodeO(APosition);
      if ACouplingDiode = Nil then
        if AreLinesCrossing(APosition) then
        begin
          DivideLines(APosition);
          DivideLines(APosition);
          RefreshLineConnectorList;
        end else
      else begin
        FLineCouplingDiodeList.Remove(ACouplingDiode);
        RefreshLineConnectorList;
      end;
    end;
  end;

  procedure TElementBox.RemoveModule(APosition: TPoint);
  var I, XI, YI: integer;
  begin
    for I := 0 to FModuleList.Count - 1 do                      // Modules
      with FModuleList[I] as TModule do
        for XI := Position.X to Position.X + Width do
          for YI := Position.Y to Position.Y + Height do
            if (XI = APosition.X) and (YI  = APosition.Y) then
            begin
              if FModuleList[I] is TClockGeneratorModule then
              begin                                 // ClockGenerator
                FClockGenerator := Nil;
                Form1.Simulation.ClockGeneratorExists := false;
                Form1.RefreshForms;
              end;
              FModuleList.Delete(I);
              exit;
            end;
    for I := 0 to FLineConnectorList.Count - 1 do               // LineConnectors
      with FLineConnectorList[I] as TLineConnector do
        if (Position.X = APosition.X) and (Position.Y = APosition.Y) then
        begin
          FLineConnectorList.Delete(I);
          CrossLinkLines(APosition);
          RefreshLineConnectorList;
          exit;
        end;
    for I := 0 to FLineCouplingDiodeList.Count - 1 do           // LineCouplingDiode
      if ((FLineCouplingDiodeList[I] as TCouplingDiode).Position.X = APosition.X)
        and ((FLineCouplingDiodeList[I] as TCouplingDiode).Position.Y = APosition.Y) then
      begin
        FLineCouplingDiodeList.Delete(I);
        CrossLinkLines(APosition);
        RefreshLineConnectorList;
        exit;
      end;
  end;

  procedure TElementBox.RemoveLine(AStartPoint, AEndPoint: TPoint);
  var AHorizontal: boolean;
      XI, YI: integer;
      TmpPoint: TPoint;
      AObject: TObject;
      ALine: TLine;
  begin
    if AStartPoint.X = AEndPoint.X then
			AHorizontal := false
		else
			AHorizontal := true;
		while FindLineCompleteInNewLine(AStartPoint, AEndPoint) <> Nil do
			FLineList.Remove(FindLineCompleteInNewLine(AStartPoint, AEndPoint));

		AObject := FindLineNewLineIsCompleteIn(AStartPoint, AEndPoint);
		if AObject <> Nil then
		begin
			with AObject as TLine do
			begin
  		  TmpPoint := EndPoint;
			  EndPoint := AStartPoint;
			  ALine := TLine.Create(AEndPoint, TmpPoint, self);
        if not ((ALine.StartPoint.X = ALine.EndPoint.X) and (ALine.StartPoint.Y = ALine.EndPoint.Y)) then
          FLineList.Add(ALine)
        else ALine.Free;
        if (StartPoint.X = EndPoint.X) and (StartPoint.Y = EndPoint.Y) then
          FLineList.Remove(AObject);
			end;
		end else begin
			if FindLineThroughPoint(AStartPoint) <> Nil then
				if ((FindLineThroughPoint(AStartPoint) as TLine).StartPoint.Y = (FindLineThroughPoint(AStartPoint) as TLine).EndPoint.Y) = AHorizontal then
					(FindLineThroughPoint(AStartPoint) as TLine).EndPoint := AStartPoint;
			if FindLineThroughPoint(AEndPoint) <> Nil then
				if ((FindLineThroughPoint(AEndPoint) as TLine).StartPoint.Y = (FindLineThroughPoint(AEndPoint) as TLine).EndPoint.Y) = AHorizontal then
					(FindLineThroughPoint(AEndPoint) as TLine).StartPoint := AEndPoint;
		end;
		DivideLines(AStartPoint);
		DivideLines(AEndPoint);

    for XI := AStartPoint.X to AEndPoint.X do
      for YI := AStartPoint.Y to AEndPoint.Y do
        if CountLinesSameHorizontal(point(XI,YI), not AHorizontal) > 1 then
          LinkLines(point(XI,YI));

		RefreshLineConnectorList;
  end;

  function TElementBox.ItemsStatus(APosition: TPoint; var AError: integer): boolean;
  // Error: 0 = all right; 1 = items have different Status; 2 = no item found //
  var I: integer;
      Found: boolean;
  begin
    AError := 0;
    Result := false;
    Found := false;
    I := 1;
    while FindStatusModule(APosition,I) <> Nil do
    begin
      if Found then
      begin
        if not ((FindStatusModule(APosition,I) as TStatusModule).Status = Result) then
          AError := 1;
      end else begin
        Result := (FindStatusModule(APosition,I) as TStatusModule).Status;
        Found := true;
      end;
      inc(I);
    end;
    if FindConnector(APosition) <> Nil then
      if Found then
      begin
        if not ((FindConnector(APosition) as TConnector).Status = Result) then
          AError := 1;
      end else begin
        Result := (FindConnector(APosition) as TConnector).Status;
        Found := true;
      end;
    I := 1;
    while FindLinePoint(APosition,I) <> Nil do
    begin
      if Found then
      begin
        if (FindLinePoint(APosition,I) as TLine).Status <> Result then
          AError := 1;
      end else begin
        Result := (FindLinePoint(APosition,I) as TLine).Status;
        Found := true;
      end;
      inc(I);
    end;
    if not Found then
      AError := 2;
  end;

  procedure TElementBox.SetStatusByClick(APosition: TPoint);
  var AStatus: boolean;
    AError: integer;
    AConnector: TObject;
    AModule: TObject;
    I: integer;
  begin
    AConnector := FindConnector(APosition);
    AModule := FindStatusModule(APosition, 1);
    if AConnector = Nil then
      if (AModule <> Nil) and (AModule is TSwitchModule) then                             // TSwitchModule
        SetItemsStatus(APosition, not (AModule as TSwitchModule).Status)
      else if (LineCount(APosition) = 1) and (FindLineThroughPoint(APosition) = Nil) then // LineEnds
        begin
          AStatus := ItemsStatus(APosition, AError);
			    if AStatus and (Error = 0) then           // true    --> switch off
			      SetItemsStatus(APosition, false)
			    else if not AStatus and (Error = 0) then  // false   --> switch on
				      SetItemsStatus(APosition, true)
			      else if Error = 1 then                  // unknown --> switch on
				      SetItemsStatus(APosition, true);
        end else                                                                          // Connector
    else if (AConnector as TConnector).IsInput and (LineCount(APosition) = 0) then  // is Inputconnector
        SetItemsStatus(APosition, not (AConnector as TConnector).Status)
      else if (AConnector as TConnector).Owner is TFlipFlopModule then  // is first Outputconnector of a FlipFlop
        begin
          I := ((AConnector as TConnector).Owner as TFlipFlopModule).FOutputConnectorList.IndexOf(AConnector);
          if I = 0 then
          begin
            ((AConnector as TConnector).Owner as TFlipFlopModule).SwapOutput;
            SetItemsStatus(APosition, (AConnector as TConnector).Status);
          end;
        end;
  end;

  procedure TElementBox.SetItemsStatus(APosition: TPoint; AStatus: Boolean);
  var AJob: ^TSetStatusStackTyp;
  begin
    New(AJob);
    AJob.Position := APosition;
    AJob.Status := AStatus;
    FSetStatusStack.Add(AJob);
    if not GIterating then
      ExecuteIteration;
  end;

  procedure TElementBox.ExecuteSetItemsStatusBuffer(var Changed: boolean);
  var AJob: ^TSetStatusStackTyp;
  begin
    while FSetStatusStack.Count > 0 do
    begin
      AJob := FSetStatusStack[0];
      if AJob <> Nil then
        SetItemsStatusExecute(AJob.Position, AJob.Status, Changed);
      AJob := Nil;
      Dispose(AJob);
      FSetStatusStack.Delete(0);
    end;
  end;

  procedure TElementBox.SetItemsStatusExecute(APosition: TPoint; AStatus: Boolean; var Changed: boolean);
  var I: integer;
      AObject: TObject;
      OneLineRightStatus: boolean;
      ALine: TLine;

    procedure ActivateLine(ALine: TObject; APosition: TPoint; AStatus:boolean; var Changed: boolean);
    (* activate the Line and the Lines on the other end (at not APosition)    *)
    begin
      with ALine as TLine do
      begin
        Status := AStatus;
        Changed := true;
        if (APosition.X = StartPoint.X) and (APosition.Y = StartPoint.Y) then   // StartPoint
          SetItemsStatusExecute(EndPoint, AStatus, Changed)
        else if (APosition.X = EndPoint.X) and (APosition.Y = EndPoint.Y) then  // EndPoint
            SetItemsStatusExecute(StartPoint, AStatus, Changed);
      end;
    end;

  begin
    if not gAbort then
    begin
    	if not gAbort then                               // StatusModule
				AObject := FindStatusModule(APosition,1)
      else AObject := Nil;
			I := 2;
			while not gAbort and (AObject <> Nil) do
			begin
        if not (AObject as TStatusModule).Status = AStatus then
        begin
				  (AObject as TStatusModule).Status := AStatus;
          Changed := true;
        end;
				AObject := FindStatusModule(APosition,I);
				inc(I);
			end;

      OneLineRightStatus := false;
      ALine := Nil;
			if not gAbort then                                  // Line
				AObject := FindLinePoint(APosition, 1)
      else AObject := Nil;
			I := 2;
			while not gAbort and (AObject <> Nil) do
			begin
				with AObject as TLine do
        begin                           // (different status) and ((no couplind diode) or (vertical))
					if not (Status = AStatus) and (not FindCouplingDiode(APosition) or not Horizontal) then
						  ActivateLine(AObject, APosition, AStatus, Changed);
              
          if FindCouplingDiode(APosition) and Horizontal then  // (coupling diode) and (horizontal)
            if Status = AStatus then
              if ALine = Nil then   // no line found, yet
                OneLineRightStatus := true
              else ActivateLine(ALine, APosition, AStatus, Changed)
            else if OneLineRightStatus then
                ActivateLine(AObject, APosition, AStatus, Changed)
              else ALine := AObject as TLine;
        end;
				AObject := FindLinePoint(APosition, I);
				inc(I);
			end;

			if not gAbort then                           // Connector
				AObject := FindConnector(APosition)
      else AObject := Nil;
			if not gAbort and (AObject <> Nil) then
				with (AObject as TConnector) do
					if IsInput and not (Status = AStatus) then
					begin
						Status := AStatus;
            Changed := true;
					end;
			if not gAbort and gSlowRePaint then
			begin
				Form1.Simulation.Paint;
				Application.ProcessMessages;
			end;
    end;
  end;

  procedure TElementBox.DivideLines(APosition: TPoint);
  var AStartPoint, AEndPoint: TPoint;
      ALine: TLine;
  begin
    if FindLineThroughPoint(APosition) <> Nil then
      with FindLineThroughPoint(APosition) as TLine do
        begin
          AStartPoint := StartPoint;
          AEndPoint := EndPoint;
          FLineList.Remove(FindLineThroughPoint(APosition));
          ALine := TLine.Create(AStartPoint, APosition, self);
          FLineList.Add(ALine);
          ALine := TLine.Create(APosition, AEndPoint, self);
          FLineList.Add(ALine);
        end;
  end;

  procedure TElementBox.LinkLines(APosition: TPoint);
  var Line1, Line2, ALine: TLine;
  begin
    if LineCount(APosition) = 2 then
    begin
      Line1 := FindLinePoint(APosition, 1) as TLine;
      Line2 := FindLinePoint(APosition, 2) as TLine;
      if (Line1.EndPoint.X = Line2.StartPoint.X)
          and (Line1.EndPoint.Y = Line2.StartPoint.Y) then
        ALine := TLine.Create(Line1.StartPoint, Line2.EndPoint, self)
      else
        ALine := TLine.Create(Line2.StartPoint, Line1.EndPoint, self);
      FLineList.Add(ALine);
      FLineList.Remove(Line1);
      FLineList.Remove(Line2);
    end;
  end;

  procedure TElementBox.CrossLinkLines(APosition: TPoint);
  var Line1, Line2: TLine;
      I: integer;
      Found, AHorizontal: boolean;
  begin
    if LineCount(APosition) = 4 then
      while LineCount(APosition) > 0 do
      begin
        Line1 := FindLinePoint(APosition, 1) as TLine;
        if Line1.StartPoint.X = Line1.EndPoint.X then
          AHorizontal := false
        else
          AHorizontal := true;
        Found := false;
        I := 2;
        while not Found and (FindLinePoint(APosition, I) <> Nil) do
        begin
          Line2 := FindLinePoint(APosition, I) as TLine;
          if not ((Line2.StartPoint.Y = Line2.EndPoint.Y) xor AHorizontal) then
          begin
            if ((Line1.EndPoint.X = Line2.StartPoint.X) and (Line1.EndPoint.Y = Line2.StartPoint.Y)) then
            begin
              Found := true;
              Line1.EndPoint := Line2.EndPoint;
              FLineList.Remove(Line2);
            end;
            if ((Line1.StartPoint.X = Line2.EndPoint.X) and (Line1.StartPoint.Y = Line2.EndPoint.Y)) then
            begin
              Found := true;
              Line1.StartPoint := Line2.StartPoint;
              FLineList.Remove(Line2);
            end;
          end;
          inc(I);
        end;
      end;
  end;

  procedure TElementBox.SwitchOn;
  var I, II: integer;
  begin
    gAbort := false;
    GIterating := false;
    for I := 0 to FModuleList.Count - 1 do
    begin                                     // switch on free InputConnectors
      if (FModuleList[I] is TAndModule) then      // for And
        for II := 0 to (FModuleList[I] as TLogicModule).FInputConnectorList.Count - 1 do
          with (FModuleList[I] as TLogicModule).FInputConnectorList[II] as TConnector do
            if not Negatived and (FindLinePoint(Position, 1) = Nil) then
              Status := true;
      if (FModuleList[I] is TFlipFlopModule) then // for FlipFlops
      begin
        (FModuleList[I] as TFlipFlopModule).Reset;
        if not ((FModuleList[I] as TLogicModule).FInputConnectorList[0] as TConnector).Negatived and (FindLinePoint(((FModuleList[I] as TLogicModule).FInputConnectorList[0] as TConnector).Position, 1) = Nil) then
          ((FModuleList[I] as TLogicModule).FInputConnectorList[0] as TConnector).Status := true;
        if not ((FModuleList[I] as TLogicModule).FInputConnectorList[2] as TConnector).Negatived and (FindLinePoint(((FModuleList[I] as TLogicModule).FInputConnectorList[2] as TConnector).Position, 1) = Nil) then
          ((FModuleList[I] as TLogicModule).FInputConnectorList[2] as TConnector).Status := true;
      end else if (FModuleList[I] is TDecoderClockModule) then   // Reset Decoder with Clock
          (FModuleList[I] as TDecoderClockModule).Reset
          else if (FModuleList[I] is TShiftingRegisterModule) then   // Reset ShiftingRegister
            (FModuleList[I] as TShiftingRegisterModule).Reset;
    end;
    ExecuteIteration;                       // Initialize
  end;

  procedure TElementBox.SwitchOff;
  var I, II: Integer;
  begin
    gAbort := true;
    for I := 0 to FModuleList.Count - 1 do
    begin
      if FModuleList[I] is TLogicModule then                  // Reset Connectors
      begin
        for II := 0 to (FModuleList[I] as TLogicModule).FInputConnectorList.Count - 1 do
        begin
          ((FModuleList[I] as TLogicModule).FInputConnectorList[II] as TConnector).FStatus := false;
          ((FModuleList[I] as TLogicModule).FInputConnectorList[II] as TConnector).FTempStatus := false;
        end;
        for II := 0 to (FModuleList[I] as TLogicModule).FOutputConnectorList.Count - 1 do
        begin
          ((FModuleList[I] as TLogicModule).FOutputConnectorList[II] as TConnector).FStatus := false;
          ((FModuleList[I] as TLogicModule).FOutputConnectorList[II] as TConnector).FTempStatus := false;
        end;
      end;
      if FModuleList[I] is TStatusModule then                 // Reset StatusModules
        (FModuleList[I] as TStatusModule).Status := false
      else if FModuleList[I] is TFlipFlopModule then                 // Reset FlipFlop
          (FModuleList[I] as TFlipFlopModule).Reset
        else if FModuleList[I] is TDecoderClockModule then           // and DecoderClock
            (FModuleList[I] as TDecoderClockModule).Reset
          else if (FModuleList[I] is TShiftingRegisterModule) then   // and ShiftingRegister
              (FModuleList[I] as TShiftingRegisterModule).Reset
            else if (FModuleList[I] is TClockGeneratorModule) then   // and ClockGenerator
                (FModuleList[I] as TClockGeneratorModule).Reset;
    end;
    for I := 0 to FLineList.Count - 1 do
      (FLineList[I] as TLine).Status := false;
  end;

  procedure TElementBox.ExecuteIteration;
  var Changed: boolean;
      I: integer;
  begin
    GIterating := true;
    repeat
      Changed := false;

      for I := 0 to FModuleList.Count - 1 do              // change TempStatus
        if (FModuleList[I] is TLogicModule) then
          (FModuleList[I] as TLogicModule).ChangedInputStatus(Changed);

      for I := 0 to FModuleList.Count - 1 do              // update Status
        if (FModuleList[I] is TLogicModule) then
          (FModuleList[I] as TLogicModule).UpdateStatus;

      ExecuteSetItemsStatusBuffer(Changed);               // update Lines
      

      for I := 0 to FLineCouplingDiodeList.Count - 1 do   // check CouplingDiodes
        (FLineCouplingDiodeList[I] as TCouplingDiode).CheckStatus(Changed);

      ExecuteSetItemsStatusBuffer(Changed);               // update Lines
      

      Form1.Simulation.Paint;      // Paint
      Application.ProcessMessages;
    until not Changed or gAbort;
    GIterating := false;

    if not gAbort and (FClockGenerator is TClockGeneratorModule) then // StartClockGenerator
      with FClockGenerator as TClockGeneratorModule do
        if tiDelay.Interval > 0 then
          tiDelay.Enabled := true
        else tiDelayTimer(self);
  end;

  function TElementBox.FindClockGenerator: TObject;
  var I: integer;
    Abr: boolean;
  begin
    Result := Nil;
    Abr := false;
    I := 0;
    while not Abr and (I < FModuleList.Count) do
    begin
      if FModuleList[I] is TClockGeneratorModule then
      begin
        Result := FModuleList[I];
        Abr := true;
      end;
      inc(I);
    end;
  end;

  {TSimulationTab}

  procedure TSimulationTab.RefreshScrollBox;
  begin
    if Resolution.X = 0 then                                          // X
    begin                     // clClient -- > equal --> no scroll
      FPaintBox.Width := FScrollBox.Width;
      FScrollBox.HorzScrollBar.Range := 0;
      gBmp.Width := FScrollBox.Width;
    end else
      if Resolution.X <= FScrollBox.Width then
      begin                     // smaller --> no scroll
        FPaintBox.Width := Resolution.X;
        FScrollBox.HorzScrollBar.Range := 0;
        gBmp.Width := FPaintBox.Width;
      end else begin            // bigger --> scroll
        FPaintBox.Width := Resolution.X;
        FScrollBox.HorzScrollBar.Range := Resolution.X;
        gBmp.Width := FScrollBox.Width;
      end;

    if Resolution.Y = 0 then                                          // Y
    begin                   // clClient -- > equal --> no scroll
      FPaintBox.Height := FScrollBox.Height;
      FScrollBox.VertScrollBar.Range := 0;
      gBmp.Height := FScrollBox.Height;
    end else
      if Resolution.Y <= FScrollBox.Height then
      begin                     // smaller --> no scroll
        FPaintBox.Height := Resolution.Y;
        FScrollBox.VertScrollBar.Range := 0;
        gBmp.Height := FPaintBox.Height;
      end else begin            // bigger --> scroll
        FPaintBox.Height := Resolution.Y;
        FScrollBox.VertScrollBar.Range := Resolution.Y;
        gBmp.Height := FScrollBox.Height;
      end;

    FVResolution := point(gBmp.Width, gBmp.Height);
    FElementBox.VResolution := FVResolution;
    FVStartPoint := point(FScrollBox.HorzScrollBar.ScrollPos, FScrollBox.VertScrollBar.ScrollPos);
    FElementBox.VStartPoint := FVStartPoint;

  end;

  procedure TSimulationTab.SetResolution(AResolution: TPoint);
  begin
    FResolution := AResolution;
    RefreshScrollBox;
    FVResolution := point(gBmp.Width, gBmp.Height);
    FElementBox.VResolution := FVResolution;
  end;

  procedure TSimulationTab.RefreshBox;
  begin
    RefreshScrollbox;
    if gAutoRePaint then
    begin
      with gBmp do begin
        Canvas.Brush.Color := clwhite;
        Canvas.FillRect(rect(0,0,Width,Height));
      end;
      FElementBox.Paint;
      FPaintBox.Canvas.Draw(FVStartPoint.X, FVStartPoint.Y, gBmp);
    end;
  end;

  procedure TSimulationTab.CatchPoint(var APoint: TPoint);
  begin
    APoint.X:= ((APoint.X+gDistance div 2) div gDistance) * gDistance;
    APoint.Y:= ((APoint.Y+gDistance div 2) div gDistance) * gDistance;
  end;

  procedure TSimulationTab.FPaintBoxPaint(Sender: TObject);
  begin
    RefreshBox;
  end;

  procedure TSimulationTab.FPaintBoxMouseDown(Sender: TObject; Button: TMouseButton;
    Shift: TShiftState; X, Y: Integer);
	var AObject: TObject;
	begin
    SetFocus;
    if Button = mbRight then
    begin
      FIsMouseDown := false;
      Paint;
    end else if Button = mbLeft then
    begin
			FClick1.x := x;
			FClick1.y := y;
			CatchPoint(FClick1);
			FClick1.X := FClick1.X div gDistance;
			FClick1.y := FClick1.y div gDistance;
			FClick2 := FClick1;
			FIsMouseDown := true;

      AObject := Nil;
      case gActAction of
        AddAnd: AObject := TAndModule.Create(FClick1, gSize, gOrientation, self);
        AddOr: AObject := TOrModule.Create(FClick1, gSize, gOrientation, self);
        AddXor: AObject := TXorModule.Create(FClick1, gSize, gOrientation, self);
        AddLed: AObject := TLedModule.Create(FClick1, clNone, self);
        AddSwitch: AObject := TSwitchModule.Create(FClick1, self);
        AddText: AObject := TTextModule.Create(FClick1, gTextCaption, gTextSize, gColor, self);
        AddInverter: AObject := TInverterModule.Create(FClick1, gOrientation, self);
        AddDelayModule: AObject := TDelayModule.Create(FClick1, gOrientation, self);
        AddHalfAdder: AObject := THalfAdderModule.Create(FClick1, gOrientation, self);
        AddFullAdder: AObject := TFullAdderModule.Create(FClick1, gOrientation, self);
        AddFlipFlop: AObject := TFlipFlopModule.Create(FClick1, gOrientation, self);
        AddDecoderNoClock: AObject := TDecoderNoClockModule.Create(point(FClick1.X, FClick1.Y - 3), gOrientation, self);
        AddDecoderClock: AObject := TDecoderClockModule.Create(point(FClick1.X, FClick1.Y - 3), gOrientation, self);
        AddShiftingRegister: AObject := TShiftingRegisterModule.Create(FClick1, gOrientation, self);
        AddClockGenerator: AObject := TClockGeneratorModule.Create(FClick1, gOrientation, self);
        //AddIc: AObject := TIcModule.Create(FClick1, self);
      end;
      FPaintBox.Canvas.Pen.Mode := pmCopy; // pmNotXor;
      try
        if AObject <> Nil then
        begin
          Paint;
          (AObject as TModule).Paint(FPaintBox.Canvas, point(0,0));
        end;
      finally
        if AObject <> Nil then
          AObject.Free;
      end;
    end;
	end;

  procedure TSimulationTab.FPaintBoxMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
	var APoint: TPoint;
      AObject: TObject;
	begin
    FClick3 := Point(x, y);                         // new Position
    CatchPoint(FClick3);
    FClick3.X := FClick3.X div gDistance;
    FClick3.y := FClick3.y div gDistance;
    
		if FIsMouseDown then
    begin
      if not ((FClick2.X = FClick3.X) and (FClick2.Y = FClick3.Y)) then
      begin
				if gActAction in [AddLine, Remove] then
				begin                           // Leitung
					APoint := Point(x, y);
					CatchPoint(APoint);
					APoint := Point(APoint.X div gDistance, APoint.Y div gDistance);
					if (FClick2.X = FClick1.X) and (FClick2.Y = FClick1.Y) then
						if FClick2.X = APoint.X then
							FHorizontal := false
						else
							FHorizontal := true;

					with FPaintBox.Canvas do
					begin
						Pen.Mode := pmNotXor;
						Pen.Width := 3;
						if gActAction = Remove then
							Pen.Color := clRed
						else
							Pen.Color := clGray;

						MoveTo(FClick1.x * gDistance, FClick1.y * gDistance);   // 1.
						if FHorizontal then
							LineTo(FClick2.x * gDistance, FClick1.y * gDistance)
						else
							LineTo(FClick1.x * gDistance, FClick2.y * gDistance);
						LineTo(FClick2.x * gDistance, FClick2.y * gDistance);

						FClick2 := APoint;
	
						MoveTo(FClick1.x * gDistance, FClick1.y * gDistance);   // 2.
						if FHorizontal then
							LineTo(FClick2.x * gDistance, FClick1.y * gDistance)
						else
							LineTo(FClick1.x * gDistance, FClick2.y * gDistance);
						LineTo(FClick2.x * gDistance, FClick2.y * gDistance);
	
						Pen.Color := clBlack;
						Pen.Width := 1;
						Pen.Mode := pmCopy;
					end;
				end else begin
					AObject := Nil;
					case gActAction of
						AddAnd: AObject := TAndModule.Create(FClick3, gSize, gOrientation, self);
						AddOr: AObject := TOrModule.Create(FClick3, gSize, gOrientation, self);
						AddXor: AObject := TXorModule.Create(FClick3, gSize, gOrientation, self);
						AddLed: AObject := TLedModule.Create(FClick3, clNone, self);
						AddSwitch: AObject := TSwitchModule.Create(FClick3, self); 
            AddText: AObject := TTextModule.Create(FClick3, gTextCaption, gTextSize, gColor, self);
						AddInverter: AObject := TInverterModule.Create(FClick3, gOrientation, self);
            AddDelayModule: AObject := TDelayModule.Create(FClick3, gOrientation, self);
            AddHalfAdder: AObject := THalfAdderModule.Create(FClick3, gOrientation, self);
            AddFullAdder: AObject := TFullAdderModule.Create(FClick3, gOrientation, self);
						AddFlipFlop: AObject := TFlipFlopModule.Create(FClick3, gOrientation, self);
            AddDecoderNoClock: AObject := TDecoderNoClockModule.Create(point(FClick3.X, FClick3.Y - 3), gOrientation, self);
            AddDecoderClock: AObject := TDecoderClockModule.Create(point(FClick3.X, FClick3.Y - 3), gOrientation, self);
            AddShiftingRegister: AObject := TShiftingRegisterModule.Create(FClick3, gOrientation, self);
            AddClockGenerator: AObject := TClockGeneratorModule.Create(FClick3, gOrientation, self);
						//AddIc: AObject := TIcModule.Create(FClick3, self);
					end;
					FPaintBox.Canvas.Pen.Mode := pmCopy; // pmNotXor;
					try
					  if AObject <> Nil then
					  begin
							Paint;
							(AObject as TModule).Paint(FPaintBox.Canvas, point(0,0));
					  end;
					finally
						if AObject <> Nil then AObject.Free;
					end;
				end;
      end;
	  end;
    Form1.StatusBar1.Panels[1].Text := Format('(%d|%d)',[FClick3.X, FClick2.Y]);
    FClick2 := FClick3;
	end;

  procedure TSimulationTab.FPaintBoxMouseUp(Sender: TObject; Button: TMouseButton;
		Shift: TShiftState; X, Y: Integer);
	var AStartPoint, AEndPoint: TPoint;
  begin
		if FIsMouseDown then
    begin
			FIsMouseDown := false;
			FClick3 := point(X,Y);
			CatchPoint(FClick3);
			FClick3 := point(FClick3.X div gDistance, FClick3.Y div gDistance);
      if (FClick3.X > 0) and (FClick3.Y > 0) or (gActAction = Remove) then
      begin
				case gActAction of
					AddLine, Remove:begin
						AStartPoint := point(0,0);
						AEndPoint := point(0,0);

						if (FClick1.X = FClick3.X) and (FClick1.Y = FClick3.Y) then
							if gActAction = Remove then
								FElementBox.RemoveModule(FClick1)
							else begin
								FElementBox.SetNegationPoint(FClick1);
								FElementBox.SetLineConnector(FClick1);
							end
						else begin
							if FHorizontal then
							begin
								if not(FClick3.x = FClick1.x) then
									if FClick1.x < FClick3.x then
									begin                          // horizontal // X<>X // X<X
										AStartPoint := point(FClick1.X, FClick1.Y);
										AEndPoint := point(FClick3.X, FClick1.y);
									end else begin                 // horizontal // X<>X // X>X
										AStartPoint := point(FClick3.X, FClick1.y);
										AEndPoint := point(FClick1.X, FClick1.Y);
									end;
							end else begin
								if not(FClick3.Y = FClick1.Y) then
									if FClick1.Y < FClick3.Y then
									begin                          // not horizontal // Y<>Y // Y<Y
										AStartPoint := point(FClick1.X, FClick1.Y);
										AEndPoint := point(FClick1.X, FClick3.y);
									end else begin                 // not horizontal // Y<>Y // Y>Y
										AStartPoint := point(FClick1.X, FClick3.y);
										AEndPoint := point(FClick1.X, FClick1.Y);
									end;
							end;

							if (not ((AStartPoint.X = AEndPoint.X) and (AStartPoint.Y = AEndPoint.Y))) then
							begin
								if gActAction = AddLine then
									FElementBox.AddLine(AStartPoint, AEndPoint)
								else FElementBox.RemoveLine(AStartPoint, AEndPoint);
								gHasChanged := true;
							end;

              AStartPoint := point(0,0);
						  AEndPoint := point(0,0);

							if FHorizontal then
							begin
								if not(FClick3.y = FClick1.y) then
									if FClick1.Y < FClick3.Y then
									begin                          // horizontal // Y<>Y // Y<Y
										AStartPoint := point(FClick3.X, FClick1.y);
										AEndPoint := point(FClick3.X, FClick3.Y);
									end else begin                 // horizontal // Y<>Y // Y>Y
										AStartPoint := point(FClick3.X, FClick3.Y);
										AEndPoint := point(FClick3.X, FClick1.y);
									end;
							end else begin
								if not(FClick3.X = FClick1.X) then
									if FClick1.X < FClick3.X then
									begin                          // not horizontal // X<>X // X<X
										AStartPoint := point(FClick1.X, FClick3.y);
										AEndPoint := point(FClick3.X, FClick3.Y);
									end else begin                 // not horizontal // X<>X // X>X
										AStartPoint := point(FClick3.X, FClick3.Y);
										AEndPoint := point(FClick1.X, FClick3.y);
									end;
							end;
		
							if (not ((AStartPoint.X = AEndPoint.X) and (AStartPoint.Y = AEndPoint.Y))) then
							begin
								if gActAction = AddLine then
									FElementBox.AddLine(AStartPoint,AEndPoint)
								else FElementBox.RemoveLine(AStartPoint,AEndPoint);
								gHasChanged := true;
							end;
						end;
					end;
					AddLineConnector: FElementBox.SetLineConnector(FClick3);
					AddCouplingDiode: FElementBox.SetCouplingDiode(FClick3);
					AddAnd: FElementBox.AddAndModule(FClick3, gOrientation, gSize);
					AddOr: FElementBox.AddOrModule(FClick3, gOrientation, gSize);
					AddXor: FElementBox.AddXorModule(FClick3, gOrientation, gSize);
					AddSwitch: FElementBox.AddSwitchModule(FClick3);  
          AddText: TDynTextEdit.Create(self.FElementBox, FClick3, FVStartPoint, gTextCaption, self);
            //FElementBox.AddTextModule(FClick3, gTextCaption, gTextSize, gColor);
					AddLed: FElementBox.AddLedModule(FClick3, gColor);
					AddInverter: FElementBox.AddInverter(FClick3, gOrientation);
          AddDelayModule: FElementBox.AddDelayModule(FClick3, gOrientation);
					AddHalfAdder: FElementBox.AddHalfAdder(FClick3, gOrientation);
					AddFullAdder: FElementBox.AddFullAdder(FClick3, gOrientation);
					AddFlipFlop: FElementBox.AddFlipFlop(FClick3, gOrientation);
					AddDecoderNoClock: if FClick3.Y > 3 then FElementBox.AddDecoderNoClock(point(FClick3.X, FClick3.Y - 3), gOrientation);
					AddDecoderClock: if FClick3.Y > 3 then FElementBox.AddDecoderClock(point(FClick3.X, FClick3.Y - 3), gOrientation);
					AddShiftingRegister: FElementBox.AddShiftingRegister(FClick3, gOrientation);
          AddClockGenerator: FElementBox.AddClockGenerator(FClick3, gOrientation);
          //AddIc: FElementBox.AddIcModule(FClick3, gIcIndex);
					SetNegationPoint: FElementBox.SetNegationPoint(FClick3);
					PowerOn: FElementBox.SetStatusByClick(FClick3);
				else
					gActAction := None;
				end;
				if not (gActAction in [None, PowerOn, AddLine]) then
					gHasChanged := true;
      end;
			Paint;
    end;
	end;

  procedure TSimulationTab.FPaintBoxMouseLeave(Sender: TObject);
  begin
    Form1.StatusBar1.Panels[1].Text := '';
  end;

  constructor TSimulationTab.Create(APageControl: TComponent; AName: string; AOwner: TObject);
  begin
    inherited Create(APageControl);
    PageControl := APageControl as TPageControl;
    Name := AName;
    Show;

    FPageControl := APageControl;
    FOwner := AOwner;

    FScrollBox := TScrollBox.Create(self);
    FScrollBox.Parent := self;
    FScrollBox.Align := alClient;
    FScrollBox.HorzScrollBar.Tracking := true;
    FScrollBox.VertScrollBar.Tracking := true;
    FScrollBox.DoubleBuffered := true;

    FPaintBox := TPaintBox.Create(FScrollBox);
    FPaintBox.Parent := FScrollBox;
    FPaintBox.Align := alCustom;
    FPaintBox.Cursor := crCross;
    FPaintBox.OnPaint := FPaintBoxPaint;
    FPaintBox.OnMouseDown := FPaintBoxMouseDown;
    FPaintBox.OnMouseMove := FPaintBoxMouseMove;
    FPaintBox.OnMouseUp := FPaintBoxMouseUp;
    FPaintBox.OnMouseLeave := FPaintBoxMouseLeave;

    FElementBox := TElementBox.Create(self, gBmp.Canvas, point(FPaintBox.Width, FPaintBox.Height));
    FElementBox.FSimulationPointer := AOwner as TObject;
    FVResolution := point(FPaintBox.Width, FPaintBox.Height);
    Resolution := point(0,0);

    if APageControl <> Nil then
      RefreshBox;
  end;

  destructor TSimulationTab.Destroy;
  begin
    FPaintBox.Free;
    FScrollBox.Free;
    inherited Destroy;
  end;

  procedure TSimulationTab.SaveToBinaryTag(Tag: TRakBinaryTag);
  begin
    FElementBox.SaveToBinaryTag(Tag.AddTag('ElementBox'));
    TRakBinaryLongInt.AddToTag(Tag,'ResolutionX',Resolution.X div GDistance);  // save Relative
    TRakBinaryLongInt.AddToTag(Tag,'ResolutionY',Resolution.Y div GDistance);  // save Relative
  end;

  procedure TSimulationTab.LoadFromBinaryTag(Tag: TRakBinaryTag);
  var TTag: TRakBinaryTag;
  begin
    TTag := Tag.FindFirstTag('ElementBox');
    if TTag <> nil then
      FElementBox.LoadFromBinaryTag(TTag);
    Resolution := point(Tag.AttributeAsInteger('ResolutionX', 0) * GDistance,Tag.AttributeAsInteger('ResolutionY', 0) * GDistance); //load Absolute
  end;

  procedure TSimulationTab.Paint;
  begin
    RefreshBox;
  end;

  {TSimulation}

  function TSimulation.GetResolution: TPoint;
  begin
    if FPageControl.Pages[FPageControl.ActivePageIndex] is TSimulationTab then
      Result := (FPageControl.Pages[FPageControl.ActivePageIndex] as TSimulationTab).Resolution
    else if FPageControl.Pages[FPageControl.ActivePageIndex] is TIcTab then
      Result := (FPageControl.Pages[FPageControl.ActivePageIndex] as TIcTab).Resolution
    else Result := point(0,0);
  end;

  procedure TSimulation.SetResolution(AResolution: TPoint);
  begin
    if FPageControl.Pages[FPageControl.ActivePageIndex] is TSimulationTab then
      with FPageControl.Pages[FPageControl.ActivePageIndex] as TSimulationTab do
        if not ((Resolution.X = AResolution.X) and (Resolution.Y = AResolution.Y)) then
          Resolution := AResolution
        else
    else if FPageControl.Pages[FPageControl.ActivePageIndex] is TIcTab then
      with FPageControl.Pages[FPageControl.ActivePageIndex] as TICTab do
        if not ((Resolution.X = AResolution.X) and (Resolution.Y = AResolution.Y)) then
          Resolution := AResolution
  end;

  function TSimulation.GetMainBox;
  begin
    Result := FMainSimulationTab.FElementBox;
  end;

  procedure TSimulation.SetMainBox(AElementBox: TElementBox);
  begin
    FMainSimulationTab.FElementBox := AElementBox;
  end;

  procedure TSimulation.SetClockGeneratorInterval(AInterval: Word);
  begin
    if ((FMainSimulationTab as TSimulationTab).FElementBox as TElementBox).FClockGenerator is TClockGeneratorModule then
      (((FMainSimulationTab as TSimulationTab).FElementBox as TElementBox).FClockGenerator as TClockGeneratorModule).tiDelay.Interval := AInterval
  end;

  procedure TSimulation.FPageControlChanging(Sender: TObject; var AllowChange: Boolean);
  var AIndex: integer;
  begin
    if not (gActAction = PowerOn) then
    begin
      Form1.RefreshForms;
      if FPageControl.ActivePage is TIcTab then
        if not (((FPageControl.ActivePage as TIcTab).FSimulation as TSimulation).FIcLinkPointer = Nil) then
          with (FPageControl.ActivePage as TIcTab).FSimulation as TSimulation do
          begin
            AIndex := ((FIcLinkPointer as TIcLink).FOwner as TIcSample).FIcLinkList.IndexOf((FIcLinkPointer as TIcLink));
            ((FIcLinkPointer as TIcLink).FOwner as TIcSample).MatchIcData(AIndex);
          end;
    end;
  end;

  constructor TSimulation.Create(APageControl: TPageControl);
  begin
    inherited Create;
    FIcTabList := TElementList.Create;
    FIcSampleList := TElementList.Create;
    FPageControl := APageControl;
    FClockGeneratorExists := false;
    if FPageControl <> Nil then
    begin
      FPageControl.OnChanging := FPageControlChanging;
      FMainSimulationTab := TSimulationTab.Create(APageControl, 'Simulation', self);
      APageControl.ActivePageIndex := 0;
      gHasChanged := false;
    end else
      FMainSimulationTab := TSimulationTab.Create(Nil, 'Simulation', self);
    FIcLinkPointer := Nil;
  end;

  destructor TSimulation.Destroy;
  begin
    FMainSimulationTab.Free;
    FIcSampleList.Free;
    FIcTabList.Free;
    inherited Destroy;
  end;

  function TSimulation.SaveToFile(AFileName: string): boolean;

    procedure SwapStreams(var StreamA, StreamB: TMemoryStream);
    var StreamT: TMemoryStream;
    begin
      StreamT := StreamA;
      StreamT.Clear;
      StreamA := StreamB;
      StreamB := StreamT;
    end;

  var
    StreamData: TRakBinaryStreamData;
    inpStream, outStream: TMemoryStream;
    Stp: boolean;
  begin
    Stp := false;
    try
      StreamData := TRakBinaryStreamData.Create;
      inpStream := TMemoryStream.Create;
      outStream := TMemoryStream.Create;
      try
        StreamData.ApplicationName := FileHeader;
        StreamData.ApplicationVersion := ProgramVersion;
        FMainSimulationTab.SaveToBinaryTag(StreamData.AddTag('MainSimulationTab'));
        //FIcTabList.SaveToBinaryTag(StreamData.AddTag('IcTabList'));
        FIcSampleList.SaveToBinaryTag(StreamData.AddTag('IcSampleList'));

        StreamData.SaveToStream(inpStream);
        if gCompress then                            // compressed
          repeat
            if CompressStream(inpStream, outStream) then
              SwapStreams(inpStream, outStream)
            else Stp := true;
          until Stp;
        inpStream.SaveToFile(AFileName);


        Result := true;
        gHasChanged := false;
      finally
        StreamData.Free;
        if inpStream <> Nil then inpStream.Free;
        if outStream <> Nil then outStream.Free;
      end;
    except
      Result := false;
    end;
  end;

  function TSimulation.LoadFromFile(AFileName: string; IsForced: boolean): string;

    procedure SwapStreams(var StreamA, StreamB: TMemoryStream);
    var StreamT: TMemoryStream;
    begin
      StreamT := StreamA;
      StreamT.Clear;
      StreamA := StreamB;
      StreamB := StreamT;
    end;

  var StreamData: TRakBinaryStreamData;
      TTag: TRakBinaryTag;
      AVersion: string;
      PV1, PV2, OV1, OV2: integer;
      inpStream, outStream: TMemoryStream;
      Stp: boolean;
  (* EString:                                               *)
  (* ''      =          OK                                  *)
  (* '0'     =          Unknown Fileformat                  *)
  (* other   =          Other Version, EString = Version    *)
  begin
    Result := '';
    Stp := false;
    StreamData := TRakBinaryStreamData.Create;
    try
      inpStream := TMemoryStream.Create;
      outStream := TMemoryStream.Create;
      try
        inpStream.LoadFromFile(AFileName);                 // load stream

        repeat                                             // try uncompress
          if DecompressStream(inpStream, outStream) then
            SwapStreams(inpStream, outStream)
          else Stp := true;
        until STP;

        if (StreamData.LoadFromStream(inpStream) <> blrOK) // try to load RakBinaryStreamData
          or ((StreamData.ApplicationName <> FileHeader) and (StreamData.ApplicationName <> 'LogicSim Christian')) then
          Result := '0';
      finally
        if inpStream <> Nil then inpStream.Free;
        if outStream <> Nil then outStream.Free;
      end;



      if Result = '' then                      // Now Load
      begin
        PV1 := StrToIntDef(ListGetAt(ProgramVersion, 0, '.'), -1);
        PV2 := StrToIntDef(ListGetAt(ProgramVersion, 1, '.'), -1);
        AVersion := StreamData.ApplicationVersion;
        OV1 := StrToIntDef(ListGetAt(AVersion, 0, '.'), -1);
        OV2 := StrToIntDef(ListGetAt(AVersion, 1, '.'), -1);
        if (OV1 > PV1) or ((OV1 = PV1) and (OV2 > PV2)) then
          Result := IntToStr(OV1) + '.' + IntToStr(OV2)
        else IsForced := true;
        if IsForced then
        begin
          TTag := StreamData.FindFirstTag('MainSimulationTab');
          if TTag <> nil then
            FMainSimulationTab.LoadFromBinaryTag(TTag);
          //TTag := StreamData.FindFirstTag('IcTabList');
          //if TTag <> nil then
          //  FIcTabList.LoadFromBinaryTag(TTag, self);
          TTag := StreamData.FindFirstTag('IcSampleList');
          if TTag <> nil then
            FIcSampleList.LoadFromBinaryTag(TTag, self);
          FPageControl.ActivePageIndex := 0;
          gHasChanged := false;
        end;
      end;
    finally
      StreamData.Free;
    end;
    // lock for clock generator
    MainBox.FClockGenerator := MainBox.FindClockGenerator;
    ClockGeneratorExists := MainBox.FClockGenerator <> nil
  end;

  procedure TSimulation.SaveToBinaryTag(Tag: TRakBinaryTag);
  begin
    FMainSimulationTab.SaveToBinaryTag(Tag.AddTag('MainSimulationTab'));
    //FIcTabList.SaveToBinaryTag(Tag.AddTag('IcTabList'));
    FIcSampleList.SaveToBinaryTag(Tag.AddTag('IcSampleList'));
  end;

  procedure TSimulation.LoadFromBinaryTag(Tag: TRakBinaryTag);
  var TTag: TRakBinaryTag;
  begin
    TTag := Tag.FindFirstTag('MainSimulationTab');
    if TTag <> nil then
      FMainSimulationTab.LoadFromBinaryTag(TTag);
    //TTag := Tag.FindFirstTag('IcTabList');
    //if TTag <> nil then
    //  FIcTabList.LoadFromBinaryTag(TTag, self);
    TTag := Tag.FindFirstTag('IcSampleList');
    if TTag <> nil then
      FIcSampleList.LoadFromBinaryTag(TTag, self);
    FPageControl.ActivePageIndex := 0;
  end;

  procedure TSimulation.Paint;
  begin
    if FPageControl.Pages[FPageControl.ActivePageIndex] is TSimulationTab then                  //<----------------
      (FPageControl.Pages[FPageControl.ActivePageIndex] as TSimulationTab).Paint;
  end;

  procedure TSimulation.SwitchOn;           // all
  begin
    FMainSimulationTab.FElementBox.SwitchOn;
  end;

  procedure TSimulation.SwitchOff;         // all
  begin
    FMainSimulationTab.FElementBox.SwitchOff;
  end;

  procedure TSimulation.AddIc(AName, ACaption: string; AIcSamplePointer: TObject);
  var AIc: TIcTab;
  begin
    if FPageControl.Pages[FPageControl.ActivePageIndex] is TSimulationTab then
    begin
      AIc := TIcTab.Create(FPageControl, AName, ACaption,self, AIcSamplePointer);       
      FIcTabList.Add(AIc);
      FPageControl.ActivePageIndex := 0;
      gHasChanged := true;
    end else if FPageControl.Pages[FPageControl.ActivePageIndex] is TIcTab then
      (FPageControl.Pages[FPageControl.ActivePageIndex] as TIcTab).AddIc(AName, ACaption, AIcSamplePointer);
  end;

  procedure TSimulation.DeleteActiveIc;
  var DoIt: boolean;
  begin
    DoIt := false;
    if FPageControl.ActivePage is TIcTab then
      with FPageControl.ActivePage as TIcTab do
        if FPageControl.ActivePage is TSimulationTab then
          DoIt := true;

    if DoIt then
    begin
      if FPageControl.ActivePageIndex > 0 then
        FIcTabList.Delete(FPageControl.ActivePageIndex - 1);
    end else
      (FPageControl.Pages[FPageControl.ActivePageIndex] as TIcTab).DeleteActiveIc
  end;

  function TSimulation.IsActiveIc;
  begin
    if FPageControl.Pages[FPageControl.ActivePageIndex] is TSimulationTab then
      Result := false
    else if FPageControl.Pages[FPageControl.ActivePageIndex] is TIcTab then
      Result := true
    else Result := false;
  end;

  function TSimulation.AddIcSample(AHeight: Integer; AWidth: Integer; ACaption: string; ACaptionHorizontal: Boolean):TObject;
  var AIcSample: TIcSample;
  begin
    Result := Nil;
    if FPageControl.Pages[FPageControl.ActivePageIndex] is TSimulationTab then
    begin
      AIcSample := TIcSample.Create(AHeight, AWidth, ACaption, ACaptionHorizontal, self);
      FIcSampleList.Add(AIcSample);    // AddsIcSample
      Result := AIcSample;
    end else if FPageControl.Pages[FPageControl.ActivePageIndex] is TIcTab then
      Result := (FPageControl.Pages[FPageControl.ActivePageIndex] as TIcTab).AddIcSample(AHeight, AWidth, ACaption, ACaptionHorizontal);
  end;

  procedure TSimulation.StartClockGenerator(AInterval: word; AClockCount: integer);
  begin
    if ((FMainSimulationTab as TSimulationTab).FElementBox as TElementBox).FClockGenerator is TClockGeneratorModule then
    begin
      with ((FMainSimulationTab as TSimulationTab).FElementBox as TElementBox).FClockGenerator as TClockGeneratorModule do
      begin
        tiDelay.Interval := AInterval div 2;
        ClockCount := AClockCount * 2;
        if AInterval > 0 then
          tiDelay.Enabled := true
        else tiDelayTimer(self);
      end;
    end;
  end;

  procedure TSimulation.StopClockGenerator;
  begin
    if ((FMainSimulationTab as TSimulationTab).FElementBox as TElementBox).FClockGenerator is TClockGeneratorModule then
      (((FMainSimulationTab as TSimulationTab).FElementBox as TElementBox).FClockGenerator as TClockGeneratorModule).Reset;
    paint;
  end;

  {TIcTab}

  function TIcTab.GetResolution: TPoint;
  begin
    Result := FSimulation.GetResolution;
  end;

  procedure TIcTab.SetResolution(AResolution: TPoint);
  begin
    FSimulation.SetResolution(AResolution);
  end;

  constructor TIcTab.Create(APageControl: TComponent; AName: string; ACaption: string; AOwner: TObject; AIcSamplePointer: TObject);
  begin
    inherited Create(APageControl);
    PageControl := APageControl as TPageControl;
    Name := AName;
    FName := AName;
    Caption := 'IC: '+ACaption;
    FCaption := 'IC: '+ACaption;

    FOwner := AOwner;

    FPageControl := TPageControl.Create(self);
    FPageControl.Parent := self;
    FPageControl.Align := alClient;
    FPageControl.Style := tsFlatButtons;
    FPageControl.Show;
    
    FSimulation := TSimulation.Create(FPageControl);
    if not (AIcSamplePointer = Nil) then
      (FSimulation as TSimulation).FIcLinkPointer := AIcSamplePointer;
  end;

  destructor TIcTab.Destroy;
  begin
    FSimulation.Free;
    FPageControl.Free;
    inherited Destroy;
  end;

  procedure TIcTab.SaveToBinaryTag(Tag: TRakBinaryTag);
  begin
    //TRakBinaryShortString.AddToTag(Tag, 'Name', 'TIcTab');
    TRakBinaryShortString.AddToTag(Tag, 'TabName', FName);
    TRakBinaryShortString.AddToTag(Tag, 'TabCaption', FCaption);
    FSimulation.SaveToBinaryTag(Tag.AddTag('Simulation'));
  end;

  procedure TIcTab.LoadFromBinaryTag(Tag: TRakBinaryTag);
  var TTag: TRakBinaryTag;
  begin
    FName := Tag.AttributeAsString('TabName', '');
    FCaption := Tag.AttributeAsString('TabCaption', '');
    if (FName <> '') and (FCaption <> '') then
    begin
      Name := FName;  
      Caption := FCaption;
    end;
    
    TTag := Tag.FindFirstTag('Simulation');
    if TTag <> nil then
      FSimulation.LoadFromBinaryTag(TTag);
  end;

  procedure TIcTab.AddIc(AName, ACaption: string; AIcSamplePointer: TObject);
  begin
    FSimulation.AddIC(AName, ACaption, AIcSamplePointer);
  end;

  procedure TIcTab.DeleteActiveIc;
  begin
    FSimulation.DeleteActiveIc;
  end;

  function TIcTab.AddIcSample(AHeight: Integer; AWidth: Integer; ACaption: string; ACaptionHorizontal: Boolean):TObject;
  begin
    Result := FSimulation.AddIcSample(AHeight, AWidth, ACaption, ACaptionHorizontal);
  end;

  {TLine}

  constructor TLine.Create(AStartPoint: TPoint; AEndPoint: TPoint; AOwner: TObject);
  begin
    inherited Create;
    Owner := AOwner;
    FStatus := false;
    StartPoint := AStartPoint;
    EndPoint := AEndPoint;
    if StartPoint.X = EndPoint.X then
      Horizontal := false
    else
      Horizontal := true;
  end;

  procedure TLine.SaveToBinaryTag(Tag: TRakBinaryTag);
  begin
    TRakBinaryShortString.AddToTag(Tag, 'Name', 'TLine');
    TRakBinaryLongInt.AddToTag(Tag, 'StartPointX', StartPoint.X);
    TRakBinaryLongInt.AddToTag(Tag, 'StartPointY', StartPoint.Y);
    TRakBinaryLongInt.AddToTag(Tag, 'EndPointX', EndPoint.X);
    TRakBinaryLongInt.AddToTag(Tag, 'EndPointY', EndPoint.Y);
  end;

  procedure TLine.LoadFromBinaryTag(Tag: TRakBinaryTag);
  begin
    StartPoint := point(Tag.AttributeAsInteger('StartPointX', 0), Tag.AttributeAsInteger('StartPointY', 0));
    EndPoint := point(Tag.AttributeAsInteger('EndPointX', 0),Tag.AttributeAsInteger('EndPointY', 0));
    Status := false;
    Horizontal := StartPoint.Y = EndPoint.Y;
  end;

  procedure TLine.Paint(ACanvas: TCanvas; AVStartPoint: TPoint);
  begin
    with ACanvas do begin
      MoveTo(StartPoint.X * gDistance - AVStartPoint.X, StartPoint.Y * gDistance - AVStartPoint.Y);
      if Status then
        Pen.Width := 3
      else
        Pen.Width := 1;
      if Status then
        LineTo(EndPoint.X * gDistance - AVStartPoint.X, EndPoint.Y * gDistance - AVStartPoint.Y)
      else if Horizontal then
        LineTo(EndPoint.X * gDistance + 1 - AVStartPoint.X, EndPoint.Y * gDistance - AVStartPoint.Y)
      else LineTo(EndPoint.X * gDistance - AVStartPoint.X, EndPoint.Y * gDistance + 1 - AVStartPoint.Y);
      Pen.Width := 1;
    end;
    if (StartPoint.X = Endpoint.X) and (StartPoint.Y = Endpoint.Y) then
      with ACanvas do begin
        brush.Color := clRed;
        FillRect(rect(StartPoint.x * gDistance - 2 - AVStartPoint. X,StartPoint.y * gDistance - 2 - AVStartPoint.Y,
            StartPoint.x * gDistance + 3 - AVStartPoint.X, StartPoint.y * gDistance + 3 - AVStartPoint.Y));
      end;
  end;

  {TConnector}

  procedure TConnector.SetStatus(AStatus: Boolean);
  begin
    if not IsInput then
      FTempStatus := AStatus
    else if not (AStatus = FStatus) then
        FStatus := AStatus;
  end;

  procedure TConnector.SetNegation(AStatus: Boolean);
  begin
    if not DenyNegativationChange then
      FNegatived := AStatus;
  end;

  constructor TConnector.Create(APosition: TPoint; AOwner: TObject);
  begin
    inherited Create;
    Position := APosition;
    Owner := AOwner;
    Status := false;
    Negatived := false;
    DenyNegativationChange := false;
    DenyConnection := false;
  end;

  procedure TConnector.SaveToBinaryTag(Tag: TRakBinaryTag);
  begin
    TRakBinaryShortString.AddToTag(Tag, 'Name', 'TConnector');
    TRakBinaryBoolean.AddToTag(Tag, 'Negatived', Negatived);
    TRakBinaryBoolean.AddToTag(Tag, 'DenyNegativationChange', DenyNegativationChange);
    TRakBinaryBoolean.AddToTag(Tag, 'DenyConnection', DenyConnection);
    TRakBinaryBoolean.AddToTag(Tag, 'IsInput', IsInput);
    TRakBinaryLongInt.AddToTag(Tag, 'Orientation', ord(Orientation));
    TRakBinaryLongInt.AddToTag(Tag, 'PositionX', Position.X);
    TRakBinaryLongInt.AddToTag(Tag, 'PositionY', Position.Y);
    if not (Caption = '') then
      TRakbinaryShortString.AddToTag(Tag, 'Caption', Caption);
  end;

  procedure TConnector.LoadFromBinaryTag(Tag: TRakBinaryTag);
  begin
    Negatived := Tag.AttributeAsBoolean('Negatived', false);
    DenyNegativationChange := Tag.AttributeAsBoolean('DenyNegativationChange', false);
    DenyConnection := Tag.AttributeAsBoolean('DenyConnection', false);
    IsInput := Tag.AttributeAsBoolean('IsInput', true);
    Orientation := TOrientation(Tag.AttributeAsInteger('Orientation', -1));
    if ord(Orientation) = -1 then
      if IsInput then
        Orientation := orLeft
      else Orientation := orRight;
    Position := point(Tag.AttributeAsInteger('PositionX', 0),Tag.AttributeAsInteger('PositionY', 0));
    Caption := Tag.AttributeAsString('Caption', '');
  end;

  procedure TConnector.Paint(ACanvas: TCanvas;AVStartPoint: TPoint);
  begin
    with ACanvas do                    //Connector
    begin
      Pen.Color := clblack;
      if Status then
        Pen.Width := 3
      else Pen.Width := 1;

      case Orientation of
        orLeft: begin
          MoveTo(Position.x * gDistance - 1 - AVStartPoint.X , Position.y * gDistance - AVStartPoint.Y);
          LineTo(Position.x * gDistance - gDistance div 2 + 1 - gDistance mod 2 - AVStartPoint.X, Position.y * gDistance - AVStartPoint.Y);
        end;
        orRight: begin
          MoveTo(Position.x * gDistance  + 1 - AVStartPoint.X, Position.y * gDistance - AVStartPoint.Y);
          LineTo(Position.x * gDistance + gDistance div 2 - 1 + gDistance mod 2 - AVStartPoint.X, Position.y * gDistance - AVStartPoint.Y);
        end;
        orUp: begin
          MoveTo(Position.x * gDistance - AVStartPoint.X, Position.y * gDistance - 1 - AVStartPoint.Y);
          LineTo(Position.x * gDistance - AVStartPoint.X, Position.y * gDistance - gDistance div 2 - AVStartPoint.Y);
        end;
        orDown: begin
          MoveTo(Position.x * gDistance - AVStartPoint.X, Position.y * gDistance + 1 - AVStartPoint.Y);
          LineTo(Position.x * gDistance - AVStartPoint.X, Position.y * gDistance + gDistance div 2 - AVStartPoint.Y);
        end;
      end;        
      Pen.Width := 1;
    end;

    if Negatived then                 // Negationpoint
      with ACanvas do
      begin
        Pen.Width := 1;
        Pen.Color := clBlack;
        Brush.Color := rgb(255,255,200);
        if gDistance > 16 then
					case Orientation of
						orLeft: Ellipse(Position.x * gDistance - 7 - AVStartPoint.X, Position.Y * gDistance - 3 - AVStartPoint.Y,
							Position.x * gDistance - AVStartPoint.X, Position.y * gDistance + 4 - AVStartPoint.Y);
						orRight: Ellipse(Position.x * gDistance + 8 - AVStartPoint.X, Position.Y * gDistance - 3 - AVStartPoint.Y,
							Position.x * gDistance + 1 - AVStartPoint.X, Position.y * gDistance + 4 - AVStartPoint.Y);
						orUp: Ellipse(Position.x * gDistance - 3 - AVStartPoint.X, Position.Y * gDistance - 7 - AVStartPoint.Y,
							Position.x * gDistance + 4 - AVStartPoint.X, Position.y * gDistance - AVStartPoint.Y);
						orDown: Ellipse(Position.x * gDistance - 3 - AVStartPoint.X, Position.Y * gDistance + 1 - AVStartPoint.Y,
							Position.x * gDistance + 4 - AVStartPoint.X, Position.y * gDistance + 8 - AVStartPoint.Y);
					end
				else
					case Orientation of
						orLeft: Ellipse(Position.x * gDistance - 5 - AVStartPoint.X, Position.Y * gDistance - 2 - AVStartPoint.Y,
							Position.x * gDistance - AVStartPoint.X, Position.y * gDistance + 3 - AVStartPoint.Y);
						orRight: Ellipse(Position.x * gDistance + 6 - AVStartPoint.X, Position.Y * gDistance - 2 - AVStartPoint.Y,
							Position.x * gDistance + 1 - AVStartPoint.X, Position.y * gDistance + 3 - AVStartPoint.Y);
						orUp: Ellipse(Position.x * gDistance - 2 - AVStartPoint.X, Position.Y * gDistance - 5 - AVStartPoint.Y,
							Position.x * gDistance + 3 - AVStartPoint.X, Position.y * gDistance - AVStartPoint.Y);
						orDown: Ellipse(Position.x * gDistance - 2 - AVStartPoint.X, Position.Y * gDistance + 1 - AVStartPoint.Y,
							Position.x * gDistance + 3 - AVStartPoint.X, Position.y * gDistance + 6 - AVStartPoint.Y);
					end;
      end; //of with ACanvas do
      
    if not (Caption = '') then                 // Caption
      with ACanvas do
      begin
        Font.Size := gDistance div 2;
        if Font.Size < 7 then
          Font.Size := 7;
        Brush.Color := rgb(255,255,128);
        case Orientation of
          orLeft: TextOut(Position.X * gDistance + Font.Size div 2 - AVStartPoint.X, Position.Y * gDistance - TextHeight(Caption) div 2 - AVStartPoint.Y, Caption);
          orRight: TextOut(Position.X * gDistance - TextWidth(Caption) - Font.Size div 2 - AVStartPoint.X, Position.Y * gDistance - TextHeight(Caption) div 2 - AVStartPoint.Y, Caption);
          orUp: TextOut(Position.X * gDistance - TextWidth(Caption) div 2 - AVStartPoint.X, Position.Y * gDistance + Font.Size div 2 - AVStartPoint.Y, Caption);
          orDown: TextOut(Position.X * gDistance - TextWidth(Caption) div 2 - AVStartPoint.X, Position.Y * gDistance - TextHeight(Caption) - Font.Size div 2  - AVStartPoint.Y, Caption);
        end;
        Font.Size := 8;
      end;
  end;

  procedure TConnector.RefreshStatus;
  begin
    if not (IsInput or (FTempStatus = FStatus)) then
    begin
      FStatus := FTempStatus;
      ((Owner as TLogicModule).Owner as TElementBox).SetItemsStatus(Position,FStatus);
    end;
  end;

  {TLineConnector}

  constructor TLineConnector.Create(APosition: TPoint; AOwner: TObject);
  begin
    inherited Create;
    Position := APosition;
    Owner := AOwner;
  end;

  procedure TLineConnector.Paint(ACanvas: TCanvas; AVStartPoint: TPoint);
  begin
    with ACanvas do
    begin
      brush.Color := clblack;
      FillRect(rect(Position.x * gDistance - 2 - AVStartPoint.X,Position.y * gDistance - 2 - AVStartPoint.Y,
                    Position.x * gDistance + 3 - AVStartPoint.X,Position.y * gDistance + 3 - AVStartPoint.Y));
    end;
  end;

  procedure TLineConnector.SaveToBinaryTag(Tag: TRakBinaryTag);
  begin
    // not supposed to be saved!
  end;

  procedure TLineConnector.LoadFromBinaryTag(Tag: TRakBinaryTag);
  begin
    // not supposed to be saved!
  end;

  {TCouplingDiode}

  constructor TCouplingDiode.Create(APosition: TPoint; AOwner: TObject);
  begin
    inherited Create;
    Position := APosition;
    Owner := AOwner;
  end;

  procedure TCouplingDiode.Paint(ACanvas: TCanvas; AVStartPoint: TPoint);
  var Edges: array of TPoint;
  begin
    with ACanvas do begin
      Pen.Color := clBlack;
      Brush.Color := clblack;

      SetLength(Edges, 3);
      Edges[0] := point(Position.x * gDistance - 5 - AVStartPoint.X, Position.y * gDistance - AVStartPoint.Y);
      Edges[1] := point(Position.x * gDistance - AVStartPoint.X, Position.y * gDistance + 5 - AVStartPoint.Y);
      Edges[2] := point(Position.x * gDistance + 5 - AVStartPoint.X, Position.y * gDistance - AVStartPoint.Y);
      Polygon(Edges);
    end;
  end;

  procedure TCouplingDiode.SaveToBinaryTag(Tag: TRakBinaryTag);
  begin
    TRakBinaryShortString.AddToTag(Tag, 'Name', 'TCouplingDiode');
    TRakBinaryLongInt.AddToTag(Tag, 'PositionX', Position.X);
    TRakBinaryLongInt.AddToTag(Tag, 'PositionY', Position.Y);
  end;

  procedure TCouplingDiode.LoadFromBinaryTag(Tag: TRakBinaryTag);
  begin
    Position := point(Tag.AttributeAsInteger('PositionX', 0),Tag.AttributeAsInteger('PositionY', 0));
  end;

  procedure TCouplingDiode.CheckStatus(var Changed: boolean);      // true means something was! something wrong
  var HorizontalOn, VerticalOn, Abr: boolean;
    I: integer;
    ALine: TLine;
  begin
    HorizontalOn := false;
    VerticalOn := false;
    Abr := false;

    I := 1;
    ALine := (Owner as TElementBox).FindLinePoint(Position, I) as TLine;
    while not Abr and (ALine <> Nil) do
    begin
      if ALine.Status then
        if ALine.Horizontal then          
          HorizontalOn := true
        else VerticalOn := true; 
      if HorizontalOn and VerticalOn then
        Abr := true
      else begin
        inc(I);
        ALine := (Owner as TElementBox).FindLinePoint(Position, I) as TLine;
      end;
    end;

    if HorizontalOn and not VerticalOn then
    begin
      (Owner as TElementBox).SetItemsStatus(Position, true);
      Changed := true;
    end;    
  end;

  {IcLink}

  constructor TIcLink.Create(AOwner: TObject);
  begin
    inherited Create;
    FOwner := AOwner;
  end;

  destructor TIcLink.Destroy;
  begin
    if not (FIcModule = Nil) then
      FIcModule.Free;
    if not (FIcTab = Nil) then
      FIcTab.Free;
    inherited Destroy;
  end;

  procedure TIcLink.SaveToBinaryTag(Tag: TRakBinaryTag);
  begin
    TRakBinaryShortString.AddToTag(Tag, 'Name', 'TIcLink');
    if not (FIcModule = Nil) then
      (FIcModule as TIcModule).SaveToBinaryTag(Tag.AddTag('IcModule'));
    if not (FIcTab = Nil) then
      (FIcTab as TIcTab).SaveToBinaryTag(Tag.AddTag('IcTab'));
    // IcConnectorList
  end;

  procedure TIcLink.LoadFromBinaryTag(Tag: TRakBinaryTag);
  var AIcTab: TIcTab;
      AElement: TElement;
      TTag: TRakBinaryTag;
  begin
    TTag := Tag.FindFirstTag('IcModule');
    if not (TTag = Nil) then
    begin
      AElement := TIcModule.Create(point(0,0), (FOwner as TIcSample).FMainBox);
      AElement.LoadFromBinaryTag(TTag);
      (AElement as TIcModule).FIcSamplePointer := FOwner as TIcSample;
      ((FOwner as TIcSample).FMainBox as TElementBox).FModuleList.Add(AElement);
      FIcModule := AElement;
    end else FIcModule := Nil;

    TTag := Tag.FindFirstTag('IcTab');
    if not (TTag = Nil) then
    begin
      AIcTab := TIcTab.Create(((FOwner as TIcSample).FOwner as TSimulation).FPageControl, 'IC', 'failed', self, FOwner);
      AIcTab.LoadFromBinaryTag(TTag);
      ((FOwner as TIcSample).FOwner as TSimulation).FICTabList.Add(AIcTab);
      FIcTab := AIcTab;
    end else FIcTab := Nil;

    // IcConnectorList
  end;

    {TModule}

    constructor TModule.Create(APosition: TPoint; AOwner: TObject);
    begin
      inherited Create;
      FPosition := APosition;
      FOwner := AOwner;
      Height := 0;
      Width := 0;
    end;

    procedure TModule.SetHeight(AHeight: integer);
    begin
      if AHeight > -1 then
        FHeight := AHeight;
    end;

    procedure TModule.SetWidth(AWidth: integer);
    begin
      if AWidth > -1 then
        FWidth := AWidth;
    end;

    procedure TModule.SaveToBinaryTag(Tag: TRakBinaryTag);
    begin
      TRakBinaryLongInt.AddToTag(Tag, 'PositionX', Position.X);
      TRakBinaryLongInt.AddToTag(Tag, 'PositionY', Position.Y);
      TRakBinaryLongInt.AddToTag(Tag, 'Height', Height);
      TRakBinaryLongInt.AddToTag(Tag, 'Width', Width);
    end;

    procedure TModule.LoadFromBinaryTag(Tag: TRakBinaryTag);
    begin
      Position := point(Tag.AttributeAsInteger('PositionX', 0),Tag.AttributeAsInteger('PositionY', 0));
      Height := Tag.AttributeAsInteger('Height', 0);
      Width := Tag.AttributeAsInteger('Width', 0);
    end;

      {TLogicModule}

      constructor TLogicModule.Create(APosition: TPoint; AOrientation: TOrientation; AOwner: TObject);
      begin
        inherited Create(APosition, AOwner);
        FInputConnectorList := TElementList.Create;
        FOutputConnectorList := TElementList.Create;
        FOrientation := AOrientation;
        Caption := '';
      end;

      destructor TLogicModule.Destroy;
      begin
        FInputConnectorList.Free;
        FOutputConnectorList.Free;
        inherited Destroy;
      end;

      procedure TLogicModule.SaveToBinaryTag(Tag: TRakBinaryTag);
      begin
        inherited SaveToBinaryTag(Tag);
        FInputConnectorList.SaveToBinaryTag(Tag.AddTag('InputConnectorList'));
        FOutputConnectorList.SaveToBinaryTag(Tag.AddTag('OutputConnectorList'));
        TRakBinaryLongInt.AddToTag(Tag, 'Orientation', ord(Orientation));
      end;

      procedure TLogicModule.LoadFromBinaryTag(Tag: TRakBinaryTag);
      var LTag: TRakBinaryTag;
      begin
        inherited LoadFromBinaryTag(Tag);
        LTag := Tag.FindFirstTag('InputConnectorList');
        if LTag <> nil then
          FInputConnectorList.LoadFromBinaryTag(LTag, self);
        LTag := Tag.FindFirstTag('OutputConnectorList');
        if LTag <> nil then
          FOutputConnectorList.LoadFromBinaryTag(LTag, self);

        Orientation := TOrientation(Tag.AttributeAsInteger('Orientation', 0));
      end;

      procedure TLogicModule.Paint(ACanvas: TCanvas; AVStartPoint: TPoint);
      var Tmp, i: integer;
        ARect: TRect;
      begin
        Tmp := gDistance div 2 - 1 + gDistance mod 2;         // Box
        if Orientation in [orLeft, orRight] then
        begin
          ARect.Left := FPosition.x * gDistance - AVStartPoint.X;
          ARect.Top := FPosition.y * gDistance - Tmp - AVStartPoint.Y;
          ARect.Right := ARect.Left + Width * gDistance + 1;
          ARect.Bottom := ARect.Top + (Height + 1) * gDistance - 1;
        end else begin
          ARect.Left := FPosition.x * gDistance - Tmp - AVStartPoint.X;
          ARect.Top := FPosition.y * gDistance - AVStartPoint.Y;
          ARect.Right := ARect.Left + (Width + 1) * gDistance - 1;
          ARect.Bottom := ARect.Top + Height * gDistance + 1;
        end;

        with ACanvas do
        begin
          Brush.Color := rgb(255,255,128);
          Pen.Color := clBlack;
          Rectangle(ARect);
        end;

        for I := 0 to FInputConnectorList.Count - 1 do      //Connector
          if not (FInputConnectorList[i] as TConnector).DenyConnection then
            (FInputConnectorList[i] as TConnector).Paint(ACanvas, AVStartPoint);
        for I := 0 to FOutputConnectorList.Count - 1 do
          if not (FOutputConnectorList[i] as TConnector).DenyConnection then
            (FOutputConnectorList[i] as TConnector).Paint(ACanvas, AVStartPoint);
      end;

      procedure TLogicModule.AddInputConnector(APosition: TPoint; AOrientation: TOrientation; AOwner: TObject);
      var AConnector: TConnector;
      begin
        AConnector := TConnector.Create(APosition, AOwner);
        AConnector.IsInput := true;
        AConnector.Orientation := AOrientation;
        FInputConnectorList.Add(AConnector);
      end;

      procedure TLogicModule.AddOutputConnector(APosition: TPoint; AOrientation: TOrientation; AOwner: TObject);
      var AConnector: TConnector;
      begin
        AConnector := TConnector.Create(APosition, AOwner);
        AConnector.IsInput := false;
        AConnector.Orientation := AOrientation;
        FOutputConnectorList.Add(AConnector);
      end;

      procedure TLogicModule.UpdateStatus;
      var I: integer;
      begin
        for I := 0 to FOutputConnectorList.Count - 1 do
          (FOutputConnectorList[I] as TConnector).RefreshStatus;
      end;

        {TStandardModule}

        constructor TStandardModule.Create(APosition: TPoint; ASize: integer; AOrientation: TOrientation; AOwner: TObject);
        var I: integer;
        begin
          inherited Create(APosition, AOrientation, AOwner);
          Position := APosition;
          FSize := ASize;
          Caption := '';

          if AOrientation in [orLeft, orRight] then
          begin
            Height := ASize - 1;
            Width := 2;
          end else begin
            Height := 2;
            Width := ASize - 1;
          end;

          for I := 0 to ASize - 1 do              // Input
            case AOrientation of
              orLeft: AddInputConnector(point(Position.X, Position.Y + I), AOrientation, self);
              orRight: AddInputConnector(point(Position.X + 2, Position.Y + I), AOrientation, self);
              orUp:  AddInputConnector(point(Position.X + I, Position.Y), AOrientation, self);
              orDown: AddInputConnector(point(Position.X + I, Position.Y + 2), AOrientation, self);
            end;
          if ASize > 0 then
            case AOrientation of                    // Output
              orLeft: AddOutputConnector(point(Position.X + 2, Position.Y + ASize div 2 - 1 + ASize mod 2), orRight, self);
              orRight: AddOutputConnector(point(Position.X, Position.Y + ASize div 2 - 1 + ASize mod 2), orLeft, self);
              orUp: AddOutputConnector(point(Position.X + ASize div 2 - 1 + ASize mod 2, Position.Y + 2), orDown, self);
              orDown: AddOutputConnector(point(Position.X + ASize div 2 - 1 + ASize mod 2, Position.Y), orUp, self);
            end;
        end;

        procedure TStandardModule.SaveToBinaryTag(Tag: TRakBinaryTag);
        begin
          inherited SaveToBinaryTag(Tag);
          TRakBinaryLongInt.AddToTag(Tag, 'FSize', FSize);
        end;

        procedure TStandardModule.LoadFromBinaryTag(Tag: TRakBinaryTag);
        begin
          inherited LoadFromBinaryTag(Tag);
          FSize := Tag.AttributeAsInteger('FSize', 3);
        end;


        procedure TStandardModule.Paint(ACanvas: TCanvas; AVStartPoint: TPoint);
        var XPosition, yPosition: integer;
        begin
          inherited Paint(ACanvas, AVStartPoint);
          XPosition := FPosition.x * gDistance - AVStartPoint.X
            + (Width * gDistance - ACanvas.TextWidth(Caption)) div 2;
          YPosition := FPosition.y * gDistance - AVStartPoint.Y
            + (Height * gDistance - ACanvas.TextHeight(Caption)) div 2;

          if FOrientation in [orLeft, orRight] then
            dec(YPosition)
          else
            dec(XPosition);

          with ACanvas do                           // Caption
          begin
            Font.Size := gDistance div 2 + 2;
            Brush.Color := rgb(255,255,128);
            if Caption = '>1' then
            begin
              SetBkMode(ACanvas.Handle, TRANSPARENT);
              TextOutW(ACanvas.Handle,
                  XPosition,
                  YPosition,
                  WideChar(8805), 1);           // "8805" = "<="
              TextOut(XPosition + TextWidth('>'), YPosition, '1');
            end else
              TextOut(XPosition, YPosition, Caption);
            Font.Size := 8;
          end;
        end;

          {TAndModule}

          constructor TAndModule.Create(APosition: TPoint; ASize: integer; AOrientation: TOrientation; AOwner: TObject);
          begin
            inherited Create(APosition, ASize, AOrientation, AOwner);
            Caption := '&';
          end;

          procedure TAndModule.SaveToBinaryTag(Tag: TRakBinaryTag);
          begin
            inherited SaveToBinaryTag(Tag);
            TRakBinaryShortString.AddToTag(Tag, 'Name', 'TAndModule');
          end;

          procedure TAndModule.ChangedInputStatus(var Changed: boolean);
          var I, TrueCount: integer;
          begin
            TrueCount := 0;
            for I := 0 to FInputConnectorList.Count - 1 do
              if (FInputConnectorList[I] as TConnector).Status xor (FInputConnectorList[I] as TConnector).Negatived then
                inc(TrueCount);

            if (TrueCount = FInputConnectorList.Count) xor (FOutputConnectorList[0] as TConnector).Negatived then
            begin
              if not (FOutputConnectorList[0] as TConnector).Status  then
              begin
                (FOutputConnectorList[0] as TConnector).Status := true;
                Changed := true;
              end;
            end
            else
              if (FOutputConnectorList[0] as TConnector).Status then
              begin
                (FOutputConnectorList[0] as TConnector).Status := false;
                Changed := true;
              end;
          end;

          {TOrModule}

          constructor TOrModule.Create(APosition: TPoint; ASize: integer; AOrientation: TOrientation; AOwner: TObject);
          begin
            inherited Create(APosition, ASize, AOrientation, AOwner);
            Caption := '>1';      // >=1
          end;

          procedure TOrModule.SaveToBinaryTag(Tag: TRakBinaryTag);
          begin
            inherited SaveToBinaryTag(Tag);
            TRakBinaryShortString.AddToTag(Tag, 'Name', 'TOrModule');
          end;

          procedure TOrModule.ChangedInputStatus(var Changed: boolean);
          var I, TrueCount: integer;
          begin
            TrueCount := 0;
            for I := 0 to FInputConnectorList.Count - 1 do
              if (FInputConnectorList[I] as TConnector).Status xor (FInputConnectorList[I] as TConnector).Negatived then
                inc(TrueCount);

            if (TrueCount >= 1) xor (FOutputConnectorList[0] as TConnector).Negatived then
            begin
              if not (FOutputConnectorList[0] as TConnector).Status  then
              begin
                (FOutputConnectorList[0] as TConnector).Status := true;
                Changed := true;
              end;
            end
            else
              if (FOutputConnectorList[0] as TConnector).Status then
              begin
                (FOutputConnectorList[0] as TConnector).Status := false;
                Changed := true;
              end;
          end;

          {TXorModule}

          constructor TXorModule.Create(APosition: TPoint; ASize: integer; AOrientation: TOrientation; AOwner: TObject);
          begin
            inherited Create(APosition, ASize, AOrientation, AOwner);
            Caption := '=1';
          end;

          procedure TXorModule.SaveToBinaryTag(Tag: TRakBinaryTag);
          begin
            inherited SaveToBinaryTag(Tag);
            TRakBinaryShortString.AddToTag(Tag, 'Name', 'TXorModule');
          end;

          procedure TXorModule.ChangedInputStatus(var Changed: boolean);
          var I, TrueCount: integer;
          begin
            TrueCount := 0;
            for I := 0 to FInputConnectorList.Count - 1 do
              if (FInputConnectorList[I] as TConnector).Status xor (FInputConnectorList[I] as TConnector).Negatived then
                inc(TrueCount);

            if (TrueCount = 1) xor (FOutputConnectorList[0] as TConnector).Negatived then
            begin
              if not (FOutputConnectorList[0] as TConnector).Status  then
              begin
                (FOutputConnectorList[0] as TConnector).Status := true;
                Changed := true;
              end;
            end
            else
              if (FOutputConnectorList[0] as TConnector).Status then
              begin
                (FOutputConnectorList[0] as TConnector).Status := false;
                Changed := true;
              end;
          end;

        {TInverterModule}

        constructor TInverterModule.Create(APosition: TPoint; AOrientation: TOrientation; AOwner: TObject);
        begin
          inherited Create(APosition, AOrientation, AOwner);
          if AOrientation in [orLeft, orRight] then
            Width := 1
          else Height := 1;

          Caption := '1';
          case AOrientation of
            orLeft: begin
              AddInputConnector(APosition, AOrientation, self);
              AddOutputConnector(point(APosition.X + 1, APosition.Y), orRight, self);
            end;
            orRight: begin
              AddInputConnector(point(APosition.X + 1, APosition.Y), AOrientation, self);
              AddOutputConnector(APosition, orLeft, self);
            end;
            orUp: begin
              AddInputConnector(APosition, AOrientation, self);
              AddOutputConnector(point(APosition.X, APosition.Y + 1), orDown, self);
            end;
            orDown: begin
              AddInputConnector(point(APosition.X, APosition.Y + 1), AOrientation, self);
              AddOutputConnector(APosition, orUp, self);
            end;
          end;               

          (FInputConnectorList[0] as TConnector).DenyNegativationChange := true;
          (FOutputConnectorList[0] as TConnector).Negatived := true;
          (FOutputConnectorList[0] as TConnector).DenyNegativationChange := true;
        end;

        procedure TInverterModule.Paint(ACanvas: TCanvas; AVStartPoint: TPoint);
        var XPosition, YPosition: integer;
        begin
          inherited Paint(ACanvas, AVStartPoint);
          XPosition := FPosition.x * gDistance - AVStartPoint.X;
          YPosition := FPosition.y * gDistance - gDistance div 2 - AVStartPoint.Y;
          with ACanvas do                           // Caption
          begin
            Font.Size := gDistance div 2;
            if gDistance div 2 <= 8 then
              Font.Size := 7;
            if gDistance <= 14 then
              Font.Size := 6;

            Brush.Color := rgb(255,255,128);
            TextOut(XPosition + (Width * gDistance - TextWidth(Caption)) div 2,
              YPosition + ((Height + 1) * gDistance - TextHeight(Caption)) div 2, Caption);
            Font.Size := 8;
          end;
        end;

        procedure TInverterModule.ChangedInputStatus(var Changed: Boolean);
        begin
          if (FInputConnectorList[0] as TConnector).Status then
            if (FOutputConnectorList[0] as TConnector).Status then
            begin
              (FOutputConnectorList[0] as TConnector).Status := false;
              Changed := true;
            end else
          else if not (FOutputConnectorList[0] as TConnector).Status then
          begin
            (FOutputConnectorList[0] as TConnector).Status := true;
            Changed := true;
          end;
        end;

        procedure TInverterModule.SaveToBinaryTag(Tag: TRakBinaryTag);
          begin
            inherited SaveToBinaryTag(Tag);
            TRakBinaryShortString.AddToTag(Tag, 'Name', 'TInverterModule');
          end;

        {TDelayModule}

        constructor TDelayModule.Create(APosition: TPoint; AOrientation: TOrientation; AOwner: TObject);
        begin
          inherited Create(APosition, AOrientation, AOwner);
          if AOrientation in [orLeft, orRight] then
            Width := 1
          else Height := 1;
          Caption := '- -';
          
          case AOrientation of
            orLeft, orRight: begin
              AddInputConnector(APosition, AOrientation, self);
              AddOutputConnector(point(APosition.X + 1, APosition.Y), orRight, self);
              AddInputConnector(point(APosition.X + 1, APosition.Y), AOrientation, self);
              AddOutputConnector(APosition, orLeft, self);
            end;
            orUp, orDown: begin
              AddInputConnector(APosition, AOrientation, self);
              AddOutputConnector(point(APosition.X, APosition.Y + 1), orDown, self);
              AddInputConnector(point(APosition.X, APosition.Y + 1), AOrientation, self);
              AddOutputConnector(APosition, orUp, self);
            end;
          end;

          (FInputConnectorList[0] as TConnector).DenyNegativationChange := true;
          (FOutputConnectorList[0] as TConnector).DenyNegativationChange := true;
          (FInputConnectorList[1] as TConnector).DenyNegativationChange := true;
          (FOutputConnectorList[1] as TConnector).DenyNegativationChange := true;
        end;

        procedure TDelayModule.Paint(ACanvas: TCanvas; AVStartPoint: TPoint);
        var XPosition, YPosition, Tmp, i: integer;
          ARect: TRect;
        begin
          for I := 0 to FInputConnectorList.Count - 1 do      //Connector
            if not (FInputConnectorList[i] as TConnector).DenyConnection then
              (FInputConnectorList[i] as TConnector).Paint(ACanvas, AVStartPoint);
          for I := 0 to FOutputConnectorList.Count - 1 do
            if not (FOutputConnectorList[i] as TConnector).DenyConnection then
              (FOutputConnectorList[i] as TConnector).Paint(ACanvas, AVStartPoint);


          Tmp := gDistance div 2 - 1 + gDistance mod 2;         // Box
          if Orientation in [orLeft, orRight] then
          begin
            ARect.Left := FPosition.x * gDistance - AVStartPoint.X;
            ARect.Top := FPosition.y * gDistance - Tmp - AVStartPoint.Y;
            ARect.Right := ARect.Left + Width * gDistance + 1;
            ARect.Bottom := ARect.Top + (Height + 1) * gDistance - 1;
          end else begin
            ARect.Left := FPosition.x * gDistance - Tmp - AVStartPoint.X;
            ARect.Top := FPosition.y * gDistance - AVStartPoint.Y;
            ARect.Right := ARect.Left + (Width + 1) * gDistance - 1;
            ARect.Bottom := ARect.Top + Height * gDistance + 1;
          end;

          with ACanvas do
          begin
            Brush.Color := rgb(255,255,128);
            Pen.Color := clBlack;
            Rectangle(ARect);
          end;

          XPosition := FPosition.x * gDistance - AVStartPoint.X
            + (Width * gDistance - ACanvas.TextWidth(Caption)) div 2;
          YPosition := FPosition.y * gDistance - AVStartPoint.Y
            + (Height * gDistance - ACanvas.TextHeight(Caption)) div 2;

          with ACanvas do                           // Caption
          begin
            Font.Size := gDistance div 2;
            if gDistance div 2 <= 8 then
              Font.Size := 7;
            if gDistance <= 14 then
              Font.Size := 6;

            Brush.Color := rgb(255,255,128);
            TextOut(XPosition, YPosition, Caption);
            Font.Size := 8;
          end;
        end;

        procedure TDelayModule.ChangedInputStatus(var Changed: Boolean);
        begin
          if (FInputConnectorList[0] as TConnector).Status then
            if not (FOutputConnectorList[0] as TConnector).Status then
            begin
              (FOutputConnectorList[0] as TConnector).Status := true;
              Changed := true;
            end else
          else if (FOutputConnectorList[0] as TConnector).Status then
          begin
            (FOutputConnectorList[0] as TConnector).Status := false;
            Changed := true;
          end;

          if (FInputConnectorList[1] as TConnector).Status then
            if not (FOutputConnectorList[1] as TConnector).Status then
            begin
              (FOutputConnectorList[1] as TConnector).Status := true;
              Changed := true;
            end else
          else if (FOutputConnectorList[1] as TConnector).Status then
          begin
            (FOutputConnectorList[1] as TConnector).Status := false;
            Changed := true;
          end;
        end;

        procedure TDelayModule.SaveToBinaryTag(Tag: TRakBinaryTag);
          begin
            inherited SaveToBinaryTag(Tag);
            TRakBinaryShortString.AddToTag(Tag, 'Name', 'TDelayModule');
          end;

        {THalfAdderModule}

        constructor THalfAdderModule.Create(APosition: TPoint; AOrientation: TOrientation; AOwner: TObject);
        var I: integer;
        begin
          inherited Create(APosition, AOrientation, AOwner);
          if AOrientation in [orLeft, orRight] then
          begin
            Width := 3;
            Height := 2;
          end else begin
            Width := 2;
            Height := 3;
          end;
          Caption := 'HA';
          for I := 0 to 2 - 1 do
          begin
            case AOrientation of
              orLeft: begin
                AddInputConnector(point(APosition.X, APosition.Y + I * 2), AOrientation, self);
                AddOutputConnector(point(APosition.X + 3, APosition.Y + I * 2), orRight, self);
              end;
              orRight: begin
                AddInputConnector(point(APosition.X + 3, APosition.Y + I * 2), AOrientation, self);
                AddOutputConnector(point(APosition.X, APosition.Y + I * 2), orLeft, self);
              end;
              orUp: begin
                AddInputConnector(point(APosition.X + I * 2, APosition.Y), AOrientation, self);
                AddOutputConnector(point(APosition.X + 2 - I * 2, APosition.Y + 3), orDown, self);
              end;
              orDown: begin
                AddInputConnector(point(APosition.X + I * 2, APosition.Y + 3), AOrientation, self);
                AddOutputConnector(point(APosition.X + 2 - I * 2, APosition.Y), orUp, self);
              end;
            end;
            (FInputConnectorList[I] as TConnector).DenyNegativationChange := true;
            (FOutputConnectorList[I] as TConnector).DenyNegativationChange := true;
          end;
          (FOutputConnectorList[0] as TConnector).Caption := 's';
          (FOutputConnectorList[1] as TConnector).Caption := 'ü';
        end;

        procedure THalfAdderModule.Paint(ACanvas: TCanvas; AVStartPoint: TPoint);
        var XPosition, yPosition: integer;
        begin
          inherited Paint(ACanvas, AVStartPoint);
          XPosition := FPosition.x * gDistance - AVStartPoint.X;
          YPosition := FPosition.y * gDistance - gDistance div 2 - AVStartPoint.Y;
          with ACanvas do                           // Caption
          begin
            Font.Size := gDistance div 2;
            if gDistance div 2 < 8 then
              Font.Size := 8;
            if gDistance < 13 then
              Font.Size := 6;
            Font.Size := Font.Size + 3;
            Brush.Color := rgb(255,255,128);
            TextOut(XPosition + (Width * gDistance - TextWidth(Caption)) div 2,
              YPosition + ((Height + 1) * gDistance - TextHeight(Caption)) div 2, Caption);
            Font.Size := 8;
          end;
        end;

        procedure THalfAdderModule.ChangedInputStatus(var Changed: Boolean);
        begin
          if (((FInputConnectorList[0] as TConnector).Status xor (FInputConnectorList[1] as TConnector).Status)
             xor (FOutputConnectorList[0] as TConnector).Status) then
          begin
            (FOutputConnectorList[0] as TConnector).Status := (FInputConnectorList[0] as TConnector).Status xor (FInputConnectorList[1] as TConnector).Status;
            Changed := true;
          end;
          if (((FInputConnectorList[0] as TConnector).Status and (FInputConnectorList[1] as TConnector).Status)
             xor (FOutputConnectorList[1] as TConnector).Status) then
          begin
            (FOutputConnectorList[1] as TConnector).Status := (FInputConnectorList[0] as TConnector).Status and (FInputConnectorList[1] as TConnector).Status;
            Changed := true;
          end;
        end;

        procedure THalfAdderModule.SaveToBinaryTag(Tag: TRakBinaryTag);
        begin
          inherited SaveToBinaryTag(Tag);
          TRakBinaryShortString.AddToTag(Tag, 'Name', 'THalfAdderModule');
        end;

        {TFullAdderModule}

        constructor TFullAdderModule.Create(APosition: TPoint; AOrientation: TOrientation; AOwner: TObject);
        var I: integer;
        begin
          inherited Create(APosition, AOrientation, AOwner);
          if AOrientation in [orLeft, orRight] then
          begin
            Width := 3;
            Height := 2;
          end else begin
            Width := 2;
            Height := 3;
          end;
          Caption := 'VA';
          for I := 0 to 3 - 1 do
          begin
            case AOrientation of
              orLeft: begin
                AddInputConnector(point(APosition.X, APosition.Y + I), AOrientation, self);
                if I < 2 then
                  AddOutputConnector(point(APosition.X + 3, APosition.Y + I * 2), orRight, self);
              end;
              orRight: begin
                AddInputConnector(point(APosition.X + 3, APosition.Y + I), AOrientation, self);
                if I < 2 then
                  AddOutputConnector(point(APosition.X, APosition.Y + I * 2), orLeft, self);
              end;
              orUp: begin
                AddInputConnector(point(APosition.X + I, APosition.Y), AOrientation, self);
                if I < 2 then
                  AddOutputConnector(point(APosition.X + 2 - I * 2, APosition.Y + 3), orDown, self);
              end;
              orDown: begin
                AddInputConnector(point(APosition.X + I, APosition.Y + 3), AOrientation, self);
                if I < 2 then
                  AddOutputConnector(point(APosition.X + 2 - I * 2, APosition.Y), orUp, self);
              end;
            end;
            (FInputConnectorList[I] as TConnector).DenyNegativationChange := true;
            if I < 2 then
              (FOutputConnectorList[I] as TConnector).DenyNegativationChange := true;
          end;

          (FOutputConnectorList[0] as TConnector).Caption := 's';
          (FOutputConnectorList[1] as TConnector).Caption := 'ü';
        end;

        procedure TFullAdderModule.Paint(ACanvas: TCanvas; AVStartPoint: TPoint);
        var XPosition, yPosition: integer;
        begin
          inherited Paint(ACanvas, AVStartPoint);
          XPosition := FPosition.x * gDistance - AVStartPoint.X;
          YPosition := FPosition.y * gDistance - gDistance div 2 - AVStartPoint.Y;
          with ACanvas do                           // Caption
          begin
            Font.Size := gDistance div 2;
            if gDistance div 2 < 8 then
              Font.Size := 8;
            if gDistance < 13 then
              Font.Size := 6;
            Font.Size := Font.Size + 3;
            Brush.Color := rgb(255,255,128);
            TextOut(XPosition + (Width * gDistance - TextWidth(Caption)) div 2,
              YPosition + ((Height + 1) * gDistance - TextHeight(Caption)) div 2, Caption);
            Font.Size := 8;
          end;
        end;

        procedure TFullAdderModule.ChangedInputStatus(var Changed: Boolean);
        var I, InpOnCount: integer;
        begin
          InpOnCount := 0;
          for I := 0 to FInputConnectorList.Count - 1 do
            if (FInputConnectorList[I] as TConnector).Status then
              inc(InpOnCount);

          if ((InpOnCount = 1) or (InpOnCount = 3)) xor (FOutputConnectorList[0] as TConnector).Status then
          begin
            (FOutputConnectorList[0] as TConnector).Status := (InpOnCount = 1) or (InpOnCount = 3);
            Changed := true;
          end;
          if (InpOnCount >= 2) xor (FOutputConnectorList[1] as TConnector).Status then
          begin
            (FOutputConnectorList[1] as TConnector).Status := InpOnCount >= 2;
            Changed := true;
          end;
        end;

        procedure TFullAdderModule.SaveToBinaryTag(Tag: TRakBinaryTag);
        begin
          inherited SaveToBinaryTag(Tag);
          TRakBinaryShortString.AddToTag(Tag, 'Name', 'TFullAdderModule');
        end;

        {TFlipFlopModule}

        constructor TFlipFlopModule.Create(APosition: TPoint; AOrientation: TOrientation; AOwner: TObject);
        begin
          inherited Create(APosition, AOrientation, AOwner);
          if AOrientation in [orLeft, orRight] then
          begin
            Width := 4;
            Height := 2;
          end else begin
            Width := 2;
            Height := 4;
          end;

          FInputMaster.J := false;
          FInputMaster.C := false;
          FInputMaster.K := false;

          case AOrientation of
            orLeft: begin
              AddInputConnector(APosition, orLeft, self);                                 // J
              AddInputConnector(point(APosition.X, APosition.Y + 1), orLeft, self);       // C
              AddInputConnector(point(APosition.X, APosition.Y + 2), orLeft, self);       // K
              AddInputConnector(point(APosition.X + 2, APosition.Y), orUp, self);         // S
              AddInputConnector(point(APosition.X + 2, APosition.Y + 2), orDown, self);   // R
              AddOutputConnector(point(APosition.X + 4, APosition.Y), orRight, self);     // Q
              AddOutputConnector(point(APosition.X + 4, APosition.Y + 2), orRight, self); // Q (neg)
            end;
            orRight: begin
              AddInputConnector(point(APosition.X + 4, APosition.Y), AOrientation, self);           // J
              AddInputConnector(point(APosition.X + 4, APosition.Y + 1), AOrientation, self);       // C
              AddInputConnector(point(APosition.X + 4, APosition.Y + 2), AOrientation, self);       // K
              AddInputConnector(point(APosition.X + 2, APosition.Y), orUp, self);               // S
              AddInputConnector(point(APosition.X + 2, APosition.Y + 2), orDown, self);   // R
              AddOutputConnector(point(APosition.X, APosition.Y), orLeft, self);     // Q
              AddOutputConnector(point(APosition.X, APosition.Y + 2), orLeft, self); // Q (neg)
            end;                                     
            orUp: begin
              AddInputConnector(point(APosition.X + 2, APosition.Y), AOrientation, self);           // J
              AddInputConnector(point(APosition.X + 1, APosition.Y), AOrientation, self);       // C
              AddInputConnector(point(APosition.X, APosition.Y), AOrientation, self);       // K
              AddInputConnector(point(APosition.X + 2, APosition.Y + 2), orRight, self);         // S
              AddInputConnector(point(APosition.X, APosition.Y + 2), orLeft, self);   // R
              AddOutputConnector(point(APosition.X + 2, APosition.Y + 4), orDown, self);     // Q
              AddOutputConnector(point(APosition.X, APosition.Y + 4), orDown, self); // Q (neg)
            end;
            orDown: begin
              AddInputConnector(point(APosition.X + 2, APosition.Y + 4), AOrientation, self);           // J
              AddInputConnector(point(APosition.X + 1, APosition.Y + 4), AOrientation, self);       // C
              AddInputConnector(point(APosition.X, APosition.Y + 4), AOrientation, self);       // K
              AddInputConnector(point(APosition.X + 2, APosition.Y + 2), orRight, self);         // S
              AddInputConnector(point(APosition.X, APosition.Y + 2), orLeft, self);   // R
              AddOutputConnector(point(APosition.X + 2, APosition.Y), orUp, self);     // Q
              AddOutputConnector(point(APosition.X, APosition.Y), orUp, self); // Q (neg)
            end;
          end;

          (FInputConnectorList[3] as TConnector).DenyNegativationChange := true;// DenyNeg.
          (FInputConnectorList[4] as TConnector).DenyNegativationChange := true;

          (FInputConnectorList[0] as TConnector).Caption := 'J';                // Set Caption
          case Orientation of
            orLeft: (FInputConnectorList[1] as TConnector).Caption := '   C';
            orRight: (FInputConnectorList[1] as TConnector).Caption := 'C   ';
          end;
          (FInputConnectorList[2] as TConnector).Caption := 'K';
          (FInputConnectorList[3] as TConnector).Caption := 'S';
          (FInputConnectorList[4] as TConnector).Caption := 'R';             
          (FOutputConnectorList[0] as TConnector).Caption := '0';
          (FOutputConnectorList[1] as TConnector).Status := true;
        end;

        procedure TFlipFlopModule.Paint(ACanvas: TCanvas; AVStartPoint: TPoint);
        var Edges: array of TPoint;
          I: integer;
          XPosition, yPosition: integer;
        begin
          inherited Paint(ACanvas, AVStartPoint);

					with ACanvas do begin
						Pen.Color := clBlack;
						Brush.Color := rgb(255,255,128);
			
						SetLength(Edges, 3);

            XPosition := 0;
            YPosition := 0;
            case Orientation of
              orLeft: begin
                Edges[0] := point(Position.x * gDistance - AVStartPoint.X,
                  (Position.y + 1) * gDistance - gDistance div 4 - AVStartPoint.Y);
						    Edges[1] := point(Position.x * gDistance + gDistance div 2 - AVStartPoint.X,
                  (Position.y + 1) * gDistance - AVStartPoint.Y);
						    Edges[2] := point(Position.x * gDistance - AVStartPoint.X,
                  (Position.y + 1) * gDistance + gDistance div 4 - AVStartPoint.Y);
                for I := 1 to 3 do
                begin
                  MoveTo((Position.x + I) * gDistance + gDistance div 2 - AVStartPoint.X,
                    (Position.y + 1) * gDistance - AVStartPoint.Y);
                  LineTo((Position.x + I + 1) * gDistance - AVStartPoint.X,
                    (Position.y + 1) * gDistance - AVStartPoint.Y);
                end;
              end;
              orRight: begin
                Edges[0] := point((Position.x + 4) * gDistance - AVStartPoint.X,
                  (Position.y + 1) * gDistance - gDistance div 4 - AVStartPoint.Y);
						    Edges[1] := point((Position.x + 4) * gDistance - gDistance div 2 - AVStartPoint.X,
                  (Position.y + 1) * gDistance - AVStartPoint.Y);
						    Edges[2] := point((Position.x + 4) * gDistance - AVStartPoint.X,
                  (Position.y + 1) * gDistance + gDistance div 4 - AVStartPoint.Y);
                for I := 1 to 3 do
                begin
                  MoveTo((Position.x + 4 - I) * gDistance - gDistance div 2 - AVStartPoint.X,
                    (Position.y + 1) * gDistance - AVStartPoint.Y);
                  LineTo((Position.x + 4 - I - 1) * gDistance - AVStartPoint.X,
                    (Position.y + 1) * gDistance - AVStartPoint.Y);
                end;
              end;
              orUp: begin
                Edges[0] := point((Position.x + 1) * gDistance - gDistance div 4 - AVStartPoint.X,
                  Position.y * gDistance - AVStartPoint.Y);
						    Edges[1] := point((Position.x + 1) * gDistance - AVStartPoint.X,
                  Position.y * gDistance + gDistance div 2 - AVStartPoint.Y);
						    Edges[2] := point((Position.x + 1) * gDistance + gDistance div 4 - AVStartPoint.X,
                  Position.y * gDistance - AVStartPoint.Y);
                for I := 1 to 3 do
                begin
                  MoveTo((Position.x + 1) * gDistance - AVStartPoint.X,
                    (Position.y + I) * gDistance + gDistance div 2 - AVStartPoint.Y);
                  LineTo((Position.x + 1) * gDistance - AVStartPoint.X,
                    (Position.y + I + 1) * gDistance - AVStartPoint.Y);
                end;
                XPosition := (FPosition.x + 1) * gDistance - AVStartPoint.X;  // Caption
                YPosition := (FPosition.y + 1) * gDistance - AVStartPoint.Y;
              end;
              orDown: begin
                Edges[0] := point((Position.x + 1) * gDistance - gDistance div 4 - AVStartPoint.X,
                  (Position.y + 4) * gDistance - AVStartPoint.Y);
						    Edges[1] := point((Position.x + 1) * gDistance - AVStartPoint.X,
                  (Position.y + 4) * gDistance - gDistance div 2 - AVStartPoint.Y);
						    Edges[2] := point((Position.x + 1) * gDistance + gDistance div 4 - AVStartPoint.X,
                  (Position.y + 4) * gDistance - AVStartPoint.Y);
                for I := 1 to 3 do
                begin
                  MoveTo((Position.x + 1) * gDistance - AVStartPoint.X,
                    (Position.y + 4 - I) * gDistance - gDistance div 2 - AVStartPoint.Y);
                  LineTo((Position.x + 1) * gDistance - AVStartPoint.X,
                    (Position.y + 4 - I - 1) * gDistance - AVStartPoint.Y);
                end;
                XPosition := (FPosition.x + 1) * gDistance - AVStartPoint.X;    // Caption
                YPosition := (FPosition.y + 3) * gDistance - AVStartPoint.Y;
              end;
            end;  // of case
            if Orientation in [orUp, orDown] then
              with ACanvas do                           // Caption
          			begin
                  Font.Size := gDistance div 2;
                  if Font.Size < 7 then
                    Font.Size := 7;

									Brush.Color := rgb(255,255,128);
									TextOut(XPosition - (TextWidth('C')) div 2,
										YPosition - (TextHeight('C')) div 2, 'C');
									Font.Size := 8;
								end;
						Polygon(Edges);
            Brush.Color := clBlack;
					end;
        end;

        procedure TFlipFlopModule.ChangedInputStatus(var Changed: Boolean);
        var AStatus: boolean;
        begin
          if (FInputConnectorList[4] as TConnector).Status then       // R
          begin
            if (FOutputConnectorList[0] as TConnector).Status then
            begin
              (FOutputConnectorList[0] as TConnector).Status := false;
              Changed := true;
            end;
            if not (FOutputConnectorList[1] as TConnector).Status then
            begin
              (FOutputConnectorList[1] as TConnector).Status := true;
              Changed := true;
            end;
            FInputMaster.J := false;
            FInputMaster.C := false;
            FInputMaster.K := false;
          end else if (FInputConnectorList[3] as TConnector).Status then  // S
          begin
            if not (FOutputConnectorList[0] as TConnector).Status then
            begin
              (FOutputConnectorList[0] as TConnector).Status := true;
              Changed := true;
            end;
            if (FOutputConnectorList[1] as TConnector).Status then
            begin
              (FOutputConnectorList[1] as TConnector).Status := false;
              Changed := true;
            end;
            FInputMaster.J := false;
            FInputMaster.C := false;
            FInputMaster.K := false;
          end else begin
          	if ((FInputConnectorList[1] as TConnector).Status xor (FInputConnectorList[1] as TConnector).Negatived)
							and not (FInputMaster.C) then                                 // rising edge
						begin
							FInputMaster.J := (FInputConnectorList[0] as TConnector).Status xor (FInputConnectorList[0] as TConnector).Negatived;
							FInputMaster.K := (FInputConnectorList[2] as TConnector).Status xor (FInputConnectorList[2] as TConnector).Negatived;
						end;
	
						if not ((FInputConnectorList[1] as TConnector).Status xor (FInputConnectorList[1] as TConnector).Negatived)
							and FInputMaster.C then                                       // trailing edge
						begin
							if FInputMaster.J xor FInputMaster.K then                         // set
							begin
								(FOutputConnectorList[0] as TConnector).Status := FInputMaster.J xor (FOutputConnectorList[0] as TConnector).Negatived;
								(FOutputConnectorList[1] as TConnector).Status := FInputMaster.K xor (FOutputConnectorList[1] as TConnector).Negatived;
								Changed := true;
							end else if FInputMaster.J and FInputMaster.K then                // exchange
								begin
									AStatus := (FOutputConnectorList[0] as TConnector).Status xor (FOutputConnectorList[0] as TConnector).Negatived;
									(FOutputConnectorList[0] as TConnector).Status := (FOutputConnectorList[1] as TConnector).Status xor (FOutputConnectorList[1] as TConnector).Negatived;
									(FOutputConnectorList[1] as TConnector).Status := AStatus;
									Changed := true;
								end;
						end;
						FInputMaster.C := (FInputConnectorList[1] as TConnector).Status;
          end;

          if Changed then                                                   // set caption
            if (FOutputConnectorList[0] as TConnector).FTempStatus then
              (FOutputConnectorList[0] as TConnector).Caption := '1'
            else (FOutputConnectorList[0] as TConnector).Caption := '0';
        end;

        procedure TFlipFlopModule.SaveToBinaryTag(Tag: TRakBinaryTag);
        begin
          inherited SaveToBinaryTag(Tag);
          TRakBinaryShortString.AddToTag(Tag, 'Name', 'TFlipFlopModule');
        end;

        procedure TFlipFlopModule.Reset;
        begin
          (FOutputConnectorList[0] as TConnector).Status := (FOutputConnectorList[0] as TConnector).Negatived;
          (FOutputConnectorList[1] as TConnector).Status := not (FOutputConnectorList[1] as TConnector).Negatived;
          (FOutputConnectorList[0] as TConnector).Caption := '0';
          FInputMaster.J := false;
          FInputMaster.C := false;
          FInputMaster.K := false;
        end;

        procedure TFlipFlopModule.SwapOutput;
        begin
          (FOutputConnectorList[0] as TConnector).Status := not (FOutputConnectorList[0] as TConnector).Status;
          (FOutputConnectorList[1] as TConnector).Status := not (FOutputConnectorList[1] as TConnector).Status;
          if (FOutputConnectorList[0] as TConnector).FTempStatus then
            (FOutputConnectorList[0] as TConnector).Caption := '1'
          else (FOutputConnectorList[0] as TConnector).Caption := '0';
          FInputMaster.J := false;
          FInputMaster.C := false;
          FInputMaster.K := false;
        end;

        {TDecoderModule}

        constructor TDecoderModule.Create(APosition: TPoint; AOrientation: TOrientation; AOwner: TObject);
        var I: integer;
        begin
          inherited Create(APosition, AOrientation, AOwner);
          if AOrientation in [orLeft, orRight] then
          begin
            Width := 3;
            Height := 9;
          end else begin
            Width := 9;
            Height := 3;
          end;

          for I := 0 to 3 - 1 do
          begin
            case Orientation of
              orLeft: AddInputConnector(point(APosition.X, APosition.Y + 3 + I), Orientation, self);
              orRight: AddInputConnector(point(APosition.X + 3, APosition.Y + 3 + I), Orientation, self);
              orUp: AddInputConnector(point(APosition.X + 3 + I, APosition.Y), Orientation, self);
              orDown: AddInputConnector(point(APosition.X + 3 + I, APosition.Y + 3), Orientation, self);
            end;
            (FInputConnectorList[I] as TConnector).DenyNegativationChange := true;
          end;

          (FInputConnectorList[0] as TConnector).Caption := '2'+chr(176);
          (FInputConnectorList[1] as TConnector).Caption := '2'+chr(185);
          (FInputConnectorList[2] as TConnector).Caption := '2'+chr(178);

          for I := 0 to 8 - 1 do
          begin
            case Orientation of
              orLeft: AddOutputConnector(point(APosition.X + 3, APosition.Y + 1 + I), orRight, self);
              orRight: AddOutputConnector(point(APosition.X, APosition.Y + 1 + I), orLeft, self);
              orUp: AddOutputConnector(point(APosition.X + 1 + I, APosition.Y + 3), orDown, self);
              orDown: AddOutputConnector(point(APosition.X + 1 + I, APosition.Y), orUp, self);
            end;
            (FOutputConnectorList[I] as TConnector).Caption := IntToStr(I);
            (FOutputConnectorList[I] as TConnector).DenyNegativationChange := true;
          end;
        end;

        procedure TDecoderModule.Paint(ACanvas: TCanvas; AVStartPoint: TPoint);
        var I, XPosition, yPosition: integer;
          Edges: array of TPoint;
        begin
          XPosition := FPosition.x * gDistance - AVStartPoint.X;           // Box
					YPosition := FPosition.y * gDistance - AVStartPoint.Y;

          SetLength(Edges, 4);
          case Orientation of
            orLeft: begin
              Edges[0] := point(XPosition, YPosition + 2 * gDistance);
              Edges[1] := point(XPosition + Width * gDistance, YPosition);
              Edges[2] := point(XPosition + Width * gDistance, YPosition + (Height) * gDistance);
              Edges[3] := point(XPosition, YPosition + 7 * gDistance);
            end;
            orRight: begin
              Edges[0] := point(XPosition, YPosition);
              Edges[1] := point(XPosition + Width * gDistance, YPosition + 2 * gDistance);
              Edges[2] := point(XPosition + Width * gDistance, YPosition + 7 * gDistance);
              Edges[3] := point(XPosition, YPosition + (Height) * gDistance);
            end;
            orUp: begin
              Edges[0] := point(XPosition + 2 * gDistance, YPosition);
              Edges[1] := point(XPosition, YPosition + Height * gDistance);
              Edges[2] := point(XPosition + (Width) * gDistance, YPosition + Height * gDistance);
              Edges[3] := point(XPosition + 7 * gDistance, YPosition);
            end;
            orDown: begin
              Edges[0] := point(XPosition, YPosition);
              Edges[1] := point(XPosition + 2 * gDistance, YPosition + Height * gDistance);
              Edges[2] := point(XPosition + 7 * gDistance, YPosition + Height * gDistance);
              Edges[3] := point(XPosition + (Width) * gDistance, YPosition);
            end;
          end;

					with ACanvas do
					begin
            Pen.Color := clBlack;
            Brush.Color := clWhite;
            FillRect(rect(XPosition, YPosition,
                XPosition + Width * gDistance + 1, YPosition + Height * gDistance + 1));
            Brush.Color := rgb(255,255,128);
            Polygon(Edges);
					end;

					for I := 0 to FInputConnectorList.Count - 1 do  //Connector
						(FInputConnectorList[I] as TConnector).Paint(ACanvas, AVStartPoint);
					for I := 0 to FOutputConnectorList.Count - 1 do
						(FOutputConnectorList[I] as TConnector).Paint(ACanvas, AVStartPoint);

        end;

        	{TDecoderNoClock}

					procedure TDecoderNoClockModule.ChangedInputStatus(var Changed: Boolean);
					var ASum: integer;
						I: Integer;
					begin
						ASum := 0;
						for I := 0 to FInputConnectorList.Count - 1 do
							if (FInputConnectorList[I] as TConnector).Status then
								Inc(ASum, Round(Power(2, I)));
						for I := 0 to FOutputConnectorList.Count - 1 do
							(FOutputConnectorList[I] as TConnector).Status := false;
						(FOutputConnectorList[ASum] as TConnector).Status := true;
					end;

					procedure TDecoderNoClockModule.SaveToBinaryTag(Tag: TRakBinaryTag);
					begin
						inherited SaveToBinaryTag(Tag);
						TRakBinaryShortString.AddToTag(Tag, 'Name', 'TDecoderNoClockModule');
					end;

          {TDecoderClockModule}

          constructor TDecoderClockModule.Create(APosition: TPoint; AOrientation: TOrientation; AOwner: TObject);
          begin
            inherited Create(APosition, AOrientation, AOwner);
            case Orientation of
              orLeft: AddInputConnector(point(APosition.X, APosition.Y + 3 + 3), Orientation, self);
              orRight: AddInputConnector(point(APosition.X + 3, APosition.Y + 3 + 3), Orientation, self);
              orUp: AddInputConnector(point(APosition.X + 3 + 3, APosition.Y), Orientation, self);
              orDown: AddInputConnector(point(APosition.X + 3 + 3, APosition.Y + 3), Orientation, self);
            end;

            (FInputConnectorList[FInputConnectorList.Count - 1] as TConnector).Caption := 'C';
            (FInputConnectorList[FInputConnectorList.Count - 1] as TConnector).DenyNegativationChange := true;

            Reset;
          end;

          procedure TDecoderClockModule.Paint(ACanvas: TCanvas; AVStartPoint: TPoint);
          begin
            inherited Paint(ACanvas, AVStartPoint);

          end;

          procedure TDecoderClockModule.ChangedInputStatus(var Changed: Boolean);
          var I: integer;
          begin
            if ((FInputConnectorList[FInputConnectorList.Count - 1] as TConnector).Status
              xor (FInputConnectorList[FInputConnectorList.Count - 1] as TConnector).Negatived)
							and not (FInputMaster. Clock) then          // rising edge
            begin
              FInputMaster.OutputSum := 0;
              for I := 0 to FInputConnectorList.Count - 2 do
                if (FInputConnectorList[I] as TConnector).Status then
                  Inc(FInputMaster.OutputSum, Round(Power(2, I)));
             end else
						if not ((FInputConnectorList[FInputConnectorList.Count - 1] as TConnector).Status
              xor (FInputConnectorList[FInputConnectorList.Count - 1] as TConnector).Negatived)
							and FInputMaster.Clock then                 // trailing edge
						begin
              for I := 0 to FOutputConnectorList.Count - 1 do
                (FOutputConnectorList[I] as TConnector).Status := false;
              (FOutputConnectorList[FInputMaster.OutputSum] as TConnector).Status := true;
						end;
            FInputMaster.Clock := (FInputConnectorList[FInputConnectorList.Count - 1] as TConnector).Status;
          end;

          procedure TDecoderClockModule.SaveToBinaryTag(Tag: TRakBinaryTag);
          begin
            inherited SaveToBinaryTag(Tag);
            TRakBinaryShortString.AddToTag(Tag, 'Name', 'TDecoderClockModule');
          end;

          procedure TDecoderClockModule.Reset;
          var I: integer;
          begin
            FInputMaster.Clock := false;
            FInputMaster.OutputSum := 0;

            for I := 0 to FOutputConnectorList.Count - 1 do
                (FOutputConnectorList[I] as TConnector).Status := false;
            (FOutputConnectorList[0] as TConnector).Status := true;
          end;

        {TShiftingRegisterModule}

        constructor TShiftingRegisterModule.Create(APosition: TPoint; AOrientation: TOrientation; AOwner: TObject);
        var I: integer;
        begin
          inherited Create(APosition, AOrientation, AOwner);
          if AOrientation in [orLeft, orRight] then
          begin
            Width := 8;
            Height := 2;
          end else begin
            Width := 2;
            Height := 8;
          end;

          for I := 0 to 2 do
            case Orientation of
              orLeft: begin
                AddInputConnector(point(APosition.X, APosition.Y + I), AOrientation, self); // 0, 1, 2
                if I < 2 then
                  AddOutputConnector(point(APosition.X + 8, APosition.Y + I * 2), orRight, self);
              end;
              orRight: begin
                AddInputConnector(point(APosition.X + 8, APosition.Y + I), AOrientation, self); // 0, 1, 2
                if I < 2 then
                  AddOutputConnector(point(APosition.X, APosition.Y + I * 2), orLeft, self);
              end;
              orUp: begin
                AddInputConnector(point(APosition.X + I, APosition.Y), AOrientation, self); // 0, 1, 2
                if I < 2 then
                  AddOutputConnector(point(APosition.X + I * 2, APosition.Y + 8), orDown, self);
              end;
              orDown: begin
                AddInputConnector(point(APosition.X + I, APosition.Y + 8), AOrientation, self); // 0, 1, 2
                if I < 2 then
                  AddOutputConnector(point(APosition.X + I * 2, APosition.Y), orUp, self);
              end;
            end;  // of case

          (FInputConnectorList[0] as TConnector).DenyNegativationChange := true;
          (FInputConnectorList[2] as TConnector).DenyNegativationChange := true;
          (FOutputConnectorList[0] as TConnector).DenyNegativationChange := true;
          (FOutputConnectorList[1] as TConnector).DenyNegativationChange := true;

          for I := 0 to 4 - 1 do
          begin
            case Orientation of
              orLeft: begin
                AddInputConnector(point(APosition.X + I * 2 + 1, APosition.Y), orUp, self);      // 3, 5, 7, 9
                AddInputConnector(point(APosition.X + I * 2 + 1, APosition.Y + 2), orDown, self);// 4, 6, 8, 10
              end;
              orRight: begin
                AddInputConnector(point(APosition.X + 8 - I * 2 - 1, APosition.Y), orUp, self);      // 3, 5, 7, 9
                AddInputConnector(point(APosition.X + 8 - I * 2 - 1, APosition.Y + 2), orDown, self);// 4, 6, 8, 10
              end;
              orUp: begin
                AddInputConnector(point(APosition.X, APosition.Y + I * 2 + 1), orUp, self);      // 3, 5, 7, 9
                AddInputConnector(point(APosition.X + 2, APosition.Y + I * 2 + 1), orDown, self);// 4, 6, 8, 10
              end;
              orDown: begin
                AddInputConnector(point(APosition.X, APosition.Y + 8 - I * 2 - 1), orUp, self);      // 3, 5, 7, 9
                AddInputConnector(point(APosition.X + 2, APosition.Y + 8 - I * 2 - 1), orDown, self);// 4, 6, 8, 10
              end;
            end;

            (FInputConnectorList[(I + 1) * 2 + 1] as TConnector).DenyConnection := true;
            (FInputConnectorList[(I + 1) * 2 + 2] as TConnector).DenyConnection := true;
            (FInputConnectorList[(I + 1) * 2 + 1] as TConnector).DenyNegativationChange := true;
            (FInputConnectorList[(I + 1) * 2 + 2] as TConnector).DenyNegativationChange := true;
          end;
          
          Reset;
        end;

        procedure TShiftingRegisterModule.Paint(ACanvas: TCanvas; AVStartPoint: TPoint);
        var Edges: array of TPoint;
          I, II: Integer;
          AChr: Char;
        begin
          inherited Paint(ACanvas, AVStartPoint);
					with ACanvas do begin
						Pen.Color := clBlack;
						Brush.Color := rgb(255,255,128);
			
						SetLength(Edges, 3);           {
						Edges[0] := point(Position.x * gDistance - AVStartPoint.X,
              (Position.y + 1) * gDistance - gDistance div 4 - AVStartPoint.Y);
						Edges[1] := point(Position.x * gDistance + gDistance div 2 - AVStartPoint.X,
              (Position.y + 1) * gDistance - AVStartPoint.Y);
						Edges[2] := point(Position.x * gDistance - AVStartPoint.X,
              (Position.y + 1) * gDistance + gDistance div 4 - AVStartPoint.Y);
             }

            case Orientation of
              orLeft: begin
                Edges[0] := point(Position.x * gDistance - AVStartPoint.X,
                  (Position.y + 1) * gDistance - gDistance div 4 - AVStartPoint.Y);
						    Edges[1] := point(Position.x * gDistance + gDistance div 2 - AVStartPoint.X,
                  (Position.y + 1) * gDistance - AVStartPoint.Y);
						    Edges[2] := point(Position.x * gDistance - AVStartPoint.X,
                  (Position.y + 1) * gDistance + gDistance div 4 - AVStartPoint.Y);
              end;
              orRight: begin
                Edges[0] := point((Position.x + 8) * gDistance - AVStartPoint.X,
                  (Position.y + 1) * gDistance - gDistance div 4 - AVStartPoint.Y);
						    Edges[1] := point((Position.x + 8) * gDistance - gDistance div 2 - AVStartPoint.X,
                  (Position.y + 1) * gDistance - AVStartPoint.Y);
						    Edges[2] := point((Position.x + 8) * gDistance - AVStartPoint.X,
                  (Position.y + 1) * gDistance + gDistance div 4 - AVStartPoint.Y);
              end;
              orUp: begin
                Edges[0] := point((Position.x + 1) * gDistance - gDistance div 4 - AVStartPoint.X,
                  Position.y * gDistance - AVStartPoint.Y);
						    Edges[1] := point((Position.x + 1) * gDistance - AVStartPoint.X,
                  Position.y * gDistance + gDistance div 2 - AVStartPoint.Y);
						    Edges[2] := point((Position.x + 1) * gDistance + gDistance div 4 - AVStartPoint.X,
                  Position.y * gDistance - AVStartPoint.Y);
              end;
              orDown: begin
                Edges[0] := point((Position.x + 1) * gDistance - gDistance div 4 - AVStartPoint.X,
                  (Position.y + 8) * gDistance - AVStartPoint.Y);
						    Edges[1] := point((Position.x + 1) * gDistance - AVStartPoint.X,
                  (Position.y + 8) * gDistance - gDistance div 2 - AVStartPoint.Y);
						    Edges[2] := point((Position.x + 1) * gDistance + gDistance div 4 - AVStartPoint.X,
                  (Position.y + 8) * gDistance - AVStartPoint.Y);
              end;
            end;  // of case
						Polygon(Edges);

            case Orientation of
              orLeft: begin
                MoveTo(Position.x * gDistance  + gDistance div 2 - AVStartPoint.X,
                  (Position.y + 1) * gDistance - AVStartPoint.Y);
                LineTo((Position.X + 8) * gDistance - AVStartPoint.X,
                  (Position.Y + 1) * gDistance - AVStartPoint.Y);
              end;
              orRight: begin
                MoveTo(Position.x * gDistance  - AVStartPoint.X,
                  (Position.y + 1) * gDistance - AVStartPoint.Y);
                LineTo((Position.X + 8) * gDistance - gDistance div 2 - AVStartPoint.X,
                  (Position.Y + 1) * gDistance - AVStartPoint.Y);
              end;
              orUp: begin
                MoveTo((Position.x + 1) * gDistance - AVStartPoint.X,
                  Position.y * gDistance + gDistance div 2 - AVStartPoint.Y);
                LineTo((Position.X + 1) * gDistance - AVStartPoint.X,
                  (Position.Y + 8) * gDistance - AVStartPoint.Y);
              end;
              orDown: begin
                MoveTo((Position.x + 1) * gDistance - AVStartPoint.X,
                  Position.y * gDistance - AVStartPoint.Y);
                LineTo((Position.X + 1) * gDistance - AVStartPoint.X,
                  (Position.Y + 8) * gDistance - gDistance div 2 - AVStartPoint.Y);
              end;
            end;  // of case

            case Orientation of
              orLeft, orRight: begin
                for I := 0 to 4 - 1 do
                begin
                  MoveTo((Position.X + (I + 1) * 2) * gDistance - AVStartPoint.X,
                    Position.Y * gDistance - gDistance div 2 + 1 - AVStartPoint.Y);
                  LineTo((Position.X + (I + 1) * 2) * gDistance - AVStartPoint.X,
                    (Position.Y + 2) * gDistance + gDistance div 2 - AVStartPoint.Y);
                end;
              end;
              orUp, orDown: begin
                for I := 0 to 4 - 1 do
                begin
                  MoveTo(Position.X * gDistance - gDistance div 2 + 1 - AVStartPoint.X,
                    (Position.Y + (I + 1) * 2) * gDistance - AVStartPoint.Y);
                  LineTo((Position.X + 2) * gDistance + gDistance div 2 - AVStartPoint.X,
                    (Position.Y + (I + 1) * 2) * gDistance - AVStartPoint.Y);
                end;
              end;
            end;   // of case

            Font.Size := gDistance div 2 + 1;      // ConnectorCaption
            if Font.Size < 7 then
              Font.Size := 7;
            Brush.Color := rgb(255,255,128);
            for I := 0 to 1 do
              for II := 0 to 4 - 1 do
              begin
                if (FInputConnectorList[(II + 1) * 2 + 1 + I] as TConnector).Status then
                  AChr := '1'
                else AChr := '0';
                case Orientation of
                  orLeft: TextOut((Position.X + 2 * II + 1)* gDistance - TextWidth(AChr) div 2 - AVStartPoint.X,
                    (Position.Y + I * 2) * gDistance - TextHeight(AChr) div 2 - AVStartPoint.Y + 1 - I * 3, AChr);
                  orRight: TextOut((Position.X + 8 - 2 * II - 1)* gDistance - TextWidth(AChr) div 2 - AVStartPoint.X,
                    (Position.Y + I * 2) * gDistance - TextHeight(AChr) div 2 - AVStartPoint.Y + 1 - I * 3, AChr);
                  orUp: TextOut((Position.X + I * 2) * gDistance - TextWidth(AChr) div 2 - AVStartPoint.X + 2 - I * 4,
                    (Position.Y + 2 * II + 1)* gDistance - TextHeight(AChr) div 2 - AVStartPoint.Y, AChr);
                  orDown: TextOut((Position.X + I * 2) * gDistance - TextWidth(AChr) div 2 - AVStartPoint.X + 2 - I * 4,
                    (Position.Y + 8 - 2 * II - 1)* gDistance - TextHeight(AChr) div 2 - AVStartPoint.Y, AChr);
                end;
              end;
            Font.Size := 8;
          end;
        end;

        procedure TShiftingRegisterModule.ChangedInputStatus(var Changed: Boolean);
        var I: Integer;
        begin
          if ((FInputConnectorList[1] as TConnector).Status
              xor (FInputConnectorList[1] as TConnector).Negatived)
							and not (FInputMaster. Clock) then          // rising edge
          begin
            FInputMaster.Inp0 := (FInputConnectorList[0] as TConnector).Status;
            FInputMaster.Inp2 := (FInputConnectorList[2] as TConnector).Status;
          end else
          if not ((FInputConnectorList[1] as TConnector).Status
            xor (FInputConnectorList[1] as TConnector).Negatived)
            and FInputMaster.Clock then                 // trailing edge
          begin
            for I := 2 downto 0 do
            begin                 // 9, 7, 5                                                     // 7, 5, 3
              (FInputConnectorList[(I + 1) * 2 + 3] as TConnector).Status := (FInputConnectorList[(I + 1) * 2 + 1] as TConnector).Status;
              (FInputConnectorList[(I + 1) * 2 + 4] as TConnector).Status := (FInputConnectorList[(I + 1) * 2 + 2] as TConnector).Status;
            end;                  // 10, 8, 6                                                    // 8, 6, 4
            (FInputConnectorList[3] as TConnector).Status := FInputMaster.Inp0;
            (FInputConnectorList[4] as TConnector).Status := FInputMaster.Inp2;
          end;

          if (FInputConnectorList[9] as TConnector).Status xor (FOutputConnectorList[0] as TConnector).Status then
            begin
              (FOutputConnectorList[0] as TConnector).Status := (FInputConnectorList[9] as TConnector).Status;
              Changed := true;
            end;
          if (FInputConnectorList[10] as TConnector).Status xor (FOutputConnectorList[1] as TConnector).Status then
          begin
            (FOutputConnectorList[1] as TConnector).Status := (FInputConnectorList[10] as TConnector).Status;
            Changed := true;
          end;
          FInputMaster.Clock := (FInputConnectorList[1] as TConnector).Status xor (FInputConnectorList[1] as TConnector).Negatived;
        end;

        procedure TShiftingRegisterModule.SaveToBinaryTag(Tag: TRakBinaryTag);
        begin
          inherited SaveToBinaryTag(Tag);
          TRakBinaryShortString.AddToTag(Tag, 'Name', 'TShiftingRegisterModule');
        end;

        procedure TShiftingRegisterModule.Reset;
        begin
          FInputMaster.Inp0 := false;
          FInputMaster.Inp2 := false;
          FInputMaster.Clock := false;
        end;

        {TClockGeneratorModule}

        procedure TClockGeneratorModule.tiDelayTimer(Sender: TObject);
        begin
          if not (FInputConnectorList[0] as TConnector).Status then
          begin
            tiDelay.Enabled := false;
            if ClockCount <> 0 then
            begin
              (FOutputConnectorList[0] as TConnector).Status := not (FOutputConnectorList[0] as TConnector).Status;
              if ClockCount > 0 then
                ClockCount := ClockCount - 1;
              if Owner is TElementBox then
                (Owner as TElementBox).SetItemsStatus((FOutputConnectorList[0] as TConnector).Position, (FOutputConnectorList[0] as TConnector).FStatus);
            end else FormOptionsClockGenerator.Done;
          end else Reset;
        end;

        constructor TClockGeneratorModule.Create(APosition: TPoint; AOrientation: TOrientation; AOwner: TObject);
        begin
          inherited Create(APosition, AOrientation, AOwner);
          if AOrientation in [orLeft, orRight] then
          begin
            Height := 2;
            Width := 3;
          end else begin
            Height := 3;
            Width := 2;
          end;
          Caption := 'TG';

          case AOrientation of
            orLeft: begin
              AddOutputConnector(point(APosition.X + 3, APosition.Y + 1), orRight, self);
              AddInputConnector(point(APosition.X + 2, APosition.Y + 2), orDown, self);
            end;
            orRight: begin
              AddOutputConnector(point(APosition.X, APosition.Y + 1), orLeft, self);
              AddInputConnector(point(APosition.X + 2, APosition.Y + 2), orDown, self);
            end;
            orUp: begin
              AddOutputConnector(point(APosition.X + 1, APosition.Y + 3), orDown, self);
              AddInputConnector(point(APosition.X + 2, APosition.Y + 2), orRight, self);
            end;
            orDown: begin
              AddOutputConnector(point(APosition.X + 1, APosition.Y), orUp, self);
              AddInputConnector(point(APosition.X + 2, APosition.Y + 2), orRight, self);
            end;
          end;

          (FOutputConnectorList[0] as TConnector).DenyNegativationChange := true;
          (FInputConnectorList[0] as TConnector).DenyNegativationChange := true;

          ClockCount := 0;
          if Owner is TElementBox then
          begin
            tiDelay := TTimer.Create(Nil);
            tiDelay.Enabled := false;
            tiDelay.Interval := 0;
            tiDelay.OnTimer := tiDelayTimer;
            tiDelay.Enabled := true;
          end;
        end;

        destructor TClockGeneratorModule.Destroy;
        begin
          inherited Destroy;
          tiDelay.Free;
        end;

        procedure TClockGeneratorModule.Paint(ACanvas: TCanvas; AVStartPoint: TPoint);
        var XPosition, yPosition: integer;
        begin
          inherited Paint(ACanvas, AVStartPoint);
          XPosition := FPosition.x * gDistance - AVStartPoint.X;
          YPosition := FPosition.y * gDistance - gDistance div 2 - AVStartPoint.Y;
          with ACanvas do                           // Caption
          begin
            Pen.Color := clBlack;
            Brush.Color := rgb(255,255,128);

            Font.Size := gDistance div 2 + 2;
            TextOut(XPosition + (Width * gDistance - TextWidth(Caption)) div 2,
              YPosition + ((Height + 1) * gDistance - TextHeight(Caption)) div 2, Caption);

            Font.Size := gDistance div 2 - 2 - gDistance div 20;
            if not (Font.Size >= 6) then
              Font.Size := 6;
            if Orientation in [orLeft, orRight] then
              TextOut((Position.x + 1) * gDistance - AVStartPoint.X,
                (Position.y + 1) * gDistance + gDistance div 2 - AVStartPoint.Y, 'Stp')
            else TextOut((Position.x + 1) * gDistance - AVStartPoint.X,
                (Position.y) * gDistance + 2 * gDistance - AVStartPoint.Y, 'Stp');

            Font.Size := 8;
          end;
        end;

        procedure TClockGeneratorModule.ChangedInputStatus(var Changed: Boolean);
        begin
          if (FInputConnectorList[0] as TConnector).Status then
          begin
            {tiDelay.Interval := 0;
            ClockCount := 0;
            (FOutputConnectorList[0] as TConnector).Status := false;}
            Reset;
            Changed := true;
          end;
        end;

        procedure TClockGeneratorModule.SaveToBinaryTag(Tag: TRakBinaryTag);
        begin
          inherited SaveToBinaryTag(Tag);
          TRakBinaryShortString.AddToTag(Tag, 'Name', 'TClockGeneratorModule');
        end;

        procedure TClockGeneratorModule.LoadFromBinaryTag(Tag: TRakBinaryTag);
        begin
          inherited LoadFromBinaryTag(Tag);
          if Owner is TElementBox then
            (Owner as TelementBox).FClockGenerator := self;
        end;

        procedure TClockGeneratorModule.Reset;
        begin
          tiDelay.Interval := 0;
          ClockCount := 0;
          //(FOutputConnectorList[0] as TConnector).FStatus := false;
          (FOutputConnectorList[0] as TConnector).Status := false;
          (FOutputConnectorList[0] as TConnector).RefreshStatus;
        end;

        {TIc}

        procedure TIc.SaveToBinaryTag(Tag: TRakBinaryTag);
        begin
          inherited SaveToBinaryTag(Tag);
          TRakBinaryShortString.AddToTag(Tag, 'IcName', FName);
          TRakBinaryShortString.AddToTag(Tag, 'Caption', Caption);
          TRakBinaryBoolean.AddToTag(Tag, 'CaptionHorizontal', CaptionHorizontal);
        end;

        procedure TIc.LoadFromBinaryTag(Tag: TRakBinaryTag);
        begin
          inherited LoadFromBinaryTag(Tag);
          FName := Tag.AttributeAsString('IcName', '');
          Caption := Tag.AttributeAsString('Caption', 'failed');
          CaptionHorizontal := Tag.AttributeAsBoolean('CaptionHorizontal', true);
        end;

        	{TIcModule}
	
					constructor TIcModule.Create(APosition: TPoint; AOwner: TObject);
					begin
						inherited Create(APosition, orLeft, AOwner);
            FIcSamplePointer := Nil;
            FIndex := 0;
            CaptionHorizontal := true;
					end;

          destructor TIcModule.Destroy;
          begin
            if FIcSamplePointer <> Nil then
              (FIcSamplePointer as TIcSample).RemoveModule(FIndex);
            inherited Destroy;
          end;

          procedure TIcModule.SaveToBinaryTag(Tag: TRakBinaryTag);
          begin
            inherited SaveToBinaryTag(Tag);
            TRakBinaryLongInt.AddToTag(Tag, 'Index', FIndex);
          end;

          procedure TIcModule.LoadFromBinaryTag(Tag: TRakBinaryTag);
          begin
            inherited LoadFromBinaryTag(Tag);
            FIndex := Tag.AttributeAsInteger('Index', 0);
          end;
	
					procedure TIcModule.Paint(ACanvas: TCanvas; AVStartPoint: TPoint);
					var I, GesHeight: integer;
							ACaption: string;
							Y: integer;
					begin
						inherited Paint(ACanvas, AVStartPoint);
						if not (Caption = '') then                 // Caption   Index
							with ACanvas do
							begin
								Font.Size := gDistance div 2 - 1;
								if Font.Size < 7 then
									Font.Size := 7;
								Brush.Color := rgb(255,255,128);
								if CaptionHorizontal then
								begin
									TextOut(Position.X * gDistance + (Width * gDistance - TextWidth(Caption)) div 2 - AVStartPoint.X,
										Position.Y * gDistance + (Height * gDistance - 5 * TextHeight(Caption) div 2) div 3 - AVStartPoint.Y, Caption);
                  if FIndex > 0 then
									  TextOut(Position.X * gDistance + (Width * gDistance - TextWidth(IntToStr(FIndex + 1))) div 2 - AVStartPoint.X,
										  Position.Y * gDistance + ((Height * gDistance - 5 * TextHeight(IntToStr(FIndex + 1)) div 2)) - AVStartPoint.Y, IntToStr(FIndex + 1));
								end else begin
									GesHeight := 0;
									if FIndex > 0 then
										ACaption := Caption + ' ' + IntToStr(FIndex + 1)
									else ACaption := Caption;
									for I := 1 to Length(ACaption) do
										Inc(GesHeight, TextHeight(ACaption[I]));
									Y := Position.Y * gDistance + (Height * gDistance - GesHeight) div 2 - AVStartPoint.Y;
									for I := 1 to Length(ACaption) do
									begin
										TextOut(Position.X * gDistance + (Width * gDistance - TextWidth(ACaption[I])) div 2 - AVStartPoint.X, Y, ACaption[I]);
										inc(Y, TextHeight(ACaption[I]));
									end;
								end;
								Font.Size := 8;
							end;
					end;
	
					procedure TIcModule.ChangedInputStatus(var Changed: Boolean);
					begin
						//
					end;

          {TIcSample}

          function TIcSample.FindIcTabByName(AName: string):TObject;
          var I: integer;
              Found: boolean;
          begin
            Result := Nil;
            I := 0;
            Found := false;
            while not Found and (I < (FOwner as TSimulation).FIcTabList.Count) do
            begin
              if ((FOwner as TSimulation).FIcTabList[I] as TIcTab).Name = AName then
              begin
                Result := (FOwner as TSimulation).FIcTabList[I];
                Found := true;
              end;
              inc(I);
            end;
          end;

          constructor TIcSample.Create(AHeight, AWidth: integer; ACaption: string; ACaptionHorizontal: boolean; AOwner: TObject);
          var AIcLink: TIcLink;
          begin
            {inherited Create(point(0,0), AOwner);
            FIcLinkList := TElementList.Create;

            FOwner := AOwner;
            FMainBox := (AOwner as TSimulation).GetMainBox;

            FPosition := point(0,0); // not used
            FHeight := AHeight;
            FWidth := AWidth;
            FCaption := ACaption;
            FName := 'IC'+IntToStr((AOwner as TSimulation).FIcSampleList.count);
            FCaptionHorizontal := ACaptionHorizontal;               }

            Create(AHeight, AWidth, ACaption, ACaptionHorizontal, AOwner, true);

            AIcLink := TIcLink.Create(self);
            FIcLinkList.Add(AIcLink);
            AIcLink.FIcModule := Nil;
            (Owner as TSimulation).AddIc(FName+'_0', ACaption, AIcLink);
            AIcLink.FIcTab := FindIcTabByName(FName+'_0');
          end;

          constructor TIcSample.Create(AHeight: Integer; AWidth: Integer; ACaption: string; ACaptionHorizontal: Boolean; AOwner: TObject; Load: Boolean);
          begin
            inherited Create(point(0,0), orLeft, AOwner);
            FIcLinkList := TElementList.Create;

            FOwner := AOwner;
            FMainBox := (AOwner as TSimulation).GetMainBox;

            FPosition := point(0,0); // not used
            FHeight := AHeight;
            FWidth := AWidth;
            FCaption := ACaption;
            FName := 'IC'+IntToStr((AOwner as TSimulation).FIcSampleList.count);
            FCaptionHorizontal := ACaptionHorizontal;
          end;

          destructor TIcSample.Destroy;
          begin
            FIcLinkList.Free;
            inherited Destroy;
          end;

          procedure TIcSample.SaveToBinaryTag(Tag: TRakBinaryTag);
          begin
            inherited SaveToBinaryTag(Tag);
            TRakBinaryShortString.AddToTag(Tag, 'Name', 'TIcSample');
            FIcLinkList.SaveToBinaryTag(Tag.AddTag('IcLinkList'));
          end;

          procedure TIcSample.LoadFromBinaryTag(Tag: TRakBinaryTag);
          var TTag: TRakBinaryTag;
          begin
            inherited LoadFromBinaryTag(Tag);
            TTag := Tag.FindFirstTag('IcLinkList');
            if TTag <> nil then
              FIcLinkList.LoadFromBinaryTag(TTag, self);
          end;

          procedure TIcSample.ChangedInputStatus(var Changed: Boolean);
          begin
            // not supposed to be called!
          end;

          function TIcSample.CreateModule(APosition: TPoint; AElementBox: TObject): TIcModule;
          var AIcModule: TIcModule;
              AConnector: TConnector;
              I: Integer;
          begin
            AIcModule := TIcModule.Create(APosition, AElementBox);
            AIcModule.Height := FHeight;
            AIcModule.Width := Width;
            AIcModule.Caption := Caption;
            AIcModule.FName := FName;
            AIcModule.FIndex := FIcLinkList.Count;
            AIcModule.FCaptionHorizontal := FCaptionHorizontal;
            for I := 0 to FInputConnectorList.Count - 1 do
            begin
              AConnector := TConnector.Create(point(APosition.X + (FInputConnectorList[I] as TConnector).FPosition.X, APosition.Y + (FInputConnectorList[I] as TConnector).FPosition.Y), AIcModule);
              AConnector.IsInput := true;
              AConnector.Orientation := (FInputConnectorList[I] as TConnector).Orientation;
              AConnector.FCaption := (FInputConnectorList[I] as TConnector).FCaption;
              AIcModule.FInputConnectorList.Add(AConnector);
            end;
            for I := 0 to FOutputConnectorList.Count - 1 do
            begin
              AConnector := TConnector.Create(point(APosition.X + (FOutputConnectorList[I] as TConnector).FPosition.X, APosition.Y + (FOutputConnectorList[I] as TConnector).FPosition.Y), AIcModule);
              AConnector.IsInput := false;
              AConnector.Orientation := (FOutputConnectorList[I] as TConnector).Orientation;
              AConnector.FCaption := (FOutputConnectorList[I] as TConnector).FCaption;
              AIcModule.FOutputConnectorList.Add(AConnector);
            end;

            Result := AIcModule
          end;

          procedure TIcSample.AddInputConnector(ARelHeight: integer; ACaption: string);
          var AConnector: TConnector;
          begin
            AConnector := TConnector.Create(point(0, ARelHeight), self);
            AConnector.IsInput := true;
            AConnector.Orientation := orLeft;
            AConnector.Caption := ACaption;
            FInputConnectorList.Add(AConnector);
          end;

          procedure TIcSample.AddOutputConnector(ARelHeight: integer; ACaption: string);
          var AConnector: TConnector;
          begin
            AConnector := TConnector.Create(point(FWidth, ARelHeight), self);
            AConnector.IsInput := false;
            AConnector.Orientation := orRight;
            AConnector.Caption := ACaption;
            FOutputConnectorList.Add(AConnector);
          end;

          procedure TIcSample.AddModule(AModule: TIcModule);
          var AIcLink: TIcLink;
          begin
            AModule.FIcSamplePointer := self;
            if (FIcLinkList[0] as TIcLink).FIcModule = Nil then
            begin
              AModule.FIndex := 0;
              (FIcLinkList[0] as TIcLink).FIcModule := AModule;
            end else begin
              AIcLink := TIcLink.Create(self);
              AIcLink.FIcModule := AModule;
              FIcLinkList.Add(AIcLink);
              (Owner as TSimulation).AddIC(FName+'_'+IntToStr(FIcLinkList.Count-1), FCaption + ' ' + IntToStr(FIcLinkList.Count), AIcLink);
              AIcLink.FIcTab := FindIcTabByName(FName+'_'+IntToStr((AIcLink.FIcModule as TIcModule).FIndex));
            end;
          end;

          procedure TIcSample.RemoveModule(AIndex: Integer);
          var I: integer;
          begin
            if FIcLinkList.Count > 1 then
            begin
              (Owner as TSimulation).FIcTabList.Remove((FIcLinkList.Items[AIndex] as TIcLink).FIcTab);
              (Owner as TSimulation).FPageControl.ActivePageIndex := 0;
              (FIcLinkList.Items[AIndex] as TIcLink).FIcModule := Nil;
              FIcLinkList.Delete(AIndex);
              for I := 0 to FIcLinkList.Count - 1 do
                with FIcLinkList[I] as TIcLink do
                begin
                  (FIcModule as TIcModule).FIndex := I;
                  (FIcTab as TIcTab).Name := FName+'_'+IntToStr(I);
                  (FIcTab as TIcTab).FName := FName+'_'+IntToStr(I);
                  if I > 0 then
                    (FIcTab as TIcTab).Caption := FCaption + ' ' + IntToStr(I + 1)
                  else (FIcTab as TIcTab).Caption := FCaption;
                  (FIcTab as TIcTab).FCaption := (FIcTab as TIcTab).Caption;
                end;
            end else
              with FIcLinkList[0] as TIcLink do
              begin
                FIcModule := Nil;
                (FIcTab as TIcTab).Name := FName+'_'+IntToStr(0);
                (FIcTab as TIcTab).FName := FName+'_'+IntToStr(0);
                (FIcTab as TIcTab).Caption := FCaption;
                (FIcTab as TIcTab).FCaption := FCaption;
              end;
          end;

          procedure TIcSample.MatchIcData(AIndex: Integer);
          var StreamDataA, StreamDataB: TRakBinaryStreamData;
              StreamA, StreamB: TMemoryStream;
              I: integer;
              TPointer: TObject;
              ASimulation: TSimulation;
          begin
            StreamDataA := TRakBinaryStreamData.Create;
            StreamDataB := TRakBinaryStreamData.Create;
            StreamA := TMemoryStream.Create;
            StreamB := TMemoryStream.Create;
            try
              ASimulation := ((FIcLinkList[AIndex] as TIcLink).FIcTab as TIcTab).FSimulation as TSimulation;
              TPointer := ASimulation.FIcLinkPointer;
              ASimulation.FIcLinkPointer := Nil;    // set Pointer to Nil = the only difference between the Tabs
              ASimulation.SaveToBinaryTag(StreamDataA);
              ASimulation.FIcLinkPointer := TPointer;

              StreamDataA.SaveToStream(StreamA);

              for I := 0 to FIcLinkList.Count - 1 do
                if not (I = AIndex) then
                begin
                  StreamDataB.Clear;
                  StreamB.Clear;
                  
                  ASimulation := ((FIcLinkList[I] as TIcLink).FIcTab as TIcTab).FSimulation as TSimulation;
                  TPointer := ASimulation.FIcLinkPointer;
                  ASimulation.FIcLinkPointer := Nil;
                  ASimulation.SaveToBinaryTag(StreamDataB);
                  ASimulation.FIcLinkPointer := TPointer;

                  StreamDataB.SaveToStream(StreamB);

                  if not CompareStreams(StreamA, StreamB) then             // compare
                  begin
                    ASimulation.Free;                                 // not equal --> copy
                    ASimulation := TSimulation.Create(((FIcLinkList[I] as TIcLink).FIcTab as TIcTab).FPageControl);
                    ASimulation.LoadFromBinaryTag(StreamDataA);
                    ASimulation.FIcLinkPointer := FIcLinkList[I];
                  end;
                end;
            finally
              StreamDataA.Free;
              StreamDataB.Free;
              StreamA.Free;
              StreamB.Free;
            end;
          end;

      {TStatusModule}

      constructor TStatusModule.Create(APosition: TPoint; AOwner: TObject);
      begin
        inherited Create(APosition, AOwner);
        Position := APosition;
        Status := false;
        Height := 0;
        Width := 0;
        if Owner is TElementBox then
        begin
          (Owner as TElementBox).DivideLines(Position);
          (Owner as TElementBox).DivideLines(Position);
          (Owner as TElementBox).RefreshLineConnectorList;
        end;
      end;

      destructor TStatusModule.Destroy;
      begin
        if Owner is TElementBox then
        begin
          (Owner as TElementBox).CrossLinkLines(Position);
          (Owner as TElementBox).LinkLines(Position);
          (Owner as TElementBox).RefreshLineConnectorList;
        end;
        inherited Destroy;
      end;

        {TSwitchModule}

        procedure TSwitchModule.SaveToBinaryTag(Tag: TRakBinaryTag);
        begin
          inherited SaveToBinaryTag(Tag);
          TRakBinaryShortString.AddToTag(Tag, 'Name', 'TSwitchModule');
        end;

        procedure TSwitchModule.Paint(ACanvas: TCanvas; AVStartPoint: TPoint);
        var text: string;
            XPosition, YPosition, Tmp: integer;
        begin
          if Status then Text := '1' else Text := '0';
          if gDistance mod 2 = 0 then
            Tmp := gDistance div 2 - 1
          else
            Tmp := gDistance div 2;
          XPosition := Position.x  * gDistance - Tmp - AVStartPoint.X;
          YPosition := Position.y * gDistance - Tmp - AVStartPoint.Y;
          with ACanvas do
          begin
            Brush.Color := rgb(255,255,128);
            FillRect(rect(XPosition,YPosition,XPosition + gDistance - 1,YPosition + gDistance - 1));
            Font.Size := gDistance div 2;
            if gDistance div 2 < 8 then
              Font.Size := 8;
            if gDistance < 13 then
              Font.Size := 6;
            TextOut(((2 * XPosition + gDistance)  - TextWidth(Text)) div 2,
              (2 * YPosition + gDistance - 1 - TextHeight(Text)) div 2, Text);
            Font.Size := 8;
            Brush.Color := clblack;
            FrameRect(rect(XPosition,YPosition,XPosition + gDistance - 1,YPosition + gDistance - 1));
          end;
        end;

        {TLedModule}

        constructor TLedModule.Create(APosition: TPoint; AColor: TColor; AOwner: TObject);
        begin
          inherited Create(APosition, AOwner);
          Color := AColor;
        end;

        procedure TLedModule.SaveToBinaryTag(Tag: TRakBinaryTag);
        begin
          inherited SaveToBinaryTag(Tag);
          TRakBinaryShortString.AddToTag(Tag, 'Name', 'TLedModule');
          TRakBinaryShortString.AddToTag(Tag, 'Color', ColorToString(Color));
        end;

        procedure TLedModule.LoadFromBinaryTag(Tag: TRakBinaryTag);
        begin
          inherited LoadFromBinaryTag(Tag);
          Color := StringToColor(Tag.AttributeAsString('Color', 'cllime'));
        end;

        procedure TLedModule.Paint(ACanvas: TCanvas; AVStartPoint: TPoint);
        var XPosition, YPosition, Tmp: integer;
        begin
          tmp := gDistance div 2 - 2 + gDistance mod 2;
          XPosition := Position.X * gDistance - AVStartPoint.X;
          YPosition := Position.Y * gDistance - AVStartPoint.Y;
          with ACanvas do
          begin
            if Status then
              Brush.Color := Color
            else Brush.Color := rgb(225,225,225);
            Pen.Color := clBlack;
            Ellipse(XPosition - Tmp, YPosition - Tmp, XPosition + tmp + 1, YPosition + tmp + 1);
            Brush.Color := clblack;
          end;
        end;

      {TTextModule}

      constructor TTextModule.Create(APosition: TPoint; ACaption: string; ASize: integer; AColor: TColor; AOwner: TObject);
      begin
        inherited Create(APosition, AOwner);
        Color := AColor;
        Caption := ACaption;
        FSize := ASize;
      end;

      procedure TTextModule.Paint(ACanvas: TCanvas; AVStartPoint: TPoint);
      var I: integer;
      begin
        with ACanvas do
        begin
          Font.Color := FColor;
          Font.Size := FSize;
          Pen.Color := clWhite;
          Brush.Style := bsClear;

          for I := 0 to (TextWidth(Caption) + 8) div gDistance do  // delete Grid
          begin
            MoveTo((Position.X + I) * gDistance, Position.Y * gDistance);
            LineTo((Position.X + I) * gDistance + 1, Position.Y * gDistance);
          end;
          
          Pen.Color := clBlack;

          Ellipse(Position.x * gDistance - 3 - AVStartPoint.X, Position.Y * gDistance - 3 - AVStartPoint.Y,
		 					Position.x * gDistance + 4 - AVStartPoint.X, Position.y * gDistance + 4 - AVStartPoint.Y);
          TextOut(Position.X * gDistance + 8 - AVStartPoint.X ,
                  Position.Y * gDistance - TextHeight(Caption) div 2 - 2 - AVStartPoint.Y, Caption);

          Font.Size := 8;
          Font.Color := clBlack;
          Brush.Style := bsSolid;
        end;
      end;

      procedure TTextModule.SaveToBinaryTag(Tag: TRakBinaryTag);
      begin
        inherited SaveToBinaryTag(Tag);
        TRakBinaryShortString.AddToTag(Tag, 'Name', 'TTextModule');
        TRakBinaryShortString.AddToTag(Tag, 'Caption', Caption);
        TRakBinaryLongint.AddToTag(Tag, 'Color', ord(Color));
        TRakBinaryLongint.AddToTag(Tag, 'Size', Size);
      end;

      procedure TTextModule.LoadFromBinaryTag(Tag: TRakBinaryTag);
      begin
        inherited LoadFromBinaryTag(Tag);
        Caption := Tag.AttributeAsString('Caption', '');
        Color := Tag.AttributeAsInteger('Color', clblack);
        Size := Tag.AttributeAsInteger('Size', 8);
      end;

end.
