(*
Options.pas
Copyright (C) 2006  Christian Brugger <brugger.christian(AT)web.de>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version. This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
details. You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA 02110, USA
*)
unit Options;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, Tabs, Registry, StdCtrls, ExtCtrls, IniFiles, ShlObj,
  ActiveX, ShellAPI;

type
  TFormMainOptions = class(TForm)
    pgOptions: TPageControl;
    tsInstallationIntegration: TTabSheet;
    btnReg: TButton;
    btnUnreg: TButton;
    gbIntegration: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    shReg: TShape;
    tShow: TTimer;
    Panel1: TPanel;
    btnOK: TButton;
    btnAbr: TButton;
    gbIniFile: TGroupBox;
    btnCreateHome: TButton;
    btnDeleteHome: TButton;
    btnCreateBinary: TButton;
    btnDeleteBinary: TButton;
    shConfigHome: TShape;
    shConfigBinary: TShape;
    cbPreferBinaryConfig: TCheckBox;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    cbDontRemindIntegration: TCheckBox;
    btnApply: TButton;
    tsGeneralOptions: TTabSheet;
    gbGeneralView: TGroupBox;
    cbClockGeneratorView: TCheckBox;
    cbStandardModulesView: TCheckBox;
    procedure FormShow(Sender: TObject);
    procedure btnRegClick(Sender: TObject);
    procedure btnUnregClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure tShowTimer(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure btnAbrClick(Sender: TObject);
    procedure btnCreateHomeClick(Sender: TObject);
    procedure btnDeleteHomeClick(Sender: TObject);
    procedure btnCreateBinaryClick(Sender: TObject);
    procedure btnDeleteBinaryClick(Sender: TObject);
    procedure btnApplyClick(Sender: TObject);
    procedure cbPreferBinaryConfigClick(Sender: TObject);
    procedure cbDontRemindIntegrationClick(Sender: TObject);
    procedure cbClockGeneratorViewClick(Sender: TObject);
    procedure cbStandardModulesViewClick(Sender: TObject);
  private
      { Global - Deklarationen }
    function InterfaceDifferFromFile: boolean;
      { Installtion and Integration - Deklarationen }
    function GetProgramPath(ASpecialFolder: integer): string;
    function InstallExt(Extension, ExtDescription, FileDescription,
      OpenWith, ParamString: string; IconIndex: Integer = 0): Boolean;
    function CheckInstallExt(Extension, ExtDescription, FileDescription,
      OpenWith, ParamString: string; IconIndex: Integer = 0): Boolean;
    function GetInstalledExt(Extension: string; var ExtDescription, FileDescription, OpenWith: string): Boolean;
    function UnInstallExt(Extension: String): Boolean;

    procedure SaveInstallationAndIntegration(AFile: string);
    procedure LoadInstallationAdnIntegration(AFile: string);
    procedure RefreshInstallationAndIntegration;
      { General Options - Deklarationen }
    procedure SaveGeneralOptions(AFile: string);
    procedure LoadGeneralOptions(AFile: string);
  public
    function GetRightConfigPath: string;
    procedure SaveSettingsToFile(AFile: string = '');
    procedure LoadSettingsFromFile(AFile: string = '');
    procedure Refresh;
  end;

const ShowNothing = 0;
      ShowOptions = 2;
      ShowBoth    = 3;

var
  FormMainOptions: TFormMainOptions;

implementation

{$R *.dfm}

uses About, Haupt, OptionsStandardModules, OptionsClockGenerator;


{*-----------------------------------Global-----------------------------------*}


function TFormMainOptions.InterfaceDifferFromFile: boolean;
var IniFile: TIniFile;
  s: string;
begin
  Result := false;
  s := GetRightConfigPath;
  if s <> '' then
  begin
    IniFile := TIniFile.Create(s);
    try
      if (cbDontRemindIntegration.Checked xor IniFile.ReadBool('General','DontRememberIntegration', false))
        or (cbClockGeneratorView.Checked xor IniFile.ReadBool('General','ClockGeneratorView', false))
        or (cbStandardModulesView.Checked xor IniFile.ReadBool('General','StandardModulesView', false)) then
        Result := true;
    finally
      IniFile.Free;
    end;
  end;
end;

function TFormMainOptions.GetRightConfigPath;
const NoConfig     = 0;
      BinaryConfig = 1;
      HomeConfig   = 2;
var LoadFrom: integer;
  IniFile: TIniFile;
begin

  LoadFrom := NoConfig;
  if FileExists(ExtractFilePath(ParamStr(0)) + 'config.ini') then
  begin
    LoadFrom := BinaryConfig;
    IniFile := TIniFile.Create(ExtractFilePath(ParamStr(0)) + 'config.ini');
    try
      if not IniFile.ReadBool('General', 'Prefer', false) 
        and FileExists(GetProgramPath(CSIDL_APPDATA)+'\LogicSim\'+'config.ini') then
        LoadFrom := HomeConfig;
    finally
      IniFile.Free;
    end;
  end else if FileExists(GetProgramPath(CSIDL_APPDATA)+'\LogicSim\'+'config.ini') then
    LoadFrom := HomeConfig;

  case LoadFrom of
    NoConfig: Result := '';
    BinaryConfig: Result := ExtractFilePath(ParamStr(0)) + 'config.ini';
    HomeConfig: Result := GetProgramPath(CSIDL_APPDATA)+'\LogicSim\'+'config.ini';
  end;
end;

procedure TFormMainOptions.SaveSettingsToFile(AFile: string = '');
begin
  Refresh;
  if AFile = '' then AFile := GetRightConfigPath;
  
  SaveInstallationAndIntegration(AFile);
  SaveGeneralOptions(AFile);

  // check forms
  with FormOptionsStandardModules do
    if cbStandardModulesView.Checked then
      if Visible and Haupt.IsPowerOn then
        Hide
      else
    else if Tag = 1 then // if hidden
        Show;
  with FormOptionsClockGenerator do
    if cbClockGeneratorView.Checked then
      if Visible and not (Haupt.IsPowerOn and Form1.Simulation.ClockGeneratorExists) then
        Hide
      else
    else if Tag = 1 then // if hidden
        Show;
end;

procedure TFormMainOptions.LoadSettingsFromFile(AFile: string = '');
begin
  if AFile = '' then AFile := GetRightConfigPath;

  LoadInstallationAdnIntegration(AFile);
  LoadGeneralOptions(AFile);

  Refresh;
end; 

procedure TFormMainOptions.Refresh;
begin
  RefreshInstallationAndIntegration;
end;

procedure TFormMainOptions.FormCreate(Sender: TObject);
var IniFile: TIniFile;
  s: string;
begin
  cbClockGeneratorView.Hint := 'Blendet das Taktgeneratorfenster aus, wenn' + #13#10 +
                    '  der Editiermodus aktiviert ist oder wenn'+ #13#10 +
                    '  kein Taktgenerator existiert';
  tShow.Tag := showNothing;
  s := GetRightConfigPath;
  if s = '' then
    tShow.Tag := ShowBoth           // not Config exists --> first start
  else if not CheckInstallExt('.sim', 'LogicSim', 'Simulation', ParamStr(0), '"%1"',1) then
  begin
    IniFile := TIniFile.Create(s);  // config exists
    try
      if IniFile.ReadBool('General','DontRememberIntegration', false) then
        tShow.Tag := ShowNothing
      else tShow.Tag := ShowOptions;
    finally
      IniFile.Free;
    end;
  end;

  if tShow.Tag <> showNothing then
    tShow.Enabled := true;
end;

procedure TFormMainOptions.FormShow(Sender: TObject);
begin
  pgOptions.ActivePage := tsInstallationIntegration;
  LoadSettingsFromFile;
end;

procedure TFormMainOptions.tShowTimer(Sender: TObject);
begin
  tShow.Enabled := false;

  if tShow.Tag = showBoth then
    AboutBox.ShowModal;
  if tShow.Tag in [showOptions, showBoth] then
    ShowModal;
end;


{*------------------------Installation and Integration------------------------*}


function TFormMainOptions.GetProgramPath(ASpecialFolder: integer): string;
var
  p: PItemIDLIst;
  Buf: array [0..MAX_PATH-1] of Char;
  ShellH: IMalloc;
begin
  if SHGetSpecialFolderLocation(Application.Handle, ASpecialFolder, p) = NOERROR   then
  try
    if SHGetPathFromIDList(p, Buf) then
      Result := Buf;
    finally
      if SHGetMalloc(ShellH) = NOERROR then
         ShellH.Free(P);
    end;
end;

function TFormMainOptions.InstallExt(Extension, ExtDescription, FileDescription,
OpenWith, ParamString: string; IconIndex: Integer = 0): Boolean;
var
  Reg: TRegistry;
begin
  Result := False;
  if Extension <> '' then
  begin
    if Extension[1] <> '.' then
      Extension := '.' + Extension;
    Reg := TRegistry.Create;
  	try
			Reg.RootKey := HKEY_CURRENT_USER;
			if Reg.OpenKey('\' + 'Software\Classes', False) then
				if Reg.OpenKey(Extension, True) then
				begin
					Reg.WriteString('', ExtDescription);
					Reg.OpenKey('\' + 'Software\Classes', False);
					if Reg.OpenKey(ExtDescription, True) then
					begin
						Reg.WriteString('', FileDescription);
						if Reg.OpenKey('DefaultIcon', True) then
						begin
							Reg.WriteString('', Format('%s,%d', [OpenWith, IconIndex]));
							Reg.OpenKey('\' + 'Software\Classes', False);
							if Reg.OpenKey(ExtDescription + '\Shell\Open\Command', True) then
							begin
								Reg.WriteString('', Format('"%s" %s', [OpenWith, ParamString]));
								Result:=True;
							end;
						end;
					end;
				end;
		finally
			Reg.Free;
		end;
  end;
end;

function TFormMainOptions.CheckInstallExt(Extension, ExtDescription, FileDescription,
OpenWith, ParamString: string; IconIndex: Integer = 0): Boolean;
var
  Reg: TRegistry;
begin
  Result := False;
  if Extension <> '' then
  begin
    if Extension[1] <> '.' then
      Extension := '.' + Extension;

    Reg := TRegistry.Create;
    try
      Reg.RootKey := HKEY_CURRENT_USER;
      if Reg.OpenKeyReadOnly('\' + 'Software\Classes')
        and Reg.OpenKeyReadOnly(Extension)
        and (ExtDescription = Reg.ReadString(''))
        and Reg.OpenKeyReadOnly('\' + 'Software\Classes')
        and Reg.OpenKeyReadOnly(ExtDescription)
        and (FileDescription = Reg.ReadString(''))
        and Reg.OpenKeyReadOnly('DefaultIcon')
        and (Format('%s,%d', [OpenWith, IconIndex]) = Reg.ReadString(''))
        and Reg.OpenKeyReadOnly('\' + 'Software\Classes')
        and Reg.OpenKeyReadOnly(ExtDescription + '\Shell\Open\Command') 
        and (Format('"%s" %s', [OpenWith, ParamString]) = Reg.ReadString('')) then
          Result:=True;
    finally
      Reg.Free;
    end;
  end;
end;

function TFormMainOptions.GetInstalledExt(Extension: string; var ExtDescription, FileDescription, OpenWith: string): Boolean;
var
  Reg: TRegistry;
  I: integer;
begin
  Result := False;
  if Extension <> '' then
  begin
    if Extension[1] <> '.' then
      Extension := '.' + Extension;
    Reg := TRegistry.Create;
    try
      Reg.RootKey := HKEY_CURRENT_USER;
      if Reg.OpenKeyReadOnly('\' + 'Software\Classes') and Reg.OpenKeyReadOnly(Extension) then
      begin
       ExtDescription := Reg.ReadString('');
       if Reg.OpenKeyReadOnly('\' + 'Software\Classes') and Reg.OpenKeyReadOnly(ExtDescription) then
       begin
         FileDescription := Reg.ReadString('');
         if Reg.OpenKeyReadOnly('DefaultIcon') then
         begin
           OpenWith := Reg.ReadString('');
           I := length(OpenWith);
           while (I > -1) and (OpenWith[I] <> ',') do
            dec(I);
           OpenWith := copy(OpenWith, 0, I - 1);
           
           Result:=True;
         end;
       end;
     end;
  finally
    Reg.Free;
  end;
  end;
end;

function TFormMainOptions.UnInstallExt(Extension: String): Boolean;
var
  Reg: TRegistry;
  Description: String;
begin
  Result := False;
  if Extension <> '' then
    begin
    if Extension[1] <> '.' then
      Extension := '.' + Extension;
    Reg := TRegistry.Create;
    try
      Reg.RootKey := HKEY_CURRENT_USER;
      if Reg.OpenKeyReadOnly('\' + 'Software\Classes')
        and Reg.KeyExists(Extension) then
      begin
        Reg.OpenKey(Extension, false);
        Description := Reg.ReadString('');
        Reg.CloseKey;
        if Reg.OpenKeyReadOnly('\' + 'Software\Classes')
          and Reg.KeyExists(Description) then
          Result := Reg.DeleteKey(Extension) and
                    Reg.OpenKeyReadOnly('\' + 'Software\Classes') and Reg.DeleteKey(Description);
      end;
    finally
      Reg.Free;
    end;
  end;
end;

procedure TFormMainOptions.btnAbrClick(Sender: TObject);
begin
  close;
end;

procedure TFormMainOptions.btnApplyClick(Sender: TObject);
begin
  SaveSettingsToFile;
  LoadSettingsFromFile;
end;

procedure TFormMainOptions.btnOKClick(Sender: TObject);
begin
  SaveSettingsToFile;  
  close;
end;

procedure TFormMainOptions.btnCreateBinaryClick(Sender: TObject);
var IniFile: TIniFile;
begin
  IniFile := TIniFile.Create(ExtractFilePath(ParamStr(0)) + 'config.ini');
  try
    IniFile.WriteBool('General','DontRememberIntegration',false);
    IniFile.WriteBool('General', 'Prefer', false);
  finally
    IniFile.Free;
  end;
  
  Refresh;
end;

procedure TFormMainOptions.btnCreateHomeClick(Sender: TObject);
var IniFile: TIniFile;
begin
  if not DirectoryExists(GetProgramPath(CSIDL_APPDATA)+'\LogicSim') then
    MkDir(GetProgramPath(CSIDL_APPDATA)+'\LogicSim');
    
  IniFile := TIniFile.Create(GetProgramPath(CSIDL_APPDATA)+'\LogicSim\'+'config.ini');
  try
    IniFile.WriteBool('General','DontRememberIntegration',false);
  finally
    IniFile.Free;
  end;
  
  Refresh;
end;

procedure TFormMainOptions.btnDeleteBinaryClick(Sender: TObject);
begin
  if FileExists(ExtractFilePath(ParamStr(0)) + 'config.ini') then
    DeleteFile(ExtractFilePath(ParamStr(0)) + 'config.ini');
  Refresh;
end;

procedure TFormMainOptions.btnDeleteHomeClick(Sender: TObject);
begin
  if FileExists(GetProgramPath(CSIDL_APPDATA)+'\LogicSim\'+'config.ini') then
  begin
    DeleteFile(GetProgramPath(CSIDL_APPDATA)+'\LogicSim\'+'config.ini');
    RemoveDir(GetProgramPath(CSIDL_APPDATA)+'\LogicSim\');
  end;
  Refresh;
end;

procedure TFormMainOptions.btnRegClick(Sender: TObject);
begin
  if not InstallExt('.sim', 'LogicSim', 'Simulation', ParamStr(0), '"%1"',1) then
    showmessage('Sie besitzen nicht die bentigten Rechte, um die Dateiendung zu registrieren.');
  Refresh;
end;

procedure TFormMainOptions.btnUnregClick(Sender: TObject);
begin
  if not UnInstallExt('.sim') then
    showmessage('Sie besitzen nicht die bentigten Rechte, um die Dateiendung zu deregistrieren.');
  Refresh;
end;

procedure TFormMainOptions.cbDontRemindIntegrationClick(Sender: TObject);
begin
  btnApply.Enabled := InterfaceDifferFromFile;
end;

procedure TFormMainOptions.cbPreferBinaryConfigClick(Sender: TObject);
var IniFile: TIniFile;
begin
  if (Sender = cbPreferBinaryConfig) and FileExists(ExtractFilePath(ParamStr(0)) + 'config.ini') then
  begin
    IniFile := TIniFile.Create(ExtractFilePath(ParamStr(0)) + 'config.ini');
    try
      if IniFile.ReadBool('General', 'Prefer', false) xor cbPreferBinaryConfig.Checked then
      begin
        Form1.SaveToIniFile(GetRightConfigPath);
        IniFile.WriteBool('General', 'Prefer', cbPreferBinaryConfig.Checked);
        Form1.LoadFromIniFile(GetRightConfigPath);
        self.BringToFront;
        LoadSettingsFromFile;
      end;
    finally
      IniFile.Free;
    end;
  end;
end;

procedure TFormMainOptions.SaveInstallationAndIntegration(AFile: string);
var IniFile: TIniFile;
begin
  if AFile <> '' then
  begin
    IniFile := TIniFile.Create(AFile);
    try
      IniFile.WriteBool('General','DontRememberIntegration', cbDontRemindIntegration.Checked);
    finally
      IniFile.Free;
    end;
  end;
  btnApply.Enabled := false;
end;

procedure TFormMainOptions.LoadInstallationAdnIntegration(AFile: string);
var IniFile: TIniFile;
begin
  if AFile <> '' then
  begin
    IniFile := TIniFile.Create(AFile);
    try
      cbDontRemindIntegration.Checked := IniFile.ReadBool('General','DontRememberIntegration', false);
      cbPreferBinaryConfig.Checked := IniFile.ReadBool('General', 'Prefer', false)
    finally
      IniFile.Free;
    end;
  end else begin
    cbPreferBinaryConfig.Checked := false;
    cbDontRemindIntegration.Checked := false;
  end;

  btnApply.Enabled := false;
end;

procedure TFormMainOptions.RefreshInstallationAndIntegration;
var ExtDescription, FileDescription, OpenWith: string;
  HomeConfigExists, BinaryConfigExists, Integrated: boolean;
  IniFile: TIniFile;
  aShape: TShape;
begin
                                        // Integration
  if CheckInstallExt('.sim', 'LogicSim', 'Simulation', ParamStr(0), '"%1"',1) then
  begin
    shReg.Brush.Color := clLime;
    shReg.Hint := 'Dateiendung registriert';
    Integrated := true;
    btnReg.Enabled := false;
    btnUnReg.Enabled := true;
  end else begin
    if GetInstalledExt('.sim', ExtDescription, FileDescription, OpenWith) then
    begin
      shReg.Brush.Color := rgb(255,128,0);
      shReg.Hint := 'Dateiendung ist fr ein anderes Programm registriert:' + #13#10 +
                    'Kurzbeschreibung: ' + ExtDescription + #13#10 +
                    'Programminfo: ' + FileDescription + #13#10 +
                    'Pfad: ' + OpenWith;
      Integrated := false;
      btnReg.Enabled := true;
      btnUnReg.Enabled := true;
    end else begin
      shReg.Brush.Color := clRed;
      shReg.Hint := 'Dateiendung nicht registriert';
      Integrated := false;
      btnReg.Enabled := true;
      btnUnReg.Enabled := false;
    end;
  end;                   
                                         // Config-File
  if FileExists(GetProgramPath(CSIDL_APPDATA)+'\LogicSim\'+'config.ini') then
  begin                 // Home
    HomeConfigExists := true;
    btnCreateHome.Enabled := false;
    btnDeleteHome.Enabled := true;
    shConfigHome.Brush.Color := cllime;
    shConfigHome.Hint := 'Config ist vorhanden und wird verwendet.';
  end else begin
    HomeConfigExists := false;
    btnCreateHome.Enabled := true;
    btnDeleteHome.Enabled := false;
    shConfigHome.Brush.Color := clred;
    shConfigHome.Hint := 'Keine Config vorhanden.';
  end;                  // Binary
  if FileExists(ExtractFilePath(ParamStr(0)) + 'config.ini') then
  begin
    BinaryConfigExists := true;
    btnCreateBinary.Enabled := false;
    btnDeleteBinary.Enabled := true;
    cbPreferBinaryConfig.Enabled := true;
    shConfigBinary.Brush.Color := cllime;
    shConfigBinary.Hint := 'Config ist vorhanden und wird verwendet.';
  end else begin
    BinaryConfigExists := false;
    btnCreateBinary.Enabled := true;
    btnDeleteBinary.Enabled := false;
    cbPreferBinaryConfig.Enabled := false;
    shConfigBinary.Brush.Color := clred;
    shConfigBinary.Hint := 'Keine Config vorhanden.';
  end;
                        // One exists
  if HomeConfigExists or BinaryConfigExists then
    if Integrated then
    begin
      cbDontRemindIntegration.Enabled := false;
      //cbDontRemindIntegration.Checked := false;
    end else cbDontRemindIntegration.Enabled := true
  else cbDontRemindIntegration.Enabled := false;
                        // Both exists
  if HomeConfigExists and BinaryConfigExists then
  begin
    IniFile := TIniFile.Create(ExtractFilePath(ParamStr(0)) + 'config.ini');
    try
      if IniFile.ReadBool('General', 'Prefer', false) then
        aShape := shConfigHome
      else aShape := shConfigBinary;

      aShape.Brush.Color := rgb(255,128,0);
      aShape.Hint := 'Die Config ist zwar vorhanden, wird allerdings nicht verwendet!';
    finally
      IniFile.Free;
    end;
  end;
end;


{*-------------------------------GeneralOptions-------------------------------*}


procedure TFormMainOptions.cbClockGeneratorViewClick(Sender: TObject);
begin
  btnApply.Enabled := InterfaceDifferFromFile;
end;

procedure TFormMainOptions.cbStandardModulesViewClick(
  Sender: TObject);
begin
  btnApply.Enabled := InterfaceDifferFromFile;
end;

procedure TFormMainOptions.SaveGeneralOptions(AFile: string);
var IniFile: TIniFile;
begin
  if AFile <> '' then
  begin
    IniFile := TIniFile.Create(AFile);
    try
      IniFile.WriteBool('General','ClockGeneratorView', cbClockGeneratorView.Checked);
      IniFile.WriteBool('General','StandardModulesView', cbStandardModulesView.Checked);
    finally
      IniFile.Free;
    end;
  end;
  btnApply.Enabled := false;
end;

procedure TFormMainOptions.LoadGeneralOptions(AFile: string);
var IniFile: TIniFile;
begin
  if AFile <> '' then
  begin
    IniFile := TIniFile.Create(AFile);
    try
      cbClockGeneratorView.Checked := IniFile.ReadBool('General','ClockGeneratorView', false);
      cbStandardModulesView.Checked := IniFile.ReadBool('General','StandardModulesView', false);
    finally
      IniFile.Free;
    end;
  end;
  btnApply.Enabled := false;
end;

end.
